// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// **************************************
// Header Files
// **************************************

#include "device.h"
#include "mymessage.h"

// **************************
// Class Declaration
// **************************

Device::Device(void)
{  // constructor
}

Device::~Device(void)
{  // destructor
}

Device::Device(const Device &dev)
{  // copy constructor

  identifier=dev.GetId();
  driver=dev.GetDriver();
  busID=dev.GetBus();
  screen=dev.GetScreen();
  videoRam=dev.GetVideoRam();

  identifierComment=dev.GetIdComment();
  driverComment=dev.GetDriverComment();
  busIDComment=dev.GetBusComment();
  screenComment=dev.GetScreenComment();
  videoRamComment=dev.GetVideoRamComment();

  comments=dev.Comments();
  options=dev.Options();
}

Device& Device::operator=(const Device &dev)
{  // copy constructor

  Clear();

  identifier=dev.GetId();
  driver=dev.GetDriver();
  busID=dev.GetBus();
  screen=dev.GetScreen();
  videoRam=dev.GetVideoRam();

  identifierComment=dev.GetIdComment();
  driverComment=dev.GetDriverComment();
  busIDComment=dev.GetBusComment();
  screenComment=dev.GetScreenComment();
  videoRamComment=dev.GetVideoRamComment();

  comments=dev.Comments();
  options=dev.Options();

  return(*this);
}

void Device::SetId(const MyString& id, const MyString& com)
{  // set identifier

  identifier=id;
  identifierComment=com;
}

void Device::SetDriver(const MyString& dr, const MyString& com)
{  // set driver

  driver=dr;
  driverComment=com;
}

void Device::SetBus(const MyString& bus, const MyString& com)
{  // set bus type

  busID=bus;
  busIDComment=com;
}

void Device::SetScreen(const MyString& scr, const MyString& com)
{  // set screen to use

  screen=scr;
  screenComment=com;
}

void Device::SetVideoRam(const MyString& ram, const MyString& com)
{  // set video ram size

  videoRam=ram;
  videoRamComment=com;
}

void Device::Clear(void)
{    // clear everything

  identifier.Empty();
  driver.Empty();
  busID.Empty();
  screen.Empty();
  videoRam.Empty();
  identifierComment.Empty();
  driverComment.Empty();
  busIDComment.Empty();
  screenComment.Empty();
  videoRamComment.Empty();
  comments.Clear();
  options.Clear();
}

const MyString& Device::GetId(void) const
{  // return identifier

  return(identifier);
}

const MyString& Device::GetDriver(void) const
{  // retun driver name

  return(driver);
}

const MyString& Device::GetBus(void) const
{  // return bus id

  return(busID);
}

const MyString& Device::GetScreen(void) const
{  // return used screen

  return(screen);
}

const MyString& Device::GetVideoRam(void) const
{  // return video ram size

  return(videoRam);
}

const MyString& Device::GetIdComment(void) const
{  // return identifier comment

  return(identifierComment);
}

const MyString& Device::GetDriverComment(void) const
{  // retun driver name comment

  return(driverComment);
}

const MyString& Device::GetBusComment(void) const
{  // return bus id comment

  return(busIDComment);
}

const MyString& Device::GetScreenComment(void) const
{  // return used screen comment

  return(screenComment);
}

const MyString& Device::GetVideoRamComment(void) const
{  // return video ram size comment

  return(videoRamComment);
}

const OptionList& Device::Options(void) const
{  // get options

  return(options);
}

OptionList& Device::Options(void)
{  // get options

  return(options);
}

const StringArray& Device::Comments(void) const
{  // return comments
  return(comments);
}

StringArray& Device::Comments(void)
{  // return comments as reference
  return(comments);
}

const bool Device::Check(void)
{  /// Check if everything has correct syntax.

  if (identifier.IsEmpty())
  {
    MyMessage::Set(ERROR_CHECK_EMPTY_DEVICE_ID, "Device::Check");
    return(true);
  }

  if (driver.IsEmpty())
  {
    MyMessage::Set(ERROR_CHECK_EMPTY_DEVICE_DRIVER, "Device::Check");
    return(true);
  }

  if (!screen.IsEmpty())
  {
    if (!screen.IsInt() || screen.toInt()<0)
    {
      MyMessage::Set(ERROR_CHECK_DEVICE_SCREEN_NUMBER, "Device::Check");
      return(true);
    }
  }

  if (!videoRam.IsEmpty())
  {
    if (!videoRam.IsInt() || videoRam.toInt()<0)
    {
      MyMessage::Set(ERROR_CHECK_DEVICE_VIDEORAM, "Device::Check");
      return(true);
    }
  }

  if (options.Check())
  {
    if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_DEVICE_OPTION, "Device::Check");
    }
    else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_SAME_DEVICE_OPTION, "Device::Check");
    }
    else
    {
      MyMessage::Append("Device::Check");
    }
    return(true);
  }

  return(false);
}

const bool Device::ConvertOptions(const DriverOptionList& list)
{  /// Compare options with list and set correct cases.

  int pos, pos2;
  MyString temp;

  // go through options
  for (unsigned int i=0; i<options.GetSize(); i++)
  {
    pos=list.Find(options(i).GetName(), true);
    if (!MyMessage::OK())
    {
      MyMessage::Append("Device::ConvertOptions");
      return(true);
    }

    if (pos!=-1)
    {
      if (options(i).GetName()!=list(pos).GetName())
      {
        options(i).SetName(list(pos).GetName());
      }
      if (!MyMessage::OK())
      {
        MyMessage::Append("Device::ConvertOptions");
        return(true);
      }

      pos2=list(pos).GetArguments().Find(options(i).GetValue(), 0, true);

      if (pos2!=-1)
      {
        if (options(i).GetValue()!=list(pos).GetArguments()(pos2))
        {
          options(i).SetValue(list(pos).GetArguments()(pos2));
        }
        if (!MyMessage::OK())
        {
          MyMessage::Append("Device::ConvertOptions");
          return(true);
        }
      } // pos2!=-1
    } // pos!=-1

    // convert all to lower case letters
    temp=options(i).GetValue().toLower();
    if (temp=="yes") options(i).SetValue("yes");
    else if (temp=="no") options(i).SetValue("no");
    else if (temp=="true") options(i).SetValue("true");
    else if (temp=="false") options(i).SetValue("false");
    else if (temp=="on") options(i).SetValue("on");
    else if (temp=="off") options(i).SetValue("off");
  } // for i

  return(false);
}

const bool Device::Check(const DriverOptionList& list)
{  /// Check if options are correct.

  int pos;
  MyString temp;

  // go through options
  for (unsigned int i=0; i<options.GetSize(); i++)
  {
    pos=list.Find(options(i).GetName(), true);
    if (!MyMessage::OK())
    {
      MyMessage::Append("Device::Check");
      return(true);
    }

    if (pos!=-1)
    {
      // check if boolean
      if (list(pos).GetValue()=="boolean")
      {
        temp=options(i).GetValue().toLower();
        if (temp!="yes" && temp!="no" && temp!="true" && temp!="false" &&
            temp!="on" && temp!="off" && temp!="0" && temp!="1")
        {
          options.SetCurrent(i);
          MyMessage::Set(ERROR_CHECK_DEVICE_OPTION_BOOL, "Device::Check");
          return(true);
        }
      }
      else  // not bool
      {
        if (list(pos).GetValue()=="integer")
        {
          if (!options(i).GetValue().IsNumber())
          {
            options.SetCurrent(i);
            MyMessage::Set(ERROR_CHECK_DEVICE_OPTION_INT, "Device::Check");
            return(true);
          }
        }

        if (list(pos).IsStrict())
        {
          if (list(pos).GetArguments().GetSize()>0 && list(pos).GetArguments().Find(options(i).GetValue(), 0, true)==-1)
          {
            options.SetCurrent(i);
            MyMessage::Set(ERROR_CHECK_DEVICE_OPTION_LIST, "Device::Check");
            return(true);
          }
        }
      }
    }
  }  // for i

  return(false);
}
