// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef DEVICE_H
#define DEVICE_H

// **************************************
// Header Files
// **************************************

#include "stringarray.h"
#include "optionlist.h"
#include "driveroptionlist.h"

// **************************
// Class Definition
// **************************

class Device
{
public:
  /// Constructor.
  Device(void);

  /// Copy Constructor.
  Device(const Device &dev);

  /// Destructor.
  ~Device(void);

  /// Assignment operator.
  Device& operator=(const Device &dev);

  /// Set device identifier and optional comment.
  void SetId(const MyString& id, const MyString& com=MyString());

  /// Set device driver and optional comment.
  void SetDriver(const MyString& dr, const MyString& com=MyString());

  /// Set device bus id and optional comment.
  void SetBus(const MyString& bus, const MyString& com=MyString());

  /// Set device screen number and optional comment.
  void SetScreen(const MyString& scre, const MyString& com=MyString());

  /// Set device video ram and optional comment.
  void SetVideoRam(const MyString& ram, const MyString& com=MyString());

  /// Get device identifier.
  const MyString& GetId(void) const;

  /// Get device driver.
  const MyString& GetDriver(void) const;

  /// Get device bus id.
  const MyString& GetBus(void) const;

  /// Get device screen number.
  const MyString& GetScreen(void) const;

  /// Get device video ram.
  const MyString& GetVideoRam(void) const;

  /// Get device identifier comment.
  const MyString& GetIdComment(void) const;

  /// Get device driver comment.
  const MyString& GetDriverComment(void) const;

  /// Get device bus id comment.
  const MyString& GetBusComment(void) const;

  /// Get device screen number comment.
  const MyString& GetScreenComment(void) const;

  /// Get device video ram comment.
  const MyString& GetVideoRamComment(void) const;

  /// Get comments.
  const StringArray& Comments(void) const;

  /// Get comments (as reference).
  StringArray& Comments(void);

  /// Get options.
  const OptionList& Options(void) const;

  /// Get options (as reference).
  OptionList& Options(void);

  /// Clear everything.
  void Clear(void);

  /// Check if everything has correct syntax.
  /**
  * Return <b>true</b> if something is wrong else <b>false</b>.
  */
  const bool Check(void);

  /// Check if options are correct.
  /**
  * Return <b>true</b> if something is wrong else <b>false</b>.
  */
  const bool Check(const DriverOptionList& list);

  /// Compare options with list and set correct cases.
  /**
  * Return <b>true</b> if something is wrong else <b>false</b>.
  */
  const bool ConvertOptions(const DriverOptionList& list);

private:
  MyString identifier;      // card identifier
  MyString driver;          // driver name
  MyString busID;           // bus type
  MyString screen;          // which screen is used
  MyString videoRam;        // video ram option

  // Note: Any additional settings will not be recognized like:
  // Chipset, Ramdac, DacSpeed, Clocks, ClockChip,
  // BiosBase, MemBase, IOBase, ChipID, ChipRev, TextClockFreq

  MyString identifierComment;
  MyString driverComment;
  MyString busIDComment;
  MyString screenComment;
  MyString videoRamComment;
  StringArray comments;       // comment for whole block
  OptionList options;        // all options
};

#endif // DEVICE_H
