// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// **************************************
// Header Files
// **************************************

#include "display.h"
#include "mymessage.h"

// **************************
// Class Declaration
// **************************

Display::Display(void)
{  // constructor
}

Display::~Display(void)
{  // destructor
}

Display::Display(const Display &dis)
{  // copy constructor

  depth=dis.GetDepth();
  viewport=dis.GetViewPort();
  virtualE=dis.GetVirtual();
  depthComment=dis.GetDepthComment();
  viewportComment=dis.GetViewPortComment();
  virtualEComment=dis.GetVirtualComment();
  comments=dis.Comments();
  options=dis.Options();
  modes=dis.Modes();

}

Display& Display::operator=(const Display &dis)
{  // copy constructor

  Clear();

  depth=dis.GetDepth();
  viewport=dis.GetViewPort();
  virtualE=dis.GetVirtual();
  depthComment=dis.GetDepthComment();
  viewportComment=dis.GetViewPortComment();
  virtualEComment=dis.GetVirtualComment();
  comments=dis.Comments();
  options=dis.Options();
  modes=dis.Modes();
  return(*this);
}

void Display::SetDepth(const MyString& dd, const MyString& com)
{  // set depth

  depth=dd;
  depthComment=com;
}

void Display::SetViewPort(const MyString& vw, const MyString& com)
{  // set viewport

  viewport=vw;
  viewportComment=com;
}

void Display::SetVirtual(const MyString& vi, const MyString& com)
{  // set virtual

  virtualE=vi;
  virtualEComment=com;
}

void Display::Clear(void)
{    // clear everything

  depth.Empty();
  virtualE.Empty();
  viewport.Empty();
  depthComment.Empty();
  virtualEComment.Empty();
  viewportComment.Empty();
  comments.Clear();
  options.Clear();
  modes.Clear();
}

const MyString& Display::GetDepth(void) const
{  // return depth

  return(depth);
}

const MyString& Display::GetViewPort(void) const
{  // return viewport

  return(viewport);
}

const MyString& Display::GetVirtual(void) const
{  // return virtual

  return(virtualE);
}

const MyString& Display::GetDepthComment(void) const
{  // return depth Comment

  return(depthComment);
}

const MyString& Display::GetViewPortComment(void) const
{  // return viewport Comment

  return(viewportComment);
}

const MyString& Display::GetVirtualComment(void) const
{  // return virtual Comment

  return(virtualEComment);
}

const OptionList& Display::Options(void) const
{  // get options

  return(options);
}

OptionList& Display::Options(void)
{  // get options

  return(options);
}

const OptionList& Display::Modes(void) const
{  // get modes

  return(modes);
}

OptionList& Display::Modes(void)
{  // get modes

  return(modes);
}

const StringArray& Display::Comments(void) const
{  // return comments
  return(comments);
}

StringArray& Display::Comments(void)
{  // return comments as reference
  return(comments);
}

const bool Display::Check(void)
{  /// Check if everything has correct syntax.

  if (depth.IsEmpty())
  {
    MyMessage::Set(ERROR_CHECK_SCREEN_DISPLAY_DEPTH, "Display::Check");
    return(true);
  }

  if (!viewport.IsEmpty())
  {
    MyString tempStr=viewport;
    tempStr.StripSpaces();
    int pos=tempStr.Find(' ');
    if (pos==-1)
    {
      MyMessage::Set(ERROR_CHECK_SCREEN_DISPLAY_VIEWPORT, "Display::Check");
      return(true);
    }

    if (!tempStr.Left(pos).IsInt() || tempStr.toInt()<0)
    {
      MyMessage::Set(ERROR_CHECK_SCREEN_DISPLAY_VIEWPORT, "Display::Check");
      return(true);
    }

    tempStr.StripRight(pos);
    if (!tempStr.IsInt() || tempStr.toInt()<0)
    {
      MyMessage::Set(ERROR_CHECK_SCREEN_DISPLAY_VIEWPORT, "Display::Check");
      return(true);
    }
  }

  if (!virtualE.IsEmpty())
  {
    MyString tempStr=virtualE;
    tempStr.StripSpaces();
    int pos=tempStr.Find(' ');
    if (pos==-1)
    {
      MyMessage::Set(ERROR_CHECK_SCREEN_DISPLAY_VIRTUAL, "Display::Check");
      return(true);
    }

    if (!tempStr.Left(pos).IsInt() || tempStr.toInt()<0)
    {
      MyMessage::Set(ERROR_CHECK_SCREEN_DISPLAY_VIRTUAL, "Display::Check");
      return(true);
    }

    tempStr.StripRight(pos);
    if (!tempStr.IsInt() || tempStr.toInt()<0)
    {
      MyMessage::Set(ERROR_CHECK_SCREEN_DISPLAY_VIRTUAL, "Display::Check");
      return(true);
    }
  }

  if (options.Check())
  {
    if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_SCREEN_DISPLAY_OPTION, "Display::Check");
    }
    else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_SAME_SCREEN_DISPLAY_OPTION, "Display::Check");
    }
    else
    {
      MyMessage::Append("Display::Check");
    }
    return(true);
  }

  if (modes.Check())
  {
    if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_SCREEN_DISPLAY_MODE, "Display::Check");
    }
    else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_SAME_SCREEN_DISPLAY_MODE, "Display::Check");
    }
    else
    {
      MyMessage::Append("Display::Check");
    }
    return(true);
  }

  return(false);
}



