// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// **************************************
// Header Files
// **************************************

#include "extensionlist.h"
#include "mymessage.h"

// **************************
// Class Declaration
// **************************

ExtensionList::ExtensionList(void) : OptionList()
{  // constructor
}

ExtensionList::~ExtensionList(void)
{  // destructor
}

ExtensionList::ExtensionList(const ExtensionList &ext) : OptionList(ext)
{  // copy constructor

  comments=ext.Comments();
}

ExtensionList& ExtensionList::operator=(const ExtensionList &ext)
{  // copy constructor

  Clear(); // clear all

  OptionList::operator=(ext);
  comments=ext.Comments();

  return(*this);
}

void ExtensionList::Clear(void)
{    // clear everything

  OptionList::Clear();
  comments.Clear();
}

const StringArray& ExtensionList::Comments(void) const
{  // return comments
  return(comments);
}

StringArray& ExtensionList::Comments(void)
{  // return comments as reference
  return(comments);
}

const bool ExtensionList::Check(void)
{  /// Check if everything has correct syntax.

  if (OptionList::Check())
  {
    if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_EXTENSION, "ExtensionList::Check");
    }
    else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_SAME_EXTENSION, "ExtensionList::Check");
    }
    else
    {
      MyMessage::Append("ExtensionList::Check");
    }
    return(true);
  }

  return(false);

}

