// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// **************************************
// Header Files
// **************************************

#include "filelist.h"
#include "mymessage.h"

// **************************
// Class Declaration
// **************************

FileList::FileList(void)
{  // constructor
}

FileList::~FileList(void)
{  // destructor
}

FileList::FileList(const FileList &list)
{  // copy constructor

  comments=list.Comments();
  fontPaths=list.FontPaths();
  modulePaths=list.ModulePaths();
  rgbPaths=list.RgbPaths();
}

FileList& FileList::operator=(const FileList &list)
{  // copy constructor

  // clear old entries
  Clear();

  comments=list.Comments();
  fontPaths=list.FontPaths();
  modulePaths=list.ModulePaths();
  rgbPaths=list.RgbPaths();

  return(*this);
}

void FileList::Clear(void)
{  // clear all

  comments.Clear();
  fontPaths.Clear();
  modulePaths.Clear();
  rgbPaths.Clear();
}

const OptionList& FileList::FontPaths(void) const
{  // get font paths

  return(fontPaths);
}

OptionList& FileList::FontPaths(void)
{  // get font paths

  return(fontPaths);
}

const OptionList& FileList::ModulePaths(void) const
{  // get module paths

  return(modulePaths);
}

OptionList& FileList::ModulePaths(void)
{  // get module paths

  return(modulePaths);
}

const OptionList& FileList::RgbPaths(void) const
{  // get rgb paths

  return(rgbPaths);
}

OptionList& FileList::RgbPaths(void)
{  // get rgb paths

  return(rgbPaths);
}

const StringArray& FileList::Comments(void) const
{  // return comments
  return(comments);
}

StringArray& FileList::Comments(void)
{  // return comments as reference
  return(comments);
}

const bool FileList::Check(void)
{  /// Check if everything has correct syntax.

  if (fontPaths.Check())
  {
    if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_FONTPATH, "FileList::Check");
    }
    else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_SAME_FONTPATH, "FileList::Check");
    }
    else
    {
      MyMessage::Append("FileList::Check");
    }
    return(true);
  }

  if (modulePaths.Check())
  {
    if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_MODULEPATH, "FileList::Check");
    }
    else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_SAME_MODULEPATH, "FileList::Check");
    }
    else
    {
      MyMessage::Append("FileList::Check");
    }
    return(true);
  }

  if (rgbPaths.Check())
  {
    if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_RGBPATH, "FileList::Check");
    }
    else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_SAME_RGBPATH, "FileList::Check");
    }
    else
    {
      MyMessage::Append("FileList::Check");
    }
    return(true);
  }

  return(false);
}

