// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef FILELIST_H
#define FILELIST_H

// **************************************
// Header Files
// **************************************

#include "optionlist.h"
#include "stringarray.h"

// **************************
// Class Definition
// **************************

class FileList
{
public:
  /// Constructor.
  FileList(void);

  /// Copy Constructor.
  FileList(const FileList &list);

  /// Destructor.
  ~FileList(void);

  /// Assignment operator.
  FileList& operator=(const FileList &list);

  /// Get comments.
  const StringArray& Comments(void) const;

  /// Get comments (as reference).
  StringArray& Comments(void);

  /// Get font paths.
  const OptionList& FontPaths(void) const;

  /// Get font paths (as reference).
  OptionList& FontPaths(void);

  /// Get module paths.
  const OptionList& ModulePaths(void) const;

  /// Get module paths (as reference).
  OptionList& ModulePaths(void);

  /// Get rgb paths.
  const OptionList& RgbPaths(void) const;

  /// Get rgb paths (as reference).
  OptionList& RgbPaths(void);

  /// Clear everything.
  void Clear(void);

  /// Check if everything has correct syntax.
  /**
  * Return <b>true</b> if something is wrong else <b>false</b>.
  */
  const bool Check(void);

private:
  StringArray comments;        // comment
  OptionList fontPaths;       // font paths
  OptionList modulePaths;       // module paths
  OptionList rgbPaths;       // rgb paths
};

#endif // FILELIST_H
