// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// **************************************
// Header Files
// **************************************

#include "inputdevice.h"
#include "mymessage.h"

// **************************
// Class Declaration
// **************************

InputDevice::InputDevice(void)
{  // constructor
}

InputDevice::~InputDevice(void)
{  // destructor
}

InputDevice::InputDevice(const InputDevice &dev)
{  // copy constructor

  identifier=dev.GetId();
  driver=dev.GetDriver();
  identifierComment=dev.GetIdComment();
  driverComment=dev.GetDriverComment();

  comments=dev.Comments();
  options=dev.Options();
}

InputDevice& InputDevice::operator=(const InputDevice &dev)
{  // copy constructor

  Clear(); // clear all

  identifier=dev.GetId();
  driver=dev.GetDriver();
  identifierComment=dev.GetIdComment();
  driverComment=dev.GetDriverComment();

  comments=dev.Comments();
  options=dev.Options();

  return(*this);
}

void InputDevice::SetId(const MyString& id, const MyString& com)
{  // set identifier

  identifier=id;
  identifierComment=com;
}

void InputDevice::SetDriver(const MyString& dr, const MyString& com)
{  // set driver

  driver=dr;
  driverComment=com;
}

const MyString& InputDevice::GetId(void) const
{  // return identifier

  return(identifier);
}

const MyString& InputDevice::GetDriver(void) const
{  // retun driver name

  return(driver);
}

const MyString& InputDevice::GetIdComment(void) const
{  // return identifier comment

  return(identifierComment);
}

const MyString& InputDevice::GetDriverComment(void) const
{  // retun driver name comment

  return(driverComment);
}

void InputDevice::Clear(void)
{    // clear everything

  identifier.Empty();
  driver.Empty();
  identifierComment.Empty();
  driverComment.Empty();
  comments.Clear();
  options.Clear();
}

const OptionList& InputDevice::Options(void) const
{  // get options

  return(options);
}

OptionList& InputDevice::Options(void)
{  // get options

  return(options);
}

const StringArray& InputDevice::Comments(void) const
{  // return comments
  return(comments);
}

StringArray& InputDevice::Comments(void)
{  // return comments as reference
  return(comments);
}

const bool InputDevice::Check(const bool checkServer)
{  /// Check if everything has correct syntax.

  if (!checkServer)
  {
    if (identifier.IsEmpty())
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_INPUTDEVICE_ID, "InputDevice::Check");
      return(true);
    }

    if (driver.IsEmpty())
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_INPUTDEVICE_DRIVER, "InputDevice::Check");
      return(true);
    }

    if (options.Check())
    {
      if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
      {
        MyMessage::Set(ERROR_CHECK_EMPTY_INPUTDEVICE_OPTION, "InputDevice::Check");
      }
      else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
      {
        MyMessage::Set(ERROR_CHECK_SAME_INPUTDEVICE_OPTION, "InputDevice::Check");
      }
      else
      {
        MyMessage::Append("InputDevice::Check");
      }
      return(true);
    }
  } // !checkServer
  else
  {
    if (identifier.IsEmpty())
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_SERVERLAYOUT_INPUTDEVICE, "InputDevice::Check");
      return(true);
    }

    if (options.Check())
    {
      if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
      {
        MyMessage::Set(ERROR_CHECK_EMPTY_SERVERLAYOUT_INPUTDEVICE_OPTION, "InputDevice::Check");
      }
      else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
      {
        MyMessage::Set(ERROR_CHECK_SAME_SERVERLAYOUT_INPUTDEVICE_OPTION, "InputDevice::Check");
      }
      else
      {
        MyMessage::Append("InputDevice::Check");
      }
      return(true);
    }
  }

  return(false);
}

const bool InputDevice::ConvertOptions(const DriverOptionList& list)
{  /// Compare options with list and set correct cases.

  int pos, pos2;
  MyString temp;

  // go through options
  for (unsigned int i=0; i<options.GetSize(); i++)
  {
    pos=list.Find(options(i).GetName(), true);
    if (!MyMessage::OK())
    {
      MyMessage::Append("InputDevice::ConvertOptions");
      return(true);
    }

    if (pos!=-1)
    {
      if (options(i).GetName()!=list(pos).GetName())
      {
        options(i).SetName(list(pos).GetName());
      }
      if (!MyMessage::OK())
      {
        MyMessage::Append("InputDevice::ConvertOptions");
        return(true);
      }

      pos2=list(pos).GetArguments().Find(options(i).GetValue(), 0, true);

      if (pos2!=-1)
      {
        if (options(i).GetValue()!=list(pos).GetArguments()(pos2))
        {
          options(i).SetValue(list(pos).GetArguments()(pos2));
        }
        if (!MyMessage::OK())
        {
          MyMessage::Append("InputDevice::ConvertOptions");
          return(true);
        }
      } // pos2!=-1
    } // pos!=-1

    // convert all to lower case letters
    temp=options(i).GetValue().toLower();
    if (temp=="yes") options(i).SetValue("yes");
    else if (temp=="no") options(i).SetValue("no");
    else if (temp=="true") options(i).SetValue("true");
    else if (temp=="false") options(i).SetValue("false");
    else if (temp=="on") options(i).SetValue("on");
    else if (temp=="off") options(i).SetValue("off");
  } // for i

  return(false);
}

const bool InputDevice::Check(const DriverOptionList& list)
{  /// Check if options are correct.

  int pos;
  MyString temp;

  // go through options
  for (unsigned int i=0; i<options.GetSize(); i++)
  {
    pos=list.Find(options(i).GetName(), true);
    if (!MyMessage::OK())
    {
      MyMessage::Append("InputDevice::Check");
      return(true);
    }

    if (pos!=-1)
    {
      // check if boolean
      if (list(pos).GetValue()=="boolean")
      {
        temp=options(i).GetValue().toLower();
        if (temp!="yes" && temp!="no" && temp!="true" && temp!="false" &&
            temp!="on" && temp!="off" && temp!="0" && temp!="1")
        {
          options.SetCurrent(i);
          MyMessage::Set(ERROR_CHECK_INPUTDEVICE_OPTION_BOOL, "InputDevice::Check");
          return(true);
        }
      }
      else  // not bool
      {
        if (list(pos).GetValue()=="integer")
        {
          if (!options(i).GetValue().IsNumber())
          {
            options.SetCurrent(i);
            MyMessage::Set(ERROR_CHECK_INPUTDEVICE_OPTION_INT, "InputDevice::Check");
            return(true);
          }
        }

        if (list(pos).IsStrict())
        {
          if (list(pos).GetArguments().GetSize()>0 && list(pos).GetArguments().Find(options(i).GetValue(), 0, true)==-1)
          {
            options.SetCurrent(i);
            MyMessage::Set(ERROR_CHECK_INPUTDEVICE_OPTION_LIST, "InputDevice::Check");
            return(true);
          }
        }
      }
    }
  }  // for i

  return(false);
}
