// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// Header Files
// **************************************

#include "module.h"
#include "mymessage.h"

// **************************
// Class Declaration
// **************************

Module::Module(void) : Option()
{  // constructor
}

Module::Module(const MyString& opt, const MyString& val, const MyString& com, const bool act) : Option(opt, val, com, act)
{  // constructor
}

Module::Module(const Module &opt) : Option(opt)
{  // copy constructor

  options=opt.Options();
}

Module::~Module(void)
{  // destructor
}

Module& Module::operator=(const Module &opt)
{  // copy constructor

  // clear old entries
  Clear();

  Option::operator=(opt);
  options=opt.Options();

  return(*this);
}

const OptionList& Module::Options(void) const
{  // get options

  return(options);
}

OptionList& Module::Options(void)
{  // get options

  return(options);
}

void Module::Clear(void)
{  /// Delete all variables.
  Option::Clear();
  options.Clear();
}

const bool Module::Check(void)
{  /// Check if everything has correct syntax.

  if (name.IsEmpty())
  {
    MyMessage::Set(ERROR_CHECK_EMPTY_MODULE, "Module::Check");
    return(true);
  }

  if (options.Check())
  {
    if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_MODULE_OPTION, "Module::Check");
    }
    else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_SAME_MODULE_OPTION, "Module::Check");
    }
    else
    {
      MyMessage::Append("Module::Check");
    }
    return(true);
  }

  return(false);
}

