// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// **************************************
// Header Files
// **************************************

#include "monitor.h"
#include "mymessage.h"

// **************************
// Class Declaration
// **************************

Monitor::Monitor(void)
{  // constructor
}

Monitor::~Monitor(void)
{  // destructor
}

Monitor::Monitor(const Monitor &mon)
{  // copy constructor

  identifier=mon.GetId();
  horSync=mon.GetHorSync();
  vertRefresh=mon.GetVertRef();
  displaySize=mon.GetDisplaySize();
  gamma=mon.GetGamma();
  identifierComment=mon.GetIdComment();
  horSyncComment=mon.GetHorSyncComment();
  vertRefreshComment=mon.GetVertRefComment();
  displaySizeComment=mon.GetDisplaySizeComment();
  gammaComment=mon.GetGammaComment();

  comments=mon.Comments();
  options=mon.Options();
  modelines=mon.Modelines();
}

Monitor& Monitor::operator=(const Monitor &mon)
{  // copy constructor

  Clear();

  identifier=mon.GetId();
  horSync=mon.GetHorSync();
  vertRefresh=mon.GetVertRef();
  displaySize=mon.GetDisplaySize();
  gamma=mon.GetGamma();
  identifierComment=mon.GetIdComment();
  horSyncComment=mon.GetHorSyncComment();
  vertRefreshComment=mon.GetVertRefComment();
  displaySizeComment=mon.GetDisplaySizeComment();
  gammaComment=mon.GetGammaComment();

  comments=mon.Comments();
  options=mon.Options();
  modelines=mon.Modelines();

  return(*this);
}

void Monitor::SetId(const MyString& id, const MyString& com)
{  // set identifier

  identifier=id;
  identifierComment=com;
}

void Monitor::SetHorSync(const MyString& hor, const MyString& com)
{  // set horizontal sync

  horSync=hor;
  horSyncComment=com;
}

void Monitor::SetVertRef(const MyString& ver, const MyString& com)
{  // set vertical refresh

  vertRefresh=ver;
  vertRefreshComment=com;
}

void Monitor::SetDisplaySize(const MyString& size, const MyString& com)
{  // set display size

  displaySize=size;
  displaySizeComment=com;
}

void Monitor::SetGamma(const MyString& gam, const MyString& com)
{  // set gamma

  gamma=gam;
  gammaComment=com;
}

void Monitor::Clear(void)
{    // clear everything

  identifier.Empty();
  horSync.Empty();
  vertRefresh.Empty();
  displaySize.Empty();
  gamma.Empty();
  identifierComment.Empty();
  horSyncComment.Empty();
  vertRefreshComment.Empty();
  displaySizeComment.Empty();
  gammaComment.Empty();
  comments.Clear();
  options.Clear();
  modelines.Clear();
}

const MyString& Monitor::GetId(void) const
{  // return identifier

  return(identifier);
}

const MyString& Monitor::GetHorSync(void) const
{  // return horizontal sync

  return(horSync);
}

const MyString& Monitor::GetVertRef(void) const
{  // return vertical refresh

  return(vertRefresh);
}

const MyString& Monitor::GetDisplaySize(void) const
{  // return display size

  return(displaySize);
}

const MyString& Monitor::GetGamma(void) const
{  // return gamma

  return(gamma);
}

const MyString& Monitor::GetIdComment(void) const
{  // return identifier Comment

  return(identifierComment);
}

const MyString& Monitor::GetHorSyncComment(void) const
{  // return horizontal sync Comment

  return(horSyncComment);
}

const MyString& Monitor::GetVertRefComment(void) const
{  // return vertical refresh Comment

  return(vertRefreshComment);
}

const MyString& Monitor::GetDisplaySizeComment(void) const
{  // return display size Comment

  return(displaySizeComment);
}

const MyString& Monitor::GetGammaComment(void) const
{  // return gamma Comment

  return(gammaComment);
}

const OptionList& Monitor::Options(void) const
{  // get options

  return(options);
}

OptionList& Monitor::Options(void)
{  // get options

  return(options);
}

const OptionList& Monitor::Modelines(void) const
{  // get modelines

  return(modelines);
}

OptionList& Monitor::Modelines(void)
{  // get modelines

  return(modelines);
}

const StringArray& Monitor::Comments(void) const
{  // return comments
  return(comments);
}

StringArray& Monitor::Comments(void)
{  // return comments as reference
  return(comments);
}

const bool Monitor::Check(void)
{  /// Check if everything has correct syntax.

  if (identifier.IsEmpty())
  {
    MyMessage::Set(ERROR_CHECK_EMPTY_MONITOR_ID, "Monitor::Check");
    return(true);
  }

  if (!displaySize.IsEmpty())
  {
    MyString tempStr=displaySize;
    tempStr.StripSpaces();
    int pos=tempStr.Find(' ');
    if (pos==-1)
    {
      MyMessage::Set(ERROR_CHECK_MONITOR_SIZE, "Monitor::CheckMonitor");
      return(true);
    }

    if (!tempStr.Left(pos).IsInt() || tempStr.toInt()<0)
    {
      MyMessage::Set(ERROR_CHECK_MONITOR_SIZE, "Monitor::CheckMonitor");
      return(true);
    }

    tempStr.StripRight(pos);
    if (!tempStr.IsInt() || tempStr.toInt()<0)
    {
      MyMessage::Set(ERROR_CHECK_MONITOR_SIZE, "Monitor::CheckMonitor");
      return(true);
    }
  }

  if (!gamma.IsEmpty())
  {
    MyString tempStr=gamma;
    tempStr.StripSpaces();

    if (!tempStr.IsDouble())
    {
      MyMessage::Set(ERROR_CHECK_MONITOR_GAMMA, "Monitor::CheckMonitor");
      return(true);
    }

    if (tempStr.toDouble()<=0.0 || tempStr.toDouble()>10.0)
    {
      MyMessage::Set(ERROR_CHECK_MONITOR_GAMMA, "Monitor::CheckMonitor");
      return(true);
    }
  }

  if (options.Check())
  {
    if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_MONITOR_OPTION, "Monitor::Check");
    }
    else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_SAME_MONITOR_OPTION, "Monitor::Check");
    }
    else
    {
      MyMessage::Append("Monitor::Check");
    }
    return(true);
  }

  if (modelines.Check())
  {
    if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_MONITOR_MODELINE, "Monitor::Check");
    }
    else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_SAME_MONITOR_MODELINE, "Monitor::Check");
    }
    else
    {
      MyMessage::Append("Monitor::Check");
    }
    return(true);
  }


  return(false);
}


