// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef MONITOR_H
#define MONITOR_H

// **************************************
// Header Files
// **************************************

#include "optionlist.h"
#include "stringarray.h"

// **************************
// Class Definition
// **************************

class Monitor
{
public:
  /// Constructor.
  Monitor(void);

  /// Copy Constructor.
  Monitor(const Monitor &mon);

  /// Destructor.
  ~Monitor(void);

  /// Assignment operator.
  Monitor& operator=(const Monitor &mon);

  /// Set monitor identifier and optional comment.
  void SetId(const MyString& id, const MyString& com=MyString());

  /// Set monitor horizontal sync and optional comment.
  void SetHorSync(const MyString& hor, const MyString& com=MyString());

  /// Set monitor vertical refresh and optional comment.
  void SetVertRef(const MyString& vert, const MyString& com=MyString());

  /// Set monitor display size and optional comment.
  void SetDisplaySize(const MyString& size, const MyString& com=MyString());

  /// Set monitor gamma and optional comment.
  void SetGamma(const MyString& gamma, const MyString& com=MyString());

  /// Get monitor identifier.
  const MyString& GetId(void) const;

  /// Get monitor horizontal sync.
  const MyString& GetHorSync(void) const;

  /// Get monitor vertical refresh.
  const MyString& GetVertRef(void) const;

  /// Get monitor display size.
  const MyString& GetDisplaySize(void) const;

  /// Get monitor gamma.
  const MyString& GetGamma(void) const;

  /// Get monitor identifier comment.
  const MyString& GetIdComment(void) const;

  /// Get monitor horizontal sync comment.
  const MyString& GetHorSyncComment(void) const;

  /// Get monitor vertical refresh comment.
  const MyString& GetVertRefComment(void) const;

  /// Get monitor display size comment.
  const MyString& GetDisplaySizeComment(void) const;

  /// Get monitor gamma comment.
  const MyString& GetGammaComment(void) const;

  /// Get comments.
  const StringArray& Comments(void) const;

  /// Get comments (as reference).
  StringArray& Comments(void);

  /// Get options.
  const OptionList& Options(void) const;

  /// Get options (as reference).
  OptionList& Options(void);

  /// Get modelines.
  const OptionList& Modelines(void) const;

  /// Get modelines (as reference).
  OptionList& Modelines(void);

  /// Clear everything.
  void Clear(void);

  /// Check if everything has correct syntax.
  /**
  * Return <b>true</b> if something is wrong else <b>false</b>.
  */
  const bool Check(void);

private:
  MyString identifier;      // Monitor identifier
  MyString horSync;         // horizontal synchronisation
  MyString vertRefresh;     // vertical refresh
  MyString displaySize;     // display size
  MyString gamma;           // gamma correction

  // Note: Any additional settings will not be recognized like:
  // VendorName, ModelName, Gamma, UseMode, Modes.

  MyString identifierComment;      // Monitor identifier comment
  MyString horSyncComment;         // horizontal synchronisation comment
  MyString vertRefreshComment;     // vertical refresh comment
  MyString displaySizeComment;     // display size comment
  MyString gammaComment;           // gamma correction comment
  StringArray comments;                  // comment
  OptionList options;     // all options
  OptionList modelines;    // all options
};

#endif // MONITOR_H
