// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// Message declaration
// **************************************

// **************************
// Headers
// **************************

#include "mymessage.h"

// **************************
// Static variables
// **************************

messageIdentifier MyMessage::messageID=EVERYTHING_OK;
StringArray MyMessage::functions;
MyString MyMessage::line;
MyString MyMessage::file;
unsigned int MyMessage::lineNr=0;

// **************************
// MyMessage Functions
// **************************

void MyMessage::Set(const messageIdentifier &mess, const MyString& func, const MyString& f, const MyString& l, const unsigned int n)    // set message
{
  Reset();
  messageID=mess;
  functions.Append(func);
  line=l;
  file=f;
  lineNr=n;
}

void MyMessage::Append(const MyString& func)
{  /// Add function where the error occured.

  functions.Append(func);
}

const messageIdentifier MyMessage::Get(void)         // get message
{
  return(messageID);
}

const MyString& MyMessage::GetFile(void)
{  /// Return file where error occured.

  return(file);
}

const MyString& MyMessage::GetLine(void)
{  /// Return file line where error occured.

  return(line);
}

const unsigned int MyMessage::GetLineNumber(void)
{  /// Return file line number where error occured.

  return(lineNr);
}

const MyString MyMessage::GetMessage(void)         // get message as string
{
  // Strings for messages IDs
  const char *messageStrings[] =
  {
  "Everything is ok.",  // 0

  // errors
  "Index out of range.",   // 1
  "Could not open file for writing.",
  "Could not create backup file. Will not save file!",
  "Could not open file for reading.",
  "Division by zero!",
  "Unknown case in switch-statement!",
  "Unknown case in if-statement!",
  "Pointer is NULL!",

  // driver list errors
  "Unrecognized line.",  // 8
  "Argument outside <option>-tag.",
  "Second <option>-tag found without closing </option> before.",
  "Option outside <section>-tag.",
  "Line does not end with '>'.",
  "</section> missing at end of file.",
  "More than one <default>-tag in one option.",
  "</default> missing at end of line.",
  "<default> is empty.",
  "Error in option name assignment.",
  "Option name is missing.",
  "Error in value assignment.",
  "Error in value assignment: Type must be 'string', 'integer' or 'boolean'.",
  "</argument> missing at end of line.",
  "<argument> is empty.",
  "Error in comment assignment.",
  "Error in strict assignment.",
  "Error in strict assignment: Type must be 'true' or 'false'.",  // 25
  "Error in description assignment.",
  "<description> is empty.",
  "No <description>-Tag before.",
  "No </description>-Tag found.",
  "Error in section assignment.",
  "Error in section assignment: Type must be 'general', 'tv' or 'misc'.",
  "No <section>-Tag before.",
  "Second <section>-tag found without closing </section> before.",
  "Error in section name assignment.",
  "Section name is missing.",
  "Option in driver list already exists!",

  // xorg errors
  "Either the file is not a valid config file or there are some options that could not be recognized.", // 37
  "No section has started.",
  "Subsection has ended without beginning.",
  "Another section has begun without ending the last one.",
  "Device identifier does not exist in device list.",
  "Screen identifier does not exist in screen list.",
  "Monitor identifier does not exist in monitor list.",
  "Input device identifier does not exist in input device list.",
  "No monitors declared. Need at least one...",
  "No devices declared. Need at least one...",
  "No or only one input devices declared. Need at least two...",
  "No screens declared. Need at least one...",
  "No server layouts declared. Need at least one...",
  "Argument or Option must not be empty.",
  "Device identifier is missing.",  // 51
  "Device driver is missing.",
  "Monitor identifier is missing.",
  "Input device identifier is missing.",
  "Input device driver is missing.",
  "Screen identifier is missing.",
  "Server layout identifier is missing.",
  "Module option given without a module.",
  "Second DRI entry found.",
  "Error in files section.",
  "Error in server flags section.",
  "Error in modules section.",
  "Error in input device section.",
  "Error in device section.",
  "Error in monitor section.",
  "Error in screen section.",
  "Error in server layout section.",
  "Error in dri section.",
  "Error in extensions section.",
  "Error in display subsection.",

  // xorg check errors
  "Device identifier must not be empty!",  // 71
  "Device identifier already exists!",
  "Device driver must not be empty!",
  "Device screen must be a non-negative integer!",
  "Device video ram must be a non-negative integer!",
  "Device option must not be empty!",
  "Device option already exists!",
  "Device option must be boolean!",
  "Device option must be integer!",
  "Device option must be in argument list!",
  "Monitor identifier must not be empty!",
  "Monitor identifier already exists!",
  "Monitor display size wrongly formatted!",
  "Monitor gamma must be double between 0.0 and 10.0!",
  "Monitor option must not be empty!",
  "Monitor option already exists!",
  "Monitor modeline must not be empty!",
  "Monitor modeline already exists!",
  "First eight monitor modeline entries must be integer!",
  "Monitor modeline must have 9 entries at least!",
  "Input device identifier must not be empty!",  // 91
  "Input device identifier already exists!",
  "Input device driver must not be empty!",
  "Input device option must not be empty!",
  "Input device option already exists!",
  "Input device option must be boolean!",
  "Input device option must be integer!",
  "Input device option must be in argument list!",
  "Screen identifier must not be empty!",
  "Screen identifier already exists!",
  "Screen depth must not be empty!",  // 101
  "Screen depth must be a non-negative integer!",
  "Screen option must not be empty!",
  "Screen option already exists!",
  "Screen display depth must not be empty!",
  "Screen display depth must be a non-negative integer!",
  "Screen display already exists!",
  "Screen display viewport must be a non-negative integer!",
  "Screen display virtual size must be a non-negative integer!",
  "Screen display option must not be empty!",
  "Screen display option already exists!",
  "Screen display mode must not be empty!",
  "Screen display mode already exists!",
  "Server layout identifier must not be empty!",
  "Server layout identifier already exists!",
  "Server layout option must not be empty!",
  "Server layout option already exists!",
  "Screen already used in this server layout. Please select another one!",
  "Screen 1 and 2 are the same. Please select another one!",
  "Server layout screen number must be a non-negative integer!",
  "Server layout screen position is wrong!",  // 121
  "I'm sorry, but the \"relative\" option for screen position is not supported yet!",
  "Input device already used in this server layout. Please select another one!",
  "Server layout input device option must not be empty!",
  "Server layout input device option already exists!",
  "Server flag must not be empty!",
  "Server flag already exists!",
  "Extension must not be empty!",
  "Extension already exists!",
  "Module name must not be empty!",
  "Module name already exists!",
  "Module option must not be empty!",
  "Module option already exists!",
  "Font path must not be empty!",
  "Font path already exists!",
  "Module path must not be empty!",
  "Module path already exists!",
  "RGB path must not be empty!",
  "RGB path already exists!",  // 139
  "Option already exists!",
  "Driver option name must not be empty!",
  "Driver option already exists!",
  "Server layout screen must not be empty!",
  "Server layout input device must not be empty!",  // 144
};

  return(messageStrings[messageID]);
}

const MyString MyMessage::GetLastFunction(void)
{             // get function

  if (functions.GetSize()==0) return(MyString());
  else return(functions(functions.GetSize()-1));
}

const MyString MyMessage::GetFirstFunction(void)
{             // get function

  if (functions.GetSize()==0) return(MyString());
  else return(functions(0));
}

void MyMessage::DeleteLast(void)
{             // get function

  if (functions.GetSize()==0) return;
  else functions.SetSize(functions.GetSize()-1);
}

const MyString MyMessage::GetStack(void)
{  /// Return error message as string.

  MyString xLine;

  xLine.Add("Function Stack:\n");
  for (int i=functions.GetSize()-1; i>=0; i--)
  {
    if (i==0) xLine.Add(functions(i)+"\n");
    else xLine.Add(functions(i)+" ->\n");
  }
  xLine.Add("\n");

  return(xLine);
}

void MyMessage::Reset(void)         // reset message
{
  messageID=EVERYTHING_OK;
  functions.Clear();
  line.Empty();
  file.Empty();
  lineNr=0;
}

const bool MyMessage::OK(void)
{             // return true if everyting is fine else false
  if (messageID==EVERYTHING_OK) return(true);
  else return(false);
}

