// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// Error class definitions
// **************************************

#if !defined(MYMESSAGE_H)
#define MYMESSAGE_H

// **************************
// Headers
// **************************

#include "stringarray.h"
#include "mystring.h"

// **************************
// Constants
// **************************

// IDs for messages

enum messageIdentifier
{
  EVERYTHING_OK=0,

  // general errors
  ERROR_INDEX_OUT_OF_RANGE,              // "Index out of range."  // 1
  ERROR_WRITE_FILE,                      // "Could not open file for writing."
  ERROR_BACKUP_FILE,                      // "Could not create backup file. Will not save file!"
  ERROR_READ_FILE,                       // "Could not open file for reading."
  ERROR_DIVISION_BY_ZERO,                // "Division by zero!"
  ERROR_UNKNOWN_CASE_IN_SWITCH,          // "Unknown case in switch-statement!"
  ERROR_UNKNOWN_CASE_IN_IF,              // "Unknown case in if-statement!"
  ERROR_POINTER_IS_NULL,                 // "Pointer is NULL!"

  // driver list errors
  ERROR_DRIVERLIST_UNKNOWN,              // "Unrecognized line."  // 8
  ERROR_DRIVERLIST_NO_OPTION,            // "Argument outside <option>-tag."
  ERROR_DRIVERLIST_TWO_OPTIONS,          // "Second <option>-tag found without closing </option> before."
  ERROR_DRIVERLIST_OPTION_WITHOUT_SECTION, // "Option outside section-Tags."
  ERROR_DRIVERLIST_NO_TAG_END,           // "Line does not end with '>'."
  ERROR_DRIVERLIST_CLOSE_SECTION,         // "</section> missing at end of file."
  ERROR_DRIVERLIST_TWO_DEFAULT,          // "More than one <default>-tag in one option."
  ERROR_DRIVERLIST_NO_DEFAULT_END,       // "</default> missing at end of line."
  ERROR_DRIVERLIST_EMPTY_DEFAULT,        // "<default> is empty."
  ERROR_DRIVERLIST_OPTION_NAME_ERROR,    // "Error in option name assignment."
  ERROR_DRIVERLIST_OPTION_NAME_MISSING,  // "Option name is missing."
  ERROR_DRIVERLIST_VALUE_ERROR,          // "Error in value assignment."
  ERROR_DRIVERLIST_VALUE_VALUE,          // "Error in value assignment: Type must be 'string', 'integer' or 'boolean'."
  ERROR_DRIVERLIST_NO_ARGUMENT_END,      // "</argument> missing at end of line."
  ERROR_DRIVERLIST_EMPTY_ARGUMENT,       // "<argument> is empty."
  ERROR_DRIVERLIST_COMMENT_ERROR,        // "Error in comment assignment."
  ERROR_DRIVERLIST_STRICT_ERROR,         // "Error in strict assignment."
  ERROR_DRIVERLIST_STRICT_VALUE,         // "Error in strict assignment: Type must be 'true' or 'false'."  // 25
  ERROR_DRIVERLIST_DESC_ERROR,           // "Error in desctiption assignment."
  ERROR_DRIVERLIST_EMPTY_DESC,           // "<description> is empty."
  ERROR_DRIVERLIST_NO_DESC,              // "No <description>-Tag before."
  ERROR_DRIVERLIST_CLOSE_DESC,           // "No </description>-Tag found."
  ERROR_DRIVERLIST_SECTION_ERROR,        // "Error in section assignment."
  ERROR_DRIVERLIST_SECTION_VALUE,        // "Error in section assignment: Type must be 'general', 'tv' or 'misc'."
  ERROR_DRIVERLIST_NO_SECTION,           // "No <section>-Tag before."
  ERROR_DRIVERLIST_TWO_SECTIONS,         // "Second <section>-tag found without closing </section> before."
  ERROR_DRIVERLIST_SECTION_NAME_ERROR,   // "Error in section name assignment."
  ERROR_DRIVERLIST_SECTION_NAME_MISSING, // "Section name is missing."
  ERROR_DRIVERLIST_DOUBLE_OPTION,      // "Option in driver list already exists!"

  // xorg errors
  ERROR_XORG_UNKNOWN,                    // "Either the file is not a valid config file or there are some options that could not be recognized."  // 37
  ERROR_XORG_NO_SECTION,                 // "No section has started.",
  ERROR_XORG_NO_SUBSECTION,              // "Subsection has ended without beginning."
  ERROR_XORG_TWO_SECTIONS,               // "Another section has begun without ending the last one."
  ERROR_XORG_NO_DEVICE,                  // "Device identifier does not exist in device list."
  ERROR_XORG_NO_SCREEN,                  // "Screen identifier does not exist in screen list."
  ERROR_XORG_NO_MONITOR,                 // "Monitor identifier does not exist in monitor list."
  ERROR_XORG_NO_INPUTDEVICE,             // "Input device identifier does not exist in input device list."
  ERROR_XORG_NO_MONITORS,                // "No monitors declared. Need at least one..."
  ERROR_XORG_NO_DEVICES,                 // "No devices declared. Need at least one..."
  ERROR_XORG_NO_INPUTDEVICES,            // "No or only one input devices declared. Need at least two..."
  ERROR_XORG_NO_SCREENS,                 // "No screens declared. Need at least one..."
  ERROR_XORG_NO_SERVERLAYOUTS,           // "No server layouts declared. Need at least one..."
  ERROR_XORG_EMPTY_ARGUMENT,             // "Argument or Option must not be empty."
  ERROR_XORG_NO_DEVICE_ID,               // "Device identifier is missing."  // 51
  ERROR_XORG_NO_DEVICE_DRIVER,           // "Device driver is missing."
  ERROR_XORG_NO_MONITOR_ID,              // "Monitor identifier is missing."
  ERROR_XORG_NO_INPUTDEVICE_ID,          // "Input device identifier is missing."
  ERROR_XORG_NO_INPUTDEVICE_DRIVER,      // "Input device driver is missing."
  ERROR_XORG_NO_SCREEN_ID,               // "Screen identifier is missing."
  ERROR_XORG_NO_SERVERLAYOUT_ID,         // "Server layout identifier is missing."
  ERROR_XORG_NO_MODUL,                   // "Module option given without a module."
  ERROR_XORG_SECOND_DRI,                 // "Second DRI entry found."
  ERROR_XORG_FILES_SECTION,              // "Error in files section."
  ERROR_XORG_SERVERFLAGS_SECTION,        // "Error in server flags section."
  ERROR_XORG_MODULE_SECTION,             // "Error in modules section."
  ERROR_XORG_INPUTDEVICE_SECTION,        // "Error in input device section."
  ERROR_XORG_DEVICE_SECTION,             // "Error in device section."
  ERROR_XORG_MONITOR_SECTION,            // "Error in monitor section."
  ERROR_XORG_SCREEN_SECTION,             // "Error in screen section."
  ERROR_XORG_SERVERLAYOUT_SECTION,       // "Error in server layout section."
  ERROR_XORG_DRI_SECTION,                // "Error in dri section."
  ERROR_XORG_EXTENSIONS_SECTION,         // "Error in extensions section."
  ERROR_XORG_DISPLAY_SECTION,            // "Error in display subsection."

  // xorg check errors
  ERROR_CHECK_EMPTY_DEVICE_ID,           // "Device identifier must not be empty!"  // 71
  ERROR_CHECK_SAME_DEVICE_ID,            // "Device identifier already exists!"
  ERROR_CHECK_EMPTY_DEVICE_DRIVER,       // "Device driver must not be empty!"
  ERROR_CHECK_DEVICE_SCREEN_NUMBER,      // "Device screen must be a non-negative integer!"
  ERROR_CHECK_DEVICE_VIDEORAM,           // "Device video ram must be a non-negative integer!"
  ERROR_CHECK_EMPTY_DEVICE_OPTION,       // "Device option must not be empty!"
  ERROR_CHECK_SAME_DEVICE_OPTION,        // "Device option already exists!"
  ERROR_CHECK_DEVICE_OPTION_BOOL,        // "Device option must be boolean!"
  ERROR_CHECK_DEVICE_OPTION_INT,         // "Device option must be integer!"
  ERROR_CHECK_DEVICE_OPTION_LIST,        // "Device option must be in argument list!"
  ERROR_CHECK_EMPTY_MONITOR_ID,          // "Monitor identifier must not be empty!"
  ERROR_CHECK_SAME_MONITOR_ID,           // "Monitor identifier already exists!"
  ERROR_CHECK_MONITOR_SIZE,              // "Monitor display size wrongly formatted!"
  ERROR_CHECK_MONITOR_GAMMA,             // "Monitor gamma must be double between 0.0 and 10.0!"
  ERROR_CHECK_EMPTY_MONITOR_OPTION,      // "Monitor option must not be empty!"
  ERROR_CHECK_SAME_MONITOR_OPTION,       // "Monitor option already exists!"
  ERROR_CHECK_EMPTY_MONITOR_MODELINE,    // "Monitor modeline must not be empty!"
  ERROR_CHECK_SAME_MONITOR_MODELINE,     // "Monitor modeline already exists!"
  ERROR_CHECK_MONITOR_MODELINE_INT,      // "First eight monitor modeline entries must be integer!"
  ERROR_CHECK_MONITOR_MODELINE_NUMBER,   // "Monitor modeline must have 9 entries at least!"
  ERROR_CHECK_EMPTY_INPUTDEVICE_ID,      // "Input device identifier must not be empty!"  // 91
  ERROR_CHECK_SAME_INPUTDEVICE_ID,       // "Input device identifier already exists!"
  ERROR_CHECK_EMPTY_INPUTDEVICE_DRIVER,  // "Input device driver must not be empty!"
  ERROR_CHECK_EMPTY_INPUTDEVICE_OPTION,  // "Input device option must not be empty!"
  ERROR_CHECK_SAME_INPUTDEVICE_OPTION,   // "Input device option already exists!"
  ERROR_CHECK_INPUTDEVICE_OPTION_BOOL,   // "Input device option must be boolean!"
  ERROR_CHECK_INPUTDEVICE_OPTION_INT,    // "Input device option must be integer!"
  ERROR_CHECK_INPUTDEVICE_OPTION_LIST,   // "Input device option must be in argument list!"
  ERROR_CHECK_EMPTY_SCREEN_ID,           // "Screen identifier must not be empty!"
  ERROR_CHECK_SAME_SCREEN_ID,            // "Screen identifier already exists!"
  ERROR_CHECK_EMPTY_SCREEN_DEPTH,        // "Screen depth must not be empty!"  // 101
  ERROR_CHECK_SCREEN_DEPTH,              // "Screen depth must be a non-negative integer!"
  ERROR_CHECK_EMPTY_SCREEN_OPTION,       // "Screen option must not be empty!"
  ERROR_CHECK_SAME_SCREEN_OPTION,        // "Screen option already exists!"
  ERROR_CHECK_EMPTY_SCREEN_DISPLAY_DEPTH, // "Screen display depth must not be empty!"
  ERROR_CHECK_SCREEN_DISPLAY_DEPTH,      // "Screen display depth must be a non-negative integer!"
  ERROR_CHECK_SAME_SCREEN_DISPLAY,       // "Screen display already exists!"
  ERROR_CHECK_SCREEN_DISPLAY_VIEWPORT,   // "Screen display viewport must be a non-negative integer!"
  ERROR_CHECK_SCREEN_DISPLAY_VIRTUAL,    // "Screen display virtual size must be a non-negative integer!"
  ERROR_CHECK_EMPTY_SCREEN_DISPLAY_OPTION, // "Screen display option must not be empty!"
  ERROR_CHECK_SAME_SCREEN_DISPLAY_OPTION, // "Screen display option already exists!"
  ERROR_CHECK_EMPTY_SCREEN_DISPLAY_MODE, // "Screen display mode must not be empty!"
  ERROR_CHECK_SAME_SCREEN_DISPLAY_MODE,  // "Screen display mode already exists!"
  ERROR_CHECK_EMPTY_SERVERLAYOUT_ID,     // "Server layout identifier must not be empty!"
  ERROR_CHECK_SAME_SERVERLAYOUT_ID,      // "Server layout identifier already exists!"
  ERROR_CHECK_EMPTY_SERVERLAYOUT_OPTION, // "Server layout option must not be empty!"
  ERROR_CHECK_SAME_SERVERLAYOUT_OPTION,  // "Server layout option already exists!"
  ERROR_CHECK_SAME_SERVERLAYOUT_SCREEN,  // "Screen already used in this server layout. Please select another one!"
  ERROR_CHECK_SAME_SERVERLAYOUT_SCREENS, // "Screen 1 and 2 are the same. Please select another one!"
  ERROR_CHECK_SERVERLAYOUT_SCREEN_NUMBER, // "Server layout screen number must be a non-negative integer!"
  ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION, // "Server layout screen position is wrong!"  // 121
  ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION_REL, // "I'm sorry, but the \"relative\" option for screen position is not supported yet!"
  ERROR_CHECK_SAME_SERVERLAYOUT_INPUTDEVICE, // "Input device already used in this server layout. Please select another one!"
  ERROR_CHECK_EMPTY_SERVERLAYOUT_INPUTDEVICE_OPTION, // "Server layout input device option must not be empty!"
  ERROR_CHECK_SAME_SERVERLAYOUT_INPUTDEVICE_OPTION, // "Server layout input device option already exists!"
  ERROR_CHECK_EMPTY_SERVERFLAG,          // "Server flag must not be empty!"
  ERROR_CHECK_SAME_SERVERFLAG,           // "Server flag already exists!"
  ERROR_CHECK_EMPTY_EXTENSION,           // "Extension must not be empty!"
  ERROR_CHECK_SAME_EXTENSION,            // "Extension already exists!"
  ERROR_CHECK_EMPTY_MODULE,              // "Module name must not be empty!"
  ERROR_CHECK_SAME_MODULE,               // "Module name already exists!"
  ERROR_CHECK_EMPTY_MODULE_OPTION,       // "Module option must not be empty!"
  ERROR_CHECK_SAME_MODULE_OPTION,        // "Module option already exists!"
  ERROR_CHECK_EMPTY_FONTPATH,            // "Font path must not be empty!"
  ERROR_CHECK_SAME_FONTPATH,             // "Font path already exists!"
  ERROR_CHECK_EMPTY_MODULEPATH,          // "Module path must not be empty!"
  ERROR_CHECK_SAME_MODULEPATH,           // "Module path already exists!"
  ERROR_CHECK_EMPTY_RGBPATH,             // "RGB path must not be empty!"
  ERROR_CHECK_SAME_RGBPATH,              // "RGB path already exists!"  // 139
  ERROR_CHECK_EMPTY_OPTION,               // "Option name must not be empty!"
  ERROR_CHECK_SAME_OPTION,                   // "Option already exists!"
  ERROR_CHECK_EMPTY_DRIVEROPTION,        // "Driver option name must not be empty!"
  ERROR_CHECK_SAME_DRIVEROPTION,            // "Driver option already exists!"
  ERROR_CHECK_EMPTY_SERVERLAYOUT_SCREEN,        // "Server layout screen must not be empty!"
  ERROR_CHECK_EMPTY_SERVERLAYOUT_INPUTDEVICE       // "Server layout input device must not be empty!"

};

class MyMessage
{

public:
  /// Reset error message to "no error".
  /**
  * <b><i>messageID</i></b> will be set to <b>EVERYTHING_OK</b>
  * and the <b>functions</b> will be cleared.
  */
  static void Reset(void);   // reset message

  /// Set first <b>messageID</b> and optional the <b>function</b> where the error occured.
  static void Set(const messageIdentifier &mess, const MyString& func, const MyString& f=MyString(), const MyString& l=MyString(), const unsigned int n=0);   // set message and function

  /// Add function where the error occured.
  static void Append(const MyString& func);

  /// Return <b>messageID</b>.
  static const messageIdentifier Get(void);

  /// Return error message as string.
  /**
  * Do not use for error output in GUI because it is not translated.
  */
  static const MyString GetMessage(void);

  /// Return <b>function</b> where the error last occured.
  static const MyString GetLastFunction(void);

  /// Return <b>function</b> where the error first occured.
  static const MyString GetFirstFunction(void);

  /// Return error message as string.
  static const MyString GetStack(void);

  /// Return file line where error occured.
  static const MyString& GetLine(void);

  /// Return file where error occured.
  static const MyString& GetFile(void);

  /// Return file line number where error occured.
  static const unsigned int GetLineNumber(void);

  /// Delete last <b>function</b> from stack.
  static void DeleteLast(void);

  /// Return <b>true</b> if <b>EVERYTHING_OK</b> is set, else <b>false</b>.
  static const bool OK(void);             // return true if everyting is fine else false

private:
  static messageIdentifier messageID;     ///< Message identifier, standard is <b>EVERYTHING_OK</b>.
  static StringArray functions;           ///< Optional functions where an error occured.
  static unsigned int lineNr;             ///< Optional line number for file errors.
  static MyString line;                   ///< Optional line for file errors.
  static MyString file;                   ///< Optional file for file errors.
};


#endif // !defined(MYMESSAGE_H)
