// *************************************************************************
// MyString class
// Copyright 2004, 2005, 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// **************************
// Headers
// **************************

#include "mystring.h"
#include "mymessage.h"     // needed for error messages
#include <stdio.h>          // needed for sprintf

// **************************
// Static variables
// **************************

char MyString::dummy=0;

// **************************
// MyString Functions
// **************************

MyString::MyString(const char *s)  // constructor
{
  if (s==NULL)           // there is nothing in this string
  {
    length=0;                    // set length of string
    string=new char[length+1];   // allocate memory
    string[length]='\0';         // set end flag
  }
  else
  {
    length=strlen(s);            // set length
    string=new char[length+1];   // allocate memory

    for (unsigned i=0; i<length; i++) string[i]=s[i];  // copy string
    string[length]='\0';                      // set end tag
  }
}

MyString::MyString(const MyString &s)  // constructor
{
  length=s.Length();            // set length
  string=new char[length+1];    // allocate memory

  for (unsigned int i=0;i<length;i++) string[i]=s[i];   // copy string
  string[length]='\0';                                  // set end tag
}

MyString::~MyString(void)   // destructor
{
  if (string) delete [] string;   // give memory free
  string=NULL;
}

MyString& MyString::operator=(const char *s)  // copy constructor
{
  if (s==NULL)           // there is nothing in this string
  {
    SetSize(0);
  }
  else
  {
    SetSize(strlen(s));
    for (unsigned i=0; i<length; i++) string[i]=s[i];  // copy string
  }

  return(*this);
}

MyString& MyString::operator=(const MyString &s)   // copy constructor
{
  SetSize(s.Length());
  for (unsigned i=0; i<length; i++) string[i]=s[i];  // copy string
  return(*this);
}

void MyString::Set(const char *s)  // copy constructor
{
  if (s==NULL)           // there is nothing in this string
  {
    SetSize(0);
  }
  else
  {
    SetSize(strlen(s));
    for (unsigned i=0; i<length; i++) string[i]=s[i];  // copy string
  }
}

void MyString::Set(const MyString &s)   // copy constructor
{
  SetSize(s.Length());
  for (unsigned i=0; i<length; i++) string[i]=s[i];  // copy string
}

MyString MyString::operator+(const MyString &s) const   // binary concatenation operator
{
  MyString r(*this);      // copy this string
  r.Add(s);               // add string s
  return(r);
}

MyString MyString::operator+(const char *s) const   // binary concatenation operator
{
  MyString r(*this);      // copy this string
  r.Add(s);               // add string s
  return(r);
}

MyString MyString::operator+(const char c) const // binary concatenation constructor
{
  MyString r(*this);      // copy this string
  r.Add(c);               // add chracter c
  return(r);
}

MyString& MyString::operator+=(const MyString &s)    // unary concatenation constructor
{
  if (s.Length()==0) return(*this);  // s ist empty

  unsigned int l=length;
  SetSize(length+s.Length());
  for (unsigned int i=0; i<s.Length(); i++) string[l+i]=s[i];   // copy new string

  return(*this);
}

MyString& MyString::operator+=(const char *s)  // unary concatenation constructor
{
  if (s==NULL) return(*this);       // s is NULL
  if (strlen(s)==0) return(*this);  // s ist empty

  unsigned int l=length;
  SetSize(length+strlen(s));
  for (unsigned int i=0; i<strlen(s); i++) string[l+i]=s[i];   // copy new string

  return(*this);
}

MyString& MyString::operator+=(const char c)   // unary concatenation constructor
{
  SetSize(length+1);
  string[length-1]=c;
  return(*this);
}

void MyString::Add(const char *s)  // add string
{
  if (!s) return;            // s is NULL
  if (strlen(s)==0) return;  // s ist empty

  unsigned int l=length;
  SetSize(length+strlen(s));
  for (unsigned int i=0; i<strlen(s); i++) string[l+i]=s[i];   // copy new string
}

void MyString::Add(const char c)   // add string
{
  SetSize(length+1);
  string[length-1]=c;
}

void MyString::Add(const MyString &s)    // add string
{
  if (s.Length()==0) return;  // s ist empty

  unsigned int l=length;
  SetSize(length+s.Length());
  for (unsigned int i=0; i<s.Length(); i++) string[l+i]=s[i];   // copy new string
}

const bool MyString::operator==(const char *s) const // compare constructor
{
  if (s==NULL || length!=strlen(s)) return(false);

  for (unsigned int i=0; i<length; i++)
  {
    if (string[i]!=s[i]) return(false);
  }
  return(true);
}

const bool MyString::operator==(const MyString &s) const // compare constructor
{
  if (length != s.Length()) return(false); // lengths are different

  for (unsigned int i=0; i<length; i++)
  {
    if (string[i]!=s[i]) return(false);
  }
  return(true);
}

const bool MyString::operator!=(const char *s) const // compare constructor
{
  if (s==NULL || length!=strlen(s)) return(true);

  for (unsigned int i=0; i<length; i++)
  {
    if (string[i]!=s[i]) return(true);
  }
  return(false);
}

const bool MyString::operator!=(const MyString &s) const // compare constructor
{
  if (length != s.Length()) return(true); // lengths are different

  for (unsigned int i=0; i<length; i++)
  {
    if (string[i]!=s[i]) return(true);
  }
  return(false);
}

const char& MyString::operator[](const unsigned int index) const   // return entry
{
  if (index>=length)         // index out of boundaries
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "MyString::operator[] const");
    return(dummy);
  }
  return (string[index]);
}

char& MyString::operator[](const unsigned int index)  // return entry
{
  if (index>=length)         // index out of boundaries
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "MyString::operator[]");
    return(dummy);
  }
  return (string[index]);
}

const char *MyString::data(void) const  // returns string as char*
{
  return (string);
}

const unsigned int MyString::Length(void) const     // returns length of string
{
  return(length);
}

const bool MyString::IsEmpty(void) const    // checks if string is empty
{
  if (length==0) return (true);
  else return(false);
}

void MyString::Empty(void)                   // clear string
{
  SetSize(0);
}

void MyString::Trunc(const unsigned int newlength)  // truncate array
{
  if (newlength<length)  // array should be smaller
  {
    char *V=new char[newlength+1];   // allocate memory

    for (unsigned int i=0; i<newlength; i++) V[i]=string[i]; // copy values
    V[newlength]='\0';                          // set end flag

    if (string) delete [] string;    // give memory free
    string=V;
    V=NULL;

    length=newlength;
  }
}

void MyString::Extend(const unsigned int newlength)   // extend array to special size
{
  if (newlength>length)  // array should be greater
  {
    char *V=new char[newlength+1];   // allocate memory
    unsigned int i;

    for (i=0; i<length; i++) V[i]=string[i];    // copy values
    for (i=length; i<newlength; i++) V[i]=' ';  // fill with spaces
    V[newlength]='\0';                          // set end flag

    if (string) delete [] string;    // give memory free
    string=V;
    V=NULL;

    length=newlength;
  }
}

void MyString::SetSize(const unsigned int newlength)    // truncate or extend array
{
  if (newlength<length) Trunc(newlength);
  else if (newlength>length) Extend(newlength);
  // if newsize==size there is nothing to do
}

const bool MyString::StartsWith(const char *s) const
{      // returns true if string starts with s

  if (s==NULL) return(false);

  if (strlen(s)>length) return(false);

  for (unsigned int i=0; i<strlen(s); i++)
  {
    if (string[i]!=s[i]) return(false);
  }

  return(true);
}

const bool MyString::StartsWith(const MyString& s) const
{    // returns true if string starts with s

  if (s.Length()>length) return(false);

  for (unsigned int i=0; i<s.Length(); i++)
  {
    if (string[i]!=s[i]) return(false);
  }

  return(true);
}


const unsigned int MyString::Contains(const char c) const  // returns numbers of c contained in the string
{
  unsigned int num=0;
  for (unsigned int i=0; i<length; i++) if (string[i]==c) num++;
  return(num);
}

const int MyString::Find(const char c, const unsigned int n) const    // finds char c, begin search at position n
{
  for (unsigned int i=n; i<length; i++)
  {
    // printf("%d: %c == %c\n", i, string[i], c);
    if (string[i]==c) return (i);
  }
  return(-1); // -1 means not found
}

const int MyString::Find(const MyString &sf, const unsigned int n) const   // finds string sf at position n
{
  unsigned int l=sf.Length();  // get length of other string

  if (length<l) return(-1);    // the other string is bigger and could not be contained

  MyString tempStr;

  for (unsigned int i=n; i<=length-l; i++)
  {
    Mid(tempStr,i,l);
    if (tempStr==sf) return(i);
  }

  return(-1); // -1 means not found
}

void MyString::Left(MyString &s, const unsigned int n) const     // set the n-th left chars
{
  unsigned int number=n;                 // number of characters to get
  if (number>length) number=length;      // cut would exceed boundaries

  s.SetSize(number);  // allocate memory if sizes do not match

  for (unsigned int i=0; i<number; i++) s[i]=string[i];
}

void MyString::Right(MyString &s, const unsigned int n) const    // set the n-th right chars
{
  unsigned int number=n;                 // number of characters to get
  if (number>length) number=length;      // cut would exceed boundaries

  s.SetSize(number);  // allocate memory if sizes do not match

  for (unsigned int i=0; i<number; i++) s[i]=string[length-(number-i)];
}

void MyString::Mid(MyString &s, const unsigned int n, const unsigned int w) const // set with w chars from pos n
{
  unsigned int begin=n;                  // first character to copy
  if (begin>length) begin=length;        // cut would exceed boundaries

  unsigned int last=begin+w;             // last character to copy
  if (last>length) last=length;          // cut would exceed boundaries

  unsigned int number=last-begin;        // number of characters to get

  s.SetSize(number);  // allocate memory if sizes do not match

  for (unsigned int i=begin; i<last; i++) s[i-begin]=string[i];
}


const MyString MyString::Left(const unsigned int n) const     // set the n-th left chars
{
  unsigned int number=n;                 // number of characters to get
  if (number>length) number=length;      // cut would exceed boundaries

  MyString s;
  s.SetSize(number); // allocate memory

  for (unsigned int i=0; i<number; i++) s[i]=string[i];
  return(s);
}

const MyString MyString::Right(const unsigned int n) const    // set the n-th right chars
{
  unsigned int number=n;            // number of characters to get
  if (length-n<0) number=length;

  MyString s;
  s.SetSize(number); // allocate memory

  for (unsigned int i=0; i<number; i++) s[i]=string[length-(number-i)];
  return(s);
}

const MyString MyString::Mid(const unsigned int n, const unsigned int w) const // set with w chars from pos n
{
  unsigned int begin=n;                  // first character to copy
  if (begin>length) begin=length;        // cut would exceed boundaries

  unsigned int last=begin+w;             // last character to copy
  if (last>length) last=length;          // cut would exceed boundaries

  unsigned int number=last-begin;        // number of characters to get

  MyString s;
  s.SetSize(number); // allocate memory

  for (unsigned int i=begin; i<last; i++) s[i-begin]=string[i];
  return(s);
}

const char MyString::Last(void) const
{   // return last char

  if (length==0) return('\0');
  else return(string[length-1]);
}

void MyString::Replace(const char c1, const char c2)
{ // replace a char with another

  if (c2=='\0') return;

  int pos=Find(c1);

  while (pos!=-1)
  {
    string[pos]=c2;
    pos=Find(c1);
  }
}

void MyString::StripSpaces(void)
{
  unsigned int index=0;
  bool start=true;
  bool space=false;

  Replace('\t', ' ');  // replace tabs with spaces

  for (unsigned int i=0; i<length; i++)
  {
    if (string[i]!=' ')
    {
      string[index]=string[i];
      index++;
      start=false;
      space=false;
    }
    else if (!start && !space)
    {
      string[index]=string[i];
      index++;
      space=true;
    }
  }

  if (index==0)
  {
    Empty();
    return;
  }

  if (string[index-1]==' ')
  {
    string[index-1]='\0';
    length=index-1;
  }
  else
  {
    string[index]='\0';
    length=index;
  }
}

void MyString::StripRight(const unsigned int num)
{  // delete n right characters

  Set(Right(length-num));
  StripSpaces();
}

void MyString::StripLeft(const unsigned int num)
{  // delete n left characters

  Set(Left(length-num));
  StripSpaces();
}

const MyString MyString::toLower(void) const     // generate lower case letters
{
  MyString str(string);
  str.MakeLower();
  return(str);
}

const MyString MyString::toUpper(void) const     // generate upper case letters
{
  MyString str(string);
  str.MakeUpper();
  return(str);
}

void MyString::toLower(MyString &str) const     // generate lower case letters
{
  str.Set(string);
  str.MakeLower();
}

void MyString::toUpper(MyString &str) const     // generate upper case letters
{
  str.Set(string);
  str.MakeUpper();
}

void MyString::MakeLower(void)
{      // generate lower case letters

  for (unsigned int i=0; i<length; i++)
  {
    if (string[i]>=65 && string[i]<=90) string[i]+=32;
  }
}

void MyString::MakeUpper(void)
{      // generate upper case letters

  for (unsigned int i=0; i<length; i++)
  {
    if (string[i]>=97 && string[i]<=122) string[i]-=32;
  }

}

const int MyString::toInt(void) const    // convert string to integer
{
  return(atoi(string));
}

const double MyString::toDouble(void) const    // convert string to double
{
  return(atof(string));
}

const MyString MyString::toString(const int value, const bool zeros)   // convert integer to string
{
  char *str=new char[50+1];  // gets 50 chars for integers < 10^50
  if (zeros) sprintf(str, "%03d", value);
  else sprintf(str, "%d", value);
  MyString temp(str);
  return(temp);
}

const MyString MyString::toString(const double value)  // convert double to string
{
  char *str=new char[50+1];  // gets 50 chars for double < 10^50
  sprintf(str, "%3.16f", value);
  MyString temp(str);
  return(temp);
}

const bool MyString::IsInt(void) const
{  // return true if string is integer

  char c;

  if (length==0) return(false);

  for (unsigned int i=0; i<length; i++)
  {
    c=string[i];
    if (c!='0' && c!='1' && c!='2' && c!='3' && c!='4' && c!='5' && c!='6'
     && c!='7' && c!='8' && c!='9' && c!='0') return(false);
  }
  return(true);
}

const bool MyString::IsDouble(void) const
{  // return true if string is double

  char c;

  if (length==0) return(false);

  for (unsigned int i=0; i<length; i++)
  {
    c=string[i];
    if (c!='0' && c!='1' && c!='2' && c!='3' && c!='4' && c!='5' && c!='6'
     && c!='7' && c!='8' && c!='9' && c!='0' && c!='.') return(false);
  }
  return(true);
}

const bool MyString::IsNumber(void) const
{  // return true if string is number (even hexeadecimal)

  char c;

  if (StartsWith("0x"))
  {  // possible hexadecimal number

    if (length<=2) return(false);

    for (unsigned int i=2; i<length; i++)
    {
      c=string[i];

      if (c!='0' && c!='1' && c!='2' && c!='3' && c!='4' && c!='5' && c!='6'
       && c!='7' && c!='8' && c!='9' && c!='0' && c!='A' && c!='B' && c!='C'
       && c!='D' && c!='E' && c!='F' && c!='a' && c!='b' && c!='c' && c!='d'
       && c!='e' && c!='f') return(false);
    }
  }
  else
  {
    if (length==0) return(false);

    for (unsigned int i=0; i<length; i++)
    {
      c=string[i];

      if (c!='0' && c!='1' && c!='2' && c!='3' && c!='4' && c!='5' && c!='6'
       && c!='7' && c!='8' && c!='9' && c!='0') return(false);
    }
  }
  return(true);
}



