// *************************************************************************
// MyString Class
// Copyright 2004, 2005, 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#if !defined(MYSTRING_H)
#define MYSTRING_H

/*
#ifdef UNICODE
#define _mT(x) L ## x
#else // !Unicode
#define _mT(x) x
#endif
*/

// **************************
// Headers
// **************************

#include <string>

// **************************
// MyString class
// **************************

// Note: The following string class is partly derived from QString of QT
// (www.trolltech.com). For compatibility with C-Strings it has also
// an end flag '\0' even if the size will be stored too.

class MyString    // new string definition
{
public:
  MyString(const char *s=NULL);              // constructor
  MyString(const MyString &s);       // constructor
  virtual ~MyString(void);           // destructor

  MyString& operator=(const char *s);             // copy constructor
  MyString& operator=(const MyString &s);    // copy constructor

  // Note: Instead of operator= you could use Set because it's faster.

  void Set(const char *s);                // set string
  void Set(const MyString &s);       // set string

  const bool operator==(const char *s) const;             // compare constructor
  const bool operator==(const MyString &s) const;    // compare constructor
  const bool operator!=(const char *s) const;             // compare constructor
  const bool operator!=(const MyString &s) const;    // compare constructor

  MyString operator+(const MyString &s) const;    // binary concatenation operator
  MyString operator+(const char *s) const;        // binary concatenation operator
  MyString operator+(const char c) const;        // binary concatenation operator

  MyString& operator+=(const char c);             // unary concatenation constructor
  MyString& operator+=(const char *s);            // unary concatenation constructor
  MyString& operator+=(const MyString &s);         // unary concatenation constructor

  // Note: Instead of operator+= you could use Add because it's faster.

  void Add(const char c);             // add character
  void Add(const char *s);            // add string
  void Add(const MyString &s);         // add string

  const char& operator[](const unsigned int index) const;   // get entry
  char& operator[](const unsigned int index);               // get entry

  const char *data(void) const;        // returns string as char*
  const unsigned int Length(void) const;    // length of the string

  const char Last(void) const;    // return last char

  const bool IsEmpty(void) const;     // checks if string is empty
  void Empty(void);             // clear string

  void SetSize(const unsigned int newsize);        // truncate or extend string
  void Extend(const unsigned int newsize);        // extend string to special size
  void Trunc(const unsigned int newsize);         // truncate string at position

  const bool StartsWith(const char *s) const;      // returns true if string starts with s
  const bool StartsWith(const MyString& s) const;    // returns true if string starts with s

  const unsigned int Contains(const char c) const;        // returns numbers of c contained in the string
  const int Find(const char c, const unsigned int n=0) const;        // finds char c at position n
  const int Find(const MyString &sf, const unsigned int n=0) const;   // finds string sf at position n

  const MyString Left(const unsigned int n) const;        // set the n-th left chars
  const MyString Right(const unsigned int n) const;       // set the n-th right chars
  const MyString Mid(const unsigned int n, const unsigned int w) const; // set with w chars from pos n

  void Left(MyString &str, const unsigned int n) const;        // set the n-th left chars
  void Right(MyString &str, const unsigned int n) const;       // set the n-th right chars
  void Mid(MyString &str, const unsigned int n, const unsigned int w) const; // set with w chars from pos n

  void StripRight(const unsigned int n);   // delete n right characters
  void StripLeft(const unsigned int n);    // delete n left characters

  void StripSpaces(void);                   // strip spaces and tabs

  void Replace(const char c1, const char c2);  // replace a char with another

  const MyString toLower(void) const;      // generate lower case letters
  const MyString toUpper(void) const;      // generate upper case letters

  void toLower(MyString &str) const;      // generate lower case letters
  void toUpper(MyString &str) const;      // generate upper case letters

  void MakeLower(void);      // generate lower case letters
  void MakeUpper(void);      // generate upper case letters

  const bool IsInt(void) const;  // return true if string is integer
  const bool IsDouble(void) const;  // return true if string is double
  const bool IsNumber(void) const;  // return true if string is number (even hexeadecimal)

  const int toInt(void) const;                  // convert string to integer
  const double toDouble(void) const;            // convert string to double
  static const MyString toString(const int value, const bool zeros);  // convert integer to string
  static const MyString toString(const double value);  // convert double to string

private:
  char *string;
  unsigned int length;     // length of string for better indexing

  static char dummy;
};

#endif // !defined(MYSTRING_H)
