// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// **************************************
// Header Files
// **************************************

#include "option.h"
#include "mymessage.h"

// **************************
// Class Declaration
// **************************

Option::Option(void)
{  // constructor
  active=true;
}

Option::Option(const MyString& opt, const MyString& val, const MyString& com, const bool act)
{  // constructor

  name=opt;
  value=val;
  comment=com;
  active=act;
}

Option::Option(const Option &opt)
{  // copy constructor

  name=opt.GetName();
  value=opt.GetValue();
  comment=opt.GetComment();
  active=opt.IsActive();
}

Option::~Option(void)
{  // destructor
}

Option& Option::operator=(const Option &opt)
{  // copy constructor

  // clear old entries
  Clear();

  name=opt.GetName();
  value=opt.GetValue();
  comment=opt.GetComment();
  active=opt.IsActive();

  return(*this);
}

void Option::Set(const MyString& opt, const MyString& val, const MyString& com, const bool act)
{  /// Set everything

  // clear old entries
  Clear();
  name=opt;
  value=val;
  comment=com;
  active=act;
}

void Option::SetName(const MyString& opt)
{ // set an Option

  name=opt;
}

void Option::SetValue(const MyString& val)
{ // set value

  value=val;
}

void Option::SetComment(const MyString& com)
{ // set comment

  comment=com;
}

void Option::SetActive(const bool act)
{ // set active status

  active=act;
}

void Option::ToogleActive(void)
{  /// Toggle active status.
  active=!active;
}

void Option::Clear(void)
{  // clear all Options and values

  name.Empty();
  value.Empty();
  comment.Empty();
  active=true;
}

const MyString& Option::GetName(void) const
{  // get single Option

  return(name);
}

const MyString& Option::GetValue(void) const
{  // get single Option value

  return(value);
}

const MyString& Option::GetComment(void) const
{  // get comment

  return(comment);
}

const bool Option::IsActive(void) const
{                            // return active status
  return(active);
}

void Option::CreateLine(MyString& line, const unsigned int numTabs, const bool val, const bool quotes, const bool com, const MyString& altName, const bool act)
{  /// Create line for output in xorg file.

  line.Empty();
  if (!active || !act) line+="#";
  for (unsigned int i=0; i<numTabs; i++)
  {
    line+="\t";
  }
  line+=altName;
  line+=" \"";
  line+=name;
  line+="\"";

  if (val && !value.IsEmpty())
  {
    line+=" ";
    if (quotes) line+="\"";
    line+=value;
    if (quotes) line+="\"";
  }
  if (com && !comment.IsEmpty())
  {
    line+=" # ";
    line+=comment;
  }
}

const bool Option::Check(void) const
{  /// Check if everything has correct syntax.

  if (name.IsEmpty())
  {
    MyMessage::Set(ERROR_CHECK_EMPTY_OPTION, "Option::Check");
    return(true);
  }

  return(false);

}
