// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// **************************************
// Header Files
// **************************************

#include "screen.h"
#include "mymessage.h"

// **************************
// Class Declaration
// **************************

Screen::Screen(void)
{  // constructor
}

Screen::~Screen(void)
{  // destructor
}

Screen::Screen(const Screen &scr)
{  // copy constructor

  identifier=scr.GetId();
  monitorEntry=scr.GetMonitor();
  deviceEntry=scr.GetDevice();
  defaultDepth=scr.GetDefaultDepth();
  identifierComment=scr.GetIdComment();
  monitorEntryComment=scr.GetMonitorComment();
  deviceEntryComment=scr.GetDeviceComment();
  defaultDepthComment=scr.GetDefaultDepthComment();
  comments=scr.Comments();
  options=scr.Options();
  displays=scr.Displays();
}

Screen& Screen::operator=(const Screen &scr)
{  // copy constructor

  Clear();

  identifier=scr.GetId();
  monitorEntry=scr.GetMonitor();
  deviceEntry=scr.GetDevice();
  defaultDepth=scr.GetDefaultDepth();
  identifierComment=scr.GetIdComment();
  monitorEntryComment=scr.GetMonitorComment();
  deviceEntryComment=scr.GetDeviceComment();
  defaultDepthComment=scr.GetDefaultDepthComment();
  comments=scr.Comments();
  options=scr.Options();
  displays=scr.Displays();

  return(*this);
}

void Screen::SetId(const MyString& id, const MyString& com)
{  // set identifier

  identifier=id;
  identifierComment=com;
}

void Screen::SetMonitor(const MyString& mon, const MyString& com)
{  // set monitor

  monitorEntry=mon;
  monitorEntryComment=com;
}

void Screen::SetDevice(const MyString& dev, const MyString& com)
{  // set device

  deviceEntry=dev;
  deviceEntryComment=com;
}

void Screen::SetDefaultDepth(const MyString& dd, const MyString& com)
{  // set default depth

  defaultDepth=dd;
  defaultDepthComment=com;
}

void Screen::SetNumber(const MyString& num)
{  /// Set screen number.
  number=num;
}

void Screen::SetPosition(const MyString& pos)
{  /// Set screen position.
  position=pos;
}

void Screen::SetSecond(const MyString& sec)
{  /// Set second screen.
  secondId=sec;
}

void Screen::Clear(void)
{    // clear everything

  identifier.Empty();
  monitorEntry.Empty();
  deviceEntry.Empty();
  defaultDepth.Empty();
  identifierComment.Empty();
  monitorEntryComment.Empty();
  deviceEntryComment.Empty();
  defaultDepthComment.Empty();
  comments.Clear();
  options.Clear();
  displays.Clear();

}

const MyString& Screen::GetId(void) const
{  // return identifier

  return(identifier);
}

const MyString& Screen::GetMonitor(void) const
{  // return monitor

  return(monitorEntry);
}

const MyString& Screen::GetDevice(void) const
{  // return device

  return(deviceEntry);
}

const MyString& Screen::GetDefaultDepth(void) const
{  // return default depth

  return(defaultDepth);
}

const MyString& Screen::GetNumber(void) const
{  /// Get screen number.
  return(number);
}

const MyString& Screen::GetPosition(void) const
{  /// Get screen position.
  return(position);
}

const MyString& Screen::GetSecond(void) const
{  /// Get second screen.
  return(secondId);
}

const MyString& Screen::GetIdComment(void) const
{  // return identifier Comment

  return(identifierComment);
}

const MyString& Screen::GetMonitorComment(void) const
{  // return monitor Comment

  return(monitorEntryComment);
}

const MyString& Screen::GetDeviceComment(void) const
{  // return device Comment

  return(deviceEntryComment);
}

const MyString& Screen::GetDefaultDepthComment(void) const
{  // return default depth Comment

  return(defaultDepthComment);
}

const OptionList& Screen::Options(void) const
{  // get options

  return(options);
}

DisplayList& Screen::Displays(void)
{  // get displays

  return(displays);
}

const DisplayList& Screen::Displays(void) const
{  // get displays

  return(displays);
}

OptionList& Screen::Options(void)
{  // get options

  return(options);
}

const StringArray& Screen::Comments(void) const
{  // return comments
  return(comments);
}

StringArray& Screen::Comments(void)
{  // return comments as reference
  return(comments);
}

const bool Screen::Check(const bool checkServer)
{  /// Check if everything has correct syntax.

  if (!checkServer)
  {
    if (identifier.IsEmpty())
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_SCREEN_ID, "Screen::Check");
      return(true);
    }

    if (defaultDepth.IsEmpty())
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_SCREEN_DEPTH, "Screen::Check");
      return(true);
    }
    else
    {
      MyString tempStr=defaultDepth;
      tempStr.StripSpaces();

      if (!tempStr.IsInt() || tempStr.toInt()<=0)
      {
        MyMessage::Set(ERROR_CHECK_SCREEN_DEPTH, "Screen::Check");
        return(true);
      }
    }

    if (options.Check())
    {
      if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
      {
        MyMessage::Set(ERROR_CHECK_EMPTY_SCREEN_OPTION, "Screen::Check");
      }
      else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
      {
        MyMessage::Set(ERROR_CHECK_SAME_SCREEN_OPTION, "Screen::Check");
      }
      else
      {
        MyMessage::Append("Screen::Check");
      }
      return(true);
    }

    if (displays.Check())
    {
      MyMessage::Append("Screen::Check");
      return(true);
    }
  } // !checkServer
  else  // checkServer
  {
    if (identifier.IsEmpty())
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_SERVERLAYOUT_SCREEN, "Screen::Check");
      return(true);
    }

    // check if double
    if (!secondId.IsEmpty() && identifier==secondId)
    {
      MyMessage::Set(ERROR_CHECK_SAME_SERVERLAYOUT_SCREENS, "Screen::Check");
      return(true);
    }

    if (!number.IsEmpty())
    {
      if (!number.IsInt() || number.toInt()<0)
      {
        MyMessage::Set(ERROR_CHECK_SERVERLAYOUT_SCREEN_NUMBER, "Screen::Check");
        return(true);
      }
    }

    MyString tempStr=position;
    tempStr.StripSpaces();
    tempStr.MakeLower();

    if (tempStr.StartsWith("absolute"))
    {
      tempStr.StripRight(8);

      int pos=tempStr.Find(' ');
      if (pos==-1)
      {
        MyMessage::Set(ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION, "Screen::Check");
        return(true);
      }

      if (!tempStr.Left(pos).IsInt() || tempStr.toInt()<0)
      {
        MyMessage::Set(ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION, "Screen::Check");
        return(true);
      }

      tempStr.StripRight(pos);
      if (!tempStr.IsInt() || tempStr.toInt()<0)
      {
        MyMessage::Set(ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION, "Screen::Check");
        return(true);
      }
    }
    else if (tempStr=="rightof" || tempStr=="leftof" || tempStr=="above" || tempStr=="below")
    {
      // everythings fine
    }
    else if (tempStr=="relative")
    {
      MyMessage::Set(ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION_REL, "Screen::Check");
      return(true);
    }
    else if (!tempStr.IsEmpty())
    {
      int pos=tempStr.Find(' ');
      if (pos==-1)
      {
        MyMessage::Set(ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION, "Screen::Check");
        return(true);
      }

      if (!tempStr.Left(pos).IsInt() || tempStr.toInt()<0)
      {
        MyMessage::Set(ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION, "Screen::Check");
        return(true);
      }

      tempStr.StripRight(pos);
      if (!tempStr.IsInt() || tempStr.toInt()<0)
      {
        MyMessage::Set(ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION, "Screen::Check");
        return(true);
      }
    }
  }  // checkServer

  return(false);
}
