// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef SCREEN_H
#define SCREEN_H

// **************************************
// Header Files
// **************************************

#include "optionlist.h"
#include "displaylist.h"
#include "stringarray.h"

// **************************
// Class Definition
// **************************

class Screen
{
public:
  /// Constructor.
  Screen(void);

  /// Copy Constructor.
  Screen(const Screen &scr);

  /// Destructor.
  ~Screen(void);

  /// Assignment operator.
  Screen& operator=(const Screen &scr);

  /// Set screen identifier and optional comment.
  void SetId(const MyString& id, const MyString& com=MyString());

  /// Set screen monitor and optional comment.
  void SetMonitor(const MyString& mon, const MyString& com=MyString());

  /// Set screen device and optional comment.
  void SetDevice(const MyString& dev, const MyString& com=MyString());

  /// Set screen depth and optional comment.
  void SetDefaultDepth(const MyString& dd, const MyString& com=MyString());

  /// Set screen number.
  void SetNumber(const MyString& num);

  /// Set screen position.
  void SetPosition(const MyString& pos);

  /// Set second screen.
  void SetSecond(const MyString& sec);

  /// Get screen identifier.
  const MyString& GetId(void) const;

  /// Get screen monitor.
  const MyString& GetMonitor(void) const;

  /// Get screen device.
  const MyString& GetDevice(void) const;

  /// Get screen depth.
  const MyString& GetDefaultDepth(void) const;

  /// Get screen number.
  const MyString& GetNumber(void) const;

  /// Get screen position.
  const MyString& GetPosition(void) const;

  /// Get second screen.
  const MyString& GetSecond(void) const;

  /// Get screen identifier comment.
  const MyString& GetIdComment(void) const;

  /// Get screen monitor comment.
  const MyString& GetMonitorComment(void) const;

  /// Get screen device comment.
  const MyString& GetDeviceComment(void) const;

  /// Get screen depth comment.
  const MyString& GetDefaultDepthComment(void) const;

  /// Get comments.
  const StringArray& Comments(void) const;

  /// Get comments (as reference).
  StringArray& Comments(void);

  /// Get options.
  const OptionList& Options(void) const;

  /// Get options (as reference).
  OptionList& Options(void);

  /// Get displays.
  const DisplayList& Displays(void) const;

  /// Get displays (as reference).
  DisplayList& Displays(void);

  /// Clear everything.
  void Clear(void);

  /// Check if everything has correct syntax.
  /**
  * Return <b>true</b> if something is wrong else <b>false</b>.
  */
  const bool Check(const bool checkServer=false);

private:
  MyString identifier;      // Screen identifier
  MyString monitorEntry;     // monitor
  MyString deviceEntry;       // device entry
  MyString defaultDepth;    // default depth

  // Note: Any additional settings will not be recognized like:
  // VideoAdaptor, DefaultFbBpp.

  MyString identifierComment;
  MyString monitorEntryComment;
  MyString deviceEntryComment;
  MyString defaultDepthComment;

  StringArray comments;
  DisplayList displays;     // all display subsections
  OptionList options;       // all options

  // elements for server layout screens
  MyString number;          // screen-number
  MyString position;        // screen position
  MyString secondId;        // second screen

};

#endif // SCREEN_H
