// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// **************************************
// Header Files
// **************************************

#include "serverlayout.h"
#include "mymessage.h"

// **************************
// Class Declaration
// **************************

ServerLayout::ServerLayout(void)
{  // constructor
}

ServerLayout::~ServerLayout(void)
{  // destructor
}

ServerLayout::ServerLayout(const ServerLayout &ser)
{  // copy constructor

  identifier=ser.GetId();
  identifierComment=ser.GetIdComment();

  comments=ser.Comments();
  options=ser.Options();
  inputDevices=ser.InputDevices();
  screens=ser.Screens();
}

ServerLayout& ServerLayout::operator=(const ServerLayout &ser)
{  // copy constructor

  Clear();

  identifier=ser.GetId();
  identifierComment=ser.GetIdComment();

  comments=ser.Comments();
  options=ser.Options();
  inputDevices=ser.InputDevices();
  screens=ser.Screens();

  return(*this);
}

void ServerLayout::SetId(const MyString& id, const MyString& com)
{  // set identifier

  identifier=id;
  identifierComment=com;
}

void ServerLayout::Clear(void)
{    // clear everything

  identifier.Empty();
  identifierComment.Empty();
  comments.Clear();
  screens.Clear();
  inputDevices.Clear();
  options.Clear();
}

const MyString& ServerLayout::GetId(void) const
{  // return identifier

  return(identifier);
}

const MyString& ServerLayout::GetIdComment(void) const
{  // return identifier comment

  return(identifierComment);
}

const OptionList& ServerLayout::Options(void) const
{  // get options

  return(options);
}

OptionList& ServerLayout::Options(void)
{  // get options

  return(options);
}

const InputDeviceList& ServerLayout::InputDevices(void) const
{  // get input devices

  return(inputDevices);
}

InputDeviceList& ServerLayout::InputDevices(void)
{  // get input devices

  return(inputDevices);
}

const ScreenList& ServerLayout::Screens(void) const
{  // get screens

  return(screens);
}

ScreenList& ServerLayout::Screens(void)
{  // get screens

  return(screens);
}

const StringArray& ServerLayout::Comments(void) const
{  // return comments
  return(comments);
}

StringArray& ServerLayout::Comments(void)
{  // return comments as reference
  return(comments);
}

const bool ServerLayout::Check(const bool checkDouble)
{  /// Check if everything has correct syntax.

  if (identifier.IsEmpty())
  {
    MyMessage::Set(ERROR_CHECK_EMPTY_SERVERLAYOUT_ID, "ServerLayout::Check");
    return(true);
  }

  if (options.Check())
  {
    if (MyMessage::Get()==ERROR_CHECK_EMPTY_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_EMPTY_SERVERLAYOUT_OPTION, "ServerLayout::Check");
    }
    else if (MyMessage::Get()==ERROR_CHECK_SAME_OPTION)
    {
      MyMessage::Set(ERROR_CHECK_SAME_SERVERLAYOUT_OPTION, "ServerLayout::Check");
    }
    else
    {
      MyMessage::Append("ServerLayout::Check");
    }
    return(true);
  }

  if (screens.Check(true))
  {
    if (MyMessage::Get()==ERROR_CHECK_SAME_SCREEN_ID)
    {
      if (checkDouble) MyMessage::Set(ERROR_CHECK_SAME_SERVERLAYOUT_SCREEN, "ServerLayout::Check");
      else
      {
        // ignore
        MyMessage::Reset();
        return(false);
      }
    }
    return(true);
  }

  if (inputDevices.Check(true))
  {
    if (MyMessage::Get()==ERROR_CHECK_SAME_INPUTDEVICE_ID)
    {
      if (checkDouble) MyMessage::Set(ERROR_CHECK_SAME_SERVERLAYOUT_INPUTDEVICE, "ServerLayout::Check");
      else
      {
        // ignore
        MyMessage::Reset();
        return(false);
      }
    }

    return(true);
  }

  return(false);
}


