// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef SERVERLAYOUT_H
#define SERVERLAYOUT_H

// **************************************
// Header Files
// **************************************

#include "optionlist.h"
#include "inputdevicelist.h"
#include "screenlist.h"
#include "stringarray.h"

// **************************
// Class Definition
// **************************

class ServerLayout
{
public:
  /// Constructor.
  ServerLayout(void);

  /// Copy Constructor.
  ServerLayout(const ServerLayout &ser);

  /// Destructor.
  ~ServerLayout(void);

  /// Assignment operator.
  ServerLayout& operator=(const ServerLayout &ser);

  /// Set server identifier and optional comment.
  void SetId(const MyString& id, const MyString& com=MyString());

  /// Get server identifier.
  const MyString& GetId(void) const;

  /// Get server identifier comment.
  const MyString& GetIdComment(void) const;

  /// Get comments.
  const StringArray& Comments(void) const;

  /// Get comments (as reference).
  StringArray& Comments(void);

  /// Get options.
  const OptionList& Options(void) const;

  /// Get options (as reference).
  OptionList& Options(void);

  /// Get input devices.
  const InputDeviceList& InputDevices(void) const;

  /// Get input devices (as reference).
  InputDeviceList& InputDevices(void);

  /// Get screen.
  const ScreenList& Screens(void) const;

  /// Get screen (as reference).
  ScreenList& Screens(void);

  /// Clear everything.
  void Clear(void);

  /// Check if everything has correct syntax.
  /**
  * Return <b>true</b> if something is wrong else <b>false</b>.
  */
  const bool Check(const bool checkDouble=false);

private:
  MyString identifier;           // identifier
  MyString identifierComment;

  StringArray comments;
  OptionList options;    // all options

  InputDeviceList inputDevices;      // input devices
  ScreenList screens;             // screens

};

#endif // SERVERLAYOUT_H
