// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// Header Files
// **************************************

#include "settings.h"
#include "mymessage.h"            // needed for error messages
#include <fstream>                // for FILE

using namespace std;

// **************************
// Class Declaration
// **************************

Settings::Settings(void)
{
  language=0; // wxLANGUAGE_DEFAULT;
  lineNr=0;
}

Settings::~Settings(void)
{
  language=0; // wxLANGUAGE_DEFAULT;
  lineNr=0;
}

Settings::Settings(const Settings& set)
{
  language=set.GetLanguage();
  progDir=set.GetProgDir();
  optionDir=set.GetOptionDir();
  curDir=set.GetCurrentDir();
  curFile=set.GetCurrentFilename();
  setFile=set.GetFilename();
  loadLastFile=set.GetLoadLastFile();
  saveSettings=set.GetSaveSettings();
  showSplash=set.GetShowSplash();
  showBlank=set.GetShowBlank();
  remTest=set.GetRememberTest();
  lineNr=0;
}

Settings& Settings::operator=(const Settings& set)
{
  language=set.GetLanguage();
  progDir=set.GetProgDir();
  optionDir=set.GetOptionDir();
  curDir=set.GetCurrentDir();
  curFile=set.GetCurrentFilename();
  setFile=set.GetFilename();
  loadLastFile=set.GetLoadLastFile();
  saveSettings=set.GetSaveSettings();
  showSplash=set.GetShowSplash();
  showBlank=set.GetShowBlank();
  remTest=set.GetRememberTest();
  lineNr=0;
  return(*this);
}

const MyString& Settings::GetProgDir(void) const
{
  return(progDir);
}

const MyString& Settings::GetOptionDir(void) const
{
  return(optionDir);
}

const MyString& Settings::GetCurrentDir(void) const
{
  return(curDir);
}

const MyString& Settings::GetCurrentFilename(void) const
{
  return(curFile);
}

const MyString& Settings::GetFilename(void) const
{
  return(setFile);
}

const MyString Settings::GetCurrentFile(void) const
{
  return(curDir+"/"+curFile);
}

const bool Settings::GetLoadLastFile(void) const
{
  return(loadLastFile);
}

const bool Settings::GetSaveSettings(void) const
{
  return(saveSettings);
}

const bool Settings::GetShowSplash(void) const
{
  return(showSplash);
}

const bool Settings::GetShowBlank(void) const
{
  return(showBlank);
}

const bool Settings::GetRememberTest(void) const
{
  return(remTest);
}

const int Settings::GetLanguage(void) const
{
  return(language);
}

void Settings::SetProgDir(const MyString& value)
{
  progDir=value;
}

void Settings::SetOptionDir(const MyString& value)
{
  optionDir=value;
}

void Settings::SetCurrentDir(const MyString& value)
{
  curDir=value;
}

void Settings::SetCurrentFilename(const MyString& value)
{
  curFile=value;
}

void Settings::SetFilename(const MyString& value)
{
  setFile=value;
}

void Settings::SetLoadLastFile(const bool value)
{
  loadLastFile=value;
}

void Settings::SetSaveSettings(const bool value)
{
  saveSettings=value;
}

void Settings::SetShowSplash(const bool value)
{
  showSplash=value;
}

void Settings::SetShowBlank(const bool value)
{
  showBlank=value;
}

void Settings::SetRememberTest(const bool value)
{
  remTest=value;
}

void Settings::SetLanguage(const int lang)
{
  language=lang;
}

const bool Settings::Load(const MyString& filename)
{  // load settings in ~/.xorg-edit

  ifstream stream;                   // File access.
  char readline[1000];               // allocate memory for 1000 chars
  MyString line;                     // current line
  MyString orgLine;                  // original line
  bool ignoreError=true;             // flag if errors should be ignored
  unsigned int tempLineNr=lineNr;
  lineNr=0;                          // start at beginning of file

  if (!filename.IsEmpty()) setFile=filename;

  if (setFile.IsEmpty())
  {
    MyMessage::Set(ERROR_READ_FILE, "Settings::Load", setFile);
    return(true);
  }

  stream.open(setFile.data());  // open file, but do not create it

  if (stream==NULL)
  {
    return(Save(true));  // create new file
  }

  while ( stream.getline(readline, 1000) )
  {
    lineNr++;
    if (lineNr>tempLineNr) ignoreError=false;

    orgLine=readline;     // set line

    line=orgLine;         // copy line (we need to change it)
    line.StripSpaces();   // strip spaces

    if (!line.IsEmpty())  // line is not empty
    {
      if (line.StartsWith("LANGUAGE="))
      {
        line.StripRight(9);
        language=line.toInt();
      }
      else if (line.StartsWith("OPTDIR=\""))
      {
        line.StripRight(8);
        if (line.Last()=='"')
        {
          line.StripLeft(1);
          optionDir=line;
        }
      }
      else if (line.StartsWith("LASTDIR=\""))
      {
        line.StripRight(9);
        if (line.Last()=='"')
        {
          line.StripLeft(1);
          curDir=line;
        }
      }
      else if (line.StartsWith("LASTFILE=\""))
      {
        line.StripRight(10);
        if (line.Last()=='"')
        {
          line.StripLeft(1);
          curFile=line;
        }
      }
      else if (line.StartsWith("LOADLAST="))
      {
        line.StripRight(9);
        if (line.Length()==1)
        {
          if (line=="0")
          {
            loadLastFile=false;
          }
          else
          {
            loadLastFile=true;
          }
        }
      }
      else if (line.StartsWith("SAVESETTINGS"))
      {
        line.StripRight(12);
        if (line.Length()==1)
        {
          if (line=="0")
          {
            saveSettings=false;
          }
          else
          {
            saveSettings=true;
          }
        }
      }
      else if (line.StartsWith("SHOWSPLASH="))
      {
        line.StripRight(11);
        if (line.Length()==1)
        {
          if (line=="0")
          {
            showSplash=false;
          }
          else
          {
            showSplash=true;
          }
        }
      }
      else if (line.StartsWith("SHOWBLANKTEST="))
      {
        line.StripRight(14);
        if (line.Length()==1)
        {
          if (line=="0")
          {
            showBlank=false;
          }
          else
          {
            showBlank=true;
          }
        }
      }
      else if (line.StartsWith("TESTBEFORESAVE="))
      {
        line.StripRight(15);
        if (line.Length()==1)
        {
          if (line=="0")
          {
            remTest=false;
          }
          else
          {
            remTest=true;
          }
        }
      }
      else
      {
          // unrecognized line... ignore
      }
    } // line is not empty
  } // EOF

  stream.close();

  return(false);
}

const bool Settings::Save(const bool start, const MyString& filename)
{  // save settings in ~/.xorg-edit

  // save following things
  // - language
  // - option directory
  // - last working directory
  // - last edited file
  // - always load last
  // - save profile at end
  // - show splash screen

  if (!filename.IsEmpty()) setFile=filename;

  if (setFile.IsEmpty())
  {
    MyMessage::Set(ERROR_WRITE_FILE, "Settings::Save", setFile);
    return(true);
  }

  ofstream stream;                   // File access.
  MyString tempStr;

  stream.open(setFile.data());  // open file, but do not create it

  if (stream==NULL)
  {
    MyMessage::Set(ERROR_WRITE_FILE, "Settings::Save", setFile);
    return(true);
  }

  // save language
  stream << "LANGUAGE=" << MyString::toString(language, false).data() << "\n";

  // save option directory
  stream << "OPTDIR=\"" << optionDir.data() << "\"\n";

  // save last working directory
  stream << "LASTDIR=\"" << curDir.data() << "\"\n";

  // save last file
  stream << "LASTFILE=\"" << curFile.data() << "\"\n";

  // save flag if last file should be loaded
  stream << "LOADLAST=";
  if (loadLastFile) stream << "1";
  else stream << "0";
  stream << "\n";

  // save flag if profle should be saved at the end
  stream << "SAVESETTINGS=";
  if (saveSettings) stream << "1";
  else stream << "0";
  stream << "\n";

  // save flag if warning should be shown at start
  stream << "SHOWSPLASH=";
  if (showSplash) stream << "1";
  else stream << "0";
  stream << "\n";

  stream << "SHOWBLANKTEST=";
  if (showBlank) stream << "1";
  else stream << "0";
  stream << "\n";

  stream << "TESTBEFORESAVE=";
  if (remTest) stream << "1";
  else stream << "0";
  stream << "\n";

  stream.close();

  return(false);
}

