// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// Array classes declaration
// **************************************

// **************************
// Headers
// **************************

#include "textfile.h"
#include "mymessage.h"     // needed for error messages
#include <fstream>
#include <iostream>

using namespace std;

const bool TextFile::Load(const MyString& filename)
{  /// Load text file.

  ifstream stream;                   // File access.
  char readline[1000];               // allocate memory for 1000 chars
  unsigned int lineNr=0;            // start at beginning of file

  stream.open(filename.data());  // open file, but do not create it

  if (stream==NULL)
  {
    Clear();
    MyMessage::Set(ERROR_READ_FILE, "TextFile::Load", filename);
    return(true);
  }

  // we will allocated blocks of 100 lines
  SetSize(100);

  while ( stream.getline(readline, 1000) )
  {
    values[lineNr].Set(readline);  // set line
    lineNr++;
    if (lineNr==size)
    {
      SetSize(size+100);  // setnew size
    }
  }
  Trunc(lineNr); // truncate to correct size

  stream.close();

  return(false);
}

const bool TextFile::Save(const MyString& filename)
{  /// Save text file.

  ofstream stream;                   // File access.
  stream.open(filename.data());      // open file

  if (stream==NULL)
  {
    MyMessage::Set(ERROR_WRITE_FILE, "TextFile::Save", filename);
    return(true);
  }

  for (unsigned int i=0; i<size; i++)
  {
    stream << values[i].data() << "\n";
  }

  stream.close();
  return(false);
}

void TextFile::AppendComment(const MyString& value, const unsigned int numTabs)
{  /// Append element.

  StringArray::Append(value);
  SetComment(size-1, numTabs);
}

void TextFile::AppendComment(const StringArray& array, const unsigned int numTabs)
{  /// Append all elements from other array.

  StringArray::Append(array);

  for (unsigned int i=size-array.GetSize(); i<size; i++)
  {
    SetComment(i, numTabs);
  }
}

void TextFile::AppendComment(const TextFile& array, const unsigned int numTabs)
{  /// Append all elements from other array.

  StringArray::Append(array);
  for (unsigned int i=size-array.GetSize(); i<size; i++)
  {
    SetComment(i, numTabs);
  }
}

const bool TextFile::SetComment(const unsigned int index, const unsigned int numTabs)
{  /// Append some line as comment.

  if (index>size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "TextFile::SetComment");
    return(true);
  }

  MyString line;
  for (unsigned int i=0; i<numTabs; i++)
  {
    line+="\t";
  }
  line+="# ";

  if (!values[index].StartsWith(line))
  {  // only add if there is none
    line+=values[index];
    values[index]=line;
  }

  return(false);
}
