// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// Header Files
// **************************************

#include "xorgfile.h"
#include "mymessage.h"
#include "driveroptionlist.h"
#include <fstream>
#include <iostream>

using namespace std;

// **************************
// Class Declaration
// **************************

XorgFile::XorgFile(void)
{
  lineNr=0;
}

XorgFile::~XorgFile()
{
}

XorgFile::XorgFile(const XorgFile& xorg)
{
  files=xorg.Files();
  modules=xorg.Modules();
  serverLayouts=xorg.ServerLayouts();
  inputDevices=xorg.InputDevices();
  devices=xorg.Devices();
  monitors=xorg.Monitors();
  screens=xorg.Screens();
  serverFlags=xorg.ServerFlags();
  extensions=xorg.Extensions();
  driEntry=xorg.Dri();
  comments=xorg.Comments();
  lineNr=0;
}

XorgFile& XorgFile::operator=(const XorgFile& xorg)
{
  Clear();
  lineNr=0;

  files=xorg.Files();
  modules=xorg.Modules();
  serverLayouts=xorg.ServerLayouts();
  inputDevices=xorg.InputDevices();
  devices=xorg.Devices();
  monitors=xorg.Monitors();
  screens=xorg.Screens();
  serverFlags=xorg.ServerFlags();
  extensions=xorg.Extensions();
  driEntry=xorg.Dri();
  comments=xorg.Comments();

  return(*this);
}

const FileList& XorgFile::Files(void) const
{
  return(files);
}

const ModuleList& XorgFile::Modules(void) const
{
  return(modules);
}

const ServerLayoutList& XorgFile::ServerLayouts(void) const
{
  return(serverLayouts);
}

const InputDeviceList& XorgFile::InputDevices(void) const
{
  return(inputDevices);
}

const DeviceList& XorgFile::Devices(void) const
{
  return(devices);
}

const MonitorList& XorgFile::Monitors(void) const
{
  return(monitors);
}

const ScreenList& XorgFile::Screens(void) const
{
  return(screens);
}

const ServerFlagList& XorgFile::ServerFlags(void) const
{
  return(serverFlags);
}

const ExtensionList& XorgFile::Extensions(void) const
{
  return(extensions);
}

const DriMode& XorgFile::Dri(void) const
{
  return(driEntry);
}

const StringArray& XorgFile::Comments(void) const
{
  return(comments);
}

FileList& XorgFile::Files(void)
{
  return(files);
}

ModuleList& XorgFile::Modules(void)
{
  return(modules);
}

ServerLayoutList& XorgFile::ServerLayouts(void)
{
  return(serverLayouts);
}

InputDeviceList& XorgFile::InputDevices(void)
{
  return(inputDevices);
}

DeviceList& XorgFile::Devices(void)
{
  return(devices);
}

MonitorList& XorgFile::Monitors(void)
{
  return(monitors);
}

ScreenList& XorgFile::Screens(void)
{
  return(screens);
}

ServerFlagList& XorgFile::ServerFlags(void)
{
  return(serverFlags);
}

ExtensionList& XorgFile::Extensions(void)
{
  return(extensions);
}

DriMode& XorgFile::Dri(void)
{
  return(driEntry);
}

StringArray& XorgFile::Comments(void)
{
  return(comments);
}

void XorgFile::Clear(void)
{
  files.Clear();
  modules.Clear();
  serverLayouts.Clear();
  inputDevices.Clear();
  devices.Clear();
  monitors.Clear();
  screens.Clear();
  serverFlags.Clear();
  extensions.Clear();
  driEntry.Clear();
  comments.Clear();
}

const bool XorgFile::GetArgument(MyString &arg, const MyString &line) const
{  // get a line and extract the last argument

  MyString tempStr(line);   // copy line
  tempStr.StripSpaces();
  arg.Empty();

  int pos=tempStr.Find('"');  // search first "

  if (pos==-1) return(true); // " not found

  // delete part before first "
  tempStr.StripRight(pos+1);

  pos=tempStr.Find('"');  // search second "

  if (pos==-1) return(true); // second " not found

  arg=tempStr.Left(pos);   // get argument

  arg.StripSpaces();
  if (arg.IsEmpty()) return(true);

  return(false);
}

const bool XorgFile::GetOption(Option &opt, const MyString &line) const
{  // get a line and extract option and argument

  MyString tempStr(line);   // copy line
  tempStr.StripSpaces();
  tempStr.MakeLower();
  opt.Clear();

  if (!tempStr.StartsWith("option")) return(true);

  tempStr=line;
  tempStr.StripSpaces();
  tempStr.StripRight(6);   // delete "option"

  if (tempStr.Find('"')!=0) return(true); // " not found at first position
  tempStr.StripRight(1);

  int pos=tempStr.Find('"');  // search first "
  if (pos==-1) return(true); // " not found

  MyString tempStr2=tempStr.Left(pos);
  tempStr2.StripSpaces();

  if (tempStr2.IsEmpty()) return(true);
  opt.SetName(tempStr2);

  tempStr.StripRight(pos+1);

  if (tempStr.Find('"')==-1) return(false); // no argument found
  if (tempStr.Find('"')!=0) return(true);  // " found but not at fist position

  tempStr=tempStr.Right(tempStr.Length()-1);
  pos=tempStr.Find('"');  // search second "

  if (pos==-1) return(true); // no second " found

  tempStr2=tempStr.Left(pos);
  tempStr2.StripSpaces();

  if (tempStr2.IsEmpty()) return(true);
  opt.SetValue(tempStr2);

  return(false);
}

const bool XorgFile::FreeComments(MyString &line, MyString& comment) const
{  // delete all comments

  comment.Empty();
  line.StripSpaces();      // strip spaces

  bool first=false;        // flag if # was first char
  int pos=line.Find('#');  // find first comment sign

  if (pos>=0)  // # found
  {
    if (pos==0) first=true;
    comment=line.Right(line.Length()-pos-1);  // extract comment
    line=line.Left(pos);                      // extract line
    line.StripSpaces();
    comment.StripSpaces();

    // it may happen that the comment has further #
    // delete the ones at beginning
    unsigned int i=0;

    while (i<comment.Length())
    {
      if (comment[i]!='#') break;
      i++;
    }
    if (i>0) comment.StripRight(i);
  }

  return(first);
}

const bool XorgFile::Load(const MyString& name, const bool reload)
{  // load file

  ifstream stream;                   // File access.
  char readline[1000];               // allocate memory for 1000 chars
  MyString line;                     // current line
  MyString orgLine;                  // original line
  MyString tempLine;                 // temporary line
  MyString comment;                  // comment in file
  MyString argument;                 // argument in line
  bool lineComment=false;            // flag if line is comment
  bool ignoreError=true;             // flag if errors should be ignored
  bool failure=false;                // read failure
  int pos=-1;                        // position of some chars
  xorgSectionID section=XORG_NOSECTION;  // current section

  unsigned int tempLineNr=lineNr;
  // if (!reload) tempLineNr=0;
  lineNr=0;                          // start at beginning of file

  // elements for reading
  Device tempDevice;
  Monitor tempMonitor;
  InputDevice tempInputDevice;
  Screen tempScreen;
  Display tempDisplay;
  ServerLayout tempServerLayout;
  Option tempOption;
  Module tempModule;

  // clear all old elements first
  Clear();

  stream.open(name.data());  // open file, but do not create it

  if (stream==NULL)
  {
    MyMessage::Set(ERROR_READ_FILE, "XorgFile::Load", name);
    return(true);
  }

  while ( stream.getline(readline, 1000) )
  {
    lineNr++;
    if (!reload || lineNr>tempLineNr) ignoreError=false;
    failure=false;
    lineComment=false;

    orgLine=readline;     // set line

    line=orgLine;         // copy line (we need to change it)
    line.Replace('\n', ' ');
    line.Replace('\r', ' ');

    line.StripSpaces();   // strip spaces

    // delete all comments in line and return true or false if only a comment
    lineComment=FreeComments(line, comment);

    if (lineComment) // whole line was a comment
    {
      line=comment;
      FreeComments(line, comment); // get second comment
    }
    comment.StripSpaces();

    {
      tempLine=line;
      line.MakeLower();   // convert to lower case letters

      // get section in xorg.conf
      if (!lineComment && (line.StartsWith("section \"device\"") || line.StartsWith("section\"device\"")))
      {
        if (section!=XORG_NOSECTION && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_TWO_SECTIONS, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
        else
        {
          switch (section)
          {
            case XORG_INPUTDEVICE:
              inputDevices.Append(tempInputDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_DEVICE:
              devices.Append(tempDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_MONITOR:
              monitors.Append(tempMonitor);
              section=XORG_NOSECTION;
              break;
            case XORG_SCREEN:
              screens.Append(tempScreen);
              section=XORG_NOSECTION;
              break;
            case XORG_SERVERLAYOUT:
              serverLayouts.Append(tempServerLayout);
              section=XORG_NOSECTION;
              break;
            default:  // all other sections are stored directly
              section=XORG_NOSECTION;
              break;
          }
          section=XORG_DEVICE;
          tempDevice.Clear();   // clear device
        }
      }
      else if (!lineComment && (line.StartsWith("section \"monitor\"") || line.StartsWith("section\"monitor\"")))
      {
        if (section!=XORG_NOSECTION && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_TWO_SECTIONS, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
        else
        {
          switch (section)
          {
            case XORG_INPUTDEVICE:
              inputDevices.Append(tempInputDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_DEVICE:
              devices.Append(tempDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_MONITOR:
              monitors.Append(tempMonitor);
              section=XORG_NOSECTION;
              break;
            case XORG_SCREEN:
              screens.Append(tempScreen);
              section=XORG_NOSECTION;
              break;
            case XORG_SERVERLAYOUT:
              serverLayouts.Append(tempServerLayout);
              section=XORG_NOSECTION;
              break;
            default:  // all other sections are stored directly
              section=XORG_NOSECTION;
              break;
          }
          section=XORG_MONITOR;
          tempMonitor.Clear();
        }
      }
      else if (!lineComment && (line.StartsWith("subsection \"display\"") || line.StartsWith("subsection\"display\"")))
      {
        if (section!=XORG_SCREEN && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_SCREEN_SECTION, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
        else
        {
          section=XORG_DISPLAY;
          tempDisplay.Clear();
        }
      }
      else if (!lineComment && (line.StartsWith("section \"inputdevice\"") || line.StartsWith("section\"inputdevice\"")))
      {
        if (section!=XORG_NOSECTION && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_TWO_SECTIONS, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
        else
        {
          switch (section)
          {
            case XORG_INPUTDEVICE:
              inputDevices.Append(tempInputDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_DEVICE:
              devices.Append(tempDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_MONITOR:
              monitors.Append(tempMonitor);
              section=XORG_NOSECTION;
              break;
            case XORG_SCREEN:
              screens.Append(tempScreen);
              section=XORG_NOSECTION;
              break;
            case XORG_SERVERLAYOUT:
              serverLayouts.Append(tempServerLayout);
              section=XORG_NOSECTION;
              break;
            default:  // all other sections are stored directly
              section=XORG_NOSECTION;
              break;
          }
          section=XORG_INPUTDEVICE;
          tempInputDevice.Clear();
        }
      }
      else if (!lineComment && (line.StartsWith("section \"screen\"") || line.StartsWith("section\"screen\"")))
      {
        if (section!=XORG_NOSECTION && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_TWO_SECTIONS, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
        else
        {
          switch (section)
          {
            case XORG_INPUTDEVICE:
              inputDevices.Append(tempInputDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_DEVICE:
              devices.Append(tempDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_MONITOR:
              monitors.Append(tempMonitor);
              section=XORG_NOSECTION;
              break;
            case XORG_SCREEN:
              screens.Append(tempScreen);
              section=XORG_NOSECTION;
              break;
            case XORG_SERVERLAYOUT:
              serverLayouts.Append(tempServerLayout);
              section=XORG_NOSECTION;
              break;
            default:  // all other sections are stored directly
              section=XORG_NOSECTION;
              break;
          }
          section=XORG_SCREEN;
          tempScreen.Clear();
        }
      }
      else if (!lineComment && (line.StartsWith("section \"serverlayout\"") || line.StartsWith("section\"serverlayout\"")))
      {
        if (section!=XORG_NOSECTION && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_TWO_SECTIONS, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
        else
        {
          switch (section)
          {
            case XORG_INPUTDEVICE:
              inputDevices.Append(tempInputDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_DEVICE:
              devices.Append(tempDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_MONITOR:
              monitors.Append(tempMonitor);
              section=XORG_NOSECTION;
              break;
            case XORG_SCREEN:
              screens.Append(tempScreen);
              section=XORG_NOSECTION;
              break;
            case XORG_SERVERLAYOUT:
              serverLayouts.Append(tempServerLayout);
              section=XORG_NOSECTION;
              break;
            default:  // all other sections are stored directly
              section=XORG_NOSECTION;
              break;
          }
          section=XORG_SERVERLAYOUT;
          tempServerLayout.Clear();
        }
      }
      else if (!lineComment && (line.StartsWith("section \"serverflags\"") || line.StartsWith("section\"serverflags\"")))
      {
        if (section!=XORG_NOSECTION && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_TWO_SECTIONS, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
        else
        {
          switch (section)
          {
            case XORG_INPUTDEVICE:
              inputDevices.Append(tempInputDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_DEVICE:
              devices.Append(tempDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_MONITOR:
              monitors.Append(tempMonitor);
              section=XORG_NOSECTION;
              break;
            case XORG_SCREEN:
              screens.Append(tempScreen);
              section=XORG_NOSECTION;
              break;
            case XORG_SERVERLAYOUT:
              serverLayouts.Append(tempServerLayout);
              section=XORG_NOSECTION;
              break;
            default:  // all other sections are stored directly
              section=XORG_NOSECTION;
              break;
          }
          section=XORG_SERVERFLAGS;
        }
      }
      else if (!lineComment && (line.StartsWith("section \"extensions\"") || line.StartsWith("section\"extensions\"")))
      {
        if (section!=XORG_NOSECTION && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_TWO_SECTIONS, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
        else
        {
          switch (section)
          {
            case XORG_INPUTDEVICE:
              inputDevices.Append(tempInputDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_DEVICE:
              devices.Append(tempDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_MONITOR:
              monitors.Append(tempMonitor);
              section=XORG_NOSECTION;
              break;
            case XORG_SCREEN:
              screens.Append(tempScreen);
              section=XORG_NOSECTION;
              break;
            case XORG_SERVERLAYOUT:
              serverLayouts.Append(tempServerLayout);
              section=XORG_NOSECTION;
              break;
            default:  // all other sections are stored directly
              section=XORG_NOSECTION;
              break;
          }
          section=XORG_EXTENSIONS;
        }
      }
      else if (!lineComment && (line.StartsWith("section \"files\"") || line.StartsWith("section\"files\"")))
      {
        if (section!=XORG_NOSECTION && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_TWO_SECTIONS, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
        else
        {
          switch (section)
          {
            case XORG_INPUTDEVICE:
              inputDevices.Append(tempInputDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_DEVICE:
              devices.Append(tempDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_MONITOR:
              monitors.Append(tempMonitor);
              section=XORG_NOSECTION;
              break;
            case XORG_SCREEN:
              screens.Append(tempScreen);
              section=XORG_NOSECTION;
              break;
            case XORG_SERVERLAYOUT:
              serverLayouts.Append(tempServerLayout);
              section=XORG_NOSECTION;
              break;
            default:  // all other sections are stored directly
              section=XORG_NOSECTION;
              break;
          }
          section=XORG_FILES;
        }
      }
      else if (!lineComment && (line.StartsWith("section \"module\"") || line.StartsWith("section\"module\"")))
      {
        if (section!=XORG_NOSECTION && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_TWO_SECTIONS, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
        else
        {
          switch (section)
          {
            case XORG_INPUTDEVICE:
              inputDevices.Append(tempInputDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_DEVICE:
              devices.Append(tempDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_MONITOR:
              monitors.Append(tempMonitor);
              section=XORG_NOSECTION;
              break;
            case XORG_SCREEN:
              screens.Append(tempScreen);
              section=XORG_NOSECTION;
              break;
            case XORG_SERVERLAYOUT:
              serverLayouts.Append(tempServerLayout);
              section=XORG_NOSECTION;
              break;
            default:  // all other sections are stored directly
              section=XORG_NOSECTION;
              break;
          }
          section=XORG_MODULE;
        }
      }
      else if (!lineComment && (line.StartsWith("section \"dri\"") || line.StartsWith("section\"dri\"")))
      {
        if (section!=XORG_NOSECTION && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_TWO_SECTIONS, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
        else
        {
          switch (section)
          {
            case XORG_INPUTDEVICE:
              inputDevices.Append(tempInputDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_DEVICE:
              devices.Append(tempDevice);
              section=XORG_NOSECTION;
              break;
            case XORG_MONITOR:
              monitors.Append(tempMonitor);
              section=XORG_NOSECTION;
              break;
            case XORG_SCREEN:
              screens.Append(tempScreen);
              section=XORG_NOSECTION;
              break;
            case XORG_SERVERLAYOUT:
              serverLayouts.Append(tempServerLayout);
              section=XORG_NOSECTION;
              break;
            default:  // all other sections are stored directly
              section=XORG_NOSECTION;
              break;
          }
          section=XORG_DRI;
        }
      }
      else if (line.StartsWith("endsubsection"))
      {
        if (!lineComment && section==XORG_DISPLAY)
        {
           tempScreen.Displays().Append(tempDisplay);
           section=XORG_SCREEN;
        }
        else if (section==XORG_MODULE);  // ignore subsections
        else if (!lineComment && !ignoreError)
        {
          MyMessage::Set(ERROR_XORG_NO_SUBSECTION, "XorgFile::Load", name, orgLine, lineNr);
          return(true);
        }
      }
      else if (!lineComment && line.StartsWith("endsection"))
      {
        switch (section)
        {
          case XORG_INPUTDEVICE:
            inputDevices.Append(tempInputDevice);
            section=XORG_NOSECTION;
            break;
          case XORG_DEVICE:
            devices.Append(tempDevice);
            section=XORG_NOSECTION;
            break;
          case XORG_MONITOR:
            monitors.Append(tempMonitor);
            section=XORG_NOSECTION;
            break;
          case XORG_SCREEN:
            screens.Append(tempScreen);
            section=XORG_NOSECTION;
            break;
          case XORG_SERVERLAYOUT:
            serverLayouts.Append(tempServerLayout);
            section=XORG_NOSECTION;
            break;
          case XORG_NOSECTION:
            if (!ignoreError)
            {
              MyMessage::Set(ERROR_XORG_NO_SECTION, "XorgFile::Load", name, orgLine, lineNr);
              return(true);
            }
            break;
          default:  // all other sections are stored directly
            section=XORG_NOSECTION;
            break;
        }
      } // select sections

      // handle each section separatly
      switch (section)
      {
        case XORG_FILES:
        {
          if (line.StartsWith("fontpath"))
          {
            line=tempLine; // restore line
            line.StripRight(8);

            pos=line.Find('"');  // search first "
            if (pos!=0) // " not found
            {
              MyMessage::Set(ERROR_XORG_FILES_SECTION, "XorgFile::Load(File Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure)
            {
              line.StripRight(1);
              pos=line.Find('"');         // search second "
              if (pos==-1)  // " not found
              {
                MyMessage::Set(ERROR_XORG_FILES_SECTION, "XorgFile::Load(File Section)", name, orgLine, lineNr);
                failure=true;
              }
            }

            if (!failure && line.Left(pos).IsEmpty())
            {
              MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(File Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure) files.FontPaths().Append(Option(line.Left(pos), "", comment, !lineComment));

            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) files.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("modulepath"))
          {
            line=tempLine; // restore line
            line.StripRight(10);
            pos=line.Find('"');  // search first "
            if (pos!=0) // " not found
            {
              MyMessage::Set(ERROR_XORG_FILES_SECTION, "XorgFile::Load(File Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure)
            {
              line.StripRight(1);
              pos=line.Find('"');  // search second "
              if (pos==-1)  // " not found
              {
                MyMessage::Set(ERROR_XORG_FILES_SECTION, "XorgFile::Load(File Section)", name, orgLine, lineNr);
                failure=true;
              }
            }
            if (!failure && line.Left(pos).IsEmpty())
            {
              MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(File Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure) files.ModulePaths().Append(Option(line.Left(pos), "", comment, !lineComment));

            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) files.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("rgbpath"))
          {
            line=tempLine; // restore line
            line.StripRight(7);
            pos=line.Find('"');  // search first "
            if (pos!=0)  // " not found
            {
              MyMessage::Set(ERROR_XORG_FILES_SECTION, "XorgFile::Load(File Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure)
            {
              line.StripRight(1);
              pos=line.Find('"');  // search second "
              if (pos==-1)  // " not found
              {
                MyMessage::Set(ERROR_XORG_FILES_SECTION, "XorgFile::Load(File Section)", name, orgLine, lineNr);
                failure=true;
              }
            }

            if (!failure && line.Left(pos).IsEmpty())
            {
              MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(File Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure) files.RgbPaths().Append(Option(line.Left(pos), "", comment, !lineComment));

            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) files.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("section \"files\"") || line.StartsWith("section\"files\""));
          else if (!line.IsEmpty() && !lineComment && !ignoreError)
          {
            MyMessage::Set(ERROR_XORG_FILES_SECTION, "XorgFile::Load(File Section)", name, orgLine, lineNr);
            return(true);
          }
          else if (lineComment || ignoreError)
          {
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) files.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          break;
        } // FILES

        case XORG_MODULE:
        {
          if (line.StartsWith("load"))
          {
            line=tempLine;  // restore line
            line.StripRight(4);
            pos=line.Find('"');  // search first "
            if (pos!=0)  // " not found
            {
              MyMessage::Set(ERROR_XORG_MODULE_SECTION, "XorgFile::Load(Module Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure)
            {
              line.StripRight(1);
              pos=line.Find('"');  // search second "
              if (pos==-1) // " not found
              {
                MyMessage::Set(ERROR_XORG_MODULE_SECTION, "XorgFile::Load(Module Section)", name, orgLine, lineNr);
                failure=true;
              }
            }
            if (!failure && line.Left(pos).IsEmpty())
            {
              MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Module Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure) modules.Append(Module(line.Left(pos), "", comment, !lineComment));

            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) modules.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("subsection"))
          {
            line=tempLine;  // restore line
            line.StripRight(10);
            pos=line.Find('"');  // search first "
            if (pos!=0) // " not found
            {
              MyMessage::Set(ERROR_XORG_MODULE_SECTION, "XorgFile::Load(Module Section)", name, orgLine, lineNr);
              failure=true;
            }
            if (!failure)
            {
              line.StripRight(1);
              pos=line.Find('"');  // search second "
              if (pos==-1) // " not found
              {
                MyMessage::Set(ERROR_XORG_MODULE_SECTION, "XorgFile::Load(Module Section)", name, orgLine, lineNr);
                failure=true;
              }
            }
            if (!failure && line.Left(pos).IsEmpty())
            {
              MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Module Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure) modules.Append(Module(line.Left(pos), "", comment, !lineComment));

            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) modules.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("option"))
          {
            line=tempLine;  // restore line
            if (GetOption(tempOption, line))
            {
              MyMessage::Set(ERROR_XORG_MODULE_SECTION, "XorgFile::Load(Module Section)", name, orgLine, lineNr);
              failure=true;
            }
            if (!failure && modules.GetSize()==0)
            {
              MyMessage::Set(ERROR_XORG_NO_MODUL, "XorgFile::Load(Module Section)", name, orgLine, lineNr);
              failure=true;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(comment);
              modules(modules.GetSize()-1).Options().Append(tempOption);
              if (!MyMessage::OK())
              {
                MyMessage::Append("XorgFile::Load(Module Section)");
                return(true);
              }
            }

            if (failure && !lineComment && !ignoreError) return(true);
            if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) modules.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("section \"module\"") || line.StartsWith("section\"module\""));
          else if (line.StartsWith("endsubsection"));
          else if (!line.IsEmpty() && !lineComment && !ignoreError)
          {
            MyMessage::Set(ERROR_XORG_MODULE_SECTION, "XorgFile::Load(Module Section)", name, orgLine, lineNr);
            return(true);
          }
          else if (lineComment || ignoreError)
          {
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) modules.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          break;
        } // MODULES
        case XORG_SERVERFLAGS:
        {
          if (line.StartsWith("option"))
          {
            line=tempLine;  // restore line
            if (GetOption(tempOption, line))
            {
              MyMessage::Set(ERROR_XORG_SERVERFLAGS_SECTION, "XorgFile::Load(Server Flags Section)", name, orgLine, lineNr);
              failure=true;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(comment);
              serverFlags.Append(tempOption); // set entry in class
            }
            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) serverFlags.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("section \"serverflags\"") || line.StartsWith("section\"serverflags\""));
          else if (!line.IsEmpty() && !lineComment && !ignoreError)
          {
            MyMessage::Set(ERROR_XORG_SERVERFLAGS_SECTION, "XorgFile::Load(Server Flags Section)", name, orgLine, lineNr);
            return(true);
          }
          else if (lineComment || ignoreError)
          {
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) serverFlags.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          break;
        } // SERVERFLAGS
        case XORG_DRI:
        {
          if (!lineComment && line.StartsWith("mode"))
          {
            line=tempLine;  // restore line
            line.StripRight(4);
            if (!driEntry.GetMode().IsEmpty())
            {
              MyMessage::Set(ERROR_XORG_SECOND_DRI, "XorgFile::Load(DRI Section)", name, orgLine, lineNr);
              failure=true;  // second dri entry found
            }
            if (!failure && line.IsEmpty())
            {
              MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(DRI Section)", name, orgLine, lineNr);
              failure=true;  // second dri entry found
            }

            if (failure && !ignoreError) return(true);
            else driEntry.SetMode(line, comment);
          }
          else if (line.StartsWith("section \"dri\"") || line.StartsWith("section\"dri\""));
          else if (!line.IsEmpty() && !lineComment && !ignoreError)
          {
            MyMessage::Set(ERROR_XORG_DRI_SECTION, "XorgFile::Load(DRI Section)", name, orgLine, lineNr);
            return(true);
          }
          else if (lineComment || ignoreError)
          {
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) driEntry.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          break;
        } // DRI
        case XORG_EXTENSIONS:
        {
          if (line.StartsWith("option"))
          {
            line=tempLine;  // restore line
            if (GetOption(tempOption, line))
            {
              MyMessage::Set(ERROR_XORG_EXTENSIONS_SECTION, "XorgFile::Load(Extentions Section)", name, orgLine, lineNr);
              failure=true;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(comment);
              extensions.Append(tempOption); // set entry in class
            }
            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) extensions.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("section \"extensions\"") || line.StartsWith("section\"extensions\""));
          else if (!line.IsEmpty() && !lineComment && !ignoreError)
          {
            MyMessage::Set(ERROR_XORG_EXTENSIONS_SECTION, "XorgFile::Load(Extentions Section)", name, orgLine, lineNr);
            return(true);
          }
          else if (lineComment || ignoreError)
          {
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) extensions.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          break;
        }
        case XORG_INPUTDEVICE:
        {
          if (!lineComment && line.StartsWith("identifier"))
          {
            line=tempLine;  // restore line
            if (GetArgument(argument, line))
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_INPUTDEVICE_SECTION, "XorgFile::Load(Input Device Section)",name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempInputDevice.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempInputDevice.SetId(argument, comment); // set entry in class
          }
          else if (!lineComment && line.StartsWith("driver"))
          {
            line=tempLine;  // restore line
            if (GetArgument(argument, line))
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_INPUTDEVICE_SECTION, "XorgFile::Load(Input Device Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempInputDevice.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempInputDevice.SetDriver(argument, comment); // set entry in class
          }
          else if (line.StartsWith("option"))
          {
            line=tempLine;  // restore line
            if (GetOption(tempOption, line))
            {
              MyMessage::Set(ERROR_XORG_INPUTDEVICE_SECTION, "XorgFile::Load(Input Device Section)", name, orgLine, lineNr);
              failure=true;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(comment);
              tempInputDevice.Options().Append(tempOption); // set entry in class
            }
            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempInputDevice.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("section \"inputdevice\"") || line.StartsWith("section\"inputdevice\""));
          else if (!line.IsEmpty() && !lineComment && !ignoreError)
          {
            MyMessage::Set(ERROR_XORG_INPUTDEVICE_SECTION, "XorgFile::Load(Input Device Section)", name, orgLine, lineNr);
            failure=true;
          }
          else if (lineComment || ignoreError)
          {
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) tempInputDevice.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          break;
        } // INPUT DEVICE
        case XORG_DEVICE:
        {
          if (!lineComment && line.StartsWith("identifier"))
          {
            line=tempLine;  // restore line
            if (GetArgument(argument, line))
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_DEVICE_SECTION, "XorgFile::Load(Device Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempDevice.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempDevice.SetId(argument, comment); // set entry in class
          }
          else if (!lineComment && line.StartsWith("driver"))
          {
            line=tempLine;  // restore line
            if (GetArgument(argument, line))
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_DEVICE_SECTION, "XorgFile::Load(Device Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempDevice.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempDevice.SetDriver(argument, comment); // set entry in class
          }
          else if (!lineComment && line.StartsWith("busid"))
          {
            line=tempLine;  // restore line
            if (GetArgument(argument, line))
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_DEVICE_SECTION, "XorgFile::Load(Device Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempDevice.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempDevice.SetBus(argument, comment); // set entry in class
          }
          else if (!lineComment && line.StartsWith("videoram"))
          {
            line=tempLine;  // restore line
            line.StripRight(8);
            if (!failure && line.IsEmpty())
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Device Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempDevice.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            tempDevice.SetVideoRam(line, comment); // set entry in class
          }
          else if (!lineComment && line.StartsWith("screen"))
          {
            line=tempLine;  // restore line
            line.StripRight(6);
            if (!failure && line.IsEmpty())
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Device Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempDevice.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempDevice.SetScreen(line, comment); // set entry in class
          }
          else if (line.StartsWith("option"))
          {
            line=tempLine;  // restore line
            if (GetOption(tempOption, line))
            {
              MyMessage::Set(ERROR_XORG_DEVICE_SECTION, "XorgFile::Load(Device Section)", name, orgLine, lineNr);
              failure=true;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(comment);
              tempDevice.Options().Append(tempOption); // set entry in class
            }
            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment ||ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempDevice.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("vendorname") || line.StartsWith("boardname") )
          {
            // ignore vendorname and boardname, because they are not needed
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) tempDevice.Comments().Append(line);
          }
          else if (line.StartsWith("section \"device\"") || line.StartsWith("section\"device\""));
          else if (!line.IsEmpty() && !lineComment && !ignoreError)
          {
            MyMessage::Set(ERROR_XORG_DEVICE_SECTION, "XorgFile::Load(Device Section)", name, orgLine, lineNr);
            return(true);
          }
          else if (lineComment || ignoreError)
          {
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) tempDevice.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          break;
        } // DEVICE

        case XORG_MONITOR:
        {
          if (!lineComment && line.StartsWith("identifier"))
          {
            line=tempLine;  // restore line
            if (GetArgument(argument, line))
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_MONITOR_SECTION, "XorgFile::Load(Monitor Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempMonitor.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempMonitor.SetId(argument, comment);
          }
          else if (!lineComment && line.StartsWith("horizsync"))
          {
            line=tempLine;  // restore line
            line.StripRight(9);
            if (!failure && line.IsEmpty())
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Monitor Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempMonitor.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempMonitor.SetHorSync(line, comment);
          }
          else if (!lineComment && line.StartsWith("vertrefresh"))
          {
            line=tempLine;  // restore line
            line.StripRight(11);
            if (!failure && line.IsEmpty())
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Monitor Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempMonitor.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempMonitor.SetVertRef(line, comment);
          }
          else if (!lineComment && line.StartsWith("gamma"))
          {
            line=tempLine;  // restore line
            line.StripRight(5);
            if (!failure && line.IsEmpty())
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Monitor Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempMonitor.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempMonitor.SetGamma(line, comment);
          }
          else if (!lineComment && line.StartsWith("displaysize"))
          {
            line=tempLine;  // restore line
            line.StripRight(11);
            if (!failure && line.IsEmpty())
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Monitor Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempMonitor.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempMonitor.SetDisplaySize(line, comment);
          }
          else if (line.StartsWith("option"))
          {
            line=tempLine;  // restore line
            if (GetOption(tempOption, line))
            {
              MyMessage::Set(ERROR_XORG_MONITOR_SECTION, "XorgFile::Load(Monitor Section)", name, orgLine, lineNr);
              failure=true;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(comment);
              tempMonitor.Options().Append(tempOption); // set entry in class
            }
            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempMonitor.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("modeline"))
          {
            line=tempLine;  // restore line
            line.StripRight(8);
            pos=line.Find('"');  // search first "
            if (pos!=0)
            {
              MyMessage::Set(ERROR_XORG_MONITOR_SECTION, "XorgFile::Load(Monitor Section)", name, orgLine, lineNr);
              failure=true;
            }
            if (!failure)
            {
              line.StripRight(1);
              pos=line.Find('"');  // search second "
              if (pos==-1) // " not found
              {
                MyMessage::Set(ERROR_XORG_MONITOR_SECTION, "XorgFile::Load(Monitor Section)", name, orgLine, lineNr);
                failure=true;
              }
            }
            if (!failure)
            {
              argument=line.Left(pos);
              line.StripRight(pos+1);

              if (!failure && (argument.IsEmpty() || line.IsEmpty()))
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Monitor Section)", name, orgLine, lineNr);
                failure=true;
              }
              if (!failure)
              {
                tempOption.SetActive(!lineComment);
                tempOption.SetName(argument);
                tempOption.SetValue(line);
                tempOption.SetComment(comment);
                tempMonitor.Modelines().Append(tempOption);
              }
            }
            if (failure && !lineComment && !ignoreError) return(true);
            if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempMonitor.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("vendorname") || line.StartsWith("modelname") )
          {
            // ignore vendorname and modelname, because they are not needed
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) tempMonitor.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          else if (line.StartsWith("section \"monitor\"") || line.StartsWith("section\"monitor\""));
          else if (!line.IsEmpty() && !lineComment && !ignoreError)
          {
            MyMessage::Set(ERROR_XORG_MONITOR_SECTION, "XorgFile::Load(Monitor Section)", name, orgLine, lineNr);
            return(true);
          }
          else if (lineComment || ignoreError)
          {
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) tempMonitor.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          break;
        } // MONITOR
        case XORG_SCREEN:
        {
          if (!lineComment && line.StartsWith("identifier"))
          {
            line=tempLine;  // restore line
            if (GetArgument(argument, line))
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_SCREEN_SECTION, "XorgFile::Load(Screen Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempScreen.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempScreen.SetId(argument, comment);
          }
          else if (!lineComment && line.StartsWith("device"))
          {
            line=tempLine;  // restore line
            if (GetArgument(argument, line))
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_SCREEN_SECTION, "XorgFile::Load(Screen Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempScreen.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempScreen.SetDevice(argument, comment);
          }
          else if (!lineComment && line.StartsWith("monitor"))
          {
            line=tempLine;  // restore line
            if (GetArgument(argument, line))
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_SCREEN_SECTION, "XorgFile::Load(Screen Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempScreen.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempScreen.SetMonitor(argument, comment);
          }
          else if (!lineComment && line.StartsWith("defaultdepth"))
          {
            line=tempLine;  // restore line
            line.StripRight(12);
            if (!failure && line.IsEmpty())
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Screen Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempScreen.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempScreen.SetDefaultDepth(line, comment);
          }
          else if (line.StartsWith("option"))
          {
            line=tempLine;  // restore line
            if (GetOption(tempOption, line))
            {
              MyMessage::Set(ERROR_XORG_SCREEN_SECTION, "XorgFile::Load(Screen Section)", name, orgLine, lineNr);
              failure=true;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(comment);
              tempScreen.Options().Append(tempOption); // set entry in class
            }
            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempScreen.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("section \"screen\"") || line.StartsWith("section\"screen\""));
          else if (line.StartsWith("endsubsection"));
          else if (!line.IsEmpty() && !lineComment && !ignoreError)
          {
            MyMessage::Set(ERROR_XORG_SCREEN_SECTION, "XorgFile::Load(Screen Section)", name, orgLine, lineNr);
            return(true);
          }
          else if (lineComment || ignoreError)
          {
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) tempScreen.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          break;
        } // SCREEN
        case XORG_DISPLAY:
        {
          if (!lineComment && line.StartsWith("depth"))
          {
            line=tempLine;  // restore line
            line.StripRight(5);
            if (!failure && line.IsEmpty())
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Display Subsection)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempDisplay.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempDisplay.SetDepth(line, comment);
          }
          else if (!lineComment && line.StartsWith("viewport"))
          {
            line=tempLine;  // restore line
            line.StripRight(8);
            if (!failure && line.IsEmpty())
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Display Subsection)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempDisplay.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempDisplay.SetViewPort(line, comment);
          }
          else if (!lineComment && line.StartsWith("virtual"))
          {
            line=tempLine;  // restore line
            line.StripRight(7);
            if (!failure && line.IsEmpty())
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Display Subsection)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempDisplay.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempDisplay.SetVirtual(line, comment);
          }
          else if (!lineComment && line.StartsWith("modes"))
          {
            line=tempLine;  // restore line
            line.StripRight(5);

            // now there should be a number of screen modes
            pos=line.Find('"');  // search first "

            while (pos!=-1)
            {
              line.StripRight(pos+1);
              pos=line.Find('"');  // search second "

              if (pos==-1)
              {  // no second " found
                MyMessage::Set(ERROR_XORG_DISPLAY_SECTION, "XorgFile::Load(Display Subsection)", name, orgLine, lineNr);
                failure=true;
                break;
              }
              argument=line.Left(pos);

              if (argument.IsEmpty())
              {
                MyMessage::Set(ERROR_XORG_EMPTY_ARGUMENT, "XorgFile::Load(Display Subsection)", name, orgLine, lineNr);
                failure=true;
                break;
              }
              if (!comment.IsEmpty())
              {
                tempDisplay.Modes().Append(Option(argument, "", comment));
                comment.Empty();
              }
              else
              {
                tempDisplay.Modes().Append(Option(argument));
              }

              line.StripRight(pos+1);
              pos=line.Find('"');  // search next first "
            }
            if (failure && ignoreError)
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempDisplay.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("option"))
          {
            line=tempLine;  // restore line
            if (GetOption(tempOption, line))
            {
              MyMessage::Set(ERROR_XORG_DISPLAY_SECTION, "XorgFile::Load(Display Subsection)", name, orgLine, lineNr);
              failure=true;
            }
            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(comment);
              tempDisplay.Options().Append(tempOption); // set entry in class
            }
            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempDisplay.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("subsection \"display\"") || line.StartsWith("subsection\"display\""));
          else if (!line.IsEmpty() && !lineComment && !ignoreError)
          {
            MyMessage::Set(ERROR_XORG_DISPLAY_SECTION, "XorgFile::Load(Display Subsection)", name, orgLine, lineNr);
            return(true);
          }
          else if (lineComment || ignoreError)
          {
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) tempDisplay.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          break;
        } // DISPLAY
        case XORG_SERVERLAYOUT:
        {
          if (!lineComment && line.StartsWith("identifier"))
          {
            line=tempLine;  // restore line
            if (GetArgument(argument, line))
            {
              if (!ignoreError)
              {
                MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
                return(true);
              }
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempServerLayout.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
            else tempServerLayout.SetId(argument, comment);
          }
          else if (!lineComment && line.StartsWith("screen"))
          {
            tempScreen.Clear();

            line=tempLine;  // restore line

            // add screens
            line.StripRight(6);

            // if the first sign is a " there is no screen number

            pos=line.Find('"');  // search first "
            if (pos==-1)   // " not found
            {
              MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure && pos!=0) // first the screen number
            {
              argument=line.Left(pos);
              argument.StripSpaces();
              tempScreen.SetNumber(argument);
              line.StripRight(pos);
            }

            if (!failure) pos=line.Find('"');  // search first " again
            if (!failure && pos!=0) // " not found at first position
            {
              MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
              failure=true;
            }

            // now comes the real screen entry
            if (!failure)
            {
              line.StripRight(1);
              pos=line.Find('"');  // search second "
            }
            if (!failure && pos==-1) // " not found
            {
              MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure)
            {
              argument=line.Left(pos);
              argument.StripSpaces();
            }

            if (!failure && argument.IsEmpty())
            {
              MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure)
            {
              tempScreen.SetId(argument);
              line.StripRight(pos+1);
            }

            if (!failure && !line.IsEmpty())  // further arguments given
            {
              pos=line.Find('"');  // search second "

              if (pos==-1)  // no other screen, absolute position
              {
                tempScreen.SetPosition(line);
              }
              else
              {  // other screen is somewhere

                argument=line.Left(pos);
                argument.StripSpaces();
                tempScreen.SetPosition(argument);

                line.StripRight(pos+1);
                pos=line.Find('"');  // search second "
                if (pos==-1) // " not found
                {
                  MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
                  failure=true;
                }

                if (!failure)
                {
                  argument=line.Left(pos);
                  argument.StripSpaces();
                }

                if (!failure && argument.IsEmpty())
                {
                  MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
                  failure=true;
                }

                if (!failure) tempScreen.SetSecond(argument);
              }
            }
            if (!failure)
            {
              if (!comment.IsEmpty()) tempScreen.Comments().Append(comment);
              tempServerLayout.Screens().Append(tempScreen);
            }
            if (failure && ignoreError)
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempServerLayout.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (!lineComment && line.StartsWith("inputdevice"))
          {
            tempInputDevice.Clear();

            line=tempLine;  // restore line
            line.StripRight(11);

            // first there is the name of the device
            pos=line.Find('"');  // search first "

            if (pos!=0)  // " not found
            {
              MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure)
            {
              line.StripRight(1);
              pos=line.Find('"');  // search second "
            }
            if (!failure && pos==-1)   // " not found
            {
              MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure) argument=line.Left(pos);

            if (!failure && argument.IsEmpty())
            {
              MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure)
            {
              tempInputDevice.SetId(argument);
              if (!comment.IsEmpty()) tempInputDevice.Comments().Append(comment);

              line.StripRight(pos+1);

              // now there may be a number of input device option
              pos=line.Find('"');  // search first "

              while (pos!=-1)
              {
                line.StripRight(pos+1);
                pos=line.Find('"');  // search second "
                if (pos==-1)   // no second " found
                {
                  MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
                  failure=true;
                  break;
                }

                argument=line.Left(pos);
                if (argument.IsEmpty())
                {
                  MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
                  failure=true;
                  break;
                }

                tempInputDevice.Options().Append(Option(argument));

                line.StripRight(pos+1);
                pos=line.Find('"');  // search first "
              } // while

              if (!failure) tempServerLayout.InputDevices().Append(tempInputDevice); // add device
            }
            if (failure && ignoreError)
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempServerLayout.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("option"))
          {
            line=tempLine;  // restore line

            if (GetOption(tempOption, line))
            {
              MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
              failure=true;
            }

            if (!failure)
            {
              tempOption.SetActive(!lineComment);
              tempOption.SetComment(comment);
              tempServerLayout.Options().Append(tempOption); // set entry in class
            }
            if (failure && !lineComment && !ignoreError) return(true);
            else if (failure && (lineComment || ignoreError))
            {
              line=tempLine;  // restore line
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (!line.IsEmpty()) tempServerLayout.Comments().Append(line);
              failure=false;
              MyMessage::Reset();
            }
          }
          else if (line.StartsWith("section \"serverlayout\"") || line.StartsWith("section\"serverlayout\""));
          else if (!line.IsEmpty() && !lineComment && !ignoreError)
          {
            MyMessage::Set(ERROR_XORG_SERVERLAYOUT_SECTION, "XorgFile::Load(Server Layout Section)", name, orgLine, lineNr);
            return(true);
          }
          else if (lineComment || ignoreError)
          {
            line=tempLine;  // restore line
            if (!comment.IsEmpty())
            {
              line+=" # ";
              line+=comment;
            }
            if (!line.IsEmpty()) tempServerLayout.Comments().Append(line);
            failure=false;
            MyMessage::Reset();
          }
          break;
        } // SERVER LAYOUT
        default:
        {
          if (lineComment || ignoreError)
          {
            if (!line.StartsWith("file edited by xorg-edit") && !line.StartsWith("endsection"))
            {
              line=tempLine;
              if (!comment.IsEmpty())
              {
                line+=" # ";
                line+=comment;
              }
              if (lineComment || !line.IsEmpty()) comments.Append(line);
            }
          } // lineComment
          else
          {
            if (!line.IsEmpty() && !line.StartsWith("endsection"))
            {
              MyMessage::Set(ERROR_XORG_UNKNOWN, "XorgFile::Load", name, orgLine, lineNr);
              return(true);
            }
          }
          break;
        } // default
      } // switch

      if (failure && !ignoreError) return(true);
      else if (failure && ignoreError)
      {
        line=tempLine;
        if (!comment.IsEmpty())
        {
          line+=" # ";
          line+=comment;
        }
        if (!line.IsEmpty()) comments.Append(line);
      }

    }
  } // while !EOF

  // check if device and monitor in SCREEN are found
  // check if input devices and screens in SERVERLAYOUT are found
  unsigned int i,j;              // loop counter

  // search screen device in device entry list
  for (i=0; i<screens.GetSize(); i++)
  {
    if (devices.Find(screens(i).GetDevice())==-1)
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Load");
        return(true);
      }

      MyMessage::Set(ERROR_XORG_NO_DEVICE, "XorgFile::Load", name);
      return(true);
    }
    else if (!MyMessage::OK())
    {
      MyMessage::Append("XorgFile::Load");
      return(true);
    }
  }

  // search screen monitor in monitor entry list
  for (i=0; i<screens.GetSize(); i++)
  {
    if (monitors.Find(screens(i).GetMonitor())==-1)
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Load");
        return(true);
      }

      MyMessage::Set(ERROR_XORG_NO_MONITOR, "XorgFile::Load", name);
      return(true);
    }
    else if (!MyMessage::OK())
    {
      MyMessage::Append("XorgFile::Load");
      return(true);
    }
  }

  // search server layout screen in screen entry list
  for (i=0; i<serverLayouts.GetSize(); i++)
  {
    for (j=0; j<serverLayouts(i).Screens().GetSize(); j++)
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Load");
        return(true);
      }

      if (screens.Find(serverLayouts(i).Screens()(j).GetId())==-1)
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append("XorgFile::Load");
          return(true);
        }

        MyMessage::Set(ERROR_XORG_NO_SCREEN, "XorgFile::Load", name);
        return(true);
      }
      else if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Load");
        return(true);
      }

      if (!serverLayouts(i).Screens()(j).GetSecond().IsEmpty() && screens.Find(serverLayouts(i).Screens()(j).GetSecond())==-1)
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append("XorgFile::Load");
          return(true);
        }

        MyMessage::Set(ERROR_XORG_NO_SCREEN, "XorgFile::Load", name);
        return(true);
      }
      else if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Load");
        return(true);
      }

    }
    if (!MyMessage::OK())
    {
      MyMessage::Append("XorgFile::Load");
      return(true);
    }
  }

  // search server layout input device in input device entry list
  for (i=0; i<serverLayouts.GetSize(); i++)
  {
    for (j=0; j<serverLayouts(i).InputDevices().GetSize(); j++)
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Load");
        return(true);
      }

      if (inputDevices.Find(serverLayouts(i).InputDevices()(j).GetId())==-1)
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append("XorgFile::Load");
          return(true);
        }

        MyMessage::Set(ERROR_XORG_NO_INPUTDEVICE, "XorgFile::Load", name);
        return(true);
      }
      else if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Load");
        return(true);
      }
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append("XorgFile::Load");
      return(true);
    }
  }

  return(false);
}

const bool XorgFile::Save(const MyString& name, const MyString& version)
{  // write data into file

  TextFile file;
  if (Preview(file, name, version))
  {
    MyMessage::Append("XorgFile::Save");
    return(true);
  }
  if (file.Save(name))
  {
    MyMessage::Append("XorgFile::Save");
    return(true);
  }
  return(false);
}

const bool XorgFile::Preview(TextFile& text, const MyString& name, const MyString& version)
{
  MyString line;             // line for writing to file
  unsigned int i,j,k;        // loop counter
  time_t* date=new time_t;   // create new time pointer

  // elements for writing
  Device tempDevice;
  Monitor tempMonitor;
  InputDevice tempInputDevice;
  Screen tempScreen;
  Display tempDisplay;
  ServerLayout tempServerLayout;
  Option tempOption;
  Module tempModule;

  text.Clear();  // clear old text
  time(date);  // get current date and time

  // Todo: It would be faster to calculate lines before and allocate memory at once
  // instead of appending lines.

  // preambel
  text.AppendComment(comments, 0);
  if (comments.GetSize()>0 && !comments(comments.GetSize()-1).IsEmpty()) text.Append("#");
  if (!MyMessage::OK())
  {
    MyMessage::Append("XorgFile::Preview");
    return(true);
  }

  line="File edited by ";
  line+=version+" at ";
  line+=ctime(date);
  text.AppendComment(line,0);
  // text.Append(""));

  delete date;
  date=NULL;

  // files section
  if (files.FontPaths().GetSize()>0 || files.ModulePaths().GetSize()>0 || files.RgbPaths().GetSize()>0 || files.Comments().GetSize()>0)
  {
    text.Append("Section \"Files\"");
    text.AppendComment(files.Comments(), 1);

    for (i=0; i<files.FontPaths().GetSize(); i++)
    {
      files.FontPaths()(i).CreateLine(line, 1, false, false, true, "FontPath");
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }
      text.Append(line);
    }

    for (i=0; i<files.ModulePaths().GetSize(); i++)
    {
      files.ModulePaths()(i).CreateLine(line, 1, false, false, true, "ModulePath");
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }
      text.Append(line);
    }

    for (i=0; i<files.RgbPaths().GetSize(); i++)
    {
      files.RgbPaths()(i).CreateLine(line, 1, false, false, true, "RgbPath");
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }
      text.Append(line);
    }

    text.Append("EndSection");
    text.Append("");
  }

  // server flags section
  if (serverFlags.GetSize()>0 || serverFlags.Comments().GetSize()>0)
  {
    text.Append("Section \"ServerFlags\"");
    text.AppendComment(serverFlags.Comments(), 1);

    for (i=0; i<serverFlags.GetSize(); i++)
    {
      serverFlags(i).CreateLine(line);
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }
      text.Append(line);
    }

    text.Append("EndSection");
    text.Append("");
  }

  // modules section
  if (modules.GetSize()>0 || modules.Comments().GetSize()>0)
  {
    text.Append("Section \"Module\"");
    text.AppendComment(modules.Comments(), 1);

    for (i=0; i<modules.GetSize(); i++)
    {
      line.Empty();
      tempModule=modules(i);
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }

      if (tempModule.Options().GetSize()>0)
      {
        if (!tempModule.IsActive()) line+="#";
        line+="\tSubSection \"";
        line+=tempModule.GetName();
        line+="\"";
        if (!tempModule.GetComment().IsEmpty())
        {
          line+=" # ";
          line+=tempModule.GetComment();
        }
        text.Append(line);

        for (j=0; j<tempModule.Options().GetSize(); j++)
        {
          tempModule.Options()(i).CreateLine(line, 2, true, true, true, "Option", tempModule.IsActive());
          if (!MyMessage::OK())
          {
            MyMessage::Append("XorgFile::Preview");
            return(true);
          }
          text.Append(line);
        }

        line.Empty();
        if (!tempModule.IsActive()) line+="#";
        line+="\tEndSubSection";
        text.Append(line);
      }
      else
      {
        if (!tempModule.IsActive()) line+="#";
        line+="\tLoad \"";
        line+=tempModule.GetName();
        line+="\"";
        if (!tempModule.GetComment().IsEmpty())
        {
          line+=" # ";
          line+=tempModule.GetComment();
        }
        text.Append(line);
      }
    }
    text.Append("EndSection");
    text.Append("");
  }

  // input device section
  for (i=0; i<inputDevices.GetSize(); i++)
  {
    tempInputDevice=inputDevices(i);
    if (!MyMessage::OK())
    {
      MyMessage::Append("XorgFile::Preview");
      return(true);
    }

    // id and driver must exist
    text.Append("Section \"InputDevice\"");
    text.AppendComment(tempInputDevice.Comments(), 1);

    line.Empty();
    line+="\tIdentifier \"";
    line+=tempInputDevice.GetId();
    line+="\"";
    if (!tempInputDevice.GetIdComment().IsEmpty())
    {
      line+=" # ";
      line+=tempInputDevice.GetIdComment();
    }
    text.Append(line);

    line.Empty();
    line+="\tDriver \"";
    line+=tempInputDevice.GetDriver();
    line+="\"";
    if (!tempInputDevice.GetDriverComment().IsEmpty())
    {
      line+=" # ";
      line+=tempInputDevice.GetDriverComment();
    }
    text.Append(line);

    for (j=0; j<tempInputDevice.Options().GetSize(); j++)
    {
      tempInputDevice.Options()(j).CreateLine(line);
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }
      text.Append(line);
    }

    text.Append("EndSection");
    text.Append("");
  }

  // device section
  for (i=0; i<devices.GetSize(); i++)
  {
    tempDevice=devices(i);
    if (!MyMessage::OK())
    {
      MyMessage::Append("XorgFile::Preview");
      return(true);
    }

    // id and driver must exist
    text.Append("Section \"Device\"");
    text.AppendComment(tempDevice.Comments(), 1);

    line.Empty();
    line+="\tIdentifier \"";
    line+=tempDevice.GetId();
    line+="\"";
    if (!tempDevice.GetIdComment().IsEmpty())
    {
      line+=" # ";
      line+=tempDevice.GetIdComment();
    }
    text.Append(line);

    line.Empty();
    line+="\tDriver \"";
    line+=tempDevice.GetDriver();
    line+="\"";
    if (!tempDevice.GetDriverComment().IsEmpty())
    {
      line+=" # ";
      line+=tempDevice.GetDriverComment();
    }
    text.Append(line);

    if (!tempDevice.GetBus().IsEmpty())
    {
      line.Empty();
      line+="\tBusID \"";
      line+=tempDevice.GetBus();
      line+="\"";
      if (!tempDevice.GetBusComment().IsEmpty())
      {
        line+=" # ";
        line+=tempDevice.GetBusComment();
      }
      text.Append(line);
    }
    if (!tempDevice.GetVideoRam().IsEmpty())
    {
      line.Empty();
      line+="\tVideoRam ";
      line+=tempDevice.GetVideoRam();
      if (!tempDevice.GetVideoRamComment().IsEmpty())
      {
        line+=" # ";
        line+=tempDevice.GetVideoRamComment();
      }
      text.Append(line);
    }
    if (!tempDevice.GetScreen().IsEmpty())
    {
      line.Empty();
      line+="\tScreen ";
      line+=tempDevice.GetScreen();
      if (!tempDevice.GetScreenComment().IsEmpty())
      {
        line+=" # ";
        line+=tempDevice.GetScreenComment();
      }
      text.Append(line);
    }

    for (j=0; j<tempDevice.Options().GetSize(); j++)
    {
      tempDevice.Options()(j).CreateLine(line);
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }
      text.Append(line);
    }

    text.Append("EndSection");
    text.Append("");
  }

  // monitor section
  for (i=0; i<monitors.GetSize(); i++)
  {
    tempMonitor=monitors(i);
    if (!MyMessage::OK())
    {
      MyMessage::Append("XorgFile::Preview");
      return(true);
    }

    // id must exist
    text.Append("Section \"Monitor\"");
    text.AppendComment(tempMonitor.Comments(), 1);

    line.Empty();
    line+="\tIdentifier \"";
    line+=tempMonitor.GetId();
    line+="\"";
    if (!tempMonitor.GetIdComment().IsEmpty())
    {
      line+=" # ";
      line+=tempMonitor.GetIdComment();
    }
    text.Append(line);

    if (!tempMonitor.GetHorSync().IsEmpty())
    {
      line.Empty();
      line+="\tHorizSync ";
      line+=tempMonitor.GetHorSync();
      if (!tempMonitor.GetHorSyncComment().IsEmpty())
      {
        line+=" # ";
        line+=tempMonitor.GetHorSyncComment();
      }
      text.Append(line);
    }
    if (!tempMonitor.GetVertRef().IsEmpty())
    {
      line.Empty();
      line+="\tVertRefresh ";
      line+=tempMonitor.GetVertRef();
      if (!tempMonitor.GetVertRefComment().IsEmpty())
      {
        line+=" # ";
        line+=tempMonitor.GetVertRefComment();
      }
      text.Append(line);
    }
    if (!tempMonitor.GetDisplaySize().IsEmpty())
    {
      line.Empty();
      line+="\tDisplaySize ";
      line+=tempMonitor.GetDisplaySize();
      if (!tempMonitor.GetDisplaySizeComment().IsEmpty())
      {
        line+=" # ";
        line+=tempMonitor.GetDisplaySizeComment();
      }
      text.Append(line);
    }
    if (!tempMonitor.GetGamma().IsEmpty())
    {
      line.Empty();
      line+="\tGamma ";
      line+=tempMonitor.GetGamma();
      if (!tempMonitor.GetGammaComment().IsEmpty())
      {
        line+=" # ";
        line+=tempMonitor.GetGammaComment();
      }
      text.Append(line);
    }

    for (j=0; j<tempMonitor.Options().GetSize(); j++)
    {
      tempMonitor.Options()(j).CreateLine(line);
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }
      text.Append(line);
    }

    for (j=0; j<tempMonitor.Modelines().GetSize(); j++)
    {
      tempMonitor.Modelines()(j).CreateLine(line, 1, true, false, true, "Modeline");

      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }
      text.Append(line);
    }

    text.Append("EndSection");
    text.Append("");
  }

  // screen section
  for (i=0; i<screens.GetSize(); i++)
  {
    tempScreen=screens(i);
    if (!MyMessage::OK())
    {
      MyMessage::Append("XorgFile::Preview");
      return(true);
    }

    // id must exist
    text.Append("Section \"Screen\"");
    text.AppendComment(tempScreen.Comments(), 1);

    line.Empty();
    line+="\tIdentifier \"";
    line+=tempScreen.GetId();
    line+="\"";
    if (!tempScreen.GetIdComment().IsEmpty())
    {
      line+=" # ";
      line+=tempScreen.GetIdComment();
    }
    text.Append(line);

    if (!tempScreen.GetDevice().IsEmpty())
    {
      line.Empty();
      line+="\tDevice \"";
      line+=tempScreen.GetDevice();
      line+="\"";
      if (!tempScreen.GetDeviceComment().IsEmpty())
      {
        line+=" # ";
        line+=tempScreen.GetDeviceComment();
      }
      text.Append(line);
    }
    if (!tempScreen.GetMonitor().IsEmpty())
    {
      line.Empty();
      line+="\tMonitor \"";
      line+=tempScreen.GetMonitor();
      line+="\"";
      if (!tempScreen.GetMonitorComment().IsEmpty())
      {
        line+=" # ";
        line+=tempScreen.GetMonitorComment();
      }
      text.Append(line);
    }
    if (!tempScreen.GetDefaultDepth().IsEmpty())
    {
      line.Empty();
      line+="\tDefaultDepth ";
      line+=tempScreen.GetDefaultDepth();
      if (!tempScreen.GetDefaultDepthComment().IsEmpty())
      {
        line+=" # ";
        line+=tempScreen.GetDefaultDepthComment();
      }
      text.Append(line);
    }

    for (j=0; j<tempScreen.Options().GetSize(); j++)
    {
      tempScreen.Options()(j).CreateLine(line);
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }
      text.Append(line);
    }

    for (j=0; j<tempScreen.Displays().GetSize(); j++)
    {
      tempDisplay=tempScreen.Displays()(j);
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }

      // depth must exist
      text.Append("\tSubSection \"Display\"");
      text.AppendComment(tempDisplay.Comments(), 2);

      line.Empty();
      line+="\t\tDepth ";
      line+=tempDisplay.GetDepth();
      if (!tempDisplay.GetDepthComment().IsEmpty())
      {
        line+=" # ";
        line+=tempDisplay.GetDepthComment();
      }
      text.Append(line);

      if (!tempDisplay.GetViewPort().IsEmpty())
      {
        line.Empty();
        line+="\t\tViewPort ";
        line+=tempDisplay.GetViewPort();
        if (!tempDisplay.GetViewPortComment().IsEmpty())
        {
          line+=" # ";
          line+=tempDisplay.GetViewPortComment();
        }
        text.Append(line);
      }
      if (!tempDisplay.GetVirtual().IsEmpty())
      {
        line.Empty();
        line+="\t\tVirtual ";
        line+=tempDisplay.GetVirtual();
        if (!tempDisplay.GetVirtualComment().IsEmpty())
        {
          line+=" # ";
          line+=tempDisplay.GetVirtualComment();
        }
        text.Append(line);
      }

      for (k=0; k<tempDisplay.Options().GetSize(); k++)
      {
        tempDisplay.Options()(k).CreateLine(line,2);
        if (!MyMessage::OK())
        {
          MyMessage::Append("XorgFile::Preview");
          return(true);
        }
        text.Append(line);
      }

      if (tempDisplay.Modes().GetSize()>0)
      {
        line="\t\tModes";
        for (k=0; k<tempDisplay.Modes().GetSize(); k++)
        {
          line+=" \"";
          line+=tempDisplay.Modes()(k).GetName();
          if (!MyMessage::OK())
          {
            MyMessage::Append("XorgFile::Preview");
            return(true);
          }
          line+="\"";
        }
        if (!tempDisplay.Modes()(0).GetComment().IsEmpty())
        {
          line+=" # ";
          line+=tempDisplay.Modes()(0).GetComment();
        }
        text.Append(line);
      }
      text.Append("\tEndSubSection");
    }

    text.Append("EndSection");
    text.Append("");
  }

  // server layout section
  for (i=0; i<serverLayouts.GetSize(); i++)
  {
    tempServerLayout=serverLayouts(i);
    if (!MyMessage::OK())
    {
      MyMessage::Append("XorgFile::Preview");
      return(true);
    }

    // id must exist
    text.Append("Section \"ServerLayout\"");
    text.AppendComment(tempServerLayout.Comments(), 1);

    line.Empty();
    line+="\tIdentifier \"";
    line+=tempServerLayout.GetId();
    line+="\"";
    if (!tempServerLayout.GetIdComment().IsEmpty())
    {
      line+=" # ";
      line+=tempServerLayout.GetIdComment();
    }
    text.Append(line);

    for (j=0; j<tempServerLayout.Screens().GetSize(); j++)
    {
      tempScreen=tempServerLayout.Screens()(j);
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }

      // screen name must exist
      line.Empty();
      line+="\tScreen";
      if (!tempScreen.GetNumber().IsEmpty())
      {
        line+=" ";
        line+=tempScreen.GetNumber();
      }
      line+=" \"";
      line+=tempScreen.GetId();
      line+="\"";
      if (!tempScreen.GetPosition().IsEmpty())
      {
        line+=" ";
        line+=tempScreen.GetPosition();
      }
      if (!tempScreen.GetSecond().IsEmpty())
      {
        line+=" \"";
        line+=tempScreen.GetSecond();
        line+="\"";
      }

      if (tempScreen.Comments().GetSize()>0)
      {
        line+=" # ";
        for (k=0; k<tempScreen.Comments().GetSize(); k++)
        {
          line+=tempScreen.Comments()(k);
          if (!MyMessage::OK())
          {
            MyMessage::Append("XorgFile::Preview");
            return(true);
          }
          line+=" ";
        }
      }
      text.Append(line);
    }

    for (j=0; j<tempServerLayout.InputDevices().GetSize(); j++)
    {
      tempInputDevice=tempServerLayout.InputDevices()(j);
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }

      // input device name name must exist
      line.Empty();
      line+="\tInputDevice \"";
      line+=tempInputDevice.GetId();
      line+="\"";

      for (k=0; k<tempInputDevice.Options().GetSize(); k++)
      {
        line+=" \"";
        line+=tempInputDevice.Options()(k).GetName();
        if (!MyMessage::OK())
        {
          MyMessage::Append("XorgFile::Preview");
          return(true);
        }
        line+="\"";
      }

      if (tempInputDevice.Comments().GetSize()>0)
      {
        line+=" # ";
        for (k=0; k<tempInputDevice.Comments().GetSize(); k++)
        {
          line+=tempInputDevice.Comments()(k);
          if (!MyMessage::OK())
          {
            MyMessage::Append("XorgFile::Preview");
            return(true);
          }
          line+=" ";
        }
      }

      text.Append(line);
    }

    for (j=0; j<tempServerLayout.Options().GetSize(); j++)
    {
      tempServerLayout.Options()(j).CreateLine(line);
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }
      text.Append(line);
    }

    text.Append("EndSection");
    text.Append("");
  }

  // extension section
  if (extensions.GetSize()>0 || extensions.Comments().GetSize()>0)
  {
    text.Append("Section \"Extensions\"");
    text.AppendComment(extensions.Comments(), 1);

    for (i=0; i<extensions.GetSize(); i++)
    {
      extensions(i).CreateLine(line);
      if (!MyMessage::OK())
      {
        MyMessage::Append("XorgFile::Preview");
        return(true);
      }
      text.Append(line);
    }
    text.Append("EndSection");
    text.Append("");
  }

  // dri section
  if (!driEntry.GetMode().IsEmpty() || driEntry.Comments().GetSize()>0)
  {
    text.Append("Section \"DRI\"");
    text.AppendComment(driEntry.Comments(), 1);

    if (!driEntry.GetMode().IsEmpty())
    {
      line.Empty();
      line+="\tMode ";
      line+=driEntry.GetMode();
      if (!driEntry.GetModeComment().IsEmpty())
      {
        line+=" # ";
        line+=driEntry.GetModeComment();
      }
      text.Append(line);
    }
    text.Append("EndSection");
  }

  return(false);
}

const bool XorgFile::CheckAll(void)
{  /// Check if everything is correct.

  if (devices.Check())
  {
    MyMessage::Append("XorgFile::CheckAll");
    return(true);
  }

  if (inputDevices.Check())
  {
    MyMessage::Append("XorgFile::CheckAll");
    return(true);
  }

  if (monitors.Check())
  {
    MyMessage::Append("XorgFile::CheckAll");
    return(true);
  }

  if (screens.Check())
  {
    MyMessage::Append("XorgFile::CheckAll");
    return(true);
  }

  if (serverLayouts.Check(true))
  {
    MyMessage::Append("XorgFile::CheckAll");
    return(true);
  }

  if (serverFlags.Check())
  {
    MyMessage::Append("XorgFile::CheckAll");
    return(true);
  }

  if (files.Check())
  {
    MyMessage::Append("XorgFile::CheckAll");
    return(true);
  }

  if (modules.Check())
  {
    MyMessage::Append("XorgFile::CheckAll");
    return(true);
  }

  if (extensions.Check())
  {
    MyMessage::Append("XorgFile::CheckAll");
    return(true);
  }

  if (driEntry.Check())
  {
    MyMessage::Append("XorgFile::CheckAll");
    return(true);
  }

  return(false);
}
