// *************************************************************************
// xorg-edit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef GUIFRAME_H
#define GUIFRAME_H

// **************************************
// Header Files
// **************************************

#include <wx/wx.h>
#include <wx/string.h>
#include <wx/intl.h>
#include <wx/notebook.h>

#include "../libxorgedit/xorgfile.h"
#include "../libxorgedit/settings.h"
#include "../libxorgedit/textfile.h"
#include "../libxorgedit/driveroptionlist.h"

// **************************
// Class Definition
// **************************

// class XorgApp;

class GuiFrame: public wxFrame
{
public:
  GuiFrame(const wxString& title);
  virtual ~GuiFrame();

  void OnNew(wxCommandEvent& event);      // create new empty xorg.conf
  void OnLoad(wxCommandEvent& event);     // load xorg.conf
  void OnSave(wxCommandEvent& event);     // save xorg.conf
  void OnSaveAs(wxCommandEvent& event);   // save config file
  void OnRestore(wxCommandEvent& event);   // restoreconfig file
  void OnClose(wxCommandEvent& event);   // close config file
  void OnQuit(wxCommandEvent& event);     // quit program
  void OnShow(wxCommandEvent& event);     // show xorg file
  void OnCheck(wxCommandEvent& event);     // check xorg file
  void OnTest(wxCommandEvent& event);     // test xorg file
  void OnDebug(wxCommandEvent& event);        // activate debug mode
  void OnAbout(wxCommandEvent& event);    // show about dialog
  void OnSettings(wxCommandEvent& event);     // show settings
  void SetLanguage(wxCommandEvent& event);    // set language
  void OnTabChange(wxNotebookEvent& event);     // some tab has changed

  // select wxComboBox elements
  void SelectDevice(wxCommandEvent& event);
  void SelectDeviceOption(wxCommandEvent& event);
  void SelectInputDevice(wxCommandEvent& event);
  void SelectInputDeviceOption(wxCommandEvent& event);
  void SelectMonitor(wxCommandEvent& event);
  void SelectMonitorOption(wxCommandEvent& event);
  void SelectMonitorModeline(wxCommandEvent& event);
  void SelectScreen(wxCommandEvent& event);
  void SelectScreenOption(wxCommandEvent& event);
  void SelectScreenDisplay(wxCommandEvent& event);
  void SelectScreenDisplayMode(wxCommandEvent& event);
  void SelectScreenDisplayOption(wxCommandEvent& event);
  void SelectServerLayout(wxCommandEvent& event);
  void SelectServerLayoutScreen(wxCommandEvent& event);
  void SelectServerLayoutInputDevice(wxCommandEvent& event);
  void SelectServerLayoutOption(wxCommandEvent& event);
  void SelectServerLayoutInputDeviceOption(wxCommandEvent& event);
  void SelectServerFlagOption(wxCommandEvent& event);
  void SelectExtension(wxCommandEvent& event);
  void SelectFontPath(wxCommandEvent& event);
  void SelectModulePath(wxCommandEvent& event);
  void SelectRgbPath(wxCommandEvent& event);
  void SelectModule(wxCommandEvent& event);
  void SelectModuleOption(wxCommandEvent& event);

  // select wxChoice elements
  void SelectScreenDevice(wxCommandEvent& event);
  void SelectScreenMonitor(wxCommandEvent& event);
  void SelectServerLayoutScreen1(wxCommandEvent& event);
  void SelectServerLayoutScreen2(wxCommandEvent& event);
  void SelectServerLayoutInputDevice1(wxCommandEvent& event);

  // enter text in wxComboBox elements
  void EnterDevice(wxCommandEvent& event);
  void EnterDeviceOption(wxCommandEvent& event);
  void EnterInputDevice(wxCommandEvent& event);
  void EnterInputDeviceOption(wxCommandEvent& event);
  void EnterMonitor(wxCommandEvent& event);
  void EnterMonitorOption(wxCommandEvent& event);
  void EnterMonitorModeline(wxCommandEvent& event);
  void EnterScreen(wxCommandEvent& event);
  void EnterScreenOption(wxCommandEvent& event);
  void EnterScreenDisplay(wxCommandEvent& event);
  void EnterScreenDisplayMode(wxCommandEvent& event);
  void EnterScreenDisplayOption(wxCommandEvent& event);
  void EnterServerLayout(wxCommandEvent& event);
  void EnterServerLayoutOption(wxCommandEvent& event);
  void EnterServerLayoutInputDeviceOption(wxCommandEvent& event);
  void EnterServerFlagOption(wxCommandEvent& event);
  void EnterExtension(wxCommandEvent& event);
  void EnterFontPath(wxCommandEvent& event);
  void EnterModulePath(wxCommandEvent& event);
  void EnterRgbPath(wxCommandEvent& event);
  void EnterModule(wxCommandEvent& event);
  void EnterModuleOption(wxCommandEvent& event);

  // enter text in text fields
  void EnterDeviceDriver(wxCommandEvent& event);
  void EnterDeviceBusID(wxCommandEvent& event);
  void EnterDeviceScreen(wxCommandEvent& event);
  void EnterDevicVideoRam(wxCommandEvent& event);
  void EnterDeviceOptValue(wxCommandEvent& event);
  void EnterMonitorHorizSync(wxCommandEvent& event);
  void EnterMonitorVertRefresh(wxCommandEvent& event);
  void EnterMonitorDisplay(wxCommandEvent& event);
  void EnterMonitorGamma(wxCommandEvent& event);
  void EnterMonitorOptValue(wxCommandEvent& event);
  void EnterMonitorModelineOption(wxCommandEvent& event);
  void EnterInputDeviceDriver(wxCommandEvent& event);
  void EnterInputDeviceOptValue(wxCommandEvent& event);
  void EnterScreenDepth(wxCommandEvent& event);
  void EnterScreenOptValue(wxCommandEvent& event);
  void EnterScreenDisplayViewport(wxCommandEvent& event);
  void EnterScreenDisplayVirtual(wxCommandEvent& event);
  void EnterScreenDisplayOptValue(wxCommandEvent& event);
  void EnterServerLayoutScreenNumber(wxCommandEvent& event);
  void EnterServerLayoutScreenPosition(wxCommandEvent& event);
  void EnterServerLayoutOptValue(wxCommandEvent& event);
  void EnterServerFlagsOptValue(wxCommandEvent& event);
  void EnterExtensionOptValue(wxCommandEvent& event);
  void EnterModuleOptValue(wxCommandEvent& event);
  void EnterDriMode(wxCommandEvent& event);

  // buttons
  void GetRefreshRate(wxCommandEvent& event);
  void GetDisplaySize(wxCommandEvent& event);
  void GenerateModeline(wxCommandEvent& event);
  void AddModeline(wxCommandEvent& event);

  void ShowWarning(void) const; // show warning message

private:
  const bool ShowError(void) const;       // show error message, return true if program should be stopped

  const bool CreateBackup(void) const;       // create backup of current file
  const bool ConvertXorg(void);       // convert all driver options in xorg file
  const bool TestXorg(void);       // test xorg.conf file
  void ClearElements(void);  // clear all elements
  const bool SetFrame(void);       // set all elements on screen
  void SetSaved(bool save);  // set saved status and set title

  const bool LoadDrivers(void);    // load driver lists

  const bool CreateGUIControls(void);  // create all gui controls, like text fields, menus, buttons

  void EnableMenuItems(const bool enable);

  void WriteDebug(const MyString& str, const bool clear=false) const;  // write into debug file
  void WriteDebug(const wxString& str, const bool clear=false) const;  // write into debug file

  // check integrity functions
  const bool CheckAllTabs(void);                                  // check all tabs
  const bool CheckTab(const unsigned int tabnr);       // check xorg devices in this tab
  const bool CheckDeviceTab(void);
  const bool CheckMonitorTab(void);
  const bool CheckInputDeviceTab(void);
  const bool CheckScreenTab(void);
  const bool CheckServerTab(void);
  const bool CheckOtherSectionTab(void);

  // Update screen functions
  const bool UpdateTab(const unsigned int tabnr);
  const bool UpdateDeviceTab(void);
  const bool UpdateMonitorTab(void);
  const bool UpdateInputDeviceTab(void);
  const bool UpdateScreenTab(void);
  const bool UpdateServerTab(void);
  const bool UpdateOtherSectionTab(void);
  const bool UpdateDevice(void);
  const bool UpdateDeviceOption(void);
  const bool UpdateInputDevice(void);
  const bool UpdateInputDeviceOption(void);
  const bool UpdateMonitor(void);
  const bool UpdateMonitorOption(void);
  const bool UpdateMonitorModeline(void);
  const bool UpdateScreen(void);
  const bool UpdateScreenOption(void);
  const bool UpdateScreenDisplay(void);
  const bool UpdateScreenDisplayMode(void);
  const bool UpdateScreenDisplayOption(void);
  const bool UpdateServerLayout(void);
  const bool UpdateServerLayoutOption(void);
  const bool UpdateServerLayoutScreen(void);
  const bool UpdateServerLayoutInputDevice(void);
  const bool UpdateServerLayoutInputDeviceOption(void);
  const bool UpdateServerFlagOption(void);
  const bool UpdateExtension(void);
  const bool UpdateFontPath(void);
  const bool UpdateModulePath(void);
  const bool UpdateRgbPath(void);
  const bool UpdateModule(void);
  const bool UpdateModuleOption(void);
  const bool UpdateDRI(void);

  // delete, add and activate functions
  const bool AddDevice(void);
  const bool DelDevice(void);
  const bool CopyDevice(void);
  const bool AddDeviceOption(void);
  const bool AddDeviceDriverOptions(void);
  const bool DelDeviceOption(void);
  const bool DeActDeviceOption(void);
  const bool AddInputDevice(void);
  const bool DelInputDevice(void);
  const bool CopyInputDevice(void);
  const bool AddInputDeviceOption(void);
  const bool AddInputDeviceDriverOptions(void);
  const bool DelInputDeviceOption(void);
  const bool DeActInputDeviceOption(void);
  const bool AddMonitor(void);
  const bool DelMonitor(void);
  const bool CopyMonitor(void);
  const bool AddMonitorOption(void);
  const bool DelMonitorOption(void);
  const bool DeActMonitorOption(void);
  const bool AddMonitorModeline(void);
  const bool DelMonitorModeline(void);
  const bool DeActMonitorModeline(void);
  const bool AddScreen(void);
  const bool DelScreen(void);
  const bool CopyScreen(void);
  const bool AddScreenOption(void);
  const bool DelScreenOption(void);
  const bool DeActScreenOption(void);
  const bool AddScreenDisplay(void);
  const bool DelScreenDisplay(void);
  const bool AddScreenDisplayMode(void);
  const bool DelScreenDisplayMode(void);
  const bool AddScreenDisplayOption(void);
  const bool DelScreenDisplayOption(void);
  const bool DeActScreenDisplayOption(void);
  const bool AddServerLayout(void);
  const bool DelServerLayout(void);
  const bool CopyServerLayout(void);
  const bool AddServerLayoutScreen(void);
  const bool DelServerLayoutScreen(void);
  const bool AddServerLayoutInputDevice(void);
  const bool DelServerLayoutInputDevice(void);
  const bool AddServerLayoutOption(void);
  const bool DelServerLayoutOption(void);
  const bool DeActServerLayoutOption(void);
  const bool AddServerLayoutInputDeviceOption(void);
  const bool DelServerLayoutInputDeviceOption(void);
  const bool AddServerFlagOption(void);
  const bool DelServerFlagOption(void);
  const bool DeActServerFlagOption(void);
  const bool AddExtension(void);
  const bool DelExtension(void);
  const bool DeActExtension(void);
  const bool AddFontPath(void);
  const bool DelFontPath(void);
  const bool DeActFontPath(void);
  const bool AddModulePath(void);
  const bool DelModulePath(void);
  const bool DeActModulePath(void);
  const bool AddRgbPath(void);
  const bool DelRgbPath(void);
  const bool DeActRgbPath(void);
  const bool AddModule(void);
  const bool DelModule(void);
  const bool DeActModule(void);
  const bool AddModuleOption(void);
  const bool DelModuleOption(void);
  const bool DeActModuleOption(void);

  // global GUI elements

  wxMenu* fileMenu;                    // file menu
  wxMenu* extrasMenu;                  // extras menu

  wxNotebook* layout;                   // layout for tabs

  wxPanel* deviceTab;                   // device tab
  wxPanel* monitorTab;                  // monitor tab
  wxPanel* inputDeviceTab;              // inputdevice tab
  wxPanel* screenTab;                   // screen tab
  wxPanel* serverTab;                   // server layout/server flags tab
  wxPanel* otherSectionTab;             // files/modules/extensions/dri tab
  wxStaticText* warningText;

  // "local" GUI elements

  // section DEVICE elements
  wxStaticBox *deviceBox;
  wxComboBox *deviceIdentifier;
  wxComboBox *deviceDriver;
  wxTextCtrl *deviceBusID;
  wxTextCtrl *deviceScreen;
  wxTextCtrl *deviceVideoRam;
  wxComboBox *deviceOption;
  wxComboBox *deviceOptValue;
  wxStaticText *deviceDriverText;
  wxStaticText *deviceBusIDText;
  wxStaticText *deviceVideoRamText;
  wxStaticText *deviceScreenNumberText;
  wxStaticText *deviceOptionText;

  // section MONITOR elements
  wxStaticBox *monitorBox;
  wxComboBox *monitorIdentifier;
  wxTextCtrl *monitorHorizSync;
  wxTextCtrl *monitorVertRefresh;
  wxTextCtrl *monitorDisplay;
  wxTextCtrl *monitorGamma;
  wxComboBox *monitorOption;
  wxTextCtrl *monitorOptValue;
  wxComboBox *monitorModelineName;
  wxTextCtrl *monitorModelineOption;
  wxStaticText *monitorHorizSyncText;
  wxStaticText *monitorVertRefreshText;
  wxStaticText *monitorDisplayText;
  wxStaticText *monitorGammaText;
  wxStaticText *monitorOptionText;
  wxStaticText *monitorModelineText;
  wxButton* monitorFreqButton;
  wxButton* monitorDisplaySizeButton;
  wxStaticBox *monitorModelineGeneratorBox;
  wxTextCtrl *monitorModelineGeneratorXRes;
  wxTextCtrl *monitorModelineGeneratorYRes;
  wxTextCtrl *monitorModelineGeneratorFreq;
  wxTextCtrl *monitorModelineGeneratorOutput1;
  wxTextCtrl *monitorModelineGeneratorOutput2;
  wxStaticText *monitorModelineGeneratorXResText;
  wxStaticText *monitorModelineGeneratorYResText;
  wxStaticText *monitorModelineGeneratorFreqText;
  wxButton* monitorModelineGeneratorButton;
  wxButton* monitorModelineGeneratorAddButton;

  // section INPUTDEVICE elements
  wxStaticBox *inputDeviceBox;
  wxComboBox *inputDeviceIdentifier;
  wxComboBox *inputDeviceDriver;
  wxComboBox *inputDeviceOption;
  wxComboBox *inputDeviceOptValue;
  wxStaticText *inputDeviceDriverText;
  wxStaticText *inputDeviceOptionText;

  // section SCREEN elements
  wxStaticBox *screenBox;
  wxComboBox *screenIdentifier;
  wxChoice *screenDevice;
  wxChoice *screenMonitor;
  wxTextCtrl *screenDepth;
  wxComboBox *screenOption;
  wxTextCtrl *screenOptValue;
  wxComboBox *screenDisplayDepth;
  wxTextCtrl *screenDisplayViewport;
  wxTextCtrl *screenDisplayVirtual;
  wxComboBox *screenDisplayModes;
  wxComboBox *screenDisplayOption;
  wxTextCtrl *screenDisplayOptValue;
  wxStaticText *screenDeviceText;
  wxStaticText *screenMonitorText;
  wxStaticText *screenDepthText;
  wxStaticText *screenOptionText;
  wxStaticText *screenDisplayDepthText;
  wxStaticText *screenDisplayViewportText;
  wxStaticText *screenDisplayVirtualText;
  wxStaticText *screenDisplayModesText;
  wxStaticText *screenDisplayOptionText;

  // section SERVERLAYOUT elements
  wxStaticBox *serverLayoutBox;
  wxComboBox *serverLayoutIdentifier;
  wxComboBox *serverLayoutScreen;
  wxChoice *serverLayoutScreen1;
  wxChoice *serverLayoutScreen2;
  wxTextCtrl *serverLayoutScreenNumber;
  wxTextCtrl *serverLayoutScreenPosition;
  wxComboBox *serverLayoutInputDevice;
  wxChoice *serverLayoutInputDevice1;
  wxComboBox *serverLayoutInputDeviceOption;
  // wxTextCtrl *serverLayoutInputDeviceOptValue;
  wxComboBox *serverLayoutOption;
  wxTextCtrl *serverLayoutOptValue;
  wxStaticText *serverLayoutScreenNumberText;
  wxStaticText *serverLayoutScreenPositionText;
  wxStaticText *serverLayoutOptionText;
  wxStaticText *serverLayoutInputDeviceOptionText;

  // section SERVERFLAGS elements
  wxStaticBox *serverFlagBox;
  wxComboBox *serverFlagsOption;
  wxTextCtrl *serverFlagsOptValue;

  // section EXTENSION elements
  wxStaticBox *extensionBox;
  wxComboBox *extensionOption;
  wxTextCtrl *extensionOptValue;

  // section MODULE elements
  wxStaticBox *moduleBox;
  wxComboBox *moduleIdentifier;
  wxComboBox *moduleOption;
  wxTextCtrl *moduleOptValue;
  wxStaticText *moduleOptionText;

  // section FILES elements
  wxStaticBox *filesBox;
  wxComboBox *fontPath;
  wxStaticText *fontPathText;
  wxComboBox *modulePath;
  wxStaticText *modulePathText;
  wxComboBox *rgbPath;
  wxStaticText *rgbPathText;

  // section DRI elements
  wxStaticBox *driBox;
  wxTextCtrl *driMode;

  // other elements
  XorgFile xorg;
  Settings settings;
  TextFile deviceDriverList;
  TextFile inputDeviceDriverList;
  unsigned int optListSize;
  unsigned int devListSize;

  wxString version;   // current version
  bool isSaved;       // flag if file is saved or there are some changes
  bool debug;         // sets debug mode on and off
  wxString debugFile; // sets debug filename

  DECLARE_EVENT_TABLE();

  // **************************
  // Constants
  // **************************

  // IDs for the controls and the menu commands in guiFrame
  enum myFrameID
  {
    // menu items
    NEWXORG=0,
    LOADXORG,
    SAVEXORG,
    SAVEAS,
    RESTORE,
    CLOSEXORG,
    QUITPROG,
    SHOWXORG,
    CHECKXORG,
    TESTXORG,
    ABOUTPROG,
    LANGGER,
    LANGENG,
    SETTINGS,
    DEBUG,

    // device items
    DEVID,
    DEVDRI,
    DEVBUS,
    DEVSCR,
    DEVRAM,
    DEVOPT,
    DEVOVAL,

    // input device items
    IDEVID,
    IDEVDRI,
    IDEVOPT,
    IDEVOVAL,

    // monitor items
    MONID,
    MONHSY,
    MONVREF,
    MONDISP,
    MONGAM,
    MONOPT,
    MONOVAL,
    MONMODL,
    MONMODLOPT,
    MONFREQ,
    MONDISPSIZE,
    MONMODLGEN,
    MONMODLADD,

    // screen items
    SCRID,
    SCRDEP,
    SCRDEV,
    SCRMON,
    SCROPT,
    SCROVAL,
    SCRDISP,
    SCRDISPVP,
    SCRDISPVIR,
    SCRDISPMOD,
    SCRDISPOPT,
    SCRDISPOVAL,

    // server layout items
    SERLID,
    SERLSCR,
    SERLSCRNUM,
    SERLSCRPOS,
    SERLSCR1,
    SERLSCR2,
    SERLIDEV,
    SERLIDEV1,
    SERLIDEVOPT,
    SERLOPT,
    SERLOVAL,

    // server flags items
    SERFOPT,
    SERFOVAL,

      // extension items
    EXTOPT,
    EXTOVAL,

    // files items
    FONTPATH,
    MODPATH,
    RGBPATH,

    // modules items
    MODID,
    MODOPT,
    MODOVAL,

    // dri items
    DRIMOD,

    // global items
    CHANGETAB
  };
};

#endif // GUIFRAME_H

