// *************************************************************************
// xorg-edit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// Header Files
// **************************************

#include "optiondialog.h"
#include "optionwindow.h"
#include "../libxorgedit/mymessage.h"

// **************************************
// Event Tables and other Macros
// **************************************

BEGIN_EVENT_TABLE(OptionDialog, wxDialog)
  EVT_NOTEBOOK_PAGE_CHANGING(CHANGE_TAB, OptionDialog::OnTabChange)
  EVT_BUTTON(wxID_OK, OptionDialog::OnOK)
  EVT_BUTTON(wxID_CANCEL, OptionDialog::OnCancel)
END_EVENT_TABLE()

// **************************
// Class Declaration
// **************************

OptionDialog::OptionDialog(const wxString& title, const DriverOptionList& list, Device& device) :
wxDialog(NULL, -1, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxMAXIMIZE_BOX )
{
  dPointer=&device;
  idPointer=NULL;
  oPointer=NULL;

  MyString funcName("OptionDialog::OptionDialog(Device)");

  if (list.GetSection(generalList, "general"))
  {
    MyMessage::Append(funcName);
    return;
  }
  if (list.GetSection(advancedList, "advanced"))
  {
    MyMessage::Append(funcName);
    return;
  }
  if (list.GetSection(dualList, "dual"))
  {
    MyMessage::Append(funcName);
    return;
  }

  unsigned int i;
  int pos;
  for (i=0; i<generalList.GetSize(); i++)
  {
    pos=device.Options().Find(generalList(i).GetName(), true);
    if (pos==-1)
    {
      generalList(i).SetActive(false);
    }
    else
    {
      generalList(i).SetActive(true);
      if (!device.Options()(pos).GetValue().IsEmpty()) generalList(i).SetDefault(device.Options()(pos).GetValue());
    }
  }

  for (i=0; i<advancedList.GetSize(); i++)
  {
    pos=device.Options().Find(advancedList(i).GetName(), true);
    if (pos==-1)
    {
      advancedList(i).SetActive(false);
    }
    else
    {
      advancedList(i).SetActive(true);
      if (!device.Options()(pos).GetValue().IsEmpty()) advancedList(i).SetDefault(device.Options()(pos).GetValue());
    }
  }

  for (i=0; i<dualList.GetSize(); i++)
  {
    pos=device.Options().Find(dualList(i).GetName(), true);
    if (pos==-1)
    {
      dualList(i).SetActive(false);
    }
    else
    {
      dualList(i).SetActive(true);
      if (!device.Options()(pos).GetValue().IsEmpty()) dualList(i).SetDefault(device.Options()(pos).GetValue());
    }
  }

  // layout for the whole frame
  wxBoxSizer *gridSizer = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *generalTabSizer=new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *advancedTabSizer=new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *dualTabSizer=new wxBoxSizer(wxVERTICAL);

  wxNotebook* layout = new wxNotebook(this, CHANGE_TAB);   // layout for tabs

  wxPanel* generalTab = new wxPanel(layout);                           // general options
  wxPanel* advancedTab = new wxPanel(layout);                        // advanced options
  wxPanel* dualTab = new wxPanel(layout);                                // dual monitor options

  optionDesc=new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(400,60), wxTE_MULTILINE | wxTE_READONLY );

  OptionWindow *generalScrollWindow=new OptionWindow(generalTab, optionDesc, generalList, wxSize(400,150));
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return;
  }

  OptionWindow *advancedWindow=new OptionWindow(advancedTab, optionDesc, advancedList, wxSize(400,150));
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return;
  }

  OptionWindow *dualScrollWindow=new OptionWindow(dualTab, optionDesc, dualList, wxSize(400,150));
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return;
  }

  generalTabSizer->Add(generalScrollWindow, 1, wxEXPAND | wxALL, 5);
  advancedTabSizer->Add(advancedWindow, 1, wxEXPAND | wxALL, 5);
  dualTabSizer->Add(dualScrollWindow, 1, wxEXPAND | wxALL, 5);

  generalTab->SetSizer(generalTabSizer);
  advancedTab->SetSizer(advancedTabSizer);
  dualTab->SetSizer(dualTabSizer);

  layout->AddPage(generalTab, _("General"));
  layout->AddPage(advancedTab, _("Advanced"));
  layout->AddPage(dualTab, _("TV / Multi Monitor"));

  wxButton *butOK=new wxButton(this, wxID_OK, _("OK"));
  wxButton *butCancel=new wxButton(this, wxID_CANCEL, _("Cancel"));

  buttonSizer->Add(butOK, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 10);
  buttonSizer->Add(butCancel, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 10);

  wxStaticText *warningText=new wxStaticText(this, -1, _("Options that are checked will be added if they do not exist.\nOptions that are unchecked will be deleted if they exist."),
                                            wxDefaultPosition, wxSize(400,-1), wxALIGN_CENTRE );

  gridSizer->Add(layout, 0, wxEXPAND);            // add notebook layout
  gridSizer->Add(optionDesc, 1, wxEXPAND | wxALL | wxALIGN_CENTER, 3);
  gridSizer->Add(warningText, 0, wxALL | wxALIGN_CENTER, 3);
  gridSizer->Add(buttonSizer, 0, wxALIGN_CENTER | wxALL, 3);            // add notebook layout

  SetSizerAndFit(gridSizer);         // sets the sizer
}

OptionDialog::OptionDialog(const wxString& title, const DriverOptionList& list, InputDevice& device) : wxDialog(NULL, -1, title)
{
  dPointer=NULL;
  idPointer=&device;
  oPointer=NULL;

  MyString funcName("OptionDialog::OptionDialog(InputDevice)");

  if (list.GetSection(generalList, "general"))
  {
    MyMessage::Append(funcName);
    return;
  }
  if (list.GetSection(advancedList, "advanced"))
  {
    MyMessage::Append(funcName);
    return;
  }

  unsigned int i;
  int pos;
  for (i=0; i<generalList.GetSize(); i++)
  {
    pos=device.Options().Find(generalList(i).GetName(), true);
    if (pos==-1)
    {
      generalList(i).SetActive(false);
    }
    else
    {
      generalList(i).SetActive(true);
      if (!device.Options()(pos).GetValue().IsEmpty()) generalList(i).SetDefault(device.Options()(pos).GetValue());
    }
  }

  for (i=0; i<advancedList.GetSize(); i++)
  {
    pos=device.Options().Find(advancedList(i).GetName(), true);
    if (pos==-1)
    {
      advancedList(i).SetActive(false);
    }
    else
    {
      advancedList(i).SetActive(true);
      if (!device.Options()(pos).GetValue().IsEmpty()) advancedList(i).SetDefault(device.Options()(pos).GetValue());
    }
  }

  // layout for the whole frame
  wxBoxSizer *gridSizer = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *generalTabSizer=new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *advancedTabSizer=new wxBoxSizer(wxVERTICAL);

  wxNotebook* layout = new wxNotebook(this, CHANGE_TAB);   // layout for tabs

  wxPanel* generalTab = new wxPanel(layout);                           // general options
  wxPanel* advancedTab = new wxPanel(layout);                        // advanced options

  optionDesc=new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(400,60), wxTE_MULTILINE | wxTE_READONLY );

  OptionWindow *generalScrollWindow=new OptionWindow(generalTab, optionDesc, generalList, wxSize(400,150));
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return;
  }

  OptionWindow *advancedWindow=new OptionWindow(advancedTab, optionDesc, advancedList, wxSize(400,150));
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return;
  }

  generalTabSizer->Add(generalScrollWindow, 1, wxEXPAND | wxALL, 5);
  advancedTabSizer->Add(advancedWindow, 1, wxEXPAND | wxALL, 5);

  generalTab->SetSizer(generalTabSizer);
  advancedTab->SetSizer(advancedTabSizer);

  layout->AddPage(generalTab, _("General"));
  layout->AddPage(advancedTab, _("Advanced"));

  wxButton *butOK=new wxButton(this, wxID_OK, _("OK"));
  wxButton *butCancel=new wxButton(this, wxID_CANCEL, _("Cancel"));

  buttonSizer->Add(butOK, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 10);
  buttonSizer->Add(butCancel, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 10);

  wxStaticText *warningText=new wxStaticText(this, -1, _("Options that are checked will be added if they do not exist.\nOptions that are unchecked will be deleted if they exist."),
                                            wxDefaultPosition, wxSize(400,-1), wxALIGN_CENTRE );

  gridSizer->Add(layout, 0, wxEXPAND);            // add notebook layout
  gridSizer->Add(optionDesc, 1, wxEXPAND | wxALL | wxALIGN_CENTER, 3);
  gridSizer->Add(warningText, 0, wxALL | wxALIGN_CENTER, 3);
  gridSizer->Add(buttonSizer, 0, wxALIGN_CENTER | wxALL, 3);            // add notebook layout

  SetSizerAndFit(gridSizer);         // sets the sizer
}

OptionDialog::OptionDialog(const wxString& title, OptionList& options) :
wxDialog(NULL, -1, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxMAXIMIZE_BOX )
{
  dPointer=NULL;
  idPointer=NULL;
  oPointer=&options;

  MyString funcName("OptionDialog::OptionDialog(Options)");

  // layout for the whole frame
  wxBoxSizer *gridSizer = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *generalTabSizer=new wxBoxSizer(wxVERTICAL);

  wxNotebook* layout = new wxNotebook(this, CHANGE_TAB);   // layout for tabs

  wxPanel* generalTab = new wxPanel(layout);                           // general options

  optionDesc=new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(400,60), wxTE_MULTILINE | wxTE_READONLY );

  OptionWindow *generalScrollWindow=new OptionWindow(generalTab, optionDesc, generalList, wxSize(400,150));
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return;
  }

  generalTabSizer->Add(generalScrollWindow, 1, wxEXPAND | wxALL, 5);

  generalTab->SetSizer(generalTabSizer);

  layout->AddPage(generalTab, _("General"));

  wxButton *butOK=new wxButton(this, wxID_OK, _("OK"));
  wxButton *butCancel=new wxButton(this, wxID_CANCEL, _("Cancel"));

  wxStaticText *warningText=new wxStaticText(this, -1, _("Options that are checked will be added if they do not exist.\nOptions that are unchecked will be deleted if they exist."),
                                            wxDefaultPosition, wxSize(400,-1), wxALIGN_CENTRE );

  buttonSizer->Add(butOK, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 10);
  buttonSizer->Add(butCancel, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 10);

  gridSizer->Add(layout, 0, wxEXPAND);            // add notebook layout
  gridSizer->Add(optionDesc, 1, wxEXPAND | wxALL | wxALIGN_CENTER, 3);
  gridSizer->Add(warningText, 0, wxALL | wxALIGN_CENTER, 3);
  gridSizer->Add(buttonSizer, 0, wxALIGN_CENTER | wxALL, 3);            // add notebook layout

  SetSizerAndFit(gridSizer);         // sets the sizer
}

OptionDialog::~OptionDialog()
{
  dPointer=NULL;
  idPointer=NULL;
  // do not delete pointers!
}

void OptionDialog::OnTabChange(wxNotebookEvent& event)    // some tab has changed
{
  optionDesc->Clear();  // clear textfield
}

void OptionDialog::OnOK(wxCommandEvent& event)
{
  MyString funcName("OptionDialog::OnOK");

  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    EndModal(wxID_CANCEL);
  }

  bool changed=false;

  if (AcceptDevice(changed))
  {
    MyMessage::Append(funcName);
    EndModal(wxID_CANCEL);
  }

  if (AcceptInputDevice(changed))
  {
    MyMessage::Append(funcName);
    EndModal(wxID_CANCEL);
  }

  if (AcceptOptions(changed))
  {
    MyMessage::Append(funcName);
    EndModal(wxID_CANCEL);
  }

  if (changed) EndModal(wxID_OK);  // end dialog
  else EndModal(wxID_CANCEL);
}

const bool OptionDialog::AcceptDevice(bool& changed)
{        // accept all changes in device

  if (dPointer==NULL) return(false);

  MyString funcName("OptionDialog::AcceptDevice");

  // accept all the changes
  unsigned int i;
  int pos;
  bool del=false;
  Option temp;

  // first delete options
  i=0;
  while (i<dPointer->Options().GetSize())
  {
    del=false;

    pos=generalList.Find(dPointer->Options()(i).GetName(), true);
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (pos!=-1)
    {  // found
      if (!generalList(pos).IsActive())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        if (dPointer->Options().Delete(i))
        {
          MyMessage::Append(funcName);
          return(true);
        }
        changed=true;
        del=true;
      }
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    } // pos!=-1

    if (!del) i++;
  } // while

  i=0;
  while (i<dPointer->Options().GetSize())
  {
    del=false;

    pos=advancedList.Find(dPointer->Options()(i).GetName(), true);
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (pos!=-1)
    {  // found
      if (!advancedList(pos).IsActive())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        if (dPointer->Options().Delete(i))
        {
          MyMessage::Append(funcName);
          return(true);
        }
        changed=true;
        del=true;
      }
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    } // pos!=-1

    if (!del) i++;
  } // while

  i=0;
  while (i<dPointer->Options().GetSize())
  {
    del=false;

    pos=dualList.Find(dPointer->Options()(i).GetName(), true);
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (pos!=-1)
    {  // found
      if (!dualList(pos).IsActive())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        if (dPointer->Options().Delete(i))
        {
          MyMessage::Append(funcName);
          return(true);
        }
        changed=true;
        del=true;
      }
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    } // pos!=-1

    if (!del) i++;
  } // while

  // check for options that have changed
  // and add new options
  for (i=0; i<generalList.GetSize(); i++)
  {
    if (generalList(i).IsActive())
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      pos=dPointer->Options().Find(generalList(i).GetName(), true);
      if (pos==-1)
      {  // not found -> add
        temp.Clear();
        temp.Set(generalList(i).GetName(), generalList(i).GetDefault());
        dPointer->Options().Append(temp);
        changed=true;
      }
      else
      {
        if (dPointer->Options()(pos).GetValue()!=generalList(i).GetDefault())
        {
          dPointer->Options()(pos).SetValue(generalList(i).GetDefault());
          changed=true;
        }
      }
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  } // for i

  for (i=0; i<advancedList.GetSize(); i++)
  {
    if (advancedList(i).IsActive())
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      pos=dPointer->Options().Find(advancedList(i).GetName(), true);
      if (pos==-1)
      {  // not found -> add
        temp.Clear();
        temp.Set(advancedList(i).GetName(), advancedList(i).GetDefault());
        dPointer->Options().Append(temp);
        changed=true;
      }
      else
      {
        if (dPointer->Options()(pos).GetValue()!=advancedList(i).GetDefault())
        {
          dPointer->Options()(pos).SetValue(advancedList(i).GetDefault());
          changed=true;
        }
      }
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  } // for i

  for (i=0; i<dualList.GetSize(); i++)
  {
    if (dualList(i).IsActive())
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      pos=dPointer->Options().Find(dualList(i).GetName(), true);
      if (pos==-1)
      {  // not found -> add
        temp.Clear();
        temp.Set(dualList(i).GetName(), dualList(i).GetDefault());
        dPointer->Options().Append(temp);
        changed=true;
      }
      else
      {
        if (dPointer->Options()(pos).GetValue()!=dualList(i).GetDefault())
        {
          dPointer->Options()(pos).SetValue(dualList(i).GetDefault());
          changed=true;
        }
      }
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  } // for i

  return(false);
}

const bool OptionDialog::AcceptInputDevice(bool& changed)
{  // accept all changes in input device

  if (idPointer==NULL) return(false);

  MyString funcName("OptionDialog::AcceptInputDevice");

  // accept all the changes
  unsigned int i;
  int pos;
  bool del=false;
  Option temp;

  // first delete options
  i=0;
  while (i<idPointer->Options().GetSize())
  {
    del=false;

    pos=generalList.Find(idPointer->Options()(i).GetName(), true);
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (pos!=-1)
    {  // found
      if (!generalList(pos).IsActive())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        if (idPointer->Options().Delete(i))
        {
          MyMessage::Append(funcName);
          return(true);
        }
        changed=true;
        del=true;
      }
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    } // pos!=-1

    if (!del) i++;
  } // while

  i=0;
  while (i<idPointer->Options().GetSize())
  {
    del=false;

    pos=advancedList.Find(idPointer->Options()(i).GetName(), true);
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (pos!=-1)
    {  // found
      if (!advancedList(pos).IsActive())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        if (idPointer->Options().Delete(i))
        {
          MyMessage::Append(funcName);
          return(true);
        }
        changed=true;
        del=true;
      }
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    } // pos!=-1

    if (!del) i++;
  } // while

  // check for options that have changed
  // and add new options
  for (i=0; i<generalList.GetSize(); i++)
  {
    if (generalList(i).IsActive())
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      pos=idPointer->Options().Find(generalList(i).GetName(), true);
      if (pos==-1)
      {  // not found
        temp.Clear();
        temp.Set(generalList(i).GetName(), generalList(i).GetDefault());
        idPointer->Options().Append(temp);
        changed=true;
      }
      else
      {
        if (idPointer->Options()(pos).GetValue()!=generalList(i).GetDefault())
        {
          idPointer->Options()(pos).SetValue(generalList(i).GetDefault());
          changed=true;
        }
      }
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  } // for i

  for (i=0; i<advancedList.GetSize(); i++)
  {
    if (advancedList(i).IsActive())
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      pos=idPointer->Options().Find(advancedList(i).GetName(), true);
      if (pos==-1)
      {  // not found
        temp.Clear();
        temp.Set(advancedList(i).GetName(), advancedList(i).GetDefault());
        idPointer->Options().Append(temp);
        changed=true;
      }
      else
      {
        if (idPointer->Options()(pos).GetValue()!=generalList(i).GetDefault())
        {
          idPointer->Options()(pos).SetValue(generalList(i).GetDefault());
          changed=true;
        }
      }
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  } // for i

  return(false);
}

const bool OptionDialog::AcceptOptions(bool& changed)
{       // accept all changes in options

  if (oPointer==NULL) return(false);

  MyString funcName("OptionDialog::AcceptOptions");

  // accept all the changes
  /*
  unsigned int i;
  int pos;
  bool del=false;
  Option temp;
  */


  return(false);
}




