// *************************************************************************
// xorg-edit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// Header Files
// **************************************

#include "optionwindow.h"
#include <wx/tooltip.h>
#include "../libxorgedit/mymessage.h"

// **************************************
// Event Tables and other Macros
// **************************************

IMPLEMENT_DYNAMIC_CLASS(OptionWindow, wxScrolledWindow)

// **************************
// Class Declaration
// **************************

OptionWindow::OptionWindow()
{}

OptionWindow::OptionWindow(wxWindow *parent, wxTextCtrl *descCtrl, DriverOptionList& list, const wxSize& size) :
  wxScrolledWindow(parent, wxID_ANY, wxDefaultPosition, size)
{
  optionPointer=&list;
  optionDesc=descCtrl;

  SetScrollRate( 10, 10 );
  // SetVirtualSize( 500, 1000 );

  wxFlexGridSizer *gridSizer=new wxFlexGridSizer(0,2,3,3);
  wxCheckBox **checkBoxPointer;
  wxString tempList[]={};

  // allocate memory für checkbox pointers
  if (optionPointer->GetSize()>0)
  {
    checkBoxPointer=new wxCheckBox*[optionPointer->GetSize()];
    defaultPointer=new wxComboBox*[optionPointer->GetSize()];
  }
  else
  {
    checkBoxPointer=NULL;
    defaultPointer=NULL;
  }

  // allocate single checkboxes
  for (unsigned int i=0; i<optionPointer->GetSize(); i++)
  {
    checkBoxPointer[i]=new wxCheckBox(this, CHECKBOX_ID+i, wxString::FromAscii(optionPointer->operator()(i).GetName().data()));
    if (!MyMessage::OK())
    {
      MyMessage::Append("OptionWindow::OptionWindow");
      return;
    }
    Connect (CHECKBOX_ID+i, wxEVT_COMMAND_CHECKBOX_CLICKED, wxCommandEventHandler(OptionWindow::OnChangeOption));
    if (!optionPointer->operator()(i).GetComment().IsEmpty())
    {
      checkBoxPointer[i]->SetToolTip(new wxToolTip(wxString::FromAscii(optionPointer->operator()(i).GetComment().data())));
      // checkBoxPointer[i]->GetToolTip()->Enable(false);
    }
    defaultPointer[i]=new wxComboBox(this, CHECKBOX_ID+i, wxEmptyString, wxDefaultPosition, wxSize(185,-1), 0, tempList, wxCB_DROPDOWN );
    Connect(CHECKBOX_ID+i, wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler(OptionWindow::EnterOptionValue));
    Connect(CHECKBOX_ID+i, wxEVT_COMMAND_COMBOBOX_SELECTED, wxCommandEventHandler(OptionWindow::SelectOptionValue));

    // add text

    if (optionPointer->operator()(i).GetValue()=="boolean")
    {
      MyString tempStr=optionPointer->operator()(i).GetDefault().toLower();

      if (tempStr.IsEmpty() || tempStr=="false" || tempStr=="true")
      {
        defaultPointer[i]->Append(_T("false"));
        defaultPointer[i]->Append(_T("true"));
      }
      else if (tempStr=="off" || tempStr=="on")
      {
        defaultPointer[i]->Append(_T("off"));
        defaultPointer[i]->Append(_T("on"));
      }
      else if (tempStr=="no" || tempStr=="yes")
      {
        defaultPointer[i]->Append(_T("no"));
        defaultPointer[i]->Append(_T("yes"));
      }
      else if (tempStr=="0" || tempStr=="1")
      {
        defaultPointer[i]->Append(_T("0"));
        defaultPointer[i]->Append(_T("1"));
      }
      defaultPointer[i]->SetEditable(false);

      if (tempStr.IsEmpty()) defaultPointer[i]->SetSelection(0);
      else defaultPointer[i]->SetValue(wxString::FromAscii(tempStr.data()));
    }
    else
    {
      // add arguments
      StringArray tempA=optionPointer->operator()(i).GetArguments();

      for (unsigned int j=0; j<tempA.GetSize(); j++)
      {
        defaultPointer[i]->Append(wxString::FromAscii(tempA(j).data()));
        if (!MyMessage::OK())
        {
          MyMessage::Append("OptionWindow::OptionWindow");
          return;
        }
      }

      MyString tempStr=optionPointer->operator()(i).GetDefault();
      if (tempStr.IsEmpty() && tempA.GetSize()!=0) defaultPointer[i]->SetSelection(0);
      else defaultPointer[i]->SetValue(wxString::FromAscii(tempStr.data()));
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append("OptionWindow::OptionWindow");
      return;
    }

    if (optionPointer->operator()(i).IsStrict())
    {
      defaultPointer[i]->SetEditable(false);
    }

    if (optionPointer->operator()(i).IsActive())
    {
      checkBoxPointer[i]->SetValue(true);
      defaultPointer[i]->Enable(true);
    }
    else
    {
      checkBoxPointer[i]->SetValue(false);
      defaultPointer[i]->Enable(false);
    }

    gridSizer->Add(checkBoxPointer[i], 0, wxEXPAND); // | wxLEFT | wxRIGHT | wxBOTTOM , 3);
    gridSizer->Add(defaultPointer[i], 0, wxEXPAND); // wxLEFT | wxRIGHT | wxBOTTOM , 3);
  }
  SetSizer(gridSizer);
}

OptionWindow::~OptionWindow()  // destructor
{
   optionPointer=NULL;  // do not delete
}

void OptionWindow::OnChangeOption(wxCommandEvent& event)
{
  optionPointer->operator()(event.GetId()-CHECKBOX_ID).SetActive(event.IsChecked());
  defaultPointer[event.GetId()-CHECKBOX_ID]->Enable(event.IsChecked());

  wxString desc=wxString::FromAscii(optionPointer->operator()(event.GetId()-CHECKBOX_ID).GetDescription().data());
  if (desc.IsEmpty())
  {
     desc=wxString::FromAscii(optionPointer->operator()(event.GetId()-CHECKBOX_ID).GetComment().data());
  }

  optionDesc->SetValue(desc);
}

void OptionWindow::EnterOptionValue(wxCommandEvent& event)
{
  MyString newId=(const char*)defaultPointer[event.GetId()-CHECKBOX_ID]->GetValue().mb_str();

  if (optionPointer->operator()(event.GetId()-CHECKBOX_ID).GetDefault()!=newId)
  {
    optionPointer->operator()(event.GetId()-CHECKBOX_ID).SetDefault(newId);
  }
}

void OptionWindow::SelectOptionValue(wxCommandEvent& event)
{
  optionPointer->operator()(event.GetId()-CHECKBOX_ID).SetDefault((const char*)defaultPointer[event.GetId()-CHECKBOX_ID]->GetValue().mb_str());
}

