// *************************************************************************
// xorg-edit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// Header Files
// **************************************

#include "settingsdialog.h"

// **************************************
// Event Tables and other Macros
// **************************************

BEGIN_EVENT_TABLE(SettingsDialog, wxDialog)
  EVT_BUTTON(wxID_OK, SettingsDialog::OnOK)
  EVT_BUTTON(wxID_CANCEL, SettingsDialog::OnCancel)
  EVT_BUTTON(BUTTON_SETTINGS_CHANGEOPTDIR, SettingsDialog::OnChangeOptionDir)
END_EVENT_TABLE()

// **************************
// Class Declaration
// **************************

SettingsDialog::SettingsDialog(wxWindow* parent, const wxString& title, Settings& settings) : wxDialog(parent, -1, title)
{
  pointer=&settings;

  wxBoxSizer *optDirSizer = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *gridSizer = new wxBoxSizer(wxVERTICAL);

  wxStaticText *textOptDir=new wxStaticText(this, wxID_ANY, _("Option Directory"));

  checkSaveSettings=new wxCheckBox(this, wxID_ANY, _("Save settings when leaving program"));
  checkLoadLast=new wxCheckBox(this, wxID_ANY, _("Open last edited file at start"));
  checkShowSplash=new wxCheckBox(this, wxID_ANY, _("Show splash screen before start"));
  checkShowBlank=new wxCheckBox(this, wxID_ANY, _("Show message that the screen goes blank when testing configuration"));
  checkRememberTest=new wxCheckBox(this, wxID_ANY, _("Remember to test the configuration before saving"));
  optDir=new wxTextCtrl(this, wxID_ANY, wxString::FromAscii(pointer->GetOptionDir().data()), wxDefaultPosition, wxSize(300,-1));

  wxButton *butOK=new wxButton(this, wxID_OK, _("OK"));
  wxButton *butCancel=new wxButton(this, wxID_CANCEL, _("Cancel"));
  wxButton *butOptDir=new wxButton(this, BUTTON_SETTINGS_CHANGEOPTDIR, _("Change Directory"));

  checkSaveSettings->SetValue(pointer->GetSaveSettings());
  checkLoadLast->SetValue(pointer->GetLoadLastFile());
  checkShowSplash->SetValue(pointer->GetShowSplash());
  checkShowBlank->SetValue(pointer->GetShowBlank());
  checkRememberTest->SetValue(pointer->GetRememberTest());

  optDirSizer->Add(textOptDir, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  optDirSizer->Add(optDir, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  optDirSizer->Add(butOptDir, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  buttonSizer->Add(butOK, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 10);
  buttonSizer->Add(butCancel, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 10);

  gridSizer->Add(optDirSizer, 0, wxALL, 3);
  gridSizer->Add(checkSaveSettings, 0, wxALL, 3);
  gridSizer->Add(checkLoadLast, 0, wxALL, 3);
  gridSizer->Add(checkShowSplash, 0, wxALL, 3);
  gridSizer->Add(checkShowBlank, 0, wxALL, 3);
  gridSizer->Add(checkRememberTest, 0, wxALL, 3);
  gridSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER, 3);

  SetSizerAndFit(gridSizer);         // sets the sizer
}

SettingsDialog::~SettingsDialog()
{
  pointer=NULL;  // do not delete
}

void SettingsDialog::OnOK(wxCommandEvent& event)
{
  pointer->SetSaveSettings(checkSaveSettings->GetValue());
  pointer->SetLoadLastFile(checkLoadLast->GetValue());
  pointer->SetShowSplash(checkShowSplash->GetValue());
  pointer->SetShowBlank(checkShowBlank->GetValue());
  pointer->SetRememberTest(checkRememberTest->GetValue());
  pointer->SetOptionDir((const char*)optDir->GetValue().mb_str());
  EndModal(wxID_OK);  // end dialog
}

void SettingsDialog::OnChangeOptionDir(wxCommandEvent& event)
{
  // directory dialog
  wxDirDialog openDirDialog(this, _("Choose a directory"), optDir->GetValue());

  if (openDirDialog.ShowModal() == wxID_OK)     // directory has been chosen
  {
    optDir->SetValue(openDirDialog.GetPath());
  }
}
