// *************************************************************************
// xorg-edit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// Header Files
// **************************************

#include <wx/datetime.h>
#include <wx/utils.h>
#include <wx/tooltip.h>
#include <wx/file.h>
#include "guiframe.h"
#include "textframe.h"
#include "optiondialog.h"
#include "settingsdialog.h"
#include "xorgapp.h"
#include "../libxorgedit/mymessage.h"

// **************************************
// Event Tables and other Macros
// **************************************

BEGIN_EVENT_TABLE(GuiFrame, wxFrame)
  EVT_MENU(LANGENG, GuiFrame::SetLanguage)
  EVT_MENU(LANGGER, GuiFrame::SetLanguage)
  EVT_NOTEBOOK_PAGE_CHANGING(CHANGETAB, GuiFrame::OnTabChange)
  EVT_MENU(NEWXORG, GuiFrame::OnNew)
  EVT_MENU(LOADXORG, GuiFrame::OnLoad)
  EVT_MENU(SAVEXORG, GuiFrame::OnSave)
  EVT_MENU(SAVEAS, GuiFrame::OnSaveAs)
  EVT_MENU(RESTORE, GuiFrame::OnRestore)
  EVT_MENU(CLOSEXORG, GuiFrame::OnClose)
  EVT_MENU(QUITPROG, GuiFrame::OnQuit)
  EVT_MENU(SETTINGS, GuiFrame::OnSettings)
  EVT_MENU(SHOWXORG, GuiFrame::OnShow)
  EVT_MENU(TESTXORG, GuiFrame::OnTest)
  EVT_MENU(CHECKXORG, GuiFrame::OnCheck)
  EVT_MENU(ABOUTPROG, GuiFrame::OnAbout)
  EVT_MENU(DEBUG, GuiFrame::OnDebug)
  EVT_BUTTON(MONFREQ, GuiFrame::GetRefreshRate)
  EVT_BUTTON(MONDISPSIZE, GuiFrame::GetDisplaySize)
  EVT_BUTTON(MONMODLGEN, GuiFrame::GenerateModeline)
  EVT_BUTTON(MONMODLADD, GuiFrame::AddModeline)
  EVT_TEXT(DEVID, GuiFrame::EnterDevice)
  EVT_TEXT(DEVOPT, GuiFrame::EnterDeviceOption)
  EVT_TEXT(IDEVID, GuiFrame::EnterInputDevice)
  EVT_TEXT(IDEVOPT, GuiFrame::EnterInputDeviceOption)
  EVT_TEXT(MONID, GuiFrame::EnterMonitor)
  EVT_TEXT(MONOPT, GuiFrame::EnterMonitorOption)
  EVT_TEXT(MONMODL, GuiFrame::EnterMonitorModeline)
  EVT_TEXT(SCRID, GuiFrame::EnterScreen)
  EVT_TEXT(SCROPT, GuiFrame::EnterScreenOption)
  EVT_TEXT(SCRDISP, GuiFrame::EnterScreenDisplay)
  EVT_TEXT(SCRDISPMOD, GuiFrame::EnterScreenDisplayMode)
  EVT_TEXT(SCRDISPOPT, GuiFrame::EnterScreenDisplayOption)
  EVT_TEXT(SERLID, GuiFrame::EnterServerLayout)
  EVT_TEXT(SERLOPT, GuiFrame::EnterServerLayoutOption)
  EVT_TEXT(SERLIDEVOPT, GuiFrame::EnterServerLayoutInputDeviceOption)
  EVT_TEXT(SERFOPT, GuiFrame::EnterServerFlagOption)
  EVT_TEXT(EXTOPT, GuiFrame::EnterExtension)
  EVT_TEXT(FONTPATH, GuiFrame::EnterFontPath)
  EVT_TEXT(MODPATH, GuiFrame::EnterModulePath)
  EVT_TEXT(RGBPATH, GuiFrame::EnterRgbPath)
  EVT_TEXT(MODID, GuiFrame::EnterModule)
  EVT_TEXT(MODOPT, GuiFrame::EnterModuleOption)
  EVT_TEXT(DEVDRI, GuiFrame::EnterDeviceDriver)
  EVT_TEXT(DEVBUS, GuiFrame::EnterDeviceBusID)
  EVT_TEXT(DEVSCR, GuiFrame::EnterDeviceScreen)
  EVT_TEXT(DEVRAM, GuiFrame::EnterDevicVideoRam)
  EVT_TEXT(DEVOVAL, GuiFrame::EnterDeviceOptValue)
  EVT_TEXT(MONHSY, GuiFrame::EnterMonitorHorizSync)
  EVT_TEXT(MONVREF, GuiFrame::EnterMonitorVertRefresh)
  EVT_TEXT(MONDISP, GuiFrame::EnterMonitorDisplay)
  EVT_TEXT(MONGAM, GuiFrame::EnterMonitorGamma)
  EVT_TEXT(MONOVAL, GuiFrame::EnterMonitorOptValue)
  EVT_TEXT(MONMODLOPT, GuiFrame::EnterMonitorModelineOption)
  EVT_TEXT(IDEVDRI, GuiFrame::EnterInputDeviceDriver)
  EVT_TEXT(IDEVOVAL, GuiFrame::EnterInputDeviceOptValue)
  EVT_TEXT(SCRDEP, GuiFrame::EnterScreenDepth)
  EVT_TEXT(SCROVAL, GuiFrame::EnterScreenOptValue)
  EVT_TEXT(SCRDISPVP, GuiFrame::EnterScreenDisplayVirtual)
  EVT_TEXT(SCRDISPVIR, GuiFrame::EnterScreenDisplayViewport)
  EVT_TEXT(SCRDISPOVAL, GuiFrame::EnterScreenDisplayOptValue)
  EVT_TEXT(SERLSCRNUM, GuiFrame::EnterServerLayoutScreenNumber)
  EVT_TEXT(SERLSCRPOS, GuiFrame::EnterServerLayoutScreenPosition)
  EVT_TEXT(SERLOVAL, GuiFrame::EnterServerLayoutOptValue)
  EVT_TEXT(SERFOVAL, GuiFrame::EnterServerFlagsOptValue)
  EVT_TEXT(EXTOVAL, GuiFrame::EnterExtensionOptValue)
  EVT_TEXT(MODOVAL, GuiFrame::EnterModuleOptValue)
  EVT_TEXT(DRIMOD, GuiFrame::EnterDriMode)
  EVT_CHOICE(SCRDEV, GuiFrame::SelectScreenDevice)
  EVT_CHOICE(SCRMON, GuiFrame::SelectScreenMonitor)
  EVT_CHOICE(SERLSCR1, GuiFrame::SelectServerLayoutScreen1)
  EVT_CHOICE(SERLSCR2, GuiFrame::SelectServerLayoutScreen2)
  EVT_CHOICE(SERLIDEV1, GuiFrame::SelectServerLayoutInputDevice1)
  EVT_COMBOBOX(DEVID, GuiFrame::SelectDevice)
  EVT_COMBOBOX(DEVOPT, GuiFrame::SelectDeviceOption)
  EVT_COMBOBOX(IDEVID, GuiFrame::SelectInputDevice)
  EVT_COMBOBOX(IDEVOPT, GuiFrame::SelectInputDeviceOption)
  EVT_COMBOBOX(MONID, GuiFrame::SelectMonitor)
  EVT_COMBOBOX(MONOPT, GuiFrame::SelectMonitorOption)
  EVT_COMBOBOX(MONMODL, GuiFrame::SelectMonitorModeline)
  EVT_COMBOBOX(SCRID, GuiFrame::SelectScreen)
  EVT_COMBOBOX(SCROPT, GuiFrame::SelectScreenOption)
  EVT_COMBOBOX(SCRDISP, GuiFrame::SelectScreenDisplay)
  EVT_COMBOBOX(SCRDISPMOD, GuiFrame::SelectScreenDisplayMode)
  EVT_COMBOBOX(SCRDISPOPT, GuiFrame::SelectScreenDisplayOption)
  EVT_COMBOBOX(SERLID, GuiFrame::SelectServerLayout)
  EVT_COMBOBOX(SERLOPT, GuiFrame::SelectServerLayoutOption)
  EVT_COMBOBOX(SERLSCR, GuiFrame::SelectServerLayoutScreen)
  EVT_COMBOBOX(SERLIDEV, GuiFrame::SelectServerLayoutInputDevice)
  EVT_COMBOBOX(SERLIDEVOPT, GuiFrame::SelectServerLayoutInputDeviceOption)
  EVT_COMBOBOX(SERFOPT, GuiFrame::SelectServerFlagOption)
  EVT_COMBOBOX(EXTOPT, GuiFrame::SelectExtension)
  EVT_COMBOBOX(FONTPATH, GuiFrame::SelectFontPath)
  EVT_COMBOBOX(MODPATH, GuiFrame::SelectModulePath)
  EVT_COMBOBOX(RGBPATH, GuiFrame::SelectRgbPath)
  EVT_COMBOBOX(MODID, GuiFrame::SelectModule)
  EVT_COMBOBOX(MODOPT, GuiFrame::SelectModuleOption)
END_EVENT_TABLE()

// **************************
// Class Declaration
// **************************

GuiFrame::GuiFrame(const wxString& title) : wxFrame(NULL, -1, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE & ~ (wxRESIZE_BORDER | wxRESIZE_BOX | wxMAXIMIZE_BOX))
{
  MyString funcName("GuiFrame::GuiFrame");

  // set constants
  optListSize=4;
  devListSize=4;

  // set title
  version=_T("xorg-edit v07.06.15");

  // debugMod off
  debug=false;

  // set all entries to null

  // layout for the whole frame
  wxBoxSizer *gridSizer = new wxBoxSizer(wxVERTICAL);

  layout = new wxNotebook(this, CHANGETAB);

  deviceTab = new wxPanel(layout);                       // device tab panel
  wxBoxSizer *deviceTabSizer=new wxBoxSizer(wxVERTICAL);

  monitorTab = new wxPanel(layout);                       // monitor tab panel
  wxBoxSizer *monitorTabSizer=new wxBoxSizer(wxVERTICAL);

  inputDeviceTab = new wxPanel(layout);                       // inputdevice tab panel
  wxBoxSizer *inputDeviceTabSizer=new wxBoxSizer(wxVERTICAL);

  screenTab = new wxPanel(layout);                       // screen tab panel
  wxBoxSizer *screenTabSizer=new wxBoxSizer(wxVERTICAL);

  serverTab = new wxPanel(layout);                       // server layout/server flags tab tab panel
  wxBoxSizer *serverTabSizer=new wxBoxSizer(wxVERTICAL);

  otherSectionTab = new wxPanel(layout);                       // iles/modules/extensions/dri tab panel
  wxBoxSizer *otherSectionTabSizer=new wxBoxSizer(wxVERTICAL);

  wxString tempList[devListSize+1];  // initial list for combo und choice elements
  wxString optList[optListSize];   // initial list for option combo und choice elements

  optList[0]=_("----------");
  optList[1]=_("Add Option");
  optList[2]=_("Delete Option");
  optList[3]=_("De/Activate Option");

  // section DEVICE elements
  deviceBox=new wxStaticBox(deviceTab, -1, _("Device"));
  wxStaticBoxSizer *deviceSizer=new wxStaticBoxSizer(deviceBox, wxVERTICAL);
  wxBoxSizer *deviceSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *deviceSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *deviceSizerLine3=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *deviceSizerLine4=new wxBoxSizer(wxHORIZONTAL);

  deviceDriverText=new wxStaticText(deviceTab, -1, _("Driver"));
  deviceBusIDText=new wxStaticText(deviceTab, -1, _("BusID"));
  deviceVideoRamText=new wxStaticText(deviceTab, -1, _("Video Ram"));
  deviceScreenNumberText=new wxStaticText(deviceTab, -1, _("Screen Number"));
  deviceOptionText=new wxStaticText(deviceTab, -1, _("Option"));

  tempList[0]=_("----------");
  tempList[1]=_("Add Device");
  tempList[2]=_("Delete Device");
  tempList[3]=_("Copy Device");

  deviceIdentifier=new wxComboBox(deviceTab, DEVID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  deviceSizerLine1->Add(deviceIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  // deviceDriver=new wxTextCtrl(deviceTab, DEVDRI, wxEmptyString, wxDefaultPosition, wxSize(140,-1));
  deviceDriver=new wxComboBox(deviceTab, DEVDRI, wxEmptyString, wxDefaultPosition, wxSize(140,-1), 0, tempList, wxCB_DROPDOWN | wxCB_SORT);
  deviceBusID=new wxTextCtrl(deviceTab, DEVBUS, wxEmptyString, wxDefaultPosition, wxSize(140,-1));

  deviceSizerLine2->Add(deviceDriverText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine2->Add(deviceDriver, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine2->Add(deviceBusIDText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
  deviceSizerLine2->Add(deviceBusID, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  deviceVideoRam=new wxTextCtrl(deviceTab, DEVRAM, wxEmptyString, wxDefaultPosition, wxSize(140,-1));
  deviceScreen=new wxTextCtrl(deviceTab, DEVSCR, wxEmptyString, wxDefaultPosition, wxSize(100,-1));

  deviceSizerLine3->Add(deviceVideoRamText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine3->Add(deviceVideoRam, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine3->Add(deviceScreenNumberText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
  deviceSizerLine3->Add(deviceScreen, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add empty Option");
  tempList[2]=_("Manage Options...");
  tempList[3]=_("Delete Option");
  tempList[4]=_("De/Activate Option");

  deviceOption=new wxComboBox(deviceTab, DEVOPT, _("----------"), wxDefaultPosition, wxSize(210,-1), devListSize+1, tempList, wxCB_DROPDOWN);
  deviceOptValue=new wxComboBox(deviceTab, DEVOVAL, wxEmptyString, wxDefaultPosition, wxSize(185,-1), 0, tempList, wxCB_DROPDOWN | wxCB_SORT);
  deviceOption->SetToolTip(new wxToolTip(wxEmptyString));
  deviceOption->GetToolTip()->Enable(false);

  deviceSizerLine4->Add(deviceOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine4->Add(deviceOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  deviceSizerLine4->Add(deviceOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  deviceSizer->Add(deviceSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  deviceSizer->Add(deviceSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  deviceSizer->Add(deviceSizerLine3, 0, wxTOP | wxBOTTOM, 3);
  deviceSizer->Add(deviceSizerLine4, 0, wxTOP | wxBOTTOM, 3);

  // section MONITOR elements
  monitorBox=new wxStaticBox(monitorTab, -1, _("Monitor"));
  wxStaticBoxSizer *monitorSizer=new wxStaticBoxSizer(monitorBox, wxVERTICAL);
  wxBoxSizer *monitorSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorSizerLine3=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorSizerLine4=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorSizerLine5=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorSizerLine6=new wxBoxSizer(wxHORIZONTAL);

  monitorHorizSyncText=new wxStaticText(monitorTab, -1, _("Hor.Sync."));
  monitorVertRefreshText=new wxStaticText(monitorTab, -1, _("Vert.Refresh"));
  monitorDisplayText=new wxStaticText(monitorTab, -1, _("Display Size"));
  monitorGammaText=new wxStaticText(monitorTab, -1, _("Gamma"));
  monitorOptionText=new wxStaticText(monitorTab, -1, _("Option"));
  monitorModelineText=new wxStaticText(monitorTab, -1, _("Modeline"));

  tempList[1]=_("Add Monitor");
  tempList[2]=_("Delete Monitor");
  tempList[3]=_("Copy Monitor");

  monitorIdentifier=new wxComboBox(monitorTab, MONID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  monitorSizerLine1->Add(monitorIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorHorizSync=new wxTextCtrl(monitorTab, MONHSY, wxEmptyString, wxDefaultPosition, wxSize(75,-1));
  monitorVertRefresh=new wxTextCtrl(monitorTab, MONVREF, wxEmptyString, wxDefaultPosition, wxSize(75,-1));
  monitorFreqButton=new wxButton(monitorTab, MONFREQ, _("Get Refresh Rate"));

  monitorSizerLine2->Add(monitorHorizSyncText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine2->Add(monitorHorizSync, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine2->Add(monitorVertRefreshText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
  monitorSizerLine2->Add(monitorVertRefresh, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine2->Add(monitorFreqButton, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorDisplay=new wxTextCtrl(monitorTab, MONDISP, wxEmptyString, wxDefaultPosition, wxSize(150,-1));
  monitorDisplaySizeButton=new wxButton(monitorTab, MONDISPSIZE, _("Get Display Size"));

  monitorSizerLine3->Add(monitorDisplayText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine3->Add(monitorDisplay, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine3->Add(monitorDisplaySizeButton, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorGamma=new wxTextCtrl(monitorTab, MONGAM, wxEmptyString, wxDefaultPosition, wxSize(120,-1));

  monitorSizerLine4->Add(monitorGammaText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
  monitorSizerLine4->Add(monitorGamma, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add Modeline");
  tempList[2]=_("Delete Modeline");
  tempList[3]=_("De/Activate Modeline");

  monitorModelineName=new wxComboBox(monitorTab, MONMODL, _("----------"), wxDefaultPosition, wxSize(145,-1), devListSize, tempList, wxCB_DROPDOWN);
  monitorModelineOption=new wxTextCtrl(monitorTab, MONMODLOPT, wxEmptyString, wxDefaultPosition, wxSize(235,-1));

  monitorSizerLine5->Add(monitorModelineText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine5->Add(monitorModelineName, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine5->Add(monitorModelineOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorOption=new wxComboBox(monitorTab, MONOPT, _("----------"), wxDefaultPosition, wxSize(210,-1), optListSize, optList, wxCB_DROPDOWN);
  monitorOptValue=new wxTextCtrl(monitorTab, MONOVAL, wxEmptyString, wxDefaultPosition, wxSize(185,-1));

  monitorSizerLine6->Add(monitorOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine6->Add(monitorOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorSizerLine6->Add(monitorOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorSizer->Add(monitorSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  monitorSizer->Add(monitorSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  monitorSizer->Add(monitorSizerLine3, 0, wxTOP | wxBOTTOM, 3);
  monitorSizer->Add(monitorSizerLine4, 0, wxTOP | wxBOTTOM, 3);
  monitorSizer->Add(monitorSizerLine5, 0, wxTOP | wxBOTTOM, 3);
  monitorSizer->Add(monitorSizerLine6, 0, wxTOP | wxBOTTOM, 3);

  monitorModelineGeneratorBox=new wxStaticBox(monitorTab, -1, _("Modeline Generator"));
  wxStaticBoxSizer *monitorModelineGeneratorSizer=new wxStaticBoxSizer(monitorModelineGeneratorBox, wxVERTICAL);
  wxBoxSizer *monitorModelineGeneratorSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *monitorModelineGeneratorSizerLine2=new wxBoxSizer(wxHORIZONTAL);

  monitorModelineGeneratorXResText=new wxStaticText(monitorTab, -1, _("X"));
  monitorModelineGeneratorYResText=new wxStaticText(monitorTab, -1, _("Y"));
  monitorModelineGeneratorFreqText=new wxStaticText(monitorTab, -1, _("Refresh"));

  monitorModelineGeneratorXRes=new wxTextCtrl(monitorTab, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(45,-1));
  monitorModelineGeneratorYRes=new wxTextCtrl(monitorTab, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(45,-1));
  monitorModelineGeneratorFreq=new wxTextCtrl(monitorTab, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(40,-1));
  monitorModelineGeneratorButton=new wxButton(monitorTab, MONMODLGEN, _("Generate"));
  monitorModelineGeneratorAddButton=new wxButton(monitorTab, MONMODLADD, _("Add"));

  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorXResText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorXRes, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorYResText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorYRes, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorFreqText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorFreq, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorButton, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine1->Add(monitorModelineGeneratorAddButton, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorModelineGeneratorOutput1=new wxTextCtrl(monitorTab, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(120,-1), wxTE_READONLY);
  monitorModelineGeneratorOutput2=new wxTextCtrl(monitorTab, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(320,-1), wxTE_READONLY);

  monitorModelineGeneratorSizerLine2->Add(monitorModelineGeneratorOutput1, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  monitorModelineGeneratorSizerLine2->Add(monitorModelineGeneratorOutput2, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  monitorModelineGeneratorSizer->Add(monitorModelineGeneratorSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  monitorModelineGeneratorSizer->Add(monitorModelineGeneratorSizerLine2, 0, wxTOP | wxBOTTOM, 3);

  // section INPUTDEVICE elements
  inputDeviceBox=new wxStaticBox(inputDeviceTab, -1, _("Input Device"));
  wxStaticBoxSizer *inputDeviceSizer=new wxStaticBoxSizer(inputDeviceBox, wxVERTICAL);
  wxBoxSizer *inputDeviceSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *inputDeviceSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *inputDeviceSizerLine3=new wxBoxSizer(wxHORIZONTAL);

  inputDeviceDriverText=new wxStaticText(inputDeviceTab, -1, _("Driver"));
  inputDeviceOptionText=new wxStaticText(inputDeviceTab, -1, _("Option"));

  tempList[1]=_("Add Input Device");
  tempList[2]=_("Delete Input Device");
  tempList[3]=_("Copy Input Device");

  inputDeviceIdentifier=new wxComboBox(inputDeviceTab, IDEVID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  inputDeviceSizerLine1->Add(inputDeviceIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  inputDeviceDriver=new wxComboBox(inputDeviceTab, IDEVDRI, wxEmptyString, wxDefaultPosition, wxSize(140,-1), 0, tempList, wxCB_DROPDOWN | wxCB_SORT);

  inputDeviceSizerLine2->Add(inputDeviceDriverText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  inputDeviceSizerLine2->Add(inputDeviceDriver, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add empty Option");
  tempList[2]=_("Manage Options...");
  tempList[3]=_("Delete Option");
  tempList[4]=_("De/Activate Option");

  inputDeviceOption=new wxComboBox(inputDeviceTab, IDEVOPT, _("----------"), wxDefaultPosition, wxSize(210,-1), devListSize+1, tempList, wxCB_DROPDOWN);
  inputDeviceOptValue=new wxComboBox(inputDeviceTab, IDEVOVAL, wxEmptyString, wxDefaultPosition, wxSize(185,-1), 0, tempList, wxCB_DROPDOWN | wxCB_SORT);
  inputDeviceOption->SetToolTip(new wxToolTip(wxEmptyString));
  inputDeviceOption->GetToolTip()->Enable(false);

  inputDeviceSizerLine3->Add(inputDeviceOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  inputDeviceSizerLine3->Add(inputDeviceOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  inputDeviceSizerLine3->Add(inputDeviceOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  inputDeviceSizer->Add(inputDeviceSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  inputDeviceSizer->Add(inputDeviceSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  inputDeviceSizer->Add(inputDeviceSizerLine3, 0, wxTOP | wxBOTTOM, 3);

  // section SCREEN elements
  screenBox=new wxStaticBox(screenTab, -1, _("Screen"));
  wxStaticBoxSizer *screenSizer=new wxStaticBoxSizer(screenBox, wxVERTICAL);
  wxBoxSizer *screenSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine3=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine4=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine5=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine6=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine7=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine8=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *screenSizerLine9=new wxBoxSizer(wxHORIZONTAL);

  tempList[1]=_("Add Screen");
  tempList[2]=_("Delete Screen");
  tempList[3]=_("Copy Screen");

  screenDeviceText=new wxStaticText(screenTab, -1, _("Device"));
  screenMonitorText=new wxStaticText(screenTab, -1, _("Monitor"));
  screenDepthText=new wxStaticText(screenTab, -1, _("Default Depth"));
  screenOptionText=new wxStaticText(screenTab, -1, _("Option"));

  screenIdentifier=new wxComboBox(screenTab, SCRID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  screenSizerLine1->Add(screenIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenDevice=new wxChoice(screenTab, SCRDEV, wxDefaultPosition, wxSize(359,-1), 0, tempList);

  screenSizerLine2->Add(screenDeviceText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine2->Add(screenDevice, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenMonitor=new wxChoice(screenTab, SCRMON, wxDefaultPosition, wxSize(359,-1), 0, tempList);

  screenSizerLine3->Add(screenMonitorText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine3->Add(screenMonitor, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenDepth=new wxTextCtrl(screenTab, SCRDEP, wxEmptyString, wxDefaultPosition, wxSize(50,-1));

  screenSizerLine4->Add(screenDepthText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine4->Add(screenDepth, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenOption=new wxComboBox(screenTab, SCROPT, _("----------"), wxDefaultPosition, wxSize(210,-1), 4, optList, wxCB_DROPDOWN);
  screenOptValue=new wxTextCtrl(screenTab, SCROVAL, wxEmptyString, wxDefaultPosition, wxSize(185,-1));

  screenSizerLine5->Add(screenOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine5->Add(screenOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine5->Add(screenOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenDisplayDepthText=new wxStaticText(screenTab, -1, _("Display Depth"));
  screenDisplayViewportText=new wxStaticText(screenTab, -1, _("Viewport"));
  screenDisplayVirtualText=new wxStaticText(screenTab, -1, _("Virtual Size"));
  screenDisplayModesText=new wxStaticText(screenTab, -1, _("Mode"));
  screenDisplayOptionText=new wxStaticText(screenTab, -1, _("Option"));

  tempList[1]=_("Add Display");
  tempList[2]=_("Delete Display");

  screenDisplayDepth=new wxComboBox(screenTab, SCRDISP, _("----------"), wxDefaultPosition, wxSize(150,-1), devListSize-1, tempList, wxCB_DROPDOWN);

  screenSizerLine6->Add(screenDisplayDepthText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine6->Add(screenDisplayDepth, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenDisplayViewport=new wxTextCtrl(screenTab, SCRDISPVP, wxEmptyString, wxDefaultPosition, wxSize(110,-1));
  screenDisplayVirtual=new wxTextCtrl(screenTab, SCRDISPVIR, wxEmptyString, wxDefaultPosition, wxSize(110,-1));

  screenSizerLine7->Add(15,1);
  screenSizerLine7->Add(screenDisplayViewportText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine7->Add(screenDisplayViewport, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine7->Add(screenDisplayVirtualText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine7->Add(screenDisplayVirtual, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add Mode");
  tempList[2]=_("Delete Mode");

  screenDisplayModes=new wxComboBox(screenTab, SCRDISPMOD, _("----------"), wxDefaultPosition, wxSize(180,-1), devListSize-1, tempList, wxCB_DROPDOWN);

  screenSizerLine8->Add(15,1);
  screenSizerLine8->Add(screenDisplayModesText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine8->Add(screenDisplayModes, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenDisplayOption=new wxComboBox(screenTab, SCRDISPOPT, _("----------"), wxDefaultPosition, wxSize(167,-1), optListSize, optList, wxCB_DROPDOWN);
  screenDisplayOptValue=new wxTextCtrl(screenTab, SCRDISPOVAL, wxEmptyString, wxDefaultPosition, wxSize(210,-1));

  screenSizerLine9->Add(15,1);
  screenSizerLine9->Add(screenDisplayOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine9->Add(screenDisplayOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  screenSizerLine9->Add(screenDisplayOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  screenSizer->Add(screenSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine3, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine4, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine5, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine6, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine7, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine8, 0, wxTOP | wxBOTTOM, 3);
  screenSizer->Add(screenSizerLine9, 0, wxTOP | wxBOTTOM, 3);

  // section SERVERLAYOUT elements
  serverLayoutBox=new wxStaticBox(serverTab, -1, _("Server Layout"));
  wxStaticBoxSizer *serverLayoutSizer=new wxStaticBoxSizer(serverLayoutBox, wxVERTICAL);
  wxBoxSizer *serverLayoutSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *serverLayoutSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *serverLayoutSizerLine3=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *serverLayoutSizerLine4=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *serverLayoutSizerLine5=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *serverLayoutSizerLine6=new wxBoxSizer(wxHORIZONTAL);

  serverLayoutScreenNumberText=new wxStaticText(serverTab, -1, _("Number"));
  serverLayoutScreenPositionText=new wxStaticText(serverTab, -1, _("Position"));
  serverLayoutOptionText=new wxStaticText(serverTab, -1, _("Option"));
  serverLayoutInputDeviceOptionText=new wxStaticText(serverTab, -1, _("Option"));

  tempList[1]=_("Add Server Layout");
  tempList[2]=_("Delete Server Layout");
  tempList[3]=_("Copy Server Layout");

  serverLayoutIdentifier=new wxComboBox(serverTab, SERLID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  serverLayoutSizerLine1->Add(serverLayoutIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add Screen");
  tempList[2]=_("Delete Screen");

  serverLayoutScreen=new wxComboBox(serverTab, SERLSCR, _("----------"), wxDefaultPosition, wxSize(175,-1), devListSize-1, tempList, wxCB_DROPDOWN | wxCB_READONLY);
  serverLayoutScreenNumber=new wxTextCtrl(serverTab, SERLSCRNUM, wxEmptyString, wxDefaultPosition, wxSize(30,-1));
  serverLayoutScreen1=new wxChoice(serverTab, SERLSCR1, wxDefaultPosition, wxSize(170,-1), 0, tempList);

  serverLayoutSizerLine2->Add(serverLayoutScreen, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine2->Add(serverLayoutScreen1, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine2->Add(serverLayoutScreenNumberText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine2->Add(serverLayoutScreenNumber, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  serverLayoutScreenPosition=new wxTextCtrl(serverTab, SERLSCRPOS, wxEmptyString, wxDefaultPosition, wxSize(104,-1));
  serverLayoutScreen2=new wxChoice(serverTab, SERLSCR2, wxDefaultPosition, wxSize(170,-1), 0, tempList);

  serverLayoutSizerLine3->Add(15, 1);
  serverLayoutSizerLine3->Add(serverLayoutScreenPositionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine3->Add(serverLayoutScreenPosition, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine3->Add(serverLayoutScreen2, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add Input Device");
  tempList[2]=_("Delete Input Device");

  serverLayoutInputDevice=new wxComboBox(serverTab, SERLIDEV, _("----------"), wxDefaultPosition, wxSize(175,-1), devListSize-1, tempList, wxCB_DROPDOWN | wxCB_READONLY);
  serverLayoutInputDevice1=new wxChoice(serverTab, SERLIDEV1, wxDefaultPosition, wxSize(170,-1), 3, tempList);

  serverLayoutSizerLine4->Add(serverLayoutInputDevice, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine4->Add(serverLayoutInputDevice1, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  serverLayoutInputDeviceOption=new wxComboBox(serverTab, SERLIDEVOPT, _("----------"), wxDefaultPosition, wxSize(210,-1), optListSize-1, optList, wxCB_DROPDOWN);
  // serverLayoutInputDeviceOptValue=new wxTextCtrl(serverTab, -1, wxEmptyString, wxDefaultPosition, wxSize(120,-1));

  serverLayoutSizerLine5->Add(15,1);
  serverLayoutSizerLine5->Add(serverLayoutInputDeviceOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine5->Add(serverLayoutInputDeviceOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  // serverLayoutSizerLine5->Add(serverLayoutInputDeviceOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  serverLayoutOption=new wxComboBox(serverTab, SERLOPT, _("----------"), wxDefaultPosition, wxSize(210,-1), optListSize-1, optList, wxCB_DROPDOWN);
  serverLayoutOptValue=new wxTextCtrl(serverTab, SERLOVAL, wxEmptyString, wxDefaultPosition, wxSize(185,-1));

  serverLayoutSizerLine6->Add(serverLayoutOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine6->Add(serverLayoutOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverLayoutSizerLine6->Add(serverLayoutOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  serverLayoutSizer->Add(serverLayoutSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  serverLayoutSizer->Add(serverLayoutSizerLine6, 0, wxTOP | wxBOTTOM, 3);
  serverLayoutSizer->Add(serverLayoutSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  serverLayoutSizer->Add(serverLayoutSizerLine3, 0, wxTOP | wxBOTTOM, 3);
  serverLayoutSizer->Add(serverLayoutSizerLine4, 0, wxTOP | wxBOTTOM, 3);
  serverLayoutSizer->Add(serverLayoutSizerLine5, 0, wxTOP | wxBOTTOM, 3);

  // section SERVERFLAGS elements
  serverFlagBox=new wxStaticBox(serverTab, -1, _("Server Flags"));
  wxStaticBoxSizer *serverFlagSizer=new wxStaticBoxSizer(serverFlagBox, wxVERTICAL);
  wxBoxSizer *serverFlagSizerLine1=new wxBoxSizer(wxHORIZONTAL);

  tempList[1]=_("Add Server Flag");
  tempList[2]=_("Delete Server Flag");
  tempList[3]=_("De/Activate Server Flag");

  serverFlagsOption=new wxComboBox(serverTab, SERFOPT, _("----------"), wxDefaultPosition, wxSize(234,-1), devListSize, tempList, wxCB_DROPDOWN);
  serverFlagsOptValue=new wxTextCtrl(serverTab, SERFOVAL, wxEmptyString, wxDefaultPosition, wxSize(210,-1));

  serverFlagSizerLine1->Add(serverFlagsOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  serverFlagSizerLine1->Add(serverFlagsOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  serverFlagSizer->Add(serverFlagSizerLine1, 0, wxTOP | wxBOTTOM, 3);

  // section EXTENSION elements
  extensionBox=new wxStaticBox(otherSectionTab, -1, _("Extensions"));
  wxStaticBoxSizer *extensionSizer=new wxStaticBoxSizer(extensionBox, wxVERTICAL);
  wxBoxSizer *extensionSizerLine1=new wxBoxSizer(wxHORIZONTAL);

  tempList[1]=_("Add Extension");
  tempList[2]=_("Delete Extension");
  tempList[3]=_("De/Activate Extension");

  extensionOption=new wxComboBox(otherSectionTab, EXTOPT, _("----------"), wxDefaultPosition, wxSize(234,-1), devListSize, tempList, wxCB_DROPDOWN);
  extensionOptValue=new wxTextCtrl(otherSectionTab, EXTOVAL, wxEmptyString, wxDefaultPosition, wxSize(210,-1));

  extensionSizerLine1->Add(extensionOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  extensionSizerLine1->Add(extensionOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  extensionSizer->Add(extensionSizerLine1, 0, wxTOP | wxBOTTOM, 3);

  // section MODULE elements
  moduleBox=new wxStaticBox(otherSectionTab, -1, _("Modules"));
  wxStaticBoxSizer *moduleSizer=new wxStaticBoxSizer(moduleBox, wxVERTICAL);
  wxBoxSizer *moduleSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *moduleSizerLine2=new wxBoxSizer(wxHORIZONTAL);

  tempList[1]=_("Add Module");
  tempList[2]=_("Delete Module");
  tempList[3]=_("De/Activate Module");

  moduleOptionText=new wxStaticText(otherSectionTab, -1, _("Option"));

  moduleIdentifier=new wxComboBox(otherSectionTab, MODID, _("----------"), wxDefaultPosition, wxSize(450,-1), devListSize, tempList, wxCB_DROPDOWN);

  moduleSizerLine1->Add(moduleIdentifier, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  moduleOption=new wxComboBox(otherSectionTab, MODOPT, _("----------"), wxDefaultPosition, wxSize(210,-1), optListSize, optList, wxCB_DROPDOWN);
  moduleOptValue=new wxTextCtrl(otherSectionTab, MODOVAL, wxEmptyString, wxDefaultPosition, wxSize(185,-1));

  moduleSizerLine2->Add(moduleOptionText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  moduleSizerLine2->Add(moduleOption, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  moduleSizerLine2->Add(moduleOptValue, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  moduleSizer->Add(moduleSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  moduleSizer->Add(moduleSizerLine2, 0, wxTOP | wxBOTTOM, 3);

  // section FILES elements
  filesBox=new wxStaticBox(otherSectionTab, -1, _("Files"));
  wxStaticBoxSizer *fileSizer=new wxStaticBoxSizer(filesBox, wxVERTICAL);
  wxBoxSizer *fileSizerLine1=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *fileSizerLine2=new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *fileSizerLine3=new wxBoxSizer(wxHORIZONTAL);

  tempList[1]=_("Add Font Path");
  tempList[2]=_("Delete Font Path");
  tempList[3]=_("De/Activate Font Path");

  fontPathText=new wxStaticText(otherSectionTab, -1, _("Font Path"));
  fontPath=new wxComboBox(otherSectionTab, FONTPATH, _("----------"), wxDefaultPosition, wxSize(375,-1), devListSize, tempList, wxCB_DROPDOWN);

  fileSizerLine1->Add(fontPathText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  fileSizerLine1->Add(fontPath, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add Module Path");
  tempList[2]=_("Delete Module Path");
  tempList[3]=_("De/Activate Module Path");

  modulePathText=new wxStaticText(otherSectionTab, -1, _("Module Path"));
  modulePath=new wxComboBox(otherSectionTab, MODPATH, _("----------"), wxDefaultPosition, wxSize(367,-1), devListSize, tempList, wxCB_DROPDOWN);

  fileSizerLine2->Add(modulePathText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  fileSizerLine2->Add(modulePath, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  tempList[1]=_("Add RGB Path");
  tempList[2]=_("Delete RGB Path");
  tempList[3]=_("De/Activate RGB Path");

  rgbPathText=new wxStaticText(otherSectionTab, -1, _("RGB Path"));
  rgbPath=new wxComboBox(otherSectionTab, RGBPATH, _("----------"), wxDefaultPosition, wxSize(365,-1), devListSize, tempList, wxCB_DROPDOWN);

  fileSizerLine3->Add(rgbPathText, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);
  fileSizerLine3->Add(rgbPath, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  fileSizer->Add(fileSizerLine1, 0, wxTOP | wxBOTTOM, 3);
  fileSizer->Add(fileSizerLine2, 0, wxTOP | wxBOTTOM, 3);
  fileSizer->Add(fileSizerLine3, 0, wxTOP | wxBOTTOM, 3);

  // section DRI elements
  driBox=new wxStaticBox(otherSectionTab, -1, _("DRI"));
  wxStaticBoxSizer *driSizer=new wxStaticBoxSizer(driBox, wxVERTICAL);
  wxBoxSizer *driSizerLine1=new wxBoxSizer(wxHORIZONTAL);

  driMode=new wxTextCtrl(otherSectionTab, DRIMOD, wxEmptyString, wxDefaultPosition, wxSize(80,-1));

  driSizerLine1->Add(driMode, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

  driSizer->Add(driSizerLine1, 0, wxTOP | wxBOTTOM, 3);

  // global sizer

  deviceTabSizer->Add(deviceSizer, 0, wxALL , 3);
  monitorTabSizer->Add(monitorSizer, 0, wxALL , 3);
  monitorTabSizer->Add(monitorModelineGeneratorSizer, 0, wxALL , 3);
  inputDeviceTabSizer->Add(inputDeviceSizer, 0, wxALL , 3);
  screenTabSizer->Add(screenSizer, 0, wxALL , 3);
  serverTabSizer->Add(serverLayoutSizer, 0, wxALL , 3);
  serverTabSizer->Add(serverFlagSizer, 0, wxALL , 3);
  otherSectionTabSizer->Add(extensionSizer, 0, wxALL , 3);
  otherSectionTabSizer->Add(moduleSizer, 0, wxALL , 3);
  otherSectionTabSizer->Add(fileSizer, 0, wxALL , 3);
  otherSectionTabSizer->Add(driSizer, 0, wxALL , 3);

  deviceTab->SetSizerAndFit(deviceTabSizer);
  monitorTab->SetSizerAndFit(monitorTabSizer);
  inputDeviceTab->SetSizerAndFit(inputDeviceTabSizer);
  screenTab->SetSizerAndFit(screenTabSizer);
  serverTab->SetSizerAndFit(serverTabSizer);
  otherSectionTab->SetSizerAndFit(otherSectionTabSizer);

  layout->AddPage(deviceTab, _("Device"));              // add device panel to the notebook
  layout->AddPage(monitorTab, _("Monitor"));              // add monitor panel to the notebook
  layout->AddPage(inputDeviceTab, _("Input Device"));         // add input device panel to the notebook
  layout->AddPage(screenTab, _("Screen"));               // add screen panel to the notebook
  layout->AddPage(serverTab, _("Server"));               // add server panel to the notebook
  layout->AddPage(otherSectionTab, _("Other Options"));              // add other options panel to the notebook

  warningText=new wxStaticText(this, -1, _("Warning: Improper use of this tool can cause damage to your\nxorg.conf and may not start your Xserver.\nYou must restart X to take the changes effect."),
                                            wxDefaultPosition, wxSize(480,65), wxALIGN_CENTRE | wxST_NO_AUTORESIZE);

  gridSizer->Add(layout);            // add notebook layout
  gridSizer->Add(warningText, 0, wxTOP | wxBOTTOM | wxALIGN_CENTER_HORIZONTAL, 3);

  // Settings

  // open xorg.conf at start if possible
  settings.SetCurrentDir("/etc/X11");
  settings.SetCurrentFilename("xorg.conf");
  settings.SetLoadLastFile(true);
  settings.SetSaveSettings(true);
  settings.SetShowSplash(true);
  settings.SetShowBlank(true);
  settings.SetRememberTest(true);
  settings.SetLanguage(wxGetApp().GetLanguage());

  settings.SetProgDir((const char*)wxGetCwd().mb_str());

  wxString name=wxGetCwd()+_T("/options");

  if (!wxDirExists(name))  // option directory does not exists
  {
    name=wxGetCwd();
  }
  settings.SetOptionDir((const char*)name.mb_str());

  if (wxGetEnv(_T("HOME"), NULL))
  {  // HOME is defined
    name=wxGetenv(_T("HOME"));
    name+=_T("/.xorg-edit");
    debugFile=wxGetenv(_T("HOME"));
    debugFile+=_T("/.xorg-edit-log");
  }
  else
  {
    name=wxGetCwd()+_T("/.xorg-edit");
    debugFile=wxGetCwd()+_T("/.xorg-edit-log");
  }

  if (settings.Load((const char*)name.mb_str()))
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  if (wxGetApp().GetLanguage()!=settings.GetLanguage())
  {
    wxGetApp().SetLanguage(settings.GetLanguage());
    // CreateGUIControls();
  }

  // Load driver lists
  if (LoadDrivers())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  if (CreateGUIControls())   // create all gui controls, like text fields, menus, buttons
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  CreateStatusBar(); // create status bar
  SetSizerAndFit(gridSizer);         // sets the sizer

  if (!settings.GetLoadLastFile())
  {
    settings.SetCurrentDir((const char*)wxGetCwd().mb_str());
    settings.SetCurrentFilename("");
    EnableMenuItems(false);
  }
  else
  {
    bool leave;
    bool error=false;
    do
    {
      leave=true;
      if (xorg.Load(settings.GetCurrentFile(), error))
      {
        error=true;
        leave=false;
        MyMessage::Append(funcName);
        if (ShowError())
        {
          settings.SetCurrentDir((const char*)wxGetCwd().mb_str());
          settings.SetCurrentFilename("");
          EnableMenuItems(false);
          leave=true;
          break;
        }
      }
      else
      {
        if (ConvertXorg())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        EnableMenuItems(true);
      }
    } while (!leave);
  }

  if (SetFrame())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
  SetSaved(true);
}

const bool GuiFrame::CreateBackup(void) const
{  // write into debug file

  MyString funcName("GuiFrame::CreateBackup");
  WriteDebug(funcName);

  wxString filename(wxString::FromAscii(settings.GetCurrentFile().data()));
  wxString date(wxDateTime::Today().FormatDate());

  // replace slashes with dots
  // this is necessary because of possible slashes
  // in English date format
  date.Replace(_T("/"), _T("."));

  wxString backupFilename(filename+_T("_")+date+_T("_")+wxDateTime::Now().FormatTime());

  if ( wxFile::Exists(filename) )
  {
    // create copy of file
    if ( !wxCopyFile( filename, backupFilename))
    {
      MyMessage::Set( ERROR_BACKUP_FILE, funcName, MyString(backupFilename.mb_str()) );
      return(true);
    }
  }

  return(false);
}

void GuiFrame::WriteDebug(const MyString& str, const bool clear) const
{  // write into debug file
  if (debug)
  {
    wxFile dFile;
    if (clear) dFile.Open(debugFile, wxFile::write);
    else dFile.Open(debugFile, wxFile::write_append);
    if (dFile.IsOpened())
    {
      dFile.Write(wxDateTime::Now().FormatTime()+_T(" > ")+wxString::FromAscii(str.data())+_T("\n"));
      dFile.Close();
    }
  }
}

void GuiFrame::WriteDebug(const wxString& str, const bool clear) const
{  // write into debug file
  if (debug)
  {
    wxFile dFile;
    if (clear) dFile.Open(debugFile, wxFile::write);
    else dFile.Open(debugFile, wxFile::write_append);
    if (dFile.IsOpened())
    {
      dFile.Write(wxDateTime::Now().FormatTime()+_T(" > ")+str+_T("\n"));
      dFile.Close();
    }
  }
}

GuiFrame::~GuiFrame()  // destructor
{
  // TODO: check if saved
  MyString funcName("GuiFrame::~GuiFrame");
  WriteDebug(funcName);

  if (settings.GetSaveSettings())
  {
    if (settings.Save())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
}

void GuiFrame::EnableMenuItems(const bool enable)
{
  MyString funcName("GuiFrame::EnableMenuItems");
  WriteDebug(funcName);

  layout->Enable(enable);
  fileMenu->Enable(SAVEXORG, enable);
  fileMenu->Enable(SAVEAS, enable);
  fileMenu->Enable(CLOSEXORG, enable);
  fileMenu->Enable(RESTORE, enable);
  extrasMenu->Enable(SHOWXORG, enable);
  extrasMenu->Enable(TESTXORG, enable);
  extrasMenu->Enable(CHECKXORG, enable);
}

void GuiFrame::ShowWarning(void) const
{  // show warning message

  MyString funcName("GuiFrame::ShowWarning");
  WriteDebug(funcName);

  if (!settings.GetShowSplash()) return;  // do not show splash

  wxString line;

  line << _("Improper use of this tool can cause damage to your xorg.conf and may not start your Xserver. ")
       << _("You then won't have a mouse, menus or any other graphical feature to undo the changes. ")
       << _("Please see readme.txt, section \"Troubleshooting\" to get X up again _before_ this happens.\n\n")
       << _("The author is not responsible for any damage caused by using this tool!");
  wxMessageBox(line, _("WARNING"), wxICON_EXCLAMATION);

}

const bool GuiFrame::ShowError(void) const
{  // show error message

  MyString funcName("GuiFrame::ShowError");
  WriteDebug(funcName);

  wxString message;
  wxString error;
  wxString title;
  wxString stack;
  wxString file;
  wxString line;

  switch (MyMessage::Get())  // check error ID
  {
    case ERROR_INDEX_OUT_OF_RANGE:
      error=_("Index out of range.");
      break;
    case ERROR_WRITE_FILE:
      error=_("Could not open file for writing.");
      file=wxString::FromAscii(MyMessage::GetFile().data());
      break;
    case ERROR_BACKUP_FILE:
      error=_("Could not create backup file.");
      file=wxString::FromAscii(MyMessage::GetFile().data());
      break;
    case ERROR_READ_FILE:
      error=_("Could not open file for reading.");
      file=wxString::FromAscii(MyMessage::GetFile().data());
      break;
    case ERROR_DIVISION_BY_ZERO:
      error=_("Division by zero!");
      break;
    case ERROR_UNKNOWN_CASE_IN_SWITCH:
      error=_("Unknown case in switch-statement!");
      break;
    case ERROR_UNKNOWN_CASE_IN_IF:
      error=_("Unknown case in if-statement!");
      break;
    case ERROR_POINTER_IS_NULL:
      error=_("Pointer is NULL!");
      break;
    case ERROR_CHECK_EMPTY_DEVICE_ID:
      error=_("Device identifier must not be empty!");
      break;
    case ERROR_CHECK_SAME_DEVICE_ID:
      error=_("Device identifier already exists!");
      break;
    case ERROR_CHECK_EMPTY_DEVICE_DRIVER:
      error=_("Device driver must not be empty!");
      break;
    case ERROR_CHECK_DEVICE_SCREEN_NUMBER:
      error=_("Device screen must be a non-negative integer!");
      break;
    case ERROR_CHECK_DEVICE_VIDEORAM:
      error=_("Device video ram must be a non-negative integer!");
      break;
    case ERROR_CHECK_EMPTY_DEVICE_OPTION:
      error=_("Device option must not be empty!");
      break;
    case ERROR_CHECK_SAME_DEVICE_OPTION:
      error=_("Device option already exists!");
      break;
    case ERROR_CHECK_DEVICE_OPTION_BOOL:
      error=_("Device option must be boolean!");
      break;
    case ERROR_CHECK_DEVICE_OPTION_INT:
      error=_("Device option must be integer!");
      break;
    case ERROR_CHECK_DEVICE_OPTION_LIST:
      error=_("Device option must be in argument list!");
      break;
    case ERROR_CHECK_EMPTY_MONITOR_ID:
      error=_("Monitor identifier must not be empty!");
      break;
    case ERROR_CHECK_SAME_MONITOR_ID:
      error=_("Monitor identifier already exists!");
      break;
    case ERROR_CHECK_MONITOR_SIZE:
      error=_("Monitor display size wrongly formatted!");
      break;
    case ERROR_CHECK_MONITOR_GAMMA:
      error=_("Monitor gamma must be double between 0.0 and 10.0!");
      break;
    case ERROR_CHECK_EMPTY_MONITOR_OPTION:
      error=_("Monitor option must not be empty!");
      break;
    case ERROR_CHECK_SAME_MONITOR_OPTION:
      error=_("Monitor option already exists!");
      break;
    case ERROR_CHECK_EMPTY_MONITOR_MODELINE:
      error=_("Monitor modeline must not be empty!");
      break;
    case ERROR_CHECK_SAME_MONITOR_MODELINE:
      error=_("Monitor modeline already exists!");
      break;
    case ERROR_CHECK_MONITOR_MODELINE_INT:
      error=_("First eight monitor modeline entries must be integer!");
      break;
    case ERROR_CHECK_MONITOR_MODELINE_NUMBER:
      error=_("Monitor modeline must have 9 entries at least!");
      break;
    case ERROR_CHECK_EMPTY_INPUTDEVICE_ID:
      error=_("Input device identifier must not be empty!");
      break;
    case ERROR_CHECK_SAME_INPUTDEVICE_ID:
       error=_("Input device identifier already exists!");
      break;
    case ERROR_CHECK_EMPTY_INPUTDEVICE_DRIVER:
      error=_("Input device driver must not be empty!");
      break;
    case ERROR_CHECK_EMPTY_INPUTDEVICE_OPTION:
      error=_("Input device option must not be empty!");
      break;
    case ERROR_CHECK_SAME_INPUTDEVICE_OPTION:
      error=_("Input device option already exists!");
      break;
    case ERROR_CHECK_INPUTDEVICE_OPTION_BOOL:
      error=_("Input device option must be boolean!");
      break;
    case ERROR_CHECK_INPUTDEVICE_OPTION_INT:
      error=_("Input device option must be integer!");
      break;
    case ERROR_CHECK_INPUTDEVICE_OPTION_LIST:
      error=_("Input device option must be in argument list!");
      break;
    case ERROR_CHECK_EMPTY_SCREEN_ID:
      error=_("Screen identifier must not be empty!");
      break;
    case ERROR_CHECK_SAME_SCREEN_ID:
      error=_("Screen identifier already exists!");
      break;
    case ERROR_CHECK_EMPTY_SCREEN_DEPTH:
      error=_("Screen depth must not be empty!");
      break;
    case ERROR_CHECK_SCREEN_DEPTH:
      error=_("Screen depth must be a non-negative integer!");
      break;
    case ERROR_CHECK_EMPTY_SCREEN_OPTION:
      error=_("Screen option must not be empty!");
      break;
    case ERROR_CHECK_SAME_SCREEN_OPTION:
      error=_("Screen option already exists!");
      break;
    case ERROR_CHECK_EMPTY_SCREEN_DISPLAY_DEPTH:
      error=_("Screen display depth must not be empty!");
      break;
    case ERROR_CHECK_SCREEN_DISPLAY_DEPTH:
      error=_("Screen display depth must be a non-negative integer!");
      break;
    case ERROR_CHECK_SAME_SCREEN_DISPLAY:
      error=_("Screen display already exists!");
      break;
    case ERROR_CHECK_SCREEN_DISPLAY_VIEWPORT:
      error=_("Screen display viewport must be a non-negative integer!");
      break;
    case ERROR_CHECK_SCREEN_DISPLAY_VIRTUAL:
      error=_("Screen display virtual size must be a non-negative integer!");
      break;
    case ERROR_CHECK_EMPTY_SCREEN_DISPLAY_OPTION:
      error=_("Screen display option must not be empty!");
      break;
    case ERROR_CHECK_SAME_SCREEN_DISPLAY_OPTION:
      error=_("Screen display option already exists!");
      break;
    case ERROR_CHECK_EMPTY_SCREEN_DISPLAY_MODE:
      error=_("Screen display mode must not be empty!");
      break;
    case ERROR_CHECK_SAME_SCREEN_DISPLAY_MODE:
      error=_("Screen display mode already exists!");
      break;
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_ID:
      error=_("Server layout identifier must not be empty!");
      break;
    case ERROR_CHECK_SAME_SERVERLAYOUT_ID:
      error=_("Server layout identifier already exists!");
      break;
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_OPTION:
      error=_("Server layout option must not be empty!");
      break;
    case ERROR_CHECK_SAME_SERVERLAYOUT_OPTION:
      error=_("Server layout option already exists!");
      break;
    case ERROR_CHECK_SAME_SERVERLAYOUT_SCREEN:
      error=_("Screen already used in this server layout. Please select another one!");
      break;
    case ERROR_CHECK_SAME_SERVERLAYOUT_SCREENS:
      error=_("Screen 1 and 2 are the same. Please select another one!");
      break;
    case ERROR_CHECK_SERVERLAYOUT_SCREEN_NUMBER:
      error=_("Server layout screen number must be a non-negative integer!");
      break;
    case ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION:
      error=_("Server layout screen position is wrong!");
      break;
    case ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION_REL:
      error=_("I'm sorry, but the \"relative\" option for screen position is not supported yet!");
      break;
    case ERROR_CHECK_SAME_SERVERLAYOUT_INPUTDEVICE:
      error=_("Input device already used in this server layout. Please select another one!");
      break;
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_INPUTDEVICE_OPTION:
      error=_("Server layout input device option must not be empty!");
      break;
    case ERROR_CHECK_SAME_SERVERLAYOUT_INPUTDEVICE_OPTION:
      error=_("Server layout input device option already exists!");
      break;
    case ERROR_CHECK_EMPTY_SERVERFLAG:
      error=_("Server flag must not be empty!");
      break;
    case ERROR_CHECK_SAME_SERVERFLAG:
      error=_("Server flag already exists!");
      break;
    case ERROR_CHECK_EMPTY_EXTENSION:
      error=_("Extension must not be empty!");
      break;
    case ERROR_CHECK_SAME_EXTENSION:
      error=_("Extension already exists!");
      break;
    case ERROR_CHECK_EMPTY_MODULE:
      error=_("Module name must not be empty!");
      break;
    case ERROR_CHECK_SAME_MODULE:
      error=_("Module name already exists!");
      break;
    case ERROR_CHECK_EMPTY_MODULE_OPTION:
      error=_("Module option must not be empty!");
      break;
    case ERROR_CHECK_SAME_MODULE_OPTION:
      error=_("Module option already exists!");
      break;
    case ERROR_CHECK_EMPTY_FONTPATH:
      error=_("Font path must not be empty!");
      break;
    case ERROR_CHECK_SAME_FONTPATH:
      error=_("Font path already exists!");
      break;
    case ERROR_CHECK_EMPTY_MODULEPATH:
      error=_("Module path must not be empty!");
      break;
    case ERROR_CHECK_SAME_MODULEPATH:
      error=_("Module path already exists!");
      break;
    case ERROR_CHECK_EMPTY_RGBPATH:
      error=_("RGB path must not be empty!");
      break;
    case ERROR_CHECK_SAME_RGBPATH:
      error=_("RGB path already exists!");
      break;
    case ERROR_CHECK_EMPTY_OPTION:
      error=_("Option name must not be empty!");
      break;
    case ERROR_CHECK_SAME_OPTION:
      error=_("Option already exists!");
      break;
    case ERROR_CHECK_EMPTY_DRIVEROPTION:
      error=_("Driver option name must not be empty!");
      break;
    case ERROR_CHECK_SAME_DRIVEROPTION:
      error=_("Driver option already exists!");
      break;
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_SCREEN:
      error=_("Server layout screen must not be empty!");
      break;
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_INPUTDEVICE:
      error=_("Server layout input device must not be empty!");
      break;
    case ERROR_XORG_UNKNOWN:
      error=_("Either the file is not a valid config file or there are some options that could not be recognized.");
      break;
    case ERROR_XORG_NO_SECTION:
      error=_("No section has started.");
      break;
    case ERROR_XORG_NO_SUBSECTION:
      error=_("Subsection has ended without beginning.");
      break;
    case ERROR_XORG_TWO_SECTIONS:
      error=_("Another section has begun without ending the last one.");
      break;
    case ERROR_XORG_NO_DEVICE:
      error=_("Device identifier does not exist in device list.");
      break;
    case ERROR_XORG_NO_SCREEN:
      error=_("Screen identifier does not exist in screen list.");
      break;
    case ERROR_XORG_NO_MONITOR:
      error=_("Monitor identifier does not exist in monitor list.");
      break;
    case ERROR_XORG_NO_INPUTDEVICE:
      error=_("Input device identifier does not exist in input device list.");
      break;
    case ERROR_XORG_NO_MONITORS:
      error=_("No monitors declared. Need at least one...");
      break;
    case ERROR_XORG_NO_DEVICES:
      error=_("No devices declared. Need at least one...");
      break;
    case ERROR_XORG_NO_INPUTDEVICES:
      error=_("No or only one input devices declared. Need at least two...");
      break;
    case ERROR_XORG_NO_SCREENS:
      error=_("No screens declared. Need at least one...");
      break;
    case ERROR_XORG_NO_SERVERLAYOUTS:
      error=_("No server layouts declared. Need at least one...");
      break;
    case ERROR_XORG_EMPTY_ARGUMENT:
      error=_("Argument or Option must not be empty.");
      break;
    case ERROR_XORG_NO_DEVICE_ID:
      error=_("Device identifier is missing." );
      break;
    case ERROR_XORG_NO_DEVICE_DRIVER:
      error=_("Device driver is missing.");
      break;
    case ERROR_XORG_NO_MONITOR_ID:
      error=_("Monitor identifier is missing.");
      break;
    case ERROR_XORG_NO_INPUTDEVICE_ID:
      error=_("Input device identifier is missing.");
      break;
    case ERROR_XORG_NO_INPUTDEVICE_DRIVER:
      error=_("Input device driver is missing.");
      break;
    case ERROR_XORG_NO_SCREEN_ID:
      error=_("Screen identifier is missing.");
      break;
    case ERROR_XORG_NO_SERVERLAYOUT_ID:
      error=_("Server layout identifier is missing.");
      break;
    case ERROR_XORG_NO_MODUL:
      error=_("Module option given without a module.");
      break;
    case ERROR_XORG_SECOND_DRI:
      error=_("Second DRI entry found.");
      break;
    case ERROR_XORG_FILES_SECTION:
      error=_("Error in files section.");
      break;
    case ERROR_XORG_SERVERFLAGS_SECTION:
      error=_("Error in server flags section.");
      break;
    case ERROR_XORG_MODULE_SECTION:
      error=_("Error in modules section.");
      break;
    case ERROR_XORG_INPUTDEVICE_SECTION:
      error=_("Error in input device section.");
      break;
    case ERROR_XORG_DEVICE_SECTION:
       error=_("Error in device section.");
      break;
    case ERROR_XORG_MONITOR_SECTION:
      error=_("Error in monitor section.");
      break;
    case ERROR_XORG_SCREEN_SECTION:
      error=_("Error in screen section.");
      break;
    case ERROR_XORG_SERVERLAYOUT_SECTION:
      error=_("Error in server layout section.");
      break;
    case ERROR_XORG_DRI_SECTION:
       error=_("Error in dri section.");
      break;
    case ERROR_XORG_EXTENSIONS_SECTION:
      error=_("Error in extensions section.");
      break;
    case ERROR_XORG_DISPLAY_SECTION:
      error=_("Error in display subsection.");
      break;
    case ERROR_DRIVERLIST_UNKNOWN:
      error=_("Unrecognized line.");
      break;
    case ERROR_DRIVERLIST_NO_OPTION:
      error=_("Argument outside <option>-tag.");
      break;
    case ERROR_DRIVERLIST_TWO_OPTIONS:
      error=_("Second <option>-tag found without closing </option> before.");
      break;
    case ERROR_DRIVERLIST_OPTION_WITHOUT_SECTION:
      error=_("Option outside <section>-tag.");
      break;
    case ERROR_DRIVERLIST_NO_TAG_END:
      error=_("Line does not end with '>'.");
      break;
    case ERROR_DRIVERLIST_CLOSE_SECTION:
      error=_("</section> missing at end of file.");
      break;
    case ERROR_DRIVERLIST_TWO_DEFAULT:
      error=_("More than one <default>-tag in one option.");
      break;
    case ERROR_DRIVERLIST_NO_DEFAULT_END:
      error=_("</default> missing at end of line.");
      break;
    case ERROR_DRIVERLIST_EMPTY_DEFAULT:
      error=_("<default> is empty.");
      break;
    case ERROR_DRIVERLIST_OPTION_NAME_ERROR:
      error=_("Error in option name assignment.");
      break;
    case ERROR_DRIVERLIST_OPTION_NAME_MISSING:
      error=_("Option name is missing.");
      break;
    case ERROR_DRIVERLIST_VALUE_ERROR:
      error=_("Error in value assignment.");
      break;
    case ERROR_DRIVERLIST_VALUE_VALUE:
      error=_("Error in value assignment: Type must be 'string', 'integer' or 'boolean'.");
      break;
    case ERROR_DRIVERLIST_NO_ARGUMENT_END:
      error=_("</argument> missing at end of line.");
      break;
    case ERROR_DRIVERLIST_EMPTY_ARGUMENT:
      error=_("<argument> is empty.");
      break;
    case ERROR_DRIVERLIST_COMMENT_ERROR:
      error=_("Error in comment assignment.");
      break;
    case ERROR_DRIVERLIST_STRICT_ERROR:
      error=_("Error in strict assignment.");
      break;
    case ERROR_DRIVERLIST_STRICT_VALUE:
      error=_("Error in strict assignment: Type must be 'true' or 'false'.");
      break;
    case ERROR_DRIVERLIST_DESC_ERROR:
      error=_("Error in description assignment.");
      break;
    case ERROR_DRIVERLIST_EMPTY_DESC:
      error=_("<description> is empty.");
      break;
    case ERROR_DRIVERLIST_NO_DESC:
      error=_("No <description>-Tag before.");
      break;
    case ERROR_DRIVERLIST_CLOSE_DESC:
      error=_("No </description>-Tag found.");
      break;
    case ERROR_DRIVERLIST_SECTION_ERROR:
      error=_("Error in section assignment.");
      break;
    case ERROR_DRIVERLIST_SECTION_VALUE:
      error=_("Error in section assignment: Type must be 'general', 'tv' or 'misc'.");
      break;
    case ERROR_DRIVERLIST_NO_SECTION:
      error=_("No <section>-Tag before.");
      break;
    case ERROR_DRIVERLIST_TWO_SECTIONS:
      error=_("Second <section>-tag found without closing </section> before.");
      break;
    case ERROR_DRIVERLIST_SECTION_NAME_ERROR:
      error=_("Error in section name assignment.");
      break;
    case ERROR_DRIVERLIST_SECTION_NAME_MISSING:
      error=_("Section name is missing.");
      break;
    case ERROR_DRIVERLIST_DOUBLE_OPTION:
      error=_("Option in driver list already exists!");
      break;
    default:
      break;
  }

  switch (MyMessage::Get())  // check error ID
  {
    case ERROR_INDEX_OUT_OF_RANGE:
    case ERROR_BACKUP_FILE:
    case ERROR_WRITE_FILE:
    case ERROR_READ_FILE:
    case ERROR_DIVISION_BY_ZERO:
    case ERROR_UNKNOWN_CASE_IN_SWITCH:
    case ERROR_UNKNOWN_CASE_IN_IF:
      title=_("General Error");
      file=wxString::FromAscii(MyMessage::GetFile().data());
      stack=wxString::FromAscii(MyMessage::GetStack().data());
      break;
    case ERROR_CHECK_EMPTY_DEVICE_ID:
    case ERROR_CHECK_SAME_DEVICE_ID:
    case ERROR_CHECK_EMPTY_DEVICE_DRIVER:
    case ERROR_CHECK_DEVICE_SCREEN_NUMBER:
    case ERROR_CHECK_DEVICE_VIDEORAM:
    case ERROR_CHECK_EMPTY_DEVICE_OPTION:
    case ERROR_CHECK_SAME_DEVICE_OPTION:
    case ERROR_CHECK_DEVICE_OPTION_BOOL:
    case ERROR_CHECK_DEVICE_OPTION_INT:
    case ERROR_CHECK_DEVICE_OPTION_LIST:
      title=_("Error in Device");
      break;
    case ERROR_CHECK_EMPTY_MONITOR_ID:
    case ERROR_CHECK_SAME_MONITOR_ID:
    case ERROR_CHECK_MONITOR_SIZE:
    case ERROR_CHECK_MONITOR_GAMMA:
    case ERROR_CHECK_EMPTY_MONITOR_OPTION:
    case ERROR_CHECK_SAME_MONITOR_OPTION:
    case ERROR_CHECK_EMPTY_MONITOR_MODELINE:
    case ERROR_CHECK_SAME_MONITOR_MODELINE:
    case ERROR_CHECK_MONITOR_MODELINE_INT:
    case ERROR_CHECK_MONITOR_MODELINE_NUMBER:
      title=_("Error in Monitor");
      break;
    case ERROR_CHECK_EMPTY_INPUTDEVICE_ID:
    case ERROR_CHECK_SAME_INPUTDEVICE_ID:
    case ERROR_CHECK_EMPTY_INPUTDEVICE_DRIVER:
    case ERROR_CHECK_EMPTY_INPUTDEVICE_OPTION:
    case ERROR_CHECK_SAME_INPUTDEVICE_OPTION:
    case ERROR_CHECK_INPUTDEVICE_OPTION_BOOL:
    case ERROR_CHECK_INPUTDEVICE_OPTION_INT:
    case ERROR_CHECK_INPUTDEVICE_OPTION_LIST:
      title=_("Error in Input Device");
      break;
    case ERROR_CHECK_EMPTY_SCREEN_ID:
    case ERROR_CHECK_SAME_SCREEN_ID:
    case ERROR_CHECK_EMPTY_SCREEN_DEPTH:
    case ERROR_CHECK_SCREEN_DEPTH:
    case ERROR_CHECK_EMPTY_SCREEN_OPTION:
    case ERROR_CHECK_SAME_SCREEN_OPTION:
    case ERROR_CHECK_EMPTY_SCREEN_DISPLAY_DEPTH:
    case ERROR_CHECK_SCREEN_DISPLAY_DEPTH:
    case ERROR_CHECK_SAME_SCREEN_DISPLAY:
    case ERROR_CHECK_SCREEN_DISPLAY_VIEWPORT:
    case ERROR_CHECK_SCREEN_DISPLAY_VIRTUAL:
    case ERROR_CHECK_EMPTY_SCREEN_DISPLAY_OPTION:
    case ERROR_CHECK_SAME_SCREEN_DISPLAY_OPTION:
    case ERROR_CHECK_EMPTY_SCREEN_DISPLAY_MODE:
    case ERROR_CHECK_SAME_SCREEN_DISPLAY_MODE:
      title=_("Error in Screen");
      break;
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_ID:
    case ERROR_CHECK_SAME_SERVERLAYOUT_ID:
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_OPTION:
    case ERROR_CHECK_SAME_SERVERLAYOUT_OPTION:
    case ERROR_CHECK_SAME_SERVERLAYOUT_SCREEN:
    case ERROR_CHECK_SAME_SERVERLAYOUT_SCREENS:
    case ERROR_CHECK_SERVERLAYOUT_SCREEN_NUMBER:
    case ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION:
    case ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION_REL:
    case ERROR_CHECK_SAME_SERVERLAYOUT_INPUTDEVICE:
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_INPUTDEVICE_OPTION:
    case ERROR_CHECK_SAME_SERVERLAYOUT_INPUTDEVICE_OPTION:
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_SCREEN:
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_INPUTDEVICE:
      title=_("Error in Server Layout");
      break;
    case ERROR_CHECK_EMPTY_SERVERFLAG:
    case ERROR_CHECK_SAME_SERVERFLAG:
      title=_("Error in Server Flag");
      break;
    case ERROR_CHECK_EMPTY_EXTENSION:
    case ERROR_CHECK_SAME_EXTENSION:
      title=_("Error in Extension");
      break;
    case ERROR_CHECK_EMPTY_MODULE:
    case ERROR_CHECK_SAME_MODULE:
    case ERROR_CHECK_EMPTY_MODULE_OPTION:
    case ERROR_CHECK_SAME_MODULE_OPTION:
      title=_("Error in Modules");
      break;
    case ERROR_CHECK_EMPTY_FONTPATH:
    case ERROR_CHECK_SAME_FONTPATH:
    case ERROR_CHECK_EMPTY_MODULEPATH:
    case ERROR_CHECK_SAME_MODULEPATH:
    case ERROR_CHECK_EMPTY_RGBPATH:
    case ERROR_CHECK_SAME_RGBPATH:
      title=_("Error in Files");
      break;
    case ERROR_CHECK_EMPTY_OPTION:
    case ERROR_CHECK_SAME_OPTION:
    case ERROR_CHECK_EMPTY_DRIVEROPTION:
    case ERROR_CHECK_SAME_DRIVEROPTION:
      title=_("Error in Options");
      break;
    case ERROR_DRIVERLIST_UNKNOWN:
    case ERROR_DRIVERLIST_NO_OPTION:
    case ERROR_DRIVERLIST_TWO_OPTIONS:
    case ERROR_DRIVERLIST_OPTION_WITHOUT_SECTION:
    case ERROR_DRIVERLIST_NO_TAG_END:
    case ERROR_DRIVERLIST_CLOSE_SECTION:
    case ERROR_DRIVERLIST_TWO_DEFAULT:
    case ERROR_DRIVERLIST_NO_DEFAULT_END:
    case ERROR_DRIVERLIST_EMPTY_DEFAULT:
    case ERROR_DRIVERLIST_OPTION_NAME_ERROR:
    case ERROR_DRIVERLIST_OPTION_NAME_MISSING:
    case ERROR_DRIVERLIST_VALUE_ERROR:
    case ERROR_DRIVERLIST_VALUE_VALUE:
    case ERROR_DRIVERLIST_NO_ARGUMENT_END:
    case ERROR_DRIVERLIST_EMPTY_ARGUMENT:
    case ERROR_DRIVERLIST_COMMENT_ERROR:
    case ERROR_DRIVERLIST_STRICT_ERROR:
    case ERROR_DRIVERLIST_STRICT_VALUE:
    case ERROR_DRIVERLIST_DESC_ERROR:
    case ERROR_DRIVERLIST_EMPTY_DESC:
    case ERROR_DRIVERLIST_NO_DESC:
    case ERROR_DRIVERLIST_CLOSE_DESC:
    case ERROR_DRIVERLIST_SECTION_ERROR:
    case ERROR_DRIVERLIST_SECTION_VALUE:
    case ERROR_DRIVERLIST_NO_SECTION:
    case ERROR_DRIVERLIST_TWO_SECTIONS:
    case ERROR_DRIVERLIST_SECTION_NAME_ERROR:
    case ERROR_DRIVERLIST_SECTION_NAME_MISSING:
    case ERROR_DRIVERLIST_DOUBLE_OPTION:
      title=_("Error in Option File");
      file=wxString::FromAscii(MyMessage::GetFile().data());
      line=wxString::FromAscii(MyMessage::GetLine().data());
      break;
    case ERROR_XORG_UNKNOWN:
    case ERROR_XORG_NO_SECTION:
    case ERROR_XORG_NO_SUBSECTION:
    case ERROR_XORG_TWO_SECTIONS:
    case ERROR_XORG_NO_DEVICE:
    case ERROR_XORG_NO_SCREEN:
    case ERROR_XORG_NO_MONITOR:
    case ERROR_XORG_NO_INPUTDEVICE:
    case ERROR_XORG_NO_MONITORS:
    case ERROR_XORG_NO_DEVICES:
    case ERROR_XORG_NO_INPUTDEVICES:
    case ERROR_XORG_NO_SCREENS:
    case ERROR_XORG_NO_SERVERLAYOUTS:
    case ERROR_XORG_EMPTY_ARGUMENT:
    case ERROR_XORG_NO_DEVICE_ID:
    case ERROR_XORG_NO_DEVICE_DRIVER:
    case ERROR_XORG_NO_MONITOR_ID:
    case ERROR_XORG_NO_INPUTDEVICE_ID:
    case ERROR_XORG_NO_INPUTDEVICE_DRIVER:
    case ERROR_XORG_NO_SCREEN_ID:
    case ERROR_XORG_NO_SERVERLAYOUT_ID:
    case ERROR_XORG_NO_MODUL:
    case ERROR_XORG_SECOND_DRI:
    case ERROR_XORG_FILES_SECTION:
    case ERROR_XORG_SERVERFLAGS_SECTION:
    case ERROR_XORG_MODULE_SECTION:
    case ERROR_XORG_INPUTDEVICE_SECTION:
    case ERROR_XORG_DEVICE_SECTION:
    case ERROR_XORG_MONITOR_SECTION:
    case ERROR_XORG_SCREEN_SECTION:
    case ERROR_XORG_SERVERLAYOUT_SECTION:
    case ERROR_XORG_DRI_SECTION:
    case ERROR_XORG_EXTENSIONS_SECTION:
    case ERROR_XORG_DISPLAY_SECTION:
      title=_("Error in Configuraton File");
      file=wxString::FromAscii(MyMessage::GetFile().data());
      line=wxString::FromAscii(MyMessage::GetLine().data());
      break;
    default: break;
  }

  if (!file.IsEmpty())
  {
    message << _("Error in file \"") << file << _T("\"");
    if (MyMessage::GetLineNumber()>0)
    {
      message << _(" in line number ") << MyMessage::GetLineNumber() << _T(": \n\n\"") << line << _T("\"\n\n");
    }
    else message << _T(".\n\n");
  }

  message << error << _T("\n");

  if (!stack.IsEmpty())
  {
    message << stack << _T("\n\n");
  }

  switch (MyMessage::Get())  // check error ID
  {
    case ERROR_INDEX_OUT_OF_RANGE:
    case ERROR_WRITE_FILE:
    case ERROR_READ_FILE:
    case ERROR_DIVISION_BY_ZERO:
    case ERROR_UNKNOWN_CASE_IN_SWITCH:
    case ERROR_UNKNOWN_CASE_IN_IF:
    case ERROR_CHECK_EMPTY_DEVICE_ID:
    case ERROR_CHECK_SAME_DEVICE_ID:
    case ERROR_CHECK_EMPTY_DEVICE_DRIVER:
    case ERROR_CHECK_DEVICE_SCREEN_NUMBER:
    case ERROR_CHECK_DEVICE_VIDEORAM:
    case ERROR_CHECK_EMPTY_DEVICE_OPTION:
    case ERROR_CHECK_SAME_DEVICE_OPTION:
    case ERROR_CHECK_DEVICE_OPTION_BOOL:
    case ERROR_CHECK_DEVICE_OPTION_INT:
    case ERROR_CHECK_DEVICE_OPTION_LIST:
    case ERROR_CHECK_EMPTY_MONITOR_ID:
    case ERROR_CHECK_SAME_MONITOR_ID:
    case ERROR_CHECK_MONITOR_SIZE:
    case ERROR_CHECK_MONITOR_GAMMA:
    case ERROR_CHECK_EMPTY_MONITOR_OPTION:
    case ERROR_CHECK_SAME_MONITOR_OPTION:
    case ERROR_CHECK_EMPTY_MONITOR_MODELINE:
    case ERROR_CHECK_SAME_MONITOR_MODELINE:
    case ERROR_CHECK_MONITOR_MODELINE_INT:
    case ERROR_CHECK_MONITOR_MODELINE_NUMBER:
    case ERROR_CHECK_EMPTY_INPUTDEVICE_ID:
    case ERROR_CHECK_SAME_INPUTDEVICE_ID:
    case ERROR_CHECK_EMPTY_INPUTDEVICE_DRIVER:
    case ERROR_CHECK_EMPTY_INPUTDEVICE_OPTION:
    case ERROR_CHECK_SAME_INPUTDEVICE_OPTION:
    case ERROR_CHECK_INPUTDEVICE_OPTION_BOOL:
    case ERROR_CHECK_INPUTDEVICE_OPTION_INT:
    case ERROR_CHECK_INPUTDEVICE_OPTION_LIST:
    case ERROR_CHECK_EMPTY_SCREEN_ID:
    case ERROR_CHECK_SAME_SCREEN_ID:
    case ERROR_CHECK_EMPTY_SCREEN_DEPTH:
    case ERROR_CHECK_SCREEN_DEPTH:
    case ERROR_CHECK_EMPTY_SCREEN_OPTION:
    case ERROR_CHECK_SAME_SCREEN_OPTION:
    case ERROR_CHECK_EMPTY_SCREEN_DISPLAY_DEPTH:
    case ERROR_CHECK_SCREEN_DISPLAY_DEPTH:
    case ERROR_CHECK_SAME_SCREEN_DISPLAY:
    case ERROR_CHECK_SCREEN_DISPLAY_VIEWPORT:
    case ERROR_CHECK_SCREEN_DISPLAY_VIRTUAL:
    case ERROR_CHECK_EMPTY_SCREEN_DISPLAY_OPTION:
    case ERROR_CHECK_SAME_SCREEN_DISPLAY_OPTION:
    case ERROR_CHECK_EMPTY_SCREEN_DISPLAY_MODE:
    case ERROR_CHECK_SAME_SCREEN_DISPLAY_MODE:
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_ID:
    case ERROR_CHECK_SAME_SERVERLAYOUT_ID:
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_OPTION:
    case ERROR_CHECK_SAME_SERVERLAYOUT_OPTION:
    case ERROR_CHECK_SAME_SERVERLAYOUT_SCREEN:
    case ERROR_CHECK_SAME_SERVERLAYOUT_SCREENS:
    case ERROR_CHECK_SERVERLAYOUT_SCREEN_NUMBER:
    case ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION:
    case ERROR_CHECK_SERVERLAYOUT_SCREEN_POSITION_REL:
    case ERROR_CHECK_SAME_SERVERLAYOUT_INPUTDEVICE:
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_INPUTDEVICE_OPTION:
    case ERROR_CHECK_SAME_SERVERLAYOUT_INPUTDEVICE_OPTION:
    case ERROR_CHECK_EMPTY_SERVERFLAG:
    case ERROR_CHECK_SAME_SERVERFLAG:
    case ERROR_CHECK_EMPTY_EXTENSION:
    case ERROR_CHECK_SAME_EXTENSION:
    case ERROR_CHECK_EMPTY_MODULE:
    case ERROR_CHECK_SAME_MODULE:
    case ERROR_CHECK_EMPTY_MODULE_OPTION:
    case ERROR_CHECK_SAME_MODULE_OPTION:
    case ERROR_CHECK_EMPTY_FONTPATH:
    case ERROR_CHECK_SAME_FONTPATH:
    case ERROR_CHECK_EMPTY_MODULEPATH:
    case ERROR_CHECK_SAME_MODULEPATH:
    case ERROR_CHECK_EMPTY_RGBPATH:
    case ERROR_CHECK_SAME_RGBPATH:
    case ERROR_CHECK_EMPTY_OPTION:
    case ERROR_CHECK_SAME_OPTION:
    case ERROR_CHECK_EMPTY_DRIVEROPTION:
    case ERROR_CHECK_SAME_DRIVEROPTION:
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_SCREEN:
    case ERROR_CHECK_EMPTY_SERVERLAYOUT_INPUTDEVICE:
      wxMessageBox(message, title, wxICON_ERROR);
      MyMessage::Reset();
      return(true);
      break;
    case ERROR_XORG_UNKNOWN:
    case ERROR_XORG_NO_SECTION:
    case ERROR_XORG_NO_SUBSECTION:
    case ERROR_XORG_TWO_SECTIONS:
    case ERROR_XORG_NO_DEVICE:
    case ERROR_XORG_NO_SCREEN:
    case ERROR_XORG_NO_MONITOR:
    case ERROR_XORG_NO_INPUTDEVICE:
    case ERROR_XORG_NO_MONITOR_ID:
    case ERROR_XORG_NO_INPUTDEVICE_ID:
    case ERROR_XORG_NO_INPUTDEVICE_DRIVER:
    case ERROR_XORG_NO_SCREEN_ID:
    case ERROR_XORG_NO_SERVERLAYOUT_ID:
    case ERROR_XORG_NO_MONITORS:
    case ERROR_XORG_NO_DEVICES:
    case ERROR_XORG_NO_INPUTDEVICES:
    case ERROR_XORG_NO_SCREENS:
    case ERROR_XORG_NO_SERVERLAYOUTS:
    case ERROR_XORG_EMPTY_ARGUMENT:
    case ERROR_XORG_NO_DEVICE_ID:
    case ERROR_XORG_NO_DEVICE_DRIVER:
    case ERROR_XORG_NO_MODUL:
    case ERROR_XORG_SECOND_DRI:
    case ERROR_XORG_FILES_SECTION:
    case ERROR_XORG_SERVERFLAGS_SECTION:
    case ERROR_XORG_MODULE_SECTION:
    case ERROR_XORG_INPUTDEVICE_SECTION:
    case ERROR_XORG_DEVICE_SECTION:
    case ERROR_XORG_MONITOR_SECTION:
    case ERROR_XORG_SCREEN_SECTION:
    case ERROR_XORG_SERVERLAYOUT_SECTION:
    case ERROR_XORG_DRI_SECTION:
    case ERROR_XORG_EXTENSIONS_SECTION:
    case ERROR_XORG_DISPLAY_SECTION:
      message << _T("\n") << _("Should I ignore this/comment this out and go on?") << _("\n(Warning: It is not sure what happens next!)");
      MyMessage::Reset();
      if (wxMessageBox(message, title, wxICON_ERROR | wxYES_NO | wxYES_DEFAULT)==wxYES) return(false);
      else return(true);
      break;
    case ERROR_DRIVERLIST_UNKNOWN:
    case ERROR_DRIVERLIST_NO_OPTION:
    case ERROR_DRIVERLIST_TWO_OPTIONS:
    case ERROR_DRIVERLIST_OPTION_WITHOUT_SECTION:
    case ERROR_DRIVERLIST_NO_TAG_END:
    case ERROR_DRIVERLIST_CLOSE_SECTION:
    case ERROR_DRIVERLIST_TWO_DEFAULT:
    case ERROR_DRIVERLIST_NO_DEFAULT_END:
    case ERROR_DRIVERLIST_EMPTY_DEFAULT:
    case ERROR_DRIVERLIST_OPTION_NAME_ERROR:
    case ERROR_DRIVERLIST_OPTION_NAME_MISSING:
    case ERROR_DRIVERLIST_VALUE_ERROR:
    case ERROR_DRIVERLIST_VALUE_VALUE:
    case ERROR_DRIVERLIST_NO_ARGUMENT_END:
    case ERROR_DRIVERLIST_EMPTY_ARGUMENT:
    case ERROR_DRIVERLIST_COMMENT_ERROR:
    case ERROR_DRIVERLIST_STRICT_ERROR:
    case ERROR_DRIVERLIST_STRICT_VALUE:
    case ERROR_DRIVERLIST_DESC_ERROR:
    case ERROR_DRIVERLIST_EMPTY_DESC:
    case ERROR_DRIVERLIST_NO_DESC:
    case ERROR_DRIVERLIST_CLOSE_DESC:
    case ERROR_DRIVERLIST_SECTION_ERROR:
    case ERROR_DRIVERLIST_SECTION_VALUE:
    case ERROR_DRIVERLIST_NO_SECTION:
    case ERROR_DRIVERLIST_TWO_SECTIONS:
    case ERROR_DRIVERLIST_SECTION_NAME_ERROR:
    case ERROR_DRIVERLIST_SECTION_NAME_MISSING:
    case ERROR_DRIVERLIST_DOUBLE_OPTION:
      wxMessageBox(message, title, wxICON_ERROR);
      MyMessage::Reset();
      return(true);
      // message << _T("\n\n") << _("Should I ignore this and go on?") << _("\n(Warning: It is not sure what happens next!)");
      // MyMessage::Reset();
      // if (wxMessageBox(message, title, wxICON_ERROR | wxYES_NO | wxYES_DEFAULT)==wxYES) return(false);
      // else return(true);
      break;
    default: break;
  } // switch

  wxMessageBox(message, title, wxICON_ERROR);
  MyMessage::Reset();
  return(true);

  // return false means stop
  // return true means go on

}

const bool GuiFrame::CreateGUIControls(void)
{  // create all gui controls, like text fields, menus, buttons

  MyString funcName("GuiFrame::CreateGUIControls");
  WriteDebug(funcName);

  wxMenuBar *formerMenuBar=GetMenuBar();  // old menu bar

  fileMenu=new wxMenu;    // create file menu
  fileMenu->Append(NEWXORG, _("&New config file\tCtrl+N"), _("Create empty config file"));
  fileMenu->AppendSeparator();
  fileMenu->Append(LOADXORG, _("&Load ...\tCtrl+L"), _("Load config file"));
  fileMenu->Append(SAVEXORG, _("&Save\tCtrl+S"), _("Save config file"));
  fileMenu->Append(SAVEAS, _("Save as..."), _("Save config file under another name"));
  fileMenu->AppendSeparator();
  fileMenu->Append(RESTORE, _("Restore file"), _("Load the config file again"));
  fileMenu->AppendSeparator();
  fileMenu->Append(CLOSEXORG, _("&Close\tCtrl+Q"), _("Close current config file"));
  fileMenu->Append(QUITPROG, _("&Quit\tAlt+F4"), _("Quit the application"));

  wxMenu *langMenu = new wxMenu;  // language menu
  langMenu->AppendRadioItem(LANGENG, _("&English"), _("Set language to English"));
  langMenu->AppendRadioItem(LANGGER, _("&German"), _("Set language to German"));

  if (settings.GetLanguage()==wxLANGUAGE_ENGLISH)
  {
    langMenu->Check(LANGENG, true);
  }
  else if (settings.GetLanguage()==wxLANGUAGE_GERMAN)
  {
    langMenu->Check(LANGGER, true);
  }
  else
  {
    langMenu->Check(LANGENG, true);
  }

  extrasMenu=new wxMenu;    // create extras menu
  extrasMenu->Append(SHOWXORG, _("&Preview...\tF5"), _("Show preview of written config file"));
  extrasMenu->Append(TESTXORG, _("&Test\tF7"), _("Test current settings"));
  extrasMenu->Append(CHECKXORG, _("&Check"), _("Check all entries for errors"));
  extrasMenu->AppendSeparator();
  extrasMenu->Append(-1, _("&Language"), langMenu);
  extrasMenu->Append(SETTINGS, _("&Settings...\tF9"), _("Show settings dialog"));
  if (!debug) extrasMenu->Append(DEBUG, _("&Debug Mode\tCtrl+D"), _("Activate debug mode"));
  else extrasMenu->Append(DEBUG, _("&Debug Mode\tCtrl+D"), _("Deactivate debug mode"));

  wxMenu *helpMenu = new wxMenu;  // help menu
  helpMenu->Append(ABOUTPROG, _("&About...\tF1"), _("Show about dialog"));

  wxMenuBar* mbar=new wxMenuBar;       // create menu bar
  mbar->Append(fileMenu, _("&File"));    // add menu item
  mbar->Append(extrasMenu, _("&Extras"));   // add menu item
  mbar->Append(helpMenu, _("&Help"));    // add menu item
  SetMenuBar(mbar);                      // set menu bar to this frame

  if (formerMenuBar) delete formerMenuBar;

  layout->SetPageText(0, _("Device"));
  layout->SetPageText(1, _("Monitor"));
  layout->SetPageText(2, _("Input Device"));
  layout->SetPageText(3, _("Screen"));
  layout->SetPageText(4, _("Server"));
  layout->SetPageText(5, _("Other Options"));

  warningText->SetLabel(_("Warning: Improper use of this tool can cause damage to your\nxorg.conf and may not start your Xserver.\nYou must restart X to take the changes effect."));

  wxString tempList[devListSize+1];  // initial list for combo und choice elements
  wxString optList[optListSize];   // initial list for option combo und choice elements
  unsigned int offset=0;
  unsigned int i;

  optList[0]=_("----------");
  optList[1]=_("Add Option");
  optList[2]=_("Delete Option");
  optList[3]=_("De/Activate Option");

  tempList[0]=_("----------");
  tempList[1]=_("Add Device");
  tempList[2]=_("Delete Device");
  tempList[3]=_("Copy Device");

  for (i=0; i<devListSize; i++)
  {
    deviceIdentifier->SetString(xorg.Devices().GetSize()+i, tempList[i]);
  }
  if (xorg.Devices().GetSize()>0)
  {
    offset=xorg.Devices().Current().Options().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  else
  {
    offset=0;
  }

  tempList[1]=_("Add empty Option");
  tempList[2]=_("Manage Options...");
  tempList[3]=_("Delete Option");
  tempList[4]=_("De/Activate Option");

  for (i=0; i<devListSize+1; i++)
  {
    deviceOption->SetString(i+offset, tempList[i]);
  }

  deviceBox->SetLabel( _("Device"));
  deviceDriverText->SetLabel(_("Driver"));
  deviceBusIDText->SetLabel(_("BusID"));
  deviceVideoRamText->SetLabel( _("Video Ram"));
  deviceScreenNumberText->SetLabel( _("Screen Number"));
  deviceOptionText->SetLabel(_("Option"));

  tempList[1]=_("Add Monitor");
  tempList[2]=_("Delete Monitor");
  tempList[3]=_("Copy Monitor");

  for (i=0; i<devListSize; i++)
  {
    monitorIdentifier->SetString(xorg.Monitors().GetSize()+i, tempList[i]);
  }
  if (xorg.Monitors().GetSize()>0)
  {
    offset=xorg.Monitors().Current().Options().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  else
  {
    offset=0;
  }
  for (i=0; i<optListSize; i++)
  {
    monitorOption->SetString(i+offset, optList[i]);
  }

  tempList[1]=_("Add Modeline");
  tempList[2]=_("Delete Modeline");
  tempList[3]=_("De/Activate Modeline");

  if (xorg.Monitors().GetSize()>0)
  {
    offset=xorg.Monitors().Current().Modelines().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  else
  {
    offset=0;
  }
  for (i=0; i<optListSize; i++)
  {
    monitorModelineName->SetString(i+offset, tempList[i]);
  }

  monitorBox->SetLabel(_("Monitor"));
  monitorHorizSyncText->SetLabel( _("Hor.Sync."));
  monitorVertRefreshText->SetLabel(_("Vert.Refresh"));
  monitorDisplayText->SetLabel(_("Display Size"));
  monitorGammaText->SetLabel(_("Gamma"));
  monitorOptionText->SetLabel(_("Option"));
  monitorModelineText->SetLabel(_("Modeline"));
  monitorFreqButton->SetLabel(_("Get Refresh Rate"));
  monitorDisplaySizeButton->SetLabel(_("Get Display Size"));
  monitorModelineGeneratorButton->SetLabel(_("Generate"));
  monitorModelineGeneratorAddButton->SetLabel(_("Add"));

  tempList[1]=_("Add Input Device");
  tempList[2]=_("Delete Input Device");
  tempList[3]=_("Copy Input Device");

  for (i=0; i<devListSize; i++)
  {
    inputDeviceIdentifier->SetString(xorg.InputDevices().GetSize()+i, tempList[i]);
  }
  if (xorg.InputDevices().GetSize()>0)
  {
    offset=xorg.InputDevices().Current().Options().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  else
  {
    offset=0;
  }

  tempList[1]=_("Add empty Option");
  tempList[2]=_("Manage Options...");
  tempList[3]=_("Delete Option");
  tempList[4]=_("De/Activate Option");

  for (i=0; i<devListSize+1; i++)
  {
    inputDeviceOption->SetString(i+offset, tempList[i]);
  }

  inputDeviceBox->SetLabel(_("Input Device"));
  inputDeviceDriverText->SetLabel( _("Driver"));
  inputDeviceOptionText->SetLabel( _("Option"));

  tempList[1]=_("Add Screen");
  tempList[2]=_("Delete Screen");
  tempList[3]=_("Copy Screen");

  for (i=0; i<devListSize; i++)
  {
    screenIdentifier->SetString(xorg.Screens().GetSize()+i, tempList[i]);
  }
  if (xorg.Screens().GetSize()>0)
  {
    offset=xorg.Screens().Current().Options().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  else
  {
    offset=0;
  }
  for (i=0; i<optListSize; i++)
  {
    screenOption->SetString(i+offset, optList[i]);
  }

  tempList[1]=_("Add Display");
  tempList[2]=_("Delete Display");

  if (xorg.Screens().GetSize()>0)
  {
    offset=xorg.Screens().Current().Displays().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  else
  {
    offset=0;
  }
  for (i=0; i<devListSize-1; i++)
  {
    screenDisplayDepth->SetString(i+offset, tempList[i]);
  }

  tempList[1]=_("Add Mode");
  tempList[2]=_("Delete Mode");


  if (xorg.Screens().GetSize()>0 && xorg.Screens().Current().Displays().GetSize()>0)
  {
    offset=xorg.Screens().Current().Displays().Current().Modes().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  else
  {
    offset=0;
  }
  for (i=0; i<devListSize-1; i++)
  {
    screenDisplayModes->SetString(i+offset, tempList[i]);
  }

  if (xorg.Screens().GetSize()>0 && xorg.Screens().Current().Displays().GetSize()>0)
  {
    offset=xorg.Screens().Current().Displays().Current().Options().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  else
  {
    offset=0;
  }
  for (i=0; i<optListSize; i++)
  {
    screenDisplayOption->SetString(i+offset, optList[i]);
  }

  screenBox->SetLabel( _("Screen"));
  screenDeviceText->SetLabel(_("Device"));
  screenMonitorText->SetLabel(_("Monitor"));
  screenDepthText->SetLabel( _("Default Depth"));
  screenOptionText->SetLabel( _("Option"));
  screenDisplayDepthText->SetLabel(_("Display Depth"));
  screenDisplayModesText->SetLabel(_("Mode"));
  screenDisplayOptionText->SetLabel(_("Option"));
  screenDisplayViewportText->SetLabel(_("Viewport"));
  screenDisplayVirtualText->SetLabel(_("Virtual Size"));

  tempList[1]=_("Add Server Layout");
  tempList[2]=_("Delete Server Layout");
  tempList[3]=_("Copy Server Layout");

  for (i=0; i<devListSize; i++)
  {
    serverLayoutIdentifier->SetString(xorg.ServerLayouts().GetSize()+i, tempList[i]);
  }
  if (xorg.ServerLayouts().GetSize()>0)
  {
    offset=xorg.ServerLayouts().Current().Options().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  else
  {
    offset=0;
  }
  for (i=0; i<optListSize; i++)
  {
    serverLayoutOption->SetString(i+offset, optList[i]);
  }

  tempList[1]=_("Add Screen");
  tempList[2]=_("Delete Screen");

  if (xorg.ServerLayouts().GetSize()>0)
  {
    offset=xorg.ServerLayouts().Current().Screens().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (i=0; i<offset; i++)
    {
      wxString tempStr;
      tempStr << _("Screen") << _T(" ") << i;
      serverLayoutScreen->SetString(i, tempStr);
    }
    //H if (xorg.ServerLayouts().Current().GetScreenSize()>0) serverLayoutScreen->SetValue(serverLayoutScreen->GetStringSelection());
  }
  else
  {
    offset=0;
  }
  for (i=0; i<devListSize-1; i++)
  {
    serverLayoutScreen->SetString(i+offset, tempList[i]);
  }

  tempList[1]=_("Add Input Device");
  tempList[2]=_("Delete Input Device");

  if (xorg.ServerLayouts().GetSize()>0)
  {
    offset=xorg.ServerLayouts().Current().InputDevices().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (i=0; i<offset; i++)
    {
      wxString tempStr;
      tempStr << _("Input Device") << _T(" ") << i;
      serverLayoutInputDevice->SetString(i, tempStr);
    }
    //H     if (xorg.ServerLayouts().Current().GetInputDeviceSize()>0) serverLayoutInputDevice->SetValue(serverLayoutInputDevice->GetStringSelection());
  }
  else
  {
    offset=0;
  }
  for (i=0; i<devListSize-1; i++)
  {
    serverLayoutInputDevice->SetString(i+offset, tempList[i]);
  }

 if (xorg.ServerLayouts().GetSize()>0 && xorg.ServerLayouts().Current().InputDevices().GetSize()>0)
  {
    offset=xorg.ServerLayouts().Current().InputDevices().Current().Options().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  else
  {
    offset=0;
  }
  for (i=0; i<optListSize-1; i++)
  {
    serverLayoutInputDeviceOption->SetString(i+offset, optList[i]);
  }

  serverLayoutBox->SetLabel(_("Server Layout"));
  serverLayoutScreenNumberText->SetLabel(_("Number"));
  serverLayoutScreenPositionText->SetLabel(_("Position"));
  serverLayoutOptionText->SetLabel(_("Option"));
  serverLayoutInputDeviceOptionText->SetLabel(_("Option"));

  tempList[1]=_("Add Server Flag");
  tempList[2]=_("Delete Server Flag");
  tempList[3]=_("De/Activate Server Flag");

  for (i=0; i<devListSize; i++)
  {
    serverFlagsOption->SetString(xorg.ServerFlags().GetSize()+i, tempList[i]);
  }

  serverFlagBox->SetLabel(_("Server Flags"));

  tempList[1]=_("Add Extension");
  tempList[2]=_("Delete Extension");
  tempList[3]=_("De/Activate Extension");

  for (i=0; i<devListSize; i++)
  {
    extensionOption->SetString(xorg.Extensions().GetSize()+i, tempList[i]);
  }

  extensionBox->SetLabel(_("Extensions"));

  tempList[1]=_("Add Module");
  tempList[2]=_("Delete Module");
  tempList[3]=_("De/Activate Module");

  for (i=0; i<devListSize; i++)
  {
    moduleIdentifier->SetString(xorg.Modules().GetSize()+i, tempList[i]);
  }
  if (xorg.Modules().GetSize()>0)
  {
    offset=xorg.Screens().Current().Options().GetSize();
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  else
  {
    offset=0;
  }
  for (i=0; i<optListSize; i++)
  {
    moduleOption->SetString(i+offset, optList[i]);
  }

  moduleBox->SetLabel( _("Modules"));
  moduleOptionText->SetLabel( _("Option"));

  filesBox->SetLabel(_("Files"));

  tempList[1]=_("Add Font Path");
  tempList[2]=_("Delete Font Path");
  tempList[3]=_("De/Activate Font Path");

  for (i=0; i<devListSize; i++)
  {
    fontPath->SetString(xorg.Files().FontPaths().GetSize()+i, tempList[i]);
  }
  fontPathText->SetLabel(_("Font Path"));

  tempList[1]=_("Add Module Path");
  tempList[2]=_("Delete Module Path");
  tempList[3]=_("De/Activate Module Path");

  for (i=0; i<devListSize; i++)
  {
    modulePath->SetString(xorg.Files().ModulePaths().GetSize()+i, tempList[i]);
  }
  modulePathText->SetLabel(_("Module Path"));

  tempList[1]=_("Add RGB Path");
  tempList[2]=_("Delete RGB Path");
  tempList[3]=_("De/Activate RGB Path");

  for (i=0; i<devListSize; i++)
  {
    rgbPath->SetString(xorg.Files().RgbPaths().GetSize()+i, tempList[i]);
  }
  rgbPathText->SetLabel(_("RGB Path"));

  driBox->SetLabel(_("DRI"));
  return(false);
}

void GuiFrame::SetSaved(const bool save)
{  // set saved status and set title

  MyString funcName("GuiFrame::SetSaved");
  WriteDebug(funcName);

  wxString title;

  isSaved=save;
  if (isSaved==false)
  {
    if (!settings.GetCurrentFilename().IsEmpty())
    {
      title=wxString::FromAscii(settings.GetCurrentFile().data());
      title+=_(" [NOT SAVED]");
    }
    else title=version;
  }
  else
  {
    if (!settings.GetCurrentFilename().IsEmpty()) title=wxString::FromAscii(settings.GetCurrentFile().data());
    else title=version;
  }
  SetTitle(title);
}

void GuiFrame::OnAbout(wxCommandEvent &event)
{  // show about dialog

  MyString funcName("GuiFrame::OnAbout");
  WriteDebug(funcName);

  wxString info;
  info << version << _T("\n");
  info << _("written by") << _T(" Dominik Wagenfuehr\n");
  info << _("Licence") << _T(": GPL\n\n");
  info << _T("Web: http://www.cyskat.de/dee/progxorg.htm\n");
  info << _T("Wiki: http://wiki.ubuntuusers.de/xorg-edit\n\n");
  info << _T("Mail: dominik (dot) wagenfuehr (at) arcor (dot) de\n");
  wxMessageBox(info, _("About xorg-edit"));

}

// update layout

const bool GuiFrame::SetFrame(void)
{ // set all elements on screen after loading file

  MyString funcName("GuiFrame::SetFrame");
  WriteDebug(funcName);

  if (UpdateTab(layout->GetSelection()))
  {
    MyMessage::Append(funcName);
    return(true);
  }
  return(false);
}

const bool GuiFrame::UpdateDeviceTab(void)
{
  MyString funcName("GuiFrame::UpdateDeviceTab");
  WriteDebug(funcName);

  if (UpdateDevice())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  return(false);
}

const bool GuiFrame::UpdateMonitorTab(void)
{
  MyString funcName("GuiFrame::UpdateMonitorTab");
  WriteDebug(funcName);

  if (UpdateMonitor())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  return(false);
}

const bool GuiFrame::UpdateInputDeviceTab(void)
{
  MyString funcName("GuiFrame::UpdateInputDeviceTab");
  WriteDebug(funcName);

  if (UpdateInputDevice())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  return(false);
}

const bool GuiFrame::UpdateScreenTab(void)
{
  MyString funcName("GuiFrame::UpdateScreenTab");
  WriteDebug(funcName);

  if (UpdateScreen())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  return(false);
}

const bool GuiFrame::UpdateServerTab(void)
{
  MyString funcName("GuiFrame::UpdateServerTab");
  WriteDebug(funcName);

  if (UpdateServerLayout())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (UpdateServerFlagOption())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  return(false);
}

const bool GuiFrame::UpdateOtherSectionTab(void)
{
  MyString funcName("GuiFrame::UpdateOtherSectionTab");
  WriteDebug(funcName);

  if (UpdateExtension())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (UpdateModule())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (UpdateFontPath())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (UpdateModulePath())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (UpdateRgbPath())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (UpdateDRI())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  return(false);
}

const bool GuiFrame::UpdateTab(const unsigned int tabnr)
{      // check xorg devices in this tab

  MyString funcName("GuiFrame::UpdateTab");
  WriteDebug(funcName);

  switch(tabnr)
  {
    case 0:  // DEVICE
      if (UpdateDeviceTab())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      break;
    case 1:  // MONITOR
      if (UpdateMonitorTab())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      break;
    case 2:  // INPUTDEVICE
      if (UpdateInputDeviceTab())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      break;
    case 3:  // SCREEN
      if (UpdateScreenTab())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      break;
    case 4:  // SERVERLAYOUT, SERVERFLAGS
      if (UpdateServerTab())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      break;
    case 5:  // EXTENSIONS, MODULES, FILES, DRI
      if (UpdateOtherSectionTab())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      break;
    default:
      MyMessage::Set(ERROR_UNKNOWN_CASE_IN_SWITCH, funcName);
      return(true);
      break;
  } // switch
  return(false);
}

void GuiFrame::ClearElements(void)
{  // clear all elements

  MyString funcName("GuiFrame::ClearElements");
  WriteDebug(funcName);

  xorg.Clear();
}

const bool GuiFrame::LoadDrivers(void)
{  // load driver lists

  MyString funcName("GuiFrame::LoadDrivers");
  WriteDebug(funcName);

  MyString filename;
  unsigned int i;

  filename=settings.GetOptionDir()+"/devices/list.xml";
  deviceDriver->Clear();
  if (deviceDriverList.Load(filename))
  {
    MyMessage::Reset();
  }
  else
  {
    for (i=0; i<deviceDriverList.GetSize(); i++)
    {
      deviceDriver->Append(wxString::FromAscii(deviceDriverList(i).data()));
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
  }

  inputDeviceDriver->Clear();
  filename=settings.GetOptionDir()+"/inputdevices/list.xml";
  if (inputDeviceDriverList.Load(filename))
  {
    MyMessage::Reset();
  }
  else
  {
    for (i=0; i<deviceDriverList.GetSize(); i++)
    {
      inputDeviceDriver->Append(wxString::FromAscii(inputDeviceDriverList(i).data()));
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
  }

  if (SetFrame())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  return(false);
}

void GuiFrame::SetLanguage(wxCommandEvent& event)
{    // set language

  MyString funcName("GuiFrame::SetLanguage");
  WriteDebug(funcName);

  if (event.GetId()==LANGENG && settings.GetLanguage()!=wxLANGUAGE_ENGLISH)
  {
    settings.SetLanguage(wxLANGUAGE_ENGLISH);
  }
  else if (event.GetId()==LANGGER && settings.GetLanguage()!=wxLANGUAGE_GERMAN)
  {
    settings.SetLanguage(wxLANGUAGE_GERMAN);
  }
  else
  {
    settings.SetLanguage(wxLANGUAGE_DEFAULT);
  }
  wxGetApp().SetLanguage(settings.GetLanguage());
  CreateGUIControls();

}

void GuiFrame::OnDebug(wxCommandEvent& event)
{  // de/activae debug mode

  MyString funcName("GuiFrame::OnDebug");
  WriteDebug(funcName);

  if (debug)
  {
    debug=false;
    extrasMenu->SetHelpString(DEBUG, _("Activate debug mode"));
  }
  else
  {
    debug=true;
    extrasMenu->SetHelpString(DEBUG, _("Deactivate debug mode"));
    WriteDebug(funcName, true);
  }

}

void GuiFrame::OnNew(wxCommandEvent& event)
{  //create empty config file

  MyString funcName("GuiFrame::OnNew");
  WriteDebug(funcName);

  if (!isSaved)
  {
    if (wxMessageBox(_("File is not saved. Do you want to discard changes?"), _("Question: Discard Changes?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT) == wxNO)
    {
      return;
    }
  }

  ClearElements();
  if (SetFrame())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
  EnableMenuItems(true);
  settings.SetCurrentDir((const char*)wxGetCwd().mb_str());
  settings.SetCurrentFilename("");
  SetSaved(false);
}

void GuiFrame::OnRestore(wxCommandEvent& event)
{  // load the current config file again

  MyString funcName("GuiFrame::OnRestore");
  WriteDebug(funcName);

  if (!isSaved)
  {
    if (wxMessageBox(_("File is not saved. Do you want to discard changes?"), _("Question: Discard Changes?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT) == wxNO)
    {
      return;
    }
  }

  bool leave;
  do
  {
    leave=xorg.Load(settings.GetCurrentFile(), true);
    MyMessage::Reset();
  } while (leave);

  if (ConvertXorg())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  EnableMenuItems(true);
  if (SetFrame())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
  SetSaved(true);
}

void GuiFrame::OnClose(wxCommandEvent& event)
{  // close current config

  MyString funcName("GuiFrame::OnClose");
  WriteDebug(funcName);

  if (!isSaved)
  {
    if (wxMessageBox(_("File is not saved. Do you want to discard changes?"), _("Question: Discard Changes?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT) == wxNO)
    {
      return;
    }
  }

  ClearElements();
  if (SetFrame())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
  settings.SetCurrentFilename("");
  SetSaved(true);

  EnableMenuItems(false);
}

void GuiFrame::OnLoad(wxCommandEvent& event)
{  // load other config file

  MyString funcName("GuiFrame::OnLoad");
  WriteDebug(funcName);

  if (!isSaved)
  {
    if (wxMessageBox(_("File is not saved. Do you want to discard changes?"), _("Question: Discard Changes?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT) == wxNO)
    {
      return;
    }
  }

  // file dialog for opening files
  wxFileDialog openFileDialog(this, _("Choose a file"), wxString::FromAscii(settings.GetCurrentDir().data()), wxEmptyString, _T("*"), wxOPEN | wxCHANGE_DIR);

  if (openFileDialog.ShowModal() == wxID_OK)     // file has been chosen
  {
    settings.SetCurrentDir((const char*)openFileDialog.GetDirectory().mb_str());
    settings.SetCurrentFilename((const char*)openFileDialog.GetFilename().mb_str());

    bool leave;
    bool error=false;
    do
    {
      leave=true;
      if (xorg.Load(settings.GetCurrentFile(), error))
      {
        leave=false;
        error=true;
        MyMessage::Append(funcName);
        if (ShowError())
        {
          EnableMenuItems(false);
          settings.SetCurrentFilename("");
          leave=true;
          break;
        }
      }
      else
      {
        if (ConvertXorg())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        EnableMenuItems(true);
      }
    } while (!leave);

    if (SetFrame())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    SetSaved(true);
  }
}

void GuiFrame::OnSave(wxCommandEvent& event)
{  // save xorg.conf

  MyString funcName("GuiFrame::OnSave");
  WriteDebug(funcName);

  if (settings.GetCurrentFilename().IsEmpty())
  {
    OnSaveAs(event);
    return;
  }

  if (CheckAllTabs())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  if (ConvertXorg())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  if (settings.GetRememberTest())
  {
    wxString tempStr;
    tempStr << _("Do you want to test your configured file before saving? ");
    tempStr << _("In case of a misconfiguration the file won't be saved!");
    if (wxMessageBox(tempStr , _("Question: Test configuration?"), wxICON_QUESTION | wxYES_NO | wxYES_DEFAULT)==wxYES)
    {
      if (TestXorg()) return;
    }
  }

  if (CreateBackup())  // create backup copy
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  if (xorg.Save(settings.GetCurrentFile(), (const char*)version.mb_str()))
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
  else SetSaved(true);

  if (UpdateTab(layout->GetSelection()))
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::OnSaveAs(wxCommandEvent& event)
{  // save other config file

  MyString funcName("GuiFrame::OnSaveAs");
  WriteDebug(funcName);

  if (CheckAllTabs())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  // file dialog for opening files
  wxFileDialog saveFileDialog(this, _("Choose a file"), wxEmptyString, wxEmptyString, wxEmptyString,  wxSAVE | wxOVERWRITE_PROMPT | wxCHANGE_DIR );

  if (saveFileDialog.ShowModal() == wxID_OK)     // file has been chosen
  {
    settings.SetCurrentDir((const char*)saveFileDialog.GetDirectory().mb_str());
    settings.SetCurrentFilename((const char*)saveFileDialog.GetFilename().mb_str());

    if (ConvertXorg())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }

    if (settings.GetRememberTest())
    {
      wxString tempStr;
      tempStr << _("Do you want to test your configured file before saving? ");
      tempStr << _("In case of a misconfiguration the file won't be saved!");
      if (wxMessageBox(tempStr , _("Question: Test configuration?"), wxICON_QUESTION | wxYES_NO | wxYES_DEFAULT)==wxYES)
      {
        if (TestXorg()) return;
      }
    }

    if (CreateBackup())  // create backup copy
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }

    if (xorg.Save(settings.GetCurrentFile(), (const char*)version.mb_str()))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    else SetSaved(true);

    if (UpdateTab(layout->GetSelection()))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
}

void GuiFrame::OnShow(wxCommandEvent& event)
{ // show config file

  MyString funcName("GuiFrame::OnShow");
  WriteDebug(funcName);

  if (!layout->IsEnabled()) return;

  TextFile preview;
  if (xorg.Preview(preview, settings.GetCurrentFile(), (const char*)version.mb_str()))
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  wxString name;
  name=wxString::FromAscii(settings.GetCurrentFile().data());
  name+=_(" - PREVIEW");

  TextFrame *xText=new TextFrame(this, name, preview);  // create frame
  xText->ShowModal();

}

void GuiFrame::OnTest(wxCommandEvent& event)
{    // test xorg file

  MyString funcName("GuiFrame::OnTest");
  WriteDebug(funcName);

  if (TestXorg())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

const bool GuiFrame::TestXorg(void)
{    // test xorg file

  MyString funcName("GuiFrame::TestXorg");
  WriteDebug(funcName);

  MyString fileName=settings.GetCurrentFilename();  // store filename
  MyString tempFile="test_";
  tempFile+=fileName;

  settings.SetCurrentFilename(tempFile);   // set temporary filename
  if (xorg.Save(settings.GetCurrentFile(), (const char*)version.mb_str()))
  {
    wxRemoveFile(wxString::FromAscii(tempFile.data()));      // delete temporary file
    settings.SetCurrentFilename(fileName);
    MyMessage::Append(funcName);
    return(true);  // write data into file without checking anything
  }

  if (settings.GetShowBlank())
  {
    wxMessageBox(_("I try to start a second XServer. The screen will go blank for a few seconds. Please be patient!"), _("Start XServer"), wxICON_INFORMATION);
  }

  wxArrayString output;  // output of x-test
  wxArrayString errors;  // output of x-test
  wxString command;
  int i;        // loop counter
  int display=1;         // start at xserver:1
  bool checked=false;
  TextFrame *xText=NULL;

  while(!checked)
  {
    if (display>=100)
    {  // checked 100 displays

      wxMessageBox(_("Sorry, no second XServer could be started."), _("No XServer!"), wxICON_ERROR);
      wxRemoveFile(wxString::FromAscii(tempFile.data()));      // delete temporary file
      settings.SetCurrentFilename(fileName);
      return(true);
    }

    checked=true;

    command.Printf(_T("X :%d"), display);
    command+=_T(" -probeonly -verbose 2 -config ");
    command+=wxString::FromAscii(tempFile.data());

    // if (wxExecute(command, output, errors, wxEXEC_SYNC)!=0)
    if (wxExecute(command, output, errors)!=0)
    {
      // Check why the command could not be executed
      for (i=0; i<(int)errors.GetCount(); i++)
      {
        if (errors[i].Find(_T("Server is already active for display"))>=0)
        {
          display++;      // check another display
          checked=false;
          errors.Clear();
          break;  // leave for i-loop
        }

        if (errors[i].Find(_T("user not authorized to run the X server"))>=0)
        {

          command=_("Sorry, no second XServer could be started.");
          command+=_("\nIt seems that you don't have the rights to.");
          command+=_("\nDo you want to see the logfile?");

          if (wxMessageBox(command, _("No XServer!"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxYES)
          {
            xText = new TextFrame(this, _("Logfile"), errors);  // create frame
            xText->ShowModal();  // show dialog modal
            delete xText;        // delete again
            xText=NULL;
          }
          wxRemoveFile(wxString::FromAscii(tempFile.data()));      // delete temporary file
          settings.SetCurrentFilename(fileName);
          return(true);
        }

        if (errors[i].Find(_T("no screens found"))>=0)
        {
          command=_("You file seems to be misconfigured. Your graphical system won't start next time!");
          command+=_("\nDo you want to see the logfile?");

          if (wxMessageBox(command, _("Misconfigured File!"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxYES)
          {
            xText = new TextFrame(this, _("Logfile"), errors);  // create frame
            xText->ShowModal();  // show dialog modal
            delete xText;        // delete again
            xText=NULL;
          }
          wxRemoveFile(wxString::FromAscii(tempFile.data()));      // delete temporary file
          settings.SetCurrentFilename(fileName);
          return(true);
        }

      } // for i

      if (!checked)
      {
        // loop is through, another error
        command=_("Sorry, no second XServer could be started.");
        command+=_("\nDo you want to see the logfile?");

        if (wxMessageBox(command, _("No XServer!"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxYES)
        {
          xText = new TextFrame(this, _("Logfile"), errors);  // create frame
          xText->ShowModal();  // show dialog modal
          delete xText;        // delete again
          xText=NULL;
        }
        wxRemoveFile(wxString::FromAscii(tempFile.data()));      // delete temporary file
        settings.SetCurrentFilename(fileName);
        return(true);
      }

    }
  }

  command=_("The new configuration seems to work fine.");
  command+=_("\nDo you want to see the logfile anyway?");

  if (wxMessageBox(command, _("Everything's ok!"), wxICON_INFORMATION | wxYES_NO | wxNO_DEFAULT)==wxYES)
  {
    xText = new TextFrame(this, _("Logfile"), errors);  // create frame
    xText->ShowModal();  // show dialog modal
    delete xText;        // delete again
    xText=NULL;
  }

  // everything's fine
  wxRemoveFile(wxString::FromAscii(tempFile.data()));      // delete temporary file
  settings.SetCurrentFilename(fileName);

  return(false);
}

void GuiFrame::OnQuit(wxCommandEvent& event)
{  // quit program

  MyString funcName("GuiFrame::OnQuit");
  WriteDebug(funcName);

  if (!isSaved)
  {
    if (wxMessageBox(_("File is not saved. Do you want to discard changes?"), _("Question: Discard Changes?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT) == wxNO)
    {
      return;
    }
  }

  Close();  // force closing window
}

const bool GuiFrame::ConvertXorg(void)
{   // convert all driver options in xorg file

  MyString funcName("GuiFrame::ConvertXorg");
  WriteDebug(funcName);

  MyString filename;      // filename for loading
  DriverOptionList list;   // driver options
  unsigned int i;          // loop counter

  for (i=0; i<xorg.Devices().GetSize(); i++)
  {
    filename=settings.GetOptionDir()+"/devices/"+xorg.Devices()(i).GetDriver()+".xml";
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (list.Load(filename))
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (xorg.Devices()(i).ConvertOptions(list))
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }

  for (i=0; i<xorg.InputDevices().GetSize(); i++)
  {
    filename=settings.GetOptionDir()+"/inputdevices/"+xorg.InputDevices()(i).GetDriver()+".xml";
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (list.Load(filename))
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (xorg.InputDevices()(i).ConvertOptions(list))
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }

  return(false);
}


// Check all changes

void GuiFrame::OnCheck(wxCommandEvent& event)
{ // check xorg file

  MyString funcName("GuiFrame::OnCheck");
  WriteDebug(funcName);

  if (CheckAllTabs())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

const bool GuiFrame::CheckAllTabs(void)
{      // check all tabs

  MyString funcName("GuiFrame::CheckAllTabs");
  WriteDebug(funcName);

  for (unsigned int i=0; i<6; i++)
  {
    if (CheckTab(i))
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  return(false);
}

const bool GuiFrame::CheckDeviceTab(void)
{
  MyString funcName("GuiFrame::CheckDeviceTab");
  WriteDebug(funcName);

  if (xorg.Devices().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  // extended check
  MyString filename;
  DriverOptionList list;

  for (unsigned int i=0; i<xorg.Devices().GetSize(); i++)
  {
    filename=settings.GetOptionDir()+"/devices/"+xorg.Devices()(i).GetDriver()+".xml";
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (list.Load(filename))
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (xorg.Devices()(i).Check(list))
    {
      if (xorg.Devices().SetCurrent(i))
      {
        MyMessage::Append(funcName);
        return(true);
      }
      MyMessage::Append(funcName);
      return(true);
    }
  }

  return(false);
}

const bool GuiFrame::CheckMonitorTab(void)
{
  MyString funcName("GuiFrame::CheckMonitorTab");
  WriteDebug(funcName);

  if (xorg.Monitors().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  return(false);
}

const bool GuiFrame::CheckInputDeviceTab(void)
{
  MyString funcName("GuiFrame::CheckInputDeviceTab");

  if (xorg.InputDevices().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  // extended check
  MyString filename;
  DriverOptionList list;

  for (unsigned int i=0; i<xorg.InputDevices().GetSize(); i++)
  {
    filename=settings.GetOptionDir()+"/inputdevices/"+xorg.InputDevices()(i).GetDriver()+".xml";
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (list.Load(filename))
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (xorg.InputDevices()(i).Check(list))
    {
      if (xorg.InputDevices().SetCurrent(i))
      {
        MyMessage::Append(funcName);
        return(true);
      }
      MyMessage::Append(funcName);
      return(true);
    }
  }

  return(false);
}

const bool GuiFrame::CheckScreenTab(void)
{
  MyString funcName("GuiFrame::CheckScreenTab");
  WriteDebug(funcName);

  if (xorg.Screens().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  return(false);
}

const bool GuiFrame::CheckServerTab(void)
{
  MyString funcName("GuiFrame::CheckServerTab");

  if (xorg.ServerLayouts().Check(true))
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.ServerFlags().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  return(false);
}

const bool GuiFrame::CheckOtherSectionTab(void)
{
  MyString funcName("GuiFrame::CheckOtherSectionTab");
  WriteDebug(funcName);

  if (xorg.Extensions().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Modules().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Files().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Dri().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  return(false);
}

const bool GuiFrame::CheckTab(const unsigned int tabnr)
{      // check xorg devices in this tab

  MyString funcName("GuiFrame::CheckTab");
  WriteDebug(funcName);

  bool error=false;

  switch(tabnr)
  {
    case 0:  // DEVICE
      if (CheckDeviceTab()) error=true;
      break;
    case 1:  // MONITOR
      if (CheckMonitorTab()) error=true;
      break;
    case 2:  // INPUTDEVICE
      if (CheckInputDeviceTab()) error=true;
      break;
    case 3:  // SCREEN
      if (CheckScreenTab()) error=true;
      break;
    case 4:  // SERVERLAYOUT, SERVERFLAGS
      if (CheckServerTab()) error=true;
      break;
    case 5:  // EXTENSIONS, MODULES, FILES, DRI
      if (CheckOtherSectionTab()) error=true;
      break;
    default:
      MyMessage::Set(ERROR_UNKNOWN_CASE_IN_SWITCH, funcName);
      return(true);
      break;
  } // switch

  if (error)
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();

    // we hope that is isn't a real error

    layout->SetSelection(tabnr); // set selection
    if (UpdateTab(tabnr))
    {
       MyMessage::Append(funcName);
       return(true);
     }

     MyMessage::Set(tempMessage, funcName);
     return(true);
  }

  return(false);
}

void GuiFrame::OnTabChange(wxNotebookEvent& event)
{    // some tab has changed, sav changes if possible

  MyString funcName("GuiFrame::OnTabChange");
  WriteDebug(funcName);

  if (event.GetSelection()!=event.GetOldSelection())  // tab has really changed
  {
    if (CheckTab(event.GetOldSelection()))
    {
        MyMessage::Append(funcName);
        if (ShowError()) event.Veto();
    }
    if (UpdateTab(event.GetSelection()))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
}

void GuiFrame::OnSettings(wxCommandEvent& event)
{    // show settings dialog

  MyString funcName("GuiFrame::OnSettings");
  WriteDebug(funcName);

  SettingsDialog dlg(this, _("Settings"), settings);
  if (dlg.ShowModal()==wxID_OK)
  {
    if (settings.Save())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    if (LoadDrivers())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
}

// ***************************
// Update Window
// ***************************

const bool GuiFrame::UpdateDevice(void)
{
  MyString funcName("GuiFrame::UpdateDevice");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (deviceIdentifier->GetCount()>(int)devListSize) deviceIdentifier->Delete(0);

  deviceIdentifier->Enable(true);

  if (xorg.Devices().GetSize()==0)
  {
    deviceDriver->Enable(false);
    deviceBusID->Enable(false);
    deviceScreen->Enable(false);
    deviceVideoRam->Enable(false);
    deviceIdentifier->SetEditable(false);
    deviceDriver->SetValue(wxEmptyString);
    deviceBusID->SetValue(wxEmptyString);
    deviceScreen->SetValue(wxEmptyString);
    deviceVideoRam->SetValue(wxEmptyString);
    deviceOptValue->SetValue(wxEmptyString);
    deviceIdentifier->SetSelection(0);
  }
  else
  {
    deviceDriver->Enable(true);
    deviceBusID->Enable(true);
    deviceScreen->Enable(true);
    deviceVideoRam->Enable(true);
    deviceIdentifier->SetEditable(true);

    deviceDriver->SetValue(wxString::FromAscii(xorg.Devices().Current().GetDriver().data()));
    deviceBusID->SetValue(wxString::FromAscii(xorg.Devices().Current().GetBus().data()));
    deviceScreen->SetValue(wxString::FromAscii(xorg.Devices().Current().GetScreen().data()));
    deviceVideoRam->SetValue(wxString::FromAscii(xorg.Devices().Current().GetVideoRam().data()));
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Devices().GetSize(); i++)
    {
      deviceIdentifier->Insert(wxString::FromAscii(xorg.Devices()(i).GetId().data()), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }

    deviceIdentifier->SetSelection(xorg.Devices().GetCurrent());
  }

  if (UpdateDeviceOption())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateDeviceOption(void)
{
  MyString funcName("GuiFrame::UpdateDeviceOption");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (deviceOption->GetCount()>(int)devListSize+1) deviceOption->Delete(0);

  if (xorg.Devices().GetSize()==0) deviceOption->Enable(false);
  else deviceOption->Enable(true);
  deviceOption->GetToolTip()->Enable(false);
  deviceOptValue->SetEditable(true);

  if (xorg.Devices().GetSize()==0 || xorg.Devices().Current().Options().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    deviceOptValue->Enable(false);
    deviceOptValue->SetValue(wxEmptyString);
    deviceOption->SetEditable(false);
    deviceOption->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (xorg.Devices().Current().Options().Current().IsActive())
    {
      deviceOptValue->Enable(true);
      deviceOption->SetEditable(true);
    }
    else
    {
      deviceOptValue->Enable(false);
      deviceOption->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Devices().Current().Options().GetSize(); i++)
    {
      if (xorg.Devices().Current().Options()(i).IsActive())
        deviceOption->Insert(wxString::FromAscii(xorg.Devices().Current().Options()(i).GetName().data()), i);
      else
        deviceOption->Insert(wxString::FromAscii(xorg.Devices().Current().Options()(i).GetName().data())+_(" [inactive]"), i);

      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }

    deviceOption->SetSelection(xorg.Devices().Current().Options().GetCurrent());

    deviceOptValue->Clear();

    MyString filename=settings.GetOptionDir()+"/devices/"+xorg.Devices().Current().GetDriver()+".xml";
    DriverOptionList list;
    if (list.Load(filename))
    {
      deviceOptValue->SetValue(wxString::FromAscii(xorg.Devices().Current().Options().Current().GetValue().data()));
      MyMessage::Append(funcName);
      return(true);
    }

    // search select option in list
    int pos=list.Find(xorg.Devices().Current().Options().Current().GetName(), true);

    if (pos>=0)
    {
      wxString tempComment=wxString::FromAscii(list(pos).GetComment().data());
      if (!tempComment.IsEmpty())
      {
        deviceOption->GetToolTip()->Enable(true);
        deviceOption->GetToolTip()->SetTip(tempComment);
      }

      if (list(pos).GetValue()=="boolean")
      {
        MyString tempStr=xorg.Devices().Current().Options().Current().GetValue().toLower();
        if (tempStr.IsEmpty())
        {
          tempStr=list(pos).GetDefault();
        }

        if (tempStr.IsEmpty() || tempStr=="false" || tempStr=="true")
        {
          deviceOptValue->Append(_T("false"));
          deviceOptValue->Append(_T("true"));
        }
        else if (tempStr=="off" || tempStr=="on")
        {
          deviceOptValue->Append(_T("off"));
          deviceOptValue->Append(_T("on"));
        }
        else if (tempStr=="no" || tempStr=="yes")
        {
          deviceOptValue->Append(_T("no"));
          deviceOptValue->Append(_T("yes"));
        }
        else if (tempStr=="0" || tempStr=="1")
        {
          deviceOptValue->Append(_T("0"));
          deviceOptValue->Append(_T("1"));
        }

        deviceOptValue->SetEditable(false);
      }
      else
      {
        // add arguments
        for (unsigned int i=0; i<list(pos).GetArguments().GetSize(); i++)
        {
          deviceOptValue->Append(wxString::FromAscii(list(pos).GetArguments()(i).data()));
          if (!MyMessage::OK())
          {
            MyMessage::Append(funcName);
            return(true);
          }
        }
      }
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }

      if (list(pos).IsStrict())
      {
        deviceOptValue->SetEditable(false);
      }
    } // pos!=-1
    else
    { // guess option type
      MyString tempStr=xorg.Devices().Current().Options().Current().GetValue().toLower();
      if (tempStr=="false" || tempStr=="true")
      {
        deviceOptValue->Append(_T("false"));
        deviceOptValue->Append(_T("true"));
      }
      else if (tempStr=="off" || tempStr=="on")
      {
        deviceOptValue->Append(_T("off"));
        deviceOptValue->Append(_T("on"));
      }
      else if (tempStr=="no" || tempStr=="yes")
      {
        deviceOptValue->Append(_T("no"));
        deviceOptValue->Append(_T("yes"));
      }
      else if (tempStr=="0" || tempStr=="1")
      {
        deviceOptValue->Append(_T("0"));
        deviceOptValue->Append(_T("1"));
      }
    }
    deviceOptValue->SetValue(wxString::FromAscii(xorg.Devices().Current().Options().Current().GetValue().data()));
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateInputDevice(void)
{
  MyString funcName("GuiFrame::UpdateInputDevice");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (inputDeviceIdentifier->GetCount()>(int)devListSize) inputDeviceIdentifier->Delete(0);

  inputDeviceIdentifier->Enable(true);

  if (xorg.InputDevices().GetSize()==0)
  {
    inputDeviceDriver->Enable(false);
    inputDeviceIdentifier->SetEditable(false);
    inputDeviceDriver->SetValue(wxEmptyString);
    inputDeviceIdentifier->SetSelection(0);
  }
  else
  {
    inputDeviceDriver->Enable(true);
    inputDeviceIdentifier->SetEditable(true);
    inputDeviceDriver->SetValue(wxString::FromAscii(xorg.InputDevices().Current().GetDriver().data()));
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.InputDevices().GetSize(); i++)
    {
      inputDeviceIdentifier->Insert(wxString::FromAscii(xorg.InputDevices()(i).GetId().data()), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    inputDeviceIdentifier->SetSelection(xorg.InputDevices().GetCurrent());
  }

  if (UpdateInputDeviceOption())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateInputDeviceOption(void)
{
  MyString funcName("GuiFrame::UpdateInputDeviceOption");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (inputDeviceOption->GetCount()>(int)devListSize+1) inputDeviceOption->Delete(0);

  if (xorg.InputDevices().GetSize()==0) inputDeviceOption->Enable(false);
  else inputDeviceOption->Enable(true);
  inputDeviceOption->GetToolTip()->Enable(false);
  inputDeviceOptValue->SetEditable(true);

  if (xorg.InputDevices().GetSize()==0 || xorg.InputDevices().Current().Options().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    inputDeviceOptValue->Enable(false);
    inputDeviceOptValue->SetValue(wxEmptyString);
    inputDeviceOption->SetEditable(false);
    inputDeviceOption->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (xorg.InputDevices().Current().Options().Current().IsActive())
    {
      inputDeviceOptValue->Enable(true);
      inputDeviceOption->SetEditable(true);
    }
    else
    {
      inputDeviceOptValue->Enable(false);
      inputDeviceOption->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.InputDevices().Current().Options().GetSize(); i++)
    {
      if (xorg.InputDevices().Current().Options()(i).IsActive())
        inputDeviceOption->Insert(wxString::FromAscii(xorg.InputDevices().Current().Options()(i).GetName().data()), i);
      else
        inputDeviceOption->Insert(wxString::FromAscii(xorg.InputDevices().Current().Options()(i).GetName().data())+_(" [inactive]"), i);

      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    inputDeviceOption->SetSelection(xorg.InputDevices().Current().Options().GetCurrent());

    inputDeviceOptValue->Clear();

    MyString filename=settings.GetOptionDir()+"/inputdevices/"+xorg.InputDevices().Current().GetDriver()+".xml";
    DriverOptionList list;
    if (list.Load(filename))
    {
      deviceOptValue->SetValue(wxString::FromAscii(xorg.InputDevices().Current().Options().Current().GetValue().data()));
      MyMessage::Append(funcName);
      return(true);
    }

    // search select option in list
    int pos=list.Find(xorg.InputDevices().Current().Options().Current().GetName().data(), true);

    if (pos>=0)
    {
      wxString tempComment=wxString::FromAscii(list(pos).GetComment().data());
      if (!tempComment.IsEmpty())
      {
        inputDeviceOption->GetToolTip()->Enable(true);
        inputDeviceOption->GetToolTip()->SetTip(tempComment);
      }

      if (list(pos).GetValue()=="boolean")
      {
        MyString tempStr=xorg.InputDevices().Current().Options().Current().GetValue().toLower();
        if (tempStr.IsEmpty())
        {
          tempStr=list(pos).GetDefault();
        }

        if (tempStr.IsEmpty() || tempStr=="false" || tempStr=="true")
        {
          inputDeviceOptValue->Append(_T("false"));
          inputDeviceOptValue->Append(_T("true"));
        }
        else if (tempStr=="off" || tempStr=="on")
        {
          inputDeviceOptValue->Append(_T("off"));
          inputDeviceOptValue->Append(_T("on"));
        }
        else if (tempStr=="no" || tempStr=="yes")
        {
          inputDeviceOptValue->Append(_T("no"));
          inputDeviceOptValue->Append(_T("yes"));
        }
        else if (tempStr=="0" || tempStr=="1")
        {
          inputDeviceOptValue->Append(_T("0"));
          inputDeviceOptValue->Append(_T("1"));
        }

        inputDeviceOptValue->SetEditable(false);
      }
      else
      {
        // add arguments
        for (unsigned int i=0; i<list(pos).GetArguments().GetSize(); i++)
        {
          inputDeviceOptValue->Append(wxString::FromAscii(list(pos).GetArguments()(i).data()));
          if (!MyMessage::OK())
          {
            MyMessage::Append(funcName);
            return(true);
          }
        }
      }
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }

      if (list(pos).IsStrict())
      {
        inputDeviceOptValue->SetEditable(false);
      }
    }
    else
    {
      MyString tempStr=xorg.InputDevices().Current().Options().Current().GetValue().toLower();
      if (tempStr=="false" || tempStr=="true")
      {
        inputDeviceOptValue->Append(_T("false"));
        inputDeviceOptValue->Append(_T("true"));
      }
      else if (tempStr=="off" || tempStr=="on")
      {
        inputDeviceOptValue->Append(_T("off"));
        inputDeviceOptValue->Append(_T("on"));
      }
      else if (tempStr=="no" || tempStr=="yes")
      {
        inputDeviceOptValue->Append(_T("no"));
        inputDeviceOptValue->Append(_T("yes"));
      }
      else if (tempStr=="0" || tempStr=="1")
      {
        inputDeviceOptValue->Append(_T("0"));
        inputDeviceOptValue->Append(_T("1"));
      }
    }
    inputDeviceOptValue->SetValue(wxString::FromAscii(xorg.InputDevices().Current().Options().Current().GetValue().data()));
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateMonitor(void)
{
  MyString funcName("GuiFrame::UpdateMonitor");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (monitorIdentifier->GetCount()>(int)devListSize) monitorIdentifier->Delete(0);

  monitorIdentifier->Enable(true);

  if (xorg.Monitors().GetSize()==0)
  {
    monitorHorizSync->Enable(false);
    monitorVertRefresh->Enable(false);
    monitorDisplay->Enable(false);
    monitorGamma->Enable(false);
    monitorIdentifier->SetEditable(false);
    monitorFreqButton->Enable(false);
    monitorDisplaySizeButton->Enable(false);
    monitorHorizSync->SetValue(wxEmptyString);
    monitorVertRefresh->SetValue(wxEmptyString);
    monitorDisplay->SetValue(wxEmptyString);
    monitorGamma->SetValue(wxEmptyString);
    monitorIdentifier->SetSelection(0);
  }
  else
  {
    monitorHorizSync->Enable(true);
    monitorVertRefresh->Enable(true);
    monitorDisplay->Enable(true);
    monitorGamma->Enable(true);
    monitorIdentifier->SetEditable(true);
    monitorFreqButton->Enable(true);
    monitorDisplaySizeButton->Enable(true);
    monitorHorizSync->SetValue(wxString::FromAscii(xorg.Monitors().Current().GetHorSync().data()));
    monitorVertRefresh->SetValue(wxString::FromAscii(xorg.Monitors().Current().GetVertRef().data()));
    monitorDisplay->SetValue(wxString::FromAscii(xorg.Monitors().Current().GetDisplaySize().data()));
    monitorGamma->SetValue(wxString::FromAscii(xorg.Monitors().Current().GetGamma().data()));

    for (unsigned int i=0; i<xorg.Monitors().GetSize(); i++)
    {
      monitorIdentifier->Insert(wxString::FromAscii(xorg.Monitors()(i).GetId().data()), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    monitorIdentifier->SetSelection(xorg.Monitors().GetCurrent());
  }

  if (UpdateMonitorOption())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  if (UpdateMonitorModeline())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateMonitorOption(void)
{
  MyString funcName("GuiFrame::UpdateMonitorOption");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (monitorOption->GetCount()>(int)optListSize) monitorOption->Delete(0);

  if (xorg.Monitors().GetSize()==0) monitorOption->Enable(false);
  else monitorOption->Enable(true);

  if (xorg.Monitors().GetSize()==0 || xorg.Monitors().Current().Options().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
    monitorOptValue->Enable(false);
    monitorOptValue->SetValue(wxEmptyString);
    monitorOption->SetEditable(false);
    monitorOption->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    monitorOptValue->SetValue(wxString::FromAscii(xorg.Monitors().Current().Options().Current().GetValue().data()));
    if (xorg.Monitors().Current().Options().Current().IsActive())
    {
      monitorOptValue->Enable(true);
      monitorOption->SetEditable(true);
    }
    else
    {
      monitorOptValue->Enable(false);
      monitorOption->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Monitors().Current().Options().GetSize(); i++)
    {
      if (xorg.Monitors().Current().Options()(i).IsActive())
        monitorOption->Insert(wxString::FromAscii(xorg.Monitors().Current().Options()(i).GetName().data()), i);
      else
        monitorOption->Insert(wxString::FromAscii(xorg.Monitors().Current().Options()(i).GetName().data())+_(" [inactive]"), i);

      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    monitorOption->SetSelection(xorg.Monitors().Current().Options().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateMonitorModeline(void)
{
  MyString funcName("GuiFrame::UpdateMonitorModeline");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (monitorModelineName->GetCount()>(int)devListSize) monitorModelineName->Delete(0);

  if (xorg.Monitors().GetSize()==0) monitorModelineName->Enable(false);
  else monitorModelineName->Enable(true);

  if (xorg.Monitors().GetSize()==0 || xorg.Monitors().Current().Modelines().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    monitorModelineOption->Enable(false);
    monitorModelineOption->SetValue(wxEmptyString);
    monitorModelineName->SetEditable(false);
    monitorModelineName->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    monitorModelineOption->SetValue(wxString::FromAscii(xorg.Monitors().Current().Modelines().Current().GetValue().data()));

    if (xorg.Monitors().Current().Modelines().Current().IsActive())
    {
      monitorModelineOption->Enable(true);
      monitorModelineName->SetEditable(true);
    }
    else
    {
      monitorModelineOption->Enable(false);
      monitorModelineName->SetEditable(false);
    }

    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Monitors().Current().Modelines().GetSize(); i++)
    {
      if (xorg.Monitors().Current().Modelines()(i).IsActive())
        monitorModelineName->Insert(wxString::FromAscii(xorg.Monitors().Current().Modelines()(i).GetName().data()), i);
      else
        monitorModelineName->Insert(wxString::FromAscii(xorg.Monitors().Current().Modelines()(i).GetName().data())+_(" [inactive]"), i);

      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    monitorModelineName->SetSelection(xorg.Monitors().Current().Modelines().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateScreen(void)
{
  MyString funcName("GuiFrame::UpdateScreen");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (screenIdentifier->GetCount()>(int)devListSize) screenIdentifier->Delete(0);
  screenDevice->Clear();
  screenMonitor->Clear();

  screenIdentifier->Enable(true);

  unsigned int i;
  if (xorg.Screens().GetSize()==0)
  {
    screenDepth->Enable(false);
    screenDevice->Enable(false);
    screenMonitor->Enable(false);
    screenIdentifier->SetEditable(false);
    screenDepth->SetValue(wxEmptyString);
    screenIdentifier->SetSelection(0);
  }
  else
  {
    screenDepth->Enable(true);
    screenDevice->Enable(true);
    screenMonitor->Enable(true);
    screenIdentifier->SetEditable(true);
    screenDepth->SetValue(wxString::FromAscii(xorg.Screens().Current().GetDefaultDepth().data()));
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (i=0; i<xorg.Devices().GetSize(); i++)
    {
      screenDevice->Append(wxString::FromAscii(xorg.Devices()(i).GetId().data()));
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    screenDevice->SetStringSelection(wxString::FromAscii(xorg.Screens().Current().GetDevice().data()));

    for (i=0; i<xorg.Monitors().GetSize(); i++)
    {
      screenMonitor->Append(wxString::FromAscii(xorg.Monitors()(i).GetId().data()));
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    screenMonitor->SetStringSelection(wxString::FromAscii(xorg.Screens().Current().GetMonitor().data()));

    for (i=0; i<xorg.Screens().GetSize(); i++)
    {
      screenIdentifier->Insert(wxString::FromAscii(xorg.Screens()(i).GetId().data()), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    screenIdentifier->SetSelection(xorg.Screens().GetCurrent());
  }

  if (UpdateScreenOption())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  if (UpdateScreenDisplay())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateScreenOption(void)
{
  MyString funcName("GuiFrame::UpdateScreenOption");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (screenOption->GetCount()>(int)optListSize) screenOption->Delete(0);

  if (xorg.Screens().GetSize()==0) screenOption->Enable(false);
  else screenOption->Enable(true);

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Options().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    screenOptValue->Enable(false);
    screenOptValue->SetValue(wxEmptyString);
    screenOption->SetEditable(false);
    screenOption->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    screenOptValue->SetValue(wxString::FromAscii(xorg.Screens().Current().Options().Current().GetValue().data()));
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (xorg.Screens().Current().Options().Current().IsActive())
    {
      screenOptValue->Enable(true);
      screenOption->SetEditable(true);
    }
    else
    {
      screenOptValue->Enable(false);
      screenOption->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Screens().Current().Options().GetSize(); i++)
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }

      if (xorg.Screens().Current().Options()(i).IsActive())
        screenOption->Insert(wxString::FromAscii(xorg.Screens().Current().Options()(i).GetName().data()), i);
      else
        screenOption->Insert(wxString::FromAscii(xorg.Screens().Current().Options()(i).GetName().data())+_(" [inactive]"), i);
    }
    screenOption->SetSelection(xorg.Screens().Current().Options().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateScreenDisplay(void)
{
  MyString funcName("GuiFrame::UpdateScreenDisplay");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (screenDisplayDepth->GetCount()>(int)devListSize-1) screenDisplayDepth->Delete(0);

  if (xorg.Screens().GetSize()==0) screenDisplayDepth->Enable(false);
  else screenDisplayDepth->Enable(true);

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Displays().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    screenDisplayViewport->Enable(false);
    screenDisplayViewport->SetValue(wxEmptyString);
    screenDisplayVirtual->Enable(false);
    screenDisplayVirtual->SetValue(wxEmptyString);
    screenDisplayDepth->SetEditable(false);
    screenDisplayDepth->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    screenDisplayViewport->Enable(true);
    screenDisplayViewport->SetValue(wxString::FromAscii(xorg.Screens().Current().Displays().Current().GetViewPort().data()));
    screenDisplayVirtual->Enable(true);
    screenDisplayVirtual->SetValue(wxString::FromAscii(xorg.Screens().Current().Displays().Current().GetVirtual().data()));
    screenDisplayDepth->SetEditable(true);

    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Screens().Current().Displays().GetSize(); i++)
    {
      screenDisplayDepth->Insert(wxString::FromAscii(xorg.Screens().Current().Displays()(i).GetDepth().data()), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    screenDisplayDepth->SetSelection(xorg.Screens().Current().Displays().GetCurrent());
  }

  if (UpdateScreenDisplayOption())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  if (UpdateScreenDisplayMode())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateScreenDisplayMode(void)
{
  MyString funcName("GuiFrame::UpdateScreenDisplayMode");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (screenDisplayModes->GetCount()>(int)devListSize-1) screenDisplayModes->Delete(0);

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Displays().GetSize()==0) screenDisplayModes->Enable(false);
  else screenDisplayModes->Enable(true);
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Displays().GetSize()==0 || xorg.Screens().Current().Displays().Current().Modes().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    screenDisplayModes->SetEditable(false);
    screenDisplayModes->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    screenDisplayModes->SetEditable(true);
    for (unsigned int i=0; i<xorg.Screens().Current().Displays().Current().Modes().GetSize(); i++)
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      screenDisplayModes->Insert(wxString::FromAscii(xorg.Screens().Current().Displays().Current().Modes()(i).GetName().data()), i);
    }
    screenDisplayModes->SetSelection(xorg.Screens().Current().Displays().Current().Modes().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateScreenDisplayOption(void)
{
  MyString funcName("GuiFrame::UpdateScreenDisplayOption");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (screenDisplayOption->GetCount()>(int)optListSize) screenDisplayOption->Delete(0);

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Displays().GetSize()==0) screenDisplayOption->Enable(false);
  else screenDisplayOption->Enable(true);
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Displays().GetSize()==0 || xorg.Screens().Current().Displays().Current().Options().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    screenDisplayOptValue->Enable(false);
    screenDisplayOptValue->SetValue(wxEmptyString);
    screenDisplayOption->SetEditable(false);
    screenDisplayOption->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    screenDisplayOptValue->SetValue(wxString::FromAscii(xorg.Screens().Current().Displays().Current().Options().Current().GetValue().data()));

    if (xorg.Screens().Current().Displays().Current().Options().Current().IsActive())
    {
      screenDisplayOptValue->Enable(true);
      screenDisplayOption->SetEditable(true);
    }
    else
    {
      screenDisplayOptValue->Enable(false);
      screenDisplayOption->SetEditable(false);
    }

    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Screens().Current().Displays().Current().Options().GetSize(); i++)
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      if (xorg.Screens().Current().Displays().Current().Options()(i).IsActive())
        screenDisplayOption->Insert(wxString::FromAscii(xorg.Screens().Current().Displays().Current().Options()(i).GetName().data()), i);
      else
        screenDisplayOption->Insert(wxString::FromAscii(xorg.Screens().Current().Displays().Current().Options()(i).GetName().data())+_(" [inactive]"), i);
    }
    screenDisplayOption->SetSelection(xorg.Screens().Current().Displays().Current().Options().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateServerLayout(void)
{
  MyString funcName("GuiFrame::UpdateServerLayout");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (serverLayoutIdentifier->GetCount()>(int)devListSize) serverLayoutIdentifier->Delete(0);

  serverLayoutIdentifier->Enable(true);

  if (xorg.ServerLayouts().GetSize()==0)
  {
    serverLayoutIdentifier->SetEditable(false);
    serverLayoutIdentifier->SetSelection(0);
  }
  else
  {
    serverLayoutIdentifier->SetEditable(true);
    for (unsigned int i=0; i<xorg.ServerLayouts().GetSize(); i++)
    {
      serverLayoutIdentifier->Insert(wxString::FromAscii(xorg.ServerLayouts()(i).GetId().data()), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    serverLayoutIdentifier->SetSelection(xorg.ServerLayouts().GetCurrent());
  }

  if (UpdateServerLayoutScreen())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  if (UpdateServerLayoutInputDevice())
  {
    MyMessage::Append(funcName);
    return(true);
  }
  if (UpdateServerLayoutOption())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateServerLayoutOption(void)
{
  MyString funcName("GuiFrame::UpdateServerLayoutOption");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (serverLayoutOption->GetCount()>(int)optListSize) serverLayoutOption->Delete(0);

  if (xorg.ServerLayouts().GetSize()==0) serverLayoutOption->Enable(false);
  else serverLayoutOption->Enable(true);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().Options().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
    serverLayoutOptValue->Enable(false);
    serverLayoutOptValue->SetValue(wxEmptyString);
    serverLayoutOption->SetEditable(false);
    serverLayoutOption->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    serverLayoutOptValue->SetValue(wxString::FromAscii(xorg.ServerLayouts().Current().Options().Current().GetValue().data()));

    if (xorg.ServerLayouts().Current().Options().Current().IsActive())
    {
      serverLayoutOptValue->Enable(true);
      serverLayoutOption->SetEditable(true);
    }
    else
    {
      serverLayoutOptValue->Enable(false);
      serverLayoutOption->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.ServerLayouts().Current().Options().GetSize(); i++)
    {
      if (xorg.ServerLayouts().Current().Options()(i).IsActive())
        serverLayoutOption->Insert(wxString::FromAscii(xorg.ServerLayouts().Current().Options()(i).GetName().data()), i);
      else
        serverLayoutOption->Insert(wxString::FromAscii(xorg.ServerLayouts().Current().Options()(i).GetName().data())+_(" [inactive]"), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    serverLayoutOption->SetSelection(xorg.ServerLayouts().Current().Options().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateServerLayoutScreen(void)
{
  MyString funcName("GuiFrame::UpdateServerLayoutScreen");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (serverLayoutScreen->GetCount()>(int)devListSize-1) serverLayoutScreen->Delete(0);

  if (xorg.ServerLayouts().GetSize()==0) serverLayoutScreen->Enable(false);
  else serverLayoutScreen->Enable(true);

  serverLayoutScreen1->Clear();
  serverLayoutScreen2->Clear();

  unsigned int i;

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().Screens().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
    serverLayoutScreen1->Enable(false);
    serverLayoutScreen2->Enable(false);
    serverLayoutScreenNumber->Enable(false);
    serverLayoutScreenPosition->Enable(false);
    serverLayoutScreenNumber->SetValue(wxEmptyString);
    serverLayoutScreenPosition->SetValue(wxEmptyString);
    serverLayoutScreen->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
    serverLayoutScreenNumber->Enable(true);
    serverLayoutScreenPosition->Enable(true);
    serverLayoutScreen1->Enable(true);
    serverLayoutScreen2->Enable(true);

    for (i=0; i<xorg.Screens().GetSize(); i++)
    {
      serverLayoutScreen1->Append(wxString::FromAscii(xorg.Screens()(i).GetId().data()));
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    serverLayoutScreen1->SetStringSelection(wxString::FromAscii(xorg.ServerLayouts().Current().Screens().Current().GetId().data()));

    serverLayoutScreenNumber->SetValue(wxString::FromAscii(xorg.ServerLayouts().Current().Screens().Current().GetNumber().data()));
    serverLayoutScreenPosition->SetValue(wxString::FromAscii(xorg.ServerLayouts().Current().Screens().Current().GetPosition().data()));
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (i=0; i<xorg.Screens().GetSize(); i++)
    {
      serverLayoutScreen2->Append(wxString::FromAscii(xorg.Screens()(i).GetId().data()));
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }

    wxString tempStr=serverLayoutScreenPosition->GetValue();
    tempStr.Trim(false);
    tempStr.Trim(true);
    tempStr.MakeLower();

    if (tempStr==_T("rightof") || tempStr==_T("leftof") || tempStr==_T("above") || tempStr==_T("below"))
    {
      serverLayoutScreen2->Enable(true);
      serverLayoutScreen2->SetStringSelection(wxString::FromAscii(xorg.ServerLayouts().Current().Screens().Current().GetSecond().data()));
    }
    else
    {
      serverLayoutScreen2->Enable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (i=0; i<xorg.ServerLayouts().Current().Screens().GetSize(); i++)
    {
      wxString tempStr;
      tempStr << _("Screen") << _T(" ") << i;
      serverLayoutScreen->Insert(tempStr, i);
    }
    serverLayoutScreen->SetSelection(xorg.ServerLayouts().Current().Screens().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateServerLayoutInputDevice(void)
{
  MyString funcName("GuiFrame::UpdateServerLayoutInputDevice");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (serverLayoutInputDevice->GetCount()>(int)devListSize-1) serverLayoutInputDevice->Delete(0);

  if (xorg.ServerLayouts().GetSize()==0) serverLayoutInputDevice->Enable(false);
  else serverLayoutInputDevice->Enable(true);

  serverLayoutInputDevice1->Clear();

  unsigned int i;

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().InputDevices().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
    serverLayoutInputDevice1->Enable(false);
    serverLayoutInputDevice->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    serverLayoutInputDevice1->Enable(true);

    for (i=0; i<xorg.InputDevices().GetSize(); i++)
    {
      serverLayoutInputDevice1->Append(wxString::FromAscii(xorg.InputDevices()(i).GetId().data()));
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    serverLayoutInputDevice1->SetStringSelection(wxString::FromAscii(xorg.ServerLayouts().Current().InputDevices().Current().GetId().data()));

    for (i=0; i<xorg.ServerLayouts().Current().InputDevices().GetSize(); i++)
    {
      wxString tempStr;
      tempStr << _("Input Device") << _T(" ") << i;
      serverLayoutInputDevice->Insert(tempStr, i);
    }
    serverLayoutInputDevice->SetSelection(xorg.ServerLayouts().Current().InputDevices().GetCurrent());
  }

  if (UpdateServerLayoutInputDeviceOption())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateServerLayoutInputDeviceOption(void)
{
  MyString funcName("GuiFrame::UpdateServerLayoutInputDeviceOption");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (serverLayoutInputDeviceOption->GetCount()>(int)optListSize-1) serverLayoutInputDeviceOption->Delete(0);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().InputDevices().GetSize()==0) serverLayoutInputDeviceOption->Enable(false);
  else serverLayoutInputDeviceOption->Enable(true);
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().InputDevices().GetSize()==0 || xorg.ServerLayouts().Current().InputDevices().Current().Options().GetSize()==0)
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
    serverLayoutInputDeviceOption->SetEditable(false);
    serverLayoutInputDeviceOption->SetSelection(0);
  }
  else
  {
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
    serverLayoutInputDeviceOption->SetEditable(true);

    for (unsigned int i=0; i<xorg.ServerLayouts().Current().InputDevices().Current().Options().GetSize(); i++)
    {
      serverLayoutInputDeviceOption->Insert(wxString::FromAscii(xorg.ServerLayouts().Current().InputDevices().Current().Options()(i).GetName().data()), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    serverLayoutInputDeviceOption->SetSelection(xorg.ServerLayouts().Current().InputDevices().Current().Options().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateServerFlagOption(void)
{
  MyString funcName("GuiFrame::UpdateServerFlagOption");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (serverFlagsOption->GetCount()>(int)devListSize) serverFlagsOption->Delete(0);

  serverFlagsOption->Enable(true);

  if (xorg.ServerFlags().GetSize()==0)
  {
    serverFlagsOptValue->Enable(false);
    serverFlagsOptValue->SetValue(wxEmptyString);
    serverFlagsOption->SetEditable(false);
    serverFlagsOption->SetSelection(0);
  }
  else
  {
    serverFlagsOptValue->SetValue(wxString::FromAscii(xorg.ServerFlags().Current().GetValue().data()));
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (xorg.ServerFlags().Current().IsActive())
    {
      serverFlagsOptValue->Enable(true);
      serverFlagsOption->SetEditable(true);
    }
    else
    {
      serverFlagsOptValue->Enable(false);
      serverFlagsOption->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.ServerFlags().GetSize(); i++)
    {
      if (xorg.ServerFlags()(i).IsActive())
        serverFlagsOption->Insert(wxString::FromAscii(xorg.ServerFlags()(i).GetName().data()), i);
      else
        serverFlagsOption->Insert(wxString::FromAscii(xorg.ServerFlags()(i).GetName().data())+_(" [inactive]"), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    serverFlagsOption->SetSelection(xorg.ServerFlags().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateExtension(void)
{
  MyString funcName("GuiFrame::UpdateExtension");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (extensionOption->GetCount()>(int)devListSize) extensionOption->Delete(0);

  extensionOption->Enable(true);

  if (xorg.Extensions().GetSize()==0)
  {
    extensionOptValue->Enable(false);
    extensionOptValue->SetValue(wxEmptyString);
    extensionOption->SetEditable(false);
    extensionOption->SetSelection(0);
  }
  else
  {
    extensionOptValue->SetValue(wxString::FromAscii(xorg.Extensions().Current().GetValue().data()));
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (xorg.Extensions().Current().IsActive())
    {
      extensionOptValue->Enable(true);
      extensionOption->SetEditable(true);
    }
    else
    {
      extensionOptValue->Enable(false);
      extensionOption->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Extensions().GetSize(); i++)
    {
      if (xorg.Extensions()(i).IsActive())
        extensionOption->Insert(wxString::FromAscii(xorg.Extensions()(i).GetName().data()), i);
      else
        extensionOption->Insert(wxString::FromAscii(xorg.Extensions()(i).GetName().data())+_(" [inactive]"), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    extensionOption->SetSelection(xorg.Extensions().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateFontPath(void)
{
  MyString funcName("GuiFrame::UpdateFontPath");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (fontPath->GetCount()>(int)devListSize) fontPath->Delete(0);

  fontPath->Enable(true);

  if (xorg.Files().FontPaths().GetSize()==0)
  {
    fontPath->SetEditable(false);
    fontPath->SetSelection(0);
  }
  else
  {
    if (xorg.Files().FontPaths().Current().IsActive())
    {
      fontPath->SetEditable(true);
    }
    else
    {
      fontPath->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Files().FontPaths().GetSize(); i++)
    {
      if (xorg.Files().FontPaths()(i).IsActive())
        fontPath->Insert(wxString::FromAscii(xorg.Files().FontPaths()(i).GetName().data()), i);
      else
        fontPath->Insert(wxString::FromAscii(xorg.Files().FontPaths()(i).GetName().data())+_(" [inactive]"), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    fontPath->SetSelection(xorg.Files().FontPaths().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateModulePath(void)
{
  MyString funcName("GuiFrame::UpdateModulePath");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (modulePath->GetCount()>(int)devListSize) modulePath->Delete(0);

  modulePath->Enable(true);

  if (xorg.Files().ModulePaths().GetSize()==0)
  {
    modulePath->SetEditable(false);
    modulePath->SetSelection(0);
  }
  else
  {
    if (xorg.Files().ModulePaths().Current().IsActive())
    {
      modulePath->SetEditable(true);
    }
    else
    {
      modulePath->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Files().ModulePaths().GetSize(); i++)
    {
      if (xorg.Files().ModulePaths()(i).IsActive())
        modulePath->Insert(wxString::FromAscii(xorg.Files().ModulePaths()(i).GetName().data()), i);
      else
        modulePath->Insert(wxString::FromAscii(xorg.Files().ModulePaths()(i).GetName().data())+_(" [inactive]"), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    modulePath->SetSelection(xorg.Files().ModulePaths().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateRgbPath(void)
{
  MyString funcName("GuiFrame::UpdateRgbPath");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (rgbPath->GetCount()>(int)devListSize) rgbPath->Delete(0);

  rgbPath->Enable(true);

  if (xorg.Files().RgbPaths().GetSize()==0)
  {
    rgbPath->SetEditable(false);
    rgbPath->SetSelection(0);
  }
  else
  {
    if (xorg.Files().RgbPaths().Current().IsActive())
    {
      rgbPath->SetEditable(true);
    }
    else
    {
      rgbPath->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Files().RgbPaths().GetSize(); i++)
    {
      if (xorg.Files().RgbPaths()(i).IsActive())
        rgbPath->Insert(wxString::FromAscii(xorg.Files().RgbPaths()(i).GetName().data()), i);
      else
        rgbPath->Insert(wxString::FromAscii(xorg.Files().RgbPaths()(i).GetName().data())+_(" [inactive]"), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    rgbPath->SetSelection(xorg.Files().RgbPaths().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateModule(void)
{
  MyString funcName("GuiFrame::UpdateModule");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (moduleIdentifier->GetCount()>(int)devListSize) moduleIdentifier->Delete(0);

  moduleIdentifier->Enable(true);

  if (xorg.Modules().GetSize()==0)
  {
    moduleIdentifier->SetEditable(false);
    moduleIdentifier->SetSelection(0);
  }
  else
  {
    if (xorg.Modules().Current().IsActive())
    {
      moduleIdentifier->SetEditable(true);
    }
    else
    {
      moduleIdentifier->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Modules().GetSize(); i++)
    {
      if (xorg.Modules()(i).IsActive())
        moduleIdentifier->Insert(wxString::FromAscii(xorg.Modules()(i).GetName().data()), i);
      else
        moduleIdentifier->Insert(wxString::FromAscii(xorg.Modules()(i).GetName().data())+_(" [inactive]"), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    moduleIdentifier->SetSelection(xorg.Modules().GetCurrent());
  }

  if (UpdateModuleOption())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateModuleOption(void)
{
  MyString funcName("GuiFrame::UpdateModuleOption");
  WriteDebug(funcName);

  bool dummy=isSaved;

  while (moduleOption->GetCount()>(int)optListSize) moduleOption->Delete(0);

  if (xorg.Modules().GetSize()==0 || !xorg.Modules().Current().IsActive()) moduleOption->Enable(false);
  else moduleOption->Enable(true);
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Modules().GetSize()==0 || xorg.Modules().Current().Options().GetSize()==0)
  {
    moduleOptValue->Enable(false);
    moduleOptValue->SetValue(wxEmptyString);
    moduleOption->SetEditable(false);
    moduleOption->SetSelection(0);
  }
  else
  {
    moduleOptValue->SetValue(wxString::FromAscii(xorg.Modules().Current().Options().Current().GetValue().data()));
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (xorg.Modules().Current().IsActive() && xorg.Modules().Current().Options().Current().IsActive())
    {
      moduleOptValue->Enable(true);
      moduleOption->SetEditable(true);
    }
    else
    {
      moduleOptValue->Enable(false);
      moduleOption->SetEditable(false);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    for (unsigned int i=0; i<xorg.Modules().Current().Options().GetSize(); i++)
    {
      if (xorg.Modules().Current().Options()(i).IsActive())
        moduleOption->Insert(wxString::FromAscii(xorg.Modules().Current().Options()(i).GetName().data()), i);
      else
        moduleOption->Insert(wxString::FromAscii(xorg.Modules().Current().Options()(i).GetName().data())+_(" [inactive]"), i);
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    moduleOption->SetSelection(xorg.Modules().Current().Options().GetCurrent());
  }

  SetSaved(dummy);

  return(false);
}

const bool GuiFrame::UpdateDRI(void)
{
  MyString funcName("GuiFrame::UpdateDRI");
  WriteDebug(funcName);

  bool dummy=isSaved;

  driMode->SetValue(wxString::FromAscii(xorg.Dri().GetMode().data()));

  SetSaved(dummy);

  return(false);
}

// *************************
// Buttons
// *************************

void GuiFrame::GetRefreshRate(wxCommandEvent& event)
{  // get refresh rate with ddcprobe

  MyString funcName("GuiFrame::GetRefreshRate");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0) return;

  wxArrayString output;  // output of ddcprobe
  unsigned int i;                // loop counter

  if (wxExecute(_T("ddcprobe"), output, wxEXEC_SYNC)!=0)
  {
    // ddcprobe could not be executed
    wxMessageBox(_("Sorry, could not determine refresh rate."), _("No result!"), wxICON_ERROR);
    return;
  }

  // search for "monitorrange"
  for (i=0; i<output.GetCount(); i++)
  {
    if (output[i].StartsWith(_T("monitorrange:")))
    {
      MyString range=(const char*)output[i].mb_str();
      range.StripRight(13);

      int pos=range.Find(',');
      if (pos==-1)
      {
        // no comma found
        wxMessageBox(_("Sorry, could not determine refresh rate."), _("No result!"), wxICON_ERROR);
        return;
      }

      MyString hor=range.Left(pos);
      hor.StripSpaces();
      MyString vert=range.Right(range.Length()-pos-1);
      vert.StripSpaces();

      xorg.Monitors().Current().SetHorSync(hor);
      xorg.Monitors().Current().SetVertRef(vert);

      if (UpdateMonitor())
      {
        if (ShowError()) return;
      }

      break;
    }
  }

  if (i==output.GetCount())
  {
    // monitorrange could not be found
    wxMessageBox(_("Sorry, could not determine refresh rate."), _("No result!"), wxICON_ERROR);
    return;
  }
}

void GuiFrame::GetDisplaySize(wxCommandEvent& event)
{  // calculate display size with xdpyinfo

  MyString funcName("GuiFrame::GetDisplaySize");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0) return;

  wxArrayString output;  // output of xdpyinfo
  unsigned int i;        // loop counter

  if (wxExecute(_T("xdpyinfo"), output, wxEXEC_SYNC)!=0)
  {
    // xdpyinfo could not be executed
    wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
    return;
  }

  // search for resolution and screen size
  for (i=0; i<output.GetCount(); i++)
  {
    if (output[i].StartsWith(_T("  dimensions:")))
    {
      MyString line=(const char*)output[i].mb_str();
      line.StripRight(13);

      int pos=line.Find("pixels");
      if (pos==-1)
      {
        // no pixel entry found
        wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
        return;
      }

      MyString resolution=line.Left(pos);
      resolution.StripSpaces();

      line.StripRight(pos+6);  // remove pixels

      if (line.IsEmpty() || line[0]!='(' || line.Last()!=')')
      {
        // no (...) found
        wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
      }

      line=line.Mid(1, line.Length()-2);
      line.StripSpaces();

      pos=line.Find("millimeters");
      if (pos==-1)
      {
        // no pixel entry found
        wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
        return;
      }

      MyString size=line.Left(pos);
      size.StripSpaces();

      pos=resolution.Find('x');
      if (pos==-1)
      {
        // no pixel entry found
        wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
        return;
      }

      MyString resHor=resolution.Left(pos);
      MyString resVer=resolution.Right(resolution.Length()-pos-1);

      pos=size.Find('x');
      if (pos==-1)
      {
        // no pixel entry found
        wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
        return;
      }

      MyString sizeHor=size.Left(pos);
      MyString sizeVer=size.Right(size.Length()-pos-1);

      // Note: resolution is unimportant here... will be used later for DPI maybe.
      xorg.Monitors().Current().SetDisplaySize(sizeHor+" "+sizeVer);

      if (UpdateMonitor())
      {
        if (ShowError()) return;
      }

      break;
    }
  }

  if (i==output.GetCount())
  {
    // resolution could not be found
    wxMessageBox(_("Sorry, could not determine display size."), _("No result!"), wxICON_ERROR);
    return;
  }
}

void GuiFrame::GenerateModeline(wxCommandEvent& event)
{  // generate modeline with gtf

  MyString funcName("GuiFrame::GenerateModeline");
  WriteDebug(funcName);

  monitorModelineGeneratorOutput1->SetValue(wxEmptyString);
  monitorModelineGeneratorOutput2->SetValue(wxEmptyString);

  if (xorg.Monitors().GetSize()==0) return;

  wxArrayString output;  // output of gtf
  unsigned int i;        // loop counter

  // check fields
  if (monitorModelineGeneratorXRes->GetValue().IsEmpty() || monitorModelineGeneratorYRes->GetValue().IsEmpty() || monitorModelineGeneratorFreq->GetValue().IsEmpty())
  {
    wxMessageBox(_("Please fill out all fields!"), _("Error generating modeline"), wxICON_ERROR);
    return;
  }


  if (wxExecute(_T("gtf ")+monitorModelineGeneratorXRes->GetValue()+_T(" ")+monitorModelineGeneratorYRes->GetValue()+_T(" ")+monitorModelineGeneratorFreq->GetValue(), output, wxEXEC_SYNC)!=0)
  {
    // gtf could not be executed
    wxMessageBox(_("Sorry, could not generate modeline."), _("No result!"), wxICON_ERROR);
    return;
  }

  // search for resolution and screen size
  for (i=0; i<output.GetCount(); i++)
  {
    if (output[i].StartsWith(_T("  Modeline")))
    {
      MyString line=(const char*)output[i].mb_str();
      line.StripRight(10);  // delete "Modeline"

      int pos=line.Find('"');
      if (pos!=0)
      {
        // no " found
        wxMessageBox(_("Sorry, could not generate modeline."), _("No result!"), wxICON_ERROR);
        return;
      }
      line.StripRight(1); // delete "

      pos=line.Find('"');
      if (pos==-1)
      {
        // no second " found
        wxMessageBox(_("Sorry, could not generate modeline."), _("No result!"), wxICON_ERROR);
        return;
      }

      MyString mode=line.Left(pos);
      mode.StripSpaces();

      line.StripRight(pos+1);

      monitorModelineGeneratorOutput1->SetValue(wxString::FromAscii(mode.data()));
      monitorModelineGeneratorOutput2->SetValue(wxString::FromAscii(line.data()));

      break;
    }
  }

  if (i==output.GetCount())
  {
    // resolution could not be found
    wxMessageBox(_("Sorry, could not generate modeline."), _("No result!"), wxICON_ERROR);
    return;
  }
}

void GuiFrame::AddModeline(wxCommandEvent& event)
{  // generate modeline with gtf and add it

  MyString funcName("GuiFrame::AddModeline");
  WriteDebug(funcName);

  monitorModelineGeneratorOutput1->SetValue(wxEmptyString);
  monitorModelineGeneratorOutput2->SetValue(wxEmptyString);

  if (xorg.Monitors().GetSize()==0) return;

  GenerateModeline(event);  // generate modeline

  if (monitorModelineGeneratorOutput1->GetValue().IsEmpty() ||
      monitorModelineGeneratorOutput2->GetValue().IsEmpty()) return;  // no modeline calculated

  if (xorg.Monitors().Current().Modelines().Current().Check())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  xorg.Monitors().Current().Modelines().Append(Option((const char*)monitorModelineGeneratorOutput1->GetValue().mb_str(), (const char*)monitorModelineGeneratorOutput2->GetValue().mb_str()));
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  if (xorg.Monitors().Current().Modelines().SetLast())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

  SetSaved(false);

  if (SetFrame())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

// *****************************
// Selection
// a) only select
// *****************************

void GuiFrame::SelectScreenDevice(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectScreenDevice");
  WriteDebug(funcName);

  MyString newString=(const char*)screenDevice->GetStringSelection().mb_str();
  if (xorg.Screens().Current().GetDevice()!=newString)
  {
    xorg.Screens().Current().SetDevice(newString);
    SetSaved(false);
  }
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectScreenMonitor(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectScreenMonitor");
  WriteDebug(funcName);

  MyString newString=(const char*)screenMonitor->GetStringSelection().mb_str();
  if (xorg.Screens().Current().GetMonitor()!=newString)
  {
    xorg.Screens().Current().SetMonitor(newString);
    SetSaved(false);
  }
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectServerLayoutScreen1(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectServerLayoutScreen1");
  WriteDebug(funcName);

  MyString newString=(const char*)serverLayoutScreen1->GetStringSelection().mb_str();
  if (xorg.ServerLayouts().Current().Screens().Current().GetId()!=newString)
  {
    xorg.ServerLayouts().Current().Screens().Current().SetId(newString);
    SetSaved(false);
  }
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectServerLayoutScreen2(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectServerLayoutScreen2");
  WriteDebug(funcName);

  MyString newString=(const char*)serverLayoutScreen2->GetStringSelection().mb_str();
  if (serverLayoutScreen2->IsEnabled())
  {
    if (xorg.ServerLayouts().Current().Screens().Current().GetSecond()!=newString)
    {
      xorg.ServerLayouts().Current().Screens().Current().SetSecond(newString);
      SetSaved(false);
    }
  }
  else
  {
    if (!xorg.ServerLayouts().Current().Screens().Current().GetSecond().IsEmpty())
    {
      xorg.ServerLayouts().Current().Screens().Current().SetSecond("");
      SetSaved(false);
    }
  }
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectServerLayoutInputDevice1(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectServerLayoutInputDevice1");
  WriteDebug(funcName);

  MyString newString=(const char*)serverLayoutInputDevice1->GetStringSelection().mb_str();
  if (xorg.ServerLayouts().Current().InputDevices().Current().GetId()!=newString)
  {
    xorg.ServerLayouts().Current().InputDevices().Current().SetId(newString);
    SetSaved(false);
  }
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

// *****************************
// Selection
// b) add/del
// *****************************

void GuiFrame::SelectDevice(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectDevice");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Devices().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Devices().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddDevice())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateDevice())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelDevice())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (CopyDevice())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateDevice())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
  }
  else
  {
    if (xorg.Devices().Current().Check())
    {
      // save error
      messageIdentifier tempMessage=MyMessage::Get();
      MyMessage::Reset();       // we hope that it isn't a real error

      if (UpdateDevice())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
      MyMessage::Set(tempMessage, funcName);
      if (ShowError()) return;
    }

    // extended check
    MyString filename;
    DriverOptionList list;

    filename=settings.GetOptionDir()+"/devices/"+xorg.Devices().Current().GetDriver()+".xml";
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }

    if (list.Load(filename))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }

    if (xorg.Devices().Current().Check(list))
    {
      // save error
      messageIdentifier tempMessage=MyMessage::Get();
      MyMessage::Reset();       // we hope that it isn't a real error

      if (UpdateDevice())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
      MyMessage::Set(tempMessage, funcName);
      if (ShowError()) return;
    }

    if (xorg.Devices().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateDevice())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }

}

void GuiFrame::SelectDeviceOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectDeviceOption");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Devices().Current().Options().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Devices().Current().Options().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddDeviceOption())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateDeviceOption())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (AddDeviceDriverOptions())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateDeviceOption())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DelDeviceOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==4)
    {
      if (DeActDeviceOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Devices().Current().Options().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateDeviceOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Devices().Current().Options().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateDeviceOption())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectInputDevice(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectInputDevice");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.InputDevices().GetSize())  // special command (add, delete)
  {
    selection-=xorg.InputDevices().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddInputDevice())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateInputDevice())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelInputDevice())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (CopyInputDevice())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateInputDevice())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
  }
  else
  {
    if (xorg.InputDevices().Current().Check())
    {
      // save error
      messageIdentifier tempMessage=MyMessage::Get();
      MyMessage::Reset();       // we hope that it isn't a real error

      if (UpdateInputDevice())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
      MyMessage::Set(tempMessage, funcName);
      if (ShowError()) return;
    }

    // extended check
    MyString filename;
    DriverOptionList list;

    filename=settings.GetOptionDir()+"/inputdevices/"+xorg.InputDevices().Current().GetDriver()+".xml";
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }

    if (list.Load(filename))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }

    if (xorg.InputDevices().Current().Check(list))
    {
      // save error
      messageIdentifier tempMessage=MyMessage::Get();
      MyMessage::Reset();       // we hope that it isn't a real error

      if (UpdateInputDevice())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
      MyMessage::Set(tempMessage, funcName);
      if (ShowError()) return;
    }

    if (xorg.InputDevices().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
  if (UpdateInputDevice())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectInputDeviceOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectInputDeviceOption");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.InputDevices().Current().Options().GetSize())  // special command (add, delete)
  {
    selection-=xorg.InputDevices().Current().Options().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddInputDeviceOption())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateInputDeviceOption())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (AddInputDeviceDriverOptions())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateInputDeviceOption())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DelInputDeviceOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==4)
    {
      if (DeActInputDeviceOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.InputDevices().Current().Options().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateInputDeviceOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.InputDevices().Current().Options().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateInputDeviceOption())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectMonitor(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectMonitor");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Monitors().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Monitors().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddMonitor())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateMonitor())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelMonitor())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (CopyMonitor())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateMonitor())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Monitors().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateMonitor())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Monitors().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateMonitor())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectMonitorOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectMonitorOption");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Monitors().Current().Options().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Monitors().Current().Options().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddMonitorOption())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateMonitorOption())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelMonitorOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActMonitorOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Monitors().Current().Options().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateMonitorOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Monitors().Current().Options().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateMonitorOption())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectMonitorModeline(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectMonitorModeline");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Monitors().Current().Modelines().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Monitors().Current().Modelines().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddMonitorModeline())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateMonitorModeline())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelMonitorModeline())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActMonitorModeline())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Monitors().Current().Modelines().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateMonitorModeline())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Monitors().Current().Modelines().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
  if (UpdateMonitorModeline())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectScreen(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectScreen");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Screens().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Screens().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddScreen())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateScreen())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelScreen())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (CopyScreen())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateScreen())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Screens().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateScreen())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Screens().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
  if (UpdateScreen())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectScreenOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectScreenOption");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Screens().Current().Options().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Screens().Current().Options().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddScreenOption())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateScreenOption())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelScreenOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActScreenOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Screens().Current().Options().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateScreenOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Screens().Current().Options().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
  if (UpdateScreenOption())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectScreenDisplay(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectScreenDisplay");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Screens().Current().Displays().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Screens().Current().Displays().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddScreenDisplay())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateScreenDisplay())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelScreenDisplay())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Screens().Current().Displays().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateScreenDisplay())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Screens().Current().Displays().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
  if (UpdateScreenDisplay())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectScreenDisplayMode(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectScreenDisplayMode");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Screens().Current().Displays().Current().Modes().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Screens().Current().Displays().Current().Modes().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddScreenDisplayMode())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateScreenDisplayMode())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelScreenDisplayMode())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Screens().Current().Displays().Current().Modes().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateScreenDisplayMode())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Screens().Current().Displays().Current().Modes().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
  if (UpdateScreenDisplayMode())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectScreenDisplayOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectScreenDisplayOption");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Screens().Current().Displays().Current().Options().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Screens().Current().Displays().Current().Options().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddScreenDisplayOption())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateScreenDisplayOption())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelScreenDisplayOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActScreenDisplayOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Screens().Current().Displays().Current().Options().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateScreenDisplayOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Screens().Current().Displays().Current().Options().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateScreenDisplayOption())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectServerLayout(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectServerLayout");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerLayouts().GetSize())  // special command (add, delete)
  {
    selection-=xorg.ServerLayouts().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddServerLayout())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateServerLayout())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelServerLayout())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (CopyServerLayout())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateServerLayout())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.ServerLayouts().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateServerLayout())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.ServerLayouts().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateServerLayout())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectServerLayoutScreen(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectServerLayoutScreen");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerLayouts().Current().Screens().GetSize())  // special command (add, delete)
  {
    selection-=xorg.ServerLayouts().Current().Screens().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddServerLayoutScreen())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateServerLayoutScreen())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelServerLayoutScreen())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.ServerLayouts().Current().Screens().Current().Check(true))
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateServerLayoutScreen())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.ServerLayouts().Current().Screens().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateServerLayoutScreen())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectServerLayoutInputDevice(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectServerLayoutInputDevice");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerLayouts().Current().InputDevices().GetSize())  // special command (add, delete)
  {
    selection-=xorg.ServerLayouts().Current().InputDevices().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddServerLayoutInputDevice())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateServerLayoutInputDevice())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelServerLayoutInputDevice())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.ServerLayouts().Current().InputDevices().Current().Check(true))
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateServerLayoutInputDevice())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.ServerLayouts().Current().InputDevices().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateServerLayoutInputDevice())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectServerLayoutOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectServerLayoutOption");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerLayouts().Current().Options().GetSize())  // special command (add, delete)
  {
    selection-=xorg.ServerLayouts().Current().Options().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddServerLayoutOption())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateServerLayoutOption())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelServerLayoutOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActServerLayoutOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.ServerLayouts().Current().Options().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateServerLayoutOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.ServerLayouts().Current().Options().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateServerLayoutOption())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectServerLayoutInputDeviceOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectServerLayoutInputDeviceOption");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerLayouts().Current().InputDevices().Current().Options().GetSize())  // special command (add, delete)
  {
    selection-=xorg.ServerLayouts().Current().InputDevices().Current().Options().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddServerLayoutInputDeviceOption())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateServerLayoutInputDeviceOption())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelServerLayoutInputDeviceOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.ServerLayouts().Current().InputDevices().Current().Options().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateServerLayoutInputDeviceOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.ServerLayouts().Current().InputDevices().Current().Options().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateServerLayoutInputDeviceOption())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectServerFlagOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectServerFlagOption");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.ServerFlags().GetSize())  // special command (add, delete)
  {
    selection-=xorg.ServerFlags().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddServerFlagOption())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateServerFlagOption())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelServerFlagOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActServerFlagOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.ServerFlags().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateServerFlagOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.ServerFlags().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateServerFlagOption())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectExtension(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectExtension");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Extensions().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Extensions().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddExtension())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateExtension())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelExtension())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActExtension())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Extensions().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateExtension())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Extensions().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateExtension())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectFontPath(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectFontPath");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Files().FontPaths().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Files().FontPaths().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddFontPath())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateFontPath())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelFontPath())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActFontPath())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Files().FontPaths().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateFontPath())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Files().FontPaths().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateFontPath())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectModulePath(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectModulePath");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Files().ModulePaths().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Files().ModulePaths().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddModulePath())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateModulePath())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelModulePath())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActModulePath())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Files().ModulePaths().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateModulePath())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Files().ModulePaths().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateModulePath())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectRgbPath(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectRgbPath");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Files().RgbPaths().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Files().RgbPaths().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddRgbPath())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateRgbPath())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelRgbPath())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActRgbPath())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Files().RgbPaths().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateRgbPath())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Files().RgbPaths().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateRgbPath())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectModule(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectModule");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Modules().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Modules().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddModule())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateModule())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelModule())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActModule())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Modules().Current().Check())
  {
    // save error
    messageIdentifier tempMessage=MyMessage::Get();
    MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateModule())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Modules().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if (UpdateModule())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

void GuiFrame::SelectModuleOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::SelectModuleOption");
  WriteDebug(funcName);

  unsigned int selection=event.GetSelection();  // get selected item

  if (selection>=xorg.Modules().Current().Options().GetSize())  // special command (add, delete)
  {
    selection-=xorg.Modules().Current().Options().GetSize();   // subtract, so that we only have cases 0,1,2

    if (selection==1)
    {
      if (AddModuleOption())
      {
        // save error
        messageIdentifier tempMessage=MyMessage::Get();
        MyMessage::Reset();       // we hope that it isn't a real error

        if (UpdateModuleOption())
        {
          MyMessage::Append(funcName);
          if (ShowError()) return;
        }
        MyMessage::Set(tempMessage, funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==2)
    {
      if (DelModuleOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
    else if (selection==3)
    {
      if (DeActModuleOption())
      {
        MyMessage::Append(funcName);
        if (ShowError()) return;
      }
    }
  }
  else if (xorg.Modules().Current().Options().Current().Check())
  {
     // save error
     messageIdentifier tempMessage=MyMessage::Get();
     MyMessage::Reset();       // we hope that it isn't a real error

    if (UpdateModuleOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
    MyMessage::Set(tempMessage, funcName);
    if (ShowError()) return;
  }
  else
  {
    if (xorg.Modules().Current().Options().SetCurrent(selection))
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

  if(UpdateModuleOption())
  {
    MyMessage::Append(funcName);
    if (ShowError()) return;
  }
}

// ************************************
// Add, Delete, Copy and De/Activate
// ************************************

const bool GuiFrame::AddDevice(void)
{
  MyString funcName("GuiFrame::AddDevice");
  WriteDebug(funcName);

  if (xorg.Devices().GetSize()>0 &&  xorg.Devices().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  Device temp;
  MyString tempStr("Default Device ");
  temp.SetId(tempStr+MyString::toString(xorg.Devices().GetSize()+1, false));
  xorg.Devices().Append(temp);

  if (xorg.Devices().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelDevice(void)
{
  MyString funcName("GuiFrame::DelDevice");
  WriteDebug(funcName);

  if (xorg.Devices().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this device?"), _("Question: Delete Device?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  bool found=false;      // entry found
  unsigned int i;        // loop counter

  // check if there are some screen that use this device
  for (i=0; i<xorg.Screens().GetSize(); i++)
  {
    if (xorg.Screens()(i).GetDevice()==xorg.Devices().Current().GetId())
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }

      found=true;
      break;  // leave for i-loop
    }
    else if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }

  if (found)
  {
    wxString tempStr;
    tempStr << _("This device is still used by screen \"") << wxString::FromAscii(xorg.Screens()(i).GetId().data()) << _T("\".");
    tempStr << _("\n\nShould I delete this references too? ");
    tempStr << _("Otherwise you must manually resolve the dependence before deleting this device.");

    if (wxMessageBox(tempStr, _("Error: Device used"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

    for (i=0; i<xorg.Screens().GetSize(); i++)
    {
      if (xorg.Screens()(i).GetDevice()==xorg.Devices().Current().GetId())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        xorg.Screens()(i).SetDevice("");
      }
      else if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
       return(true);
      }
    }
  }
  if (xorg.Devices().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::CopyDevice(void)
{
  MyString funcName("GuiFrame::CopyDevice");
  WriteDebug(funcName);

  if (xorg.Devices().GetSize()==0) return(false);

  if (xorg.Devices().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  Device temp(xorg.Devices().Current());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  temp.SetId(temp.GetId()+" (Copy)");
  xorg.Devices().Append(temp);

  if (xorg.Devices().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddDeviceOption(void)
{
  MyString funcName("GuiFrame::AddDeviceOption");
  WriteDebug(funcName);

  if (xorg.Devices().Current().Options().GetSize()>0 && xorg.Devices().Current().Options().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Devices().Current().Options().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Devices().Current().Options().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddDeviceDriverOptions(void)
{
  MyString funcName("GuiFrame::AddDeviceDriverOptions");
  WriteDebug(funcName);

  if (xorg.Devices().Current().Options().GetSize()>0 && xorg.Devices().Current().Options().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  MyString filename;      // filename for loading
  DriverOptionList list;   // driver options

  filename=settings.GetOptionDir()+"/devices/"+xorg.Devices().Current().GetDriver()+".xml";
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (list.Load(filename))
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (list.GetSize()==0)
  {
     wxMessageBox(_("Driver list is empty or does not exist.Options cannot be managed!"));
  }
  else
  {
     OptionDialog optDlg(_("Manage Options..."), list, xorg.Devices().Current());
     if (optDlg.ShowModal() == wxID_OK)
     {
       SetSaved(false);
     }
     else
     {
       if (!MyMessage::OK())
       {
          MyMessage::Append(funcName);
          return(true);
       }
     }
  }

  return(false);
}

const bool GuiFrame::DelDeviceOption(void)
{
  MyString funcName("GuiFrame::DelDeviceOption");
  WriteDebug(funcName);

  if (xorg.Devices().GetSize()==0 || xorg.Devices().Current().Options().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Devices().Current().Options().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActDeviceOption(void)
{
  MyString funcName("GuiFrame::DeActDeviceOption");
  WriteDebug(funcName);

  if (xorg.Devices().GetSize()==0 || xorg.Devices().Current().Options().GetSize()==0) return(false);
  xorg.Devices().Current().Options().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddInputDevice(void)
{
  MyString funcName("GuiFrame::AddInputDevice");
  WriteDebug(funcName);

  if (xorg.InputDevices().GetSize()>0 &&  xorg.InputDevices().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  InputDevice temp;
  MyString tempStr("Default Input Device ");
  temp.SetId(tempStr+MyString::toString(xorg.InputDevices().GetSize()+1, false));
  xorg.InputDevices().Append(temp);

  if (xorg.InputDevices().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (UpdateServerLayout())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddInputDeviceDriverOptions(void)
{
  MyString funcName("GuiFrame::AddInputDeviceDriverOptions");
  WriteDebug(funcName);

  if (xorg.InputDevices().Current().Options().GetSize()>0 && xorg.InputDevices().Current().Options().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  MyString filename;      // filename for loading
  DriverOptionList list;   // driver options

  filename=settings.GetOptionDir()+"/inputdevices/"+xorg.InputDevices().Current().GetDriver()+".xml";
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (list.Load(filename))
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (list.GetSize()==0)
  {
     wxMessageBox(_("Driver list is empty or does not exist.Options cannot be managed!"));
  }
  else
  {
     OptionDialog optDlg(_("Manage Options..."), list, xorg.InputDevices().Current());
     if (optDlg.ShowModal() == wxID_OK)
     {
       SetSaved(false);
     }
     else
     {
       if (!MyMessage::OK())
       {
          MyMessage::Append(funcName);
          return(true);
       }
     }
  }

  return(false);
}

const bool GuiFrame::DelInputDevice(void)
{
  MyString funcName("GuiFrame::DelInputDevice");
  WriteDebug(funcName);

  if (xorg.InputDevices().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this input device?"), _("Question: Delete Input Device?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  bool found=false;      // entry found
  unsigned int i,j;        // loop counter

  // check if there are some server layout that use this input device
  for (i=0; i<xorg.ServerLayouts().GetSize(); i++)
  {
    for (j=0; j<xorg.ServerLayouts()(i).InputDevices().GetSize(); j++)
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      if (xorg.ServerLayouts()(i).InputDevices()(j).GetId()==xorg.InputDevices().Current().GetId())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        found=true;
        break; // leave for j-loop
      }
      else if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    if (found) break; // leave for i-loop
  }

  if (found)
  {
    wxString tempStr;
    tempStr << _("This input device is still used by server layout \"") << wxString::FromAscii(xorg.ServerLayouts()(i).GetId().data()) << _T("\".");
    tempStr << _("\n\nShould I delete this references too? ");
    tempStr << _("Otherwise you must manually resolve the dependence before deleting this input device.");

    if (wxMessageBox(tempStr, _("Error: Input Device used"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

    for (i=0; i<xorg.ServerLayouts().GetSize(); i++)
    {
      for (j=0; j<xorg.ServerLayouts()(i).InputDevices().GetSize(); j++)
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        if (xorg.ServerLayouts()(i).InputDevices()(j).GetId()==xorg.InputDevices().Current().GetId())
        {
          if (!MyMessage::OK())
          {
            MyMessage::Append(funcName);
            return(true);
          }
          if (xorg.ServerLayouts()(i).InputDevices().Delete(j))
          {
            MyMessage::Append(funcName);
            return(true);
          }
          j--;
        }
        else if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
      }
    }
  }
  if (xorg.InputDevices().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::CopyInputDevice(void)
{
  MyString funcName("GuiFrame::CopyInputDevice");
  WriteDebug(funcName);

  if (xorg.InputDevices().GetSize()==0) return(false);

  if (xorg.InputDevices().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  InputDevice temp(xorg.InputDevices().Current());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  temp.SetId(temp.GetId()+" (Copy)");
  xorg.InputDevices().Append(temp);

  if (xorg.InputDevices().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddInputDeviceOption(void)
{
  MyString funcName("GuiFrame::AddInputDeviceOption");
  WriteDebug(funcName);

  if (xorg.InputDevices().Current().Options().GetSize()>0 && xorg.InputDevices().Current().Options().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.InputDevices().Current().Options().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.InputDevices().Current().Options().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelInputDeviceOption(void)
{
  MyString funcName("GuiFrame::DelInputDeviceOption");
  WriteDebug(funcName);

  if (xorg.InputDevices().GetSize()==0 || xorg.InputDevices().Current().Options().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.InputDevices().Current().Options().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActInputDeviceOption(void)
{
  MyString funcName("GuiFrame::DeActInputDeviceOption");
  WriteDebug(funcName);

  if (xorg.InputDevices().GetSize()==0 || xorg.InputDevices().Current().Options().GetSize()==0) return(false);
  xorg.InputDevices().Current().Options().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddMonitor(void)
{
  MyString funcName("GuiFrame::AddMonitor");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()>0 && xorg.Monitors().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  Monitor temp;
  MyString tempStr("Default Monitor ");
  temp.SetId(tempStr+MyString::toString(xorg.Monitors().GetSize()+1, false));
  xorg.Monitors().Append(temp);

  if (xorg.Monitors().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelMonitor(void)
{
  MyString funcName("GuiFrame::DelMonitor");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this monitor?"), _("Question: Delete Monitor?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  bool found=false;      // entry found
  unsigned int i;        // loop counter

  // check if there are some screen that use this device
  for (i=0; i<xorg.Screens().GetSize(); i++)
  {
    if (xorg.Screens()(i).GetMonitor()==xorg.Monitors().Current().GetId())
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }

      found=true;
      break;  // leave for i-loop
    }
    else if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }

  if (found)
  {
    wxString tempStr;
    tempStr << _("This monitor is still used by screen \"") << wxString::FromAscii(xorg.Screens()(i).GetId().data()) << _T("\".");
    tempStr << _("\n\nShould I delete this references too? ");
    tempStr << _("Otherwise you must manually resolve the dependence before deleting this monitor.");

    if (wxMessageBox(tempStr, _("Error: Monitor used"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

    for (i=0; i<xorg.Screens().GetSize(); i++)
    {
      if (xorg.Screens()(i).GetMonitor()==xorg.Monitors().Current().GetId())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        xorg.Screens()(i).SetMonitor("");
      }
      else if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
       return(true);
      }
    }
  }
  if (xorg.Monitors().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::CopyMonitor(void)
{
  MyString funcName("GuiFrame::CopyMonitor");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0) return(false);

  if (xorg.Monitors().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  Monitor temp(xorg.Monitors().Current());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  temp.SetId(temp.GetId()+" (Copy)");
  xorg.Monitors().Append(temp);

  if (xorg.Monitors().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddMonitorOption(void)
{
  MyString funcName("GuiFrame::AddMonitorOption");
  WriteDebug(funcName);

  if (xorg.Monitors().Current().Options().GetSize()>0 && xorg.Monitors().Current().Options().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Monitors().Current().Options().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Monitors().Current().Options().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelMonitorOption(void)
{
  MyString funcName("GuiFrame::DelMonitorOption");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0 || xorg.Monitors().Current().Options().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Monitors().Current().Options().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActMonitorOption(void)
{
  MyString funcName("GuiFrame::DeActMonitorOption");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0 || xorg.Monitors().Current().Options().GetSize()==0) return(false);
  xorg.Monitors().Current().Options().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddMonitorModeline(void)
{
  MyString funcName("GuiFrame::AddMonitorModeline");
  WriteDebug(funcName);

  if (xorg.Monitors().Current().Modelines().GetSize()>0 && xorg.Monitors().Current().Modelines().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Monitors().Current().Modelines().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Monitors().Current().Modelines().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelMonitorModeline(void)
{
  MyString funcName("GuiFrame::DelMonitorModeline");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0 || xorg.Monitors().Current().Modelines().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this modeline?"), _("Question: Delete Modeline?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Monitors().Current().Modelines().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActMonitorModeline(void)
{
  MyString funcName("GuiFrame::DeActMonitorModeline");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0 || xorg.Monitors().Current().Modelines().GetSize()==0) return(false);
  xorg.Monitors().Current().Modelines().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddScreen(void)
{
  MyString funcName("GuiFrame::AddScreen");
  WriteDebug(funcName);

  if (xorg.Screens().GetSize()>0 && xorg.Screens().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

/*
  if (xorg.Devices().GetSize()==0)
  {
    wxMessageBox(_("No devices available to add!"), _("Error in Screen"), wxICON_ERROR);
    return(false);
  }

  if (xorg.Monitors().GetSize()==0)
  {
    wxMessageBox(_("No monitors available to add!"), _("Error in Screen"), wxICON_ERROR);
    return(false);
  }
*/

  Screen temp;
  MyString tempStr("Default Screen ");
  temp.SetId(tempStr+MyString::toString(xorg.Screens().GetSize()+1, false));

  if (xorg.Devices().GetSize()>0)
  {
    temp.SetDevice(xorg.Devices()(0).GetId());   // get first device
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  if (xorg.Monitors().GetSize()>0)
  {
    temp.SetMonitor(xorg.Monitors()(0).GetId());   // get first monitor
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }
  }
  xorg.Screens().Append(temp);

  if (xorg.Screens().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelScreen(void)
{
  MyString funcName("GuiFrame::DelScreen");
  WriteDebug(funcName);

  if (xorg.Screens().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this screen?"), _("Question: Delete Screen?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  bool found=false;      // entry found
  unsigned int i,j;        // loop counter

  // check if there are some server layout that use this screen
  for (i=0; i<xorg.ServerLayouts().GetSize(); i++)
  {
    for (j=0; j<xorg.ServerLayouts()(i).Screens().GetSize(); j++)
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      if (xorg.ServerLayouts()(i).Screens()(j).GetId()==xorg.Screens().Current().GetId())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        found=true;
        break; // leave for j-loop
      }
      else if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    if (found) break; // leave for i-loop
  }

  if (found)
  {
    wxString tempStr;
    tempStr << _("This screen is still used by server layout \"") << wxString::FromAscii(xorg.ServerLayouts()(i).GetId().data()) << _T("\".");
    tempStr << _("\n\nShould I delete this references too? ");
    tempStr << _("Otherwise you must manually resolve the dependence before deleting this screen.");

    if (wxMessageBox(tempStr, _("Error: Screen used"), wxICON_ERROR | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

    for (i=0; i<xorg.ServerLayouts().GetSize(); i++)
    {
      for (j=0; j<xorg.ServerLayouts()(i).Screens().GetSize(); j++)
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        if (xorg.ServerLayouts()(i).Screens()(j).GetId()==xorg.Screens().Current().GetId())
        {
          if (!MyMessage::OK())
          {
            MyMessage::Append(funcName);
            return(true);
          }
          if (xorg.ServerLayouts()(i).Screens().Delete(j))
          {
            MyMessage::Append(funcName);
            return(true);
          }
          j--;
        }
        else if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
      }
    }
  }
  if (xorg.Screens().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::CopyScreen(void)
{
  MyString funcName("GuiFrame::CopyScreen");
  WriteDebug(funcName);

  if (xorg.Screens().GetSize()==0) return(false);

  if (xorg.Screens().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  Screen temp(xorg.Screens().Current());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  temp.SetId(temp.GetId()+" (Copy)");
  xorg.Screens().Append(temp);

  if (xorg.Screens().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddScreenOption(void)
{
  MyString funcName("GuiFrame::AddScreenOption");
  WriteDebug(funcName);

  if (xorg.Screens().Current().Options().GetSize()>0 && xorg.Screens().Current().Options().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Screens().Current().Options().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Screens().Current().Options().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelScreenOption(void)
{
  MyString funcName("GuiFrame::DelScreenOption");
  WriteDebug(funcName);

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Options().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Screens().Current().Options().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActScreenOption(void)
{
  MyString funcName("GuiFrame::DeActScreenOption");
  WriteDebug(funcName);

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Options().GetSize()==0) return(false);
  xorg.Screens().Current().Options().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddScreenDisplay(void)
{
  MyString funcName("GuiFrame::AddScreenDisplay");
  WriteDebug(funcName);

  if (xorg.Screens().Current().Displays().GetSize()>0 && xorg.Screens().Current().Displays().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Screens().Current().Displays().Append(Display());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Screens().Current().Displays().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelScreenDisplay(void)
{
  MyString funcName("GuiFrame::DelScreenDisplay");
  WriteDebug(funcName);

  if (xorg.Screens().Current().Displays().GetSize()==0) return(false);
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (wxMessageBox(_("Do you really want to delete this display?"), _("Question: Delete Display?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Screens().Current().Displays().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddScreenDisplayMode(void)
{
  MyString funcName("GuiFrame::AddScreenDisplayMode");
  WriteDebug(funcName);

  if (xorg.Screens().Current().Displays().Current().Modes().GetSize()>0 && xorg.Screens().Current().Displays().Current().Modes().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Screens().Current().Displays().Current().Modes().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Screens().Current().Displays().Current().Modes().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelScreenDisplayMode(void)
{
  MyString funcName("GuiFrame::DelScreenDisplayMode");
  WriteDebug(funcName);

  if (xorg.Screens().Current().Displays().GetSize()==0 || xorg.Screens().Current().Displays().Current().Modes().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this mode?"), _("Question: Delete Mode?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Screens().Current().Displays().Current().Modes().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddScreenDisplayOption(void)
{
  MyString funcName("GuiFrame::AddScreenDisplayOption");
  WriteDebug(funcName);

  if (xorg.Screens().Current().Displays().Current().Options().GetSize()>0 && xorg.Screens().Current().Displays().Current().Options().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Screens().Current().Displays().Current().Options().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Screens().Current().Displays().Current().Options().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelScreenDisplayOption(void)
{
  MyString funcName("GuiFrame::DelScreenDisplayOption");
  WriteDebug(funcName);

  if (xorg.Screens().Current().Displays().GetSize()==0 || xorg.Screens().Current().Displays().Current().Options().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Screens().Current().Displays().Current().Options().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActScreenDisplayOption(void)
{
  MyString funcName("GuiFrame::DeActScreenDisplayOption");
  WriteDebug(funcName);

  if (xorg.Screens().Current().Displays().GetSize()==0 || xorg.Screens().Current().Displays().Current().Options().GetSize()==0) return(false);
  xorg.Screens().Current().Displays().Current().Options().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddServerLayout(void)
{
  MyString funcName("GuiFrame::AddServerLayout");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().GetSize()>0 && xorg.ServerLayouts().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  ServerLayout temp;
  MyString tempStr("Default Layout ");
  temp.SetId(tempStr+MyString::toString(xorg.ServerLayouts().GetSize()+1, false));
  xorg.ServerLayouts().Append(temp);

  if (xorg.ServerLayouts().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelServerLayout(void)
{
  MyString funcName("GuiFrame::DelServerLayout");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this server layout?"), _("Question: Delete Server Layout?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.ServerLayouts().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::CopyServerLayout(void)
{
  MyString funcName("GuiFrame::CopyServerLayout");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().GetSize()==0) return(false);

  if (xorg.ServerLayouts().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  ServerLayout temp(xorg.ServerLayouts().Current());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  temp.SetId(temp.GetId()+" (Copy)");
  xorg.ServerLayouts().Append(temp);

  if (xorg.ServerLayouts().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddServerLayoutScreen(void)
{
  MyString funcName("GuiFrame::AddServerLayoutScreen");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().Current().Screens().GetSize()>0 && xorg.ServerLayouts().Current().Screens().Current().Check(true))
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.ServerLayouts().Current().Screens().GetSize()>=xorg.Screens().GetSize())
  {
    wxMessageBox(_("No (more) screens available to add!"), _("Error in Server Layout"), wxICON_ERROR);
    return(false);
  }

  // search correct screen to add
  unsigned int i, j;

  for (i=0; i<xorg.Screens().GetSize(); i++)
  {
    for (j=0; j<xorg.ServerLayouts().Current().Screens().GetSize(); j++)
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      if (xorg.Screens()(i).GetId()==xorg.ServerLayouts().Current().Screens()(j).GetId())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        break;  // leave for j-loop
      }
      else if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (j==xorg.ServerLayouts().Current().Screens().GetSize())
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      break; // leave for i-loop
    }
  }
  if (i==xorg.Screens().GetSize())
  {
    wxMessageBox(_("No (more) screens available to add!"), _("Error in Server Layout"), wxICON_ERROR);
    return(false);
  }

  xorg.ServerLayouts().Current().Screens().Append(xorg.Screens()(i));
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.ServerLayouts().Current().Screens().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelServerLayoutScreen(void)
{
  MyString funcName("GuiFrame::DelServerLayoutScreen");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().Screens().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this screen?"), _("Question: Delete Screen?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.ServerLayouts().Current().Screens().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddServerLayoutInputDevice(void)
{
  MyString funcName("GuiFrame::AddServerLayoutInputDevice");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().Current().InputDevices().GetSize()>0 && xorg.ServerLayouts().Current().InputDevices().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.ServerLayouts().Current().InputDevices().GetSize()>=xorg.Screens().GetSize())
  {
    wxMessageBox(_("No (more) input devices available to add!"), _("Error in Server Layout"), wxICON_ERROR);
    return(false);
  }

  // search correct screen to add
  unsigned int i, j;

  for (i=0; i<xorg.InputDevices().GetSize(); i++)
  {
    for (j=0; j<xorg.ServerLayouts().Current().InputDevices().GetSize(); j++)
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      if (xorg.InputDevices()(i).GetId()==xorg.ServerLayouts().Current().InputDevices()(j).GetId())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append(funcName);
          return(true);
        }
        break;  // leave for j-loop
      }
      else if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append(funcName);
      return(true);
    }

    if (j==xorg.ServerLayouts().Current().InputDevices().GetSize())
    {
      if (!MyMessage::OK())
      {
        MyMessage::Append(funcName);
        return(true);
      }
      break; // leave for i-loop
    }
  }
  if (i==xorg.InputDevices().GetSize())
  {
    wxMessageBox(_("No (more) input devices available to add!"), _("Error in Server Layout"), wxICON_ERROR);
    return(false);
  }

  xorg.ServerLayouts().Current().InputDevices().Append(xorg.InputDevices()(i));
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.ServerLayouts().Current().InputDevices().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelServerLayoutInputDevice(void)
{
  MyString funcName("GuiFrame::DelServerLayoutInputDevice");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().InputDevices().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this input device?"), _("Question: Delete Input Device?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.ServerLayouts().Current().InputDevices().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddServerLayoutOption(void)
{
  MyString funcName("GuiFrame::AddServerLayoutOption");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().Current().Options().GetSize()>0 && xorg.ServerLayouts().Current().Options().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.ServerLayouts().Current().Options().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.ServerLayouts().Current().Options().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelServerLayoutOption(void)
{
  MyString funcName("GuiFrame::DelServerLayoutOption");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().Options().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.ServerLayouts().Current().Options().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActServerLayoutOption(void)
{
  MyString funcName("GuiFrame::DeActServerLayoutOption");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().Options().GetSize()==0) return(false);
  xorg.ServerLayouts().Current().Options().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddServerLayoutInputDeviceOption(void)
{
  MyString funcName("GuiFrame::AddServerLayoutInputDeviceOption");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().Current().InputDevices().Current().Options().GetSize()>0 && xorg.ServerLayouts().Current().InputDevices().Current().Options().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.ServerLayouts().Current().InputDevices().Current().Options().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.ServerLayouts().Current().InputDevices().Current().Options().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelServerLayoutInputDeviceOption(void)
{
  MyString funcName("GuiFrame::DelServerLayoutInputDeviceOption");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().InputDevices().GetSize()==0 || xorg.ServerLayouts().Current().InputDevices().Current().Options().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.ServerLayouts().Current().InputDevices().Current().Options().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddServerFlagOption(void)
{
  MyString funcName("GuiFrame::AddServerFlagOption");
  WriteDebug(funcName);

  if (xorg.ServerFlags().GetSize()>0 && xorg.ServerFlags().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.ServerFlags().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.ServerFlags().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelServerFlagOption(void)
{
  MyString funcName("GuiFrame::DelServerFlagOption");
  WriteDebug(funcName);

  if (xorg.ServerFlags().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this server flag?"), _("Question: Delete Server Flag?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.ServerFlags().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActServerFlagOption(void)
{
  MyString funcName("GuiFrame::DeActServerFlagOption");
  WriteDebug(funcName);

  if (xorg.ServerFlags().GetSize()==0) return(false);
  xorg.ServerFlags().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddExtension(void)
{
  MyString funcName("GuiFrame::AddExtension");
  WriteDebug(funcName);

  if (xorg.Extensions().GetSize()>0 && xorg.Extensions().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Extensions().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Extensions().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelExtension(void)
{
  MyString funcName("GuiFrame::DelExtension");
  WriteDebug(funcName);

  if (xorg.Extensions().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this extension?"), _("Question: Delete Extension?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Extensions().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActExtension(void)
{
  MyString funcName("GuiFrame::DeActExtension");
  WriteDebug(funcName);

  if (xorg.Extensions().GetSize()==0) return(false);
  xorg.Extensions().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddFontPath(void)
{
  MyString funcName("GuiFrame::AddFontPath");
  WriteDebug(funcName);

  if (xorg.Files().FontPaths().GetSize()>0 && xorg.Files().FontPaths().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Files().FontPaths().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Files().FontPaths().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelFontPath(void)
{
  MyString funcName("GuiFrame::DelFontPath");
  WriteDebug(funcName);

  if (xorg.Files().FontPaths().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this font path?"), _("Question: Delete Font Path?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Files().FontPaths().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActFontPath(void)
{
  MyString funcName("GuiFrame::DeActFontPath");
  WriteDebug(funcName);

  if (xorg.Files().FontPaths().GetSize()==0) return(false);
  xorg.Files().FontPaths().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddModulePath(void)
{
  MyString funcName("GuiFrame::AddModulePath");
  WriteDebug(funcName);

  if (xorg.Files().ModulePaths().GetSize()>0 && xorg.Files().ModulePaths().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Files().ModulePaths().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Files().ModulePaths().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelModulePath(void)
{
  MyString funcName("GuiFrame::DelModulePath");
  WriteDebug(funcName);

  if (xorg.Files().ModulePaths().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this module path?"), _("Question: Delete Module Path?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Files().ModulePaths().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActModulePath(void)
{
  MyString funcName("GuiFrame::DeActModulePath");
  WriteDebug(funcName);

  if (xorg.Files().ModulePaths().GetSize()==0) return(false);
  xorg.Files().ModulePaths().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddRgbPath(void)
{
  MyString funcName("GuiFrame::AddRgbPath");
  WriteDebug(funcName);

  if (xorg.Files().RgbPaths().GetSize()>0 && xorg.Files().RgbPaths().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Files().RgbPaths().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Files().RgbPaths().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelRgbPath(void)
{
  MyString funcName("GuiFrame::DelRgbPath");
  WriteDebug(funcName);

  if (xorg.Files().RgbPaths().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this RGB path?"), _("Question: Delete RGB Path?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Files().RgbPaths().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActRgbPath(void)
{
  MyString funcName("GuiFrame::DeActRgbPath");
  WriteDebug(funcName);

  if (xorg.Files().RgbPaths().GetSize()==0) return(false);
  xorg.Files().RgbPaths().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddModule(void)
{
  MyString funcName("GuiFrame::AddModule");
  WriteDebug(funcName);

  if (xorg.Modules().GetSize()>0 && xorg.Modules().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Modules().Append(Module());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Modules().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelModule(void)
{
  MyString funcName("GuiFrame::DelModule");
  WriteDebug(funcName);

  if (xorg.Modules().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this module?"), _("Question: Delete Module?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Modules().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActModule(void)
{
  MyString funcName("GuiFrame::DeActModule");
  WriteDebug(funcName);

  if (xorg.Modules().GetSize()==0) return(false);
  xorg.Modules().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

const bool GuiFrame::AddModuleOption(void)
{
  MyString funcName("GuiFrame::AddModuleOption");
  WriteDebug(funcName);

  if (xorg.Modules().Current().Options().GetSize()>0 && xorg.Modules().Current().Options().Current().Check())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  xorg.Modules().Current().Options().Append(Option());
  if (!MyMessage::OK())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  if (xorg.Modules().Current().Options().SetLast())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DelModuleOption(void)
{
  MyString funcName("GuiFrame::DelModuleOption");
  WriteDebug(funcName);

  if (xorg.Modules().GetSize()==0 || xorg.Modules().Current().Options().GetSize()==0) return(false);

  if (wxMessageBox(_("Do you really want to delete this option?"), _("Question: Delete Option?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT)==wxNO) return(false);

  if (xorg.Modules().Current().Options().DeleteCurrent())
  {
    MyMessage::Append(funcName);
    return(true);
  }

  SetSaved(false);
  return(false);
}

const bool GuiFrame::DeActModuleOption(void)
{
  MyString funcName("GuiFrame::DeActModuleOption");
  WriteDebug(funcName);

  if (xorg.Modules().GetSize()==0 || xorg.Modules().Current().Options().GetSize()==0) return(false);
  xorg.Modules().Current().Options().Current().ToogleActive();
  SetSaved(false);
  return(false);
}

// ************************
// Enter text fields
// a) combo boxes
// **************************

void GuiFrame::EnterDevice(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterDevice");
  WriteDebug(funcName);

  unsigned int selection=deviceIdentifier->GetSelection();
  if (selection>=xorg.Devices().GetSize()) return;
  if (selection!= xorg.Devices().GetCurrent()) return;

  MyString newId =(const char*)deviceIdentifier->GetValue().mb_str();

  if (xorg.Devices().Current().GetId()!=newId)
  {
    deviceIdentifier->SetString(selection, deviceIdentifier->GetValue());

    // check if there are some screen that use this device and change the name there
    for (unsigned int i=0; i<xorg.Screens().GetSize(); i++)
    {
      if (xorg.Screens()(i).GetDevice()==xorg.Devices().Current().GetId())
      {
        xorg.Screens()(i).SetDevice(newId);
      }
    }
    xorg.Devices().Current().SetId(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterDeviceOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterDeviceOption");
  WriteDebug(funcName);

  unsigned int selection=deviceOption->GetSelection();
  if (selection>=xorg.Devices().Current().Options().GetSize()) return;
  if (selection!= xorg.Devices().Current().Options().GetCurrent()) return;

  MyString newId=(const char*)deviceOption->GetValue().mb_str();

  if (xorg.Devices().Current().Options().Current().GetName()!=newId)
  {
    deviceOption->SetString(selection, deviceOption->GetValue());

    xorg.Devices().Current().Options().Current().SetName(newId);
    SetSaved(false);

    if (UpdateDeviceOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }

}

void GuiFrame::EnterInputDevice(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterInputDevice");
  WriteDebug(funcName);

  unsigned int selection=inputDeviceIdentifier->GetSelection();
  if (selection>=xorg.InputDevices().GetSize()) return;
  if (selection!= xorg.InputDevices().GetCurrent()) return;

  MyString newId =(const char*)inputDeviceIdentifier->GetValue().mb_str();

  if (xorg.InputDevices().Current().GetId()!=newId)
  {
    inputDeviceIdentifier->SetString(selection, inputDeviceIdentifier->GetValue());

    // check if there are some sever layout that use this device and change the name there
    for (unsigned int i=0; i<xorg.ServerLayouts().GetSize(); i++)
    {
      for (unsigned int j=0; j<xorg.ServerLayouts()(i).InputDevices().GetSize(); j++)
      {
        if (xorg.ServerLayouts()(i).InputDevices()(j).GetId()==xorg.InputDevices().Current().GetId())
        {
          xorg.ServerLayouts()(i).InputDevices()(j).SetId(newId);
        }
      }
    }
    xorg.InputDevices().Current().SetId(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterInputDeviceOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterInputDeviceOption");
  WriteDebug(funcName);

  unsigned int selection=inputDeviceOption->GetSelection();
  if (selection>=xorg.InputDevices().Current().Options().GetSize()) return;
  if (selection!= xorg.InputDevices().Current().Options().GetCurrent()) return;

  MyString newId=(const char*)inputDeviceOption->GetValue().mb_str();

  if (xorg.InputDevices().Current().Options().Current().GetName()!=newId)
  {
    inputDeviceOption->SetString(selection, inputDeviceOption->GetValue());

    xorg.InputDevices().Current().Options().Current().SetName(newId);
    SetSaved(false);

    if (UpdateInputDeviceOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
}

void GuiFrame::EnterMonitor(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterMonitor");
  WriteDebug(funcName);

  unsigned int selection=monitorIdentifier->GetSelection();
  if (selection>=xorg.Monitors().GetSize()) return;
  if (selection!= xorg.Monitors().GetCurrent()) return;

  MyString newId =(const char*)monitorIdentifier->GetValue().mb_str();

  if (xorg.Monitors().Current().GetId()!=newId)
  {
    monitorIdentifier->SetString(selection, monitorIdentifier->GetValue());

    // check if there are some screen that use this device and change the name there
    for (unsigned int i=0; i<xorg.Screens().GetSize(); i++)
    {
      if (xorg.Screens()(i).GetMonitor()==xorg.Monitors().Current().GetId())
      {
        xorg.Screens()(i).SetMonitor(newId);
      }
    }
    xorg.Monitors().Current().SetId(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterMonitorOption");
  WriteDebug(funcName);

  unsigned int selection=monitorOption->GetSelection();
  if (selection>=xorg.Monitors().Current().Options().GetSize()) return;
  if (selection!= xorg.Monitors().Current().Options().GetCurrent()) return;

  MyString newId=(const char*)monitorOption->GetValue().mb_str();

  if (xorg.Monitors().Current().Options().Current().GetName()!=newId)
  {
    monitorOption->SetString(selection, monitorOption->GetValue());

    xorg.Monitors().Current().Options().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorModeline(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterMonitorModeline");
  WriteDebug(funcName);

  unsigned int selection=monitorModelineName->GetSelection();
  if (selection>=xorg.Monitors().Current().Modelines().GetSize()) return;
  if (selection!= xorg.Monitors().Current().Modelines().GetCurrent()) return;

  MyString newId=(const char*)monitorModelineName->GetValue().mb_str();

  if (xorg.Monitors().Current().Modelines().Current().GetName()!=newId)
  {
    monitorModelineName->SetString(selection, monitorModelineName->GetValue());

    xorg.Monitors().Current().Modelines().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterScreen(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterScreen");
  WriteDebug(funcName);

  unsigned int selection=screenIdentifier->GetSelection();
  if (selection>=xorg.Screens().GetSize()) return;
  if (selection!= xorg.Screens().GetCurrent()) return;

  MyString newId =(const char*)screenIdentifier->GetValue().mb_str();

  if (xorg.Screens().Current().GetId()!=newId)
  {
    screenIdentifier->SetString(selection, screenIdentifier->GetValue());

    // check if there are some sever layout that use this device and change the name there
    for (unsigned int i=0; i<xorg.ServerLayouts().GetSize(); i++)
    {
      for (unsigned int j=0; j<xorg.ServerLayouts()(i).Screens().GetSize(); j++)
      {
        if (xorg.ServerLayouts()(i).Screens()(j).GetId()==xorg.Screens().Current().GetId())
        {
          xorg.ServerLayouts()(i).Screens()(j).SetId(newId);
        }
      }
    }
    xorg.Screens().Current().SetId(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterScreenOption");
  WriteDebug(funcName);

  unsigned int selection=screenOption->GetSelection();
  if (selection>=xorg.Screens().Current().Options().GetSize()) return;
  if (selection!= xorg.Screens().Current().Options().GetCurrent()) return;

  MyString newId=(const char*)screenOption->GetValue().mb_str();

  if (xorg.Screens().Current().Options().Current().GetName()!=newId)
  {
    screenOption->SetString(selection, screenOption->GetValue());

    xorg.Screens().Current().Options().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenDisplay(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterScreenDisplay");
  WriteDebug(funcName);

  unsigned int selection=screenDisplayDepth->GetSelection();
  if (selection>=xorg.Screens().Current().Displays().GetSize()) return;
  if (selection!= xorg.Screens().Current().Displays().GetCurrent()) return;

  MyString newId=(const char*)screenDisplayDepth->GetValue().mb_str();

  if (xorg.Screens().Current().Displays().Current().GetDepth()!=newId)
  {
    screenDisplayDepth->SetString(selection, screenDisplayDepth->GetValue());

    xorg.Screens().Current().Displays().Current().SetDepth(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenDisplayMode(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterScreenDisplayMode");
  WriteDebug(funcName);

  unsigned int selection=screenDisplayModes->GetSelection();
  if (selection>=xorg.Screens().Current().Displays().Current().Modes().GetSize()) return;
  if (selection!= xorg.Screens().Current().Displays().Current().Modes().GetCurrent()) return;

  MyString newId=(const char*)screenDisplayModes->GetValue().mb_str();

  if (xorg.Screens().Current().Displays().Current().Modes().Current().GetName()!=newId)
  {
    screenDisplayModes->SetString(selection, screenDisplayModes->GetValue());

    xorg.Screens().Current().Displays().Current().Modes().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenDisplayOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterScreenDisplayOption");
  WriteDebug(funcName);

  unsigned int selection=screenDisplayOption->GetSelection();
  if (selection>=xorg.Screens().Current().Displays().Current().Options().GetSize()) return;
  if (selection!= xorg.Screens().Current().Displays().Current().Options().GetCurrent()) return;

  MyString newId=(const char*)screenDisplayOption->GetValue().mb_str();

  if (xorg.Screens().Current().Displays().Current().Options().Current().GetName()!=newId)
  {
    screenDisplayOption->SetString(selection, screenDisplayOption->GetValue());

    xorg.Screens().Current().Displays().Current().Options().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterServerLayout(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterServerLayout");
  WriteDebug(funcName);

  unsigned int selection=serverLayoutIdentifier->GetSelection();
  if (selection>=xorg.ServerLayouts().GetSize()) return;
  if (selection!= xorg.ServerLayouts().GetCurrent()) return;

  MyString newId =(const char*)serverLayoutIdentifier->GetValue().mb_str();

  if (xorg.ServerLayouts().Current().GetId()!=newId)
  {
    serverLayoutIdentifier->SetString(selection, serverLayoutIdentifier->GetValue());

    xorg.ServerLayouts().Current().SetId(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterServerLayoutOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterServerLayoutOption");
  WriteDebug(funcName);

  unsigned int selection=serverLayoutOption->GetSelection();
  if (selection>=xorg.ServerLayouts().Current().Options().GetSize()) return;
  if (selection!= xorg.ServerLayouts().Current().Options().GetCurrent()) return;

  MyString newId=(const char*)serverLayoutOption->GetValue().mb_str();

  if (xorg.ServerLayouts().Current().Options().Current().GetName()!=newId)
  {
    serverLayoutOption->SetString(selection, serverLayoutOption->GetValue());

    xorg.ServerLayouts().Current().Options().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterServerLayoutInputDeviceOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterServerLayoutInputDeviceOption");
  WriteDebug(funcName);

  unsigned int selection=serverLayoutInputDeviceOption->GetSelection();
  if (selection>=xorg.ServerLayouts().Current().InputDevices().Current().Options().GetSize()) return;
  if (selection!= xorg.ServerLayouts().Current().InputDevices().Current().Options().GetCurrent()) return;

  MyString newId=(const char*)serverLayoutInputDeviceOption->GetValue().mb_str();

  if (xorg.ServerLayouts().Current().InputDevices().Current().Options().Current().GetName()!=newId)
  {
    serverLayoutInputDeviceOption->SetString(selection, serverLayoutInputDeviceOption->GetValue());

    xorg.ServerLayouts().Current().InputDevices().Current().Options().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterServerFlagOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterServerFlagOption");
  WriteDebug(funcName);

  unsigned int selection=serverFlagsOption->GetSelection();
  if (selection>=xorg.ServerFlags().GetSize()) return;
  if (selection!= xorg.ServerFlags().GetCurrent()) return;

  MyString newId=(const char*)serverFlagsOption->GetValue().mb_str();

  if (xorg.ServerFlags().Current().GetName()!=newId)
  {
    serverFlagsOption->SetString(selection, serverFlagsOption->GetValue());

    xorg.ServerFlags().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterExtension(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterExtension");
  WriteDebug(funcName);

  unsigned int selection=extensionOption->GetSelection();
  if (selection>=xorg.Extensions().GetSize()) return;
  if (selection!= xorg.Extensions().GetCurrent()) return;

  MyString newId=(const char*)extensionOption->GetValue().mb_str();

  if (xorg.Extensions().Current().GetName()!=newId)
  {
    extensionOption->SetString(selection, extensionOption->GetValue());

    xorg.Extensions().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterFontPath(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterFontPath");
  WriteDebug(funcName);

  unsigned int selection=fontPath->GetSelection();
  if (selection>=xorg.Files().FontPaths().GetSize()) return;
  if (selection!= xorg.Files().FontPaths().GetCurrent()) return;

  MyString newId=(const char*)fontPath->GetValue().mb_str();

  if (xorg.Files().FontPaths().Current().GetName()!=newId)
  {
    fontPath->SetString(selection, fontPath->GetValue());

    xorg.Files().FontPaths().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterModulePath(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterModulePath");
  WriteDebug(funcName);

  unsigned int selection=modulePath->GetSelection();
  if (selection>=xorg.Files().ModulePaths().GetSize()) return;
  if (selection!= xorg.Files().ModulePaths().GetCurrent()) return;

  MyString newId=(const char*)modulePath->GetValue().mb_str();

  if (xorg.Files().ModulePaths().Current().GetName()!=newId)
  {
    modulePath->SetString(selection, modulePath->GetValue());

    xorg.Files().ModulePaths().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterRgbPath(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterRgbPath");
  WriteDebug(funcName);

  unsigned int selection=rgbPath->GetSelection();
  if (selection>=xorg.Files().RgbPaths().GetSize()) return;
  if (selection!= xorg.Files().RgbPaths().GetCurrent()) return;

  MyString newId=(const char*)rgbPath->GetValue().mb_str();

  if (xorg.Files().RgbPaths().Current().GetName()!=newId)
  {
    rgbPath->SetString(selection, rgbPath->GetValue());

    xorg.Files().RgbPaths().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterModule(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterModule");
  WriteDebug(funcName);

  unsigned int selection=moduleIdentifier->GetSelection();
  if (selection>=xorg.Modules().GetSize()) return;
  if (selection!= xorg.Modules().GetCurrent()) return;

  MyString newId=(const char*)moduleIdentifier->GetValue().mb_str();

  if (xorg.Modules().Current().GetName()!=newId)
  {
    moduleIdentifier->SetString(selection, moduleIdentifier->GetValue());

    xorg.Modules().Current().SetName(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterModuleOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterModuleOption");
  WriteDebug(funcName);

  unsigned int selection=moduleOption->GetSelection();
  if (selection>=xorg.Modules().Current().Options().GetSize()) return;
  if (selection!= xorg.Modules().Current().Options().GetCurrent()) return;

  MyString newId=(const char*)moduleOption->GetValue().mb_str();

  if (xorg.Modules().Current().Options().Current().GetName()!=newId)
  {
    moduleOption->SetString(selection, moduleOption->GetValue());

    xorg.Modules().Current().Options().Current().SetName(newId);
    SetSaved(false);
  }
}

// ************************
// Enter text fields
// b) text fields
// **************************

void GuiFrame::EnterDeviceDriver(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterDeviceDriver");
  WriteDebug(funcName);

  if (xorg.Devices().GetSize()==0) return;

  MyString newId=(const char*)deviceDriver->GetValue().mb_str();

  if (xorg.Devices().Current().GetDriver()!=newId)
  {
    xorg.Devices().Current().SetDriver(newId);
    SetSaved(false);

    if (UpdateDeviceOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
}

void GuiFrame::EnterDeviceBusID(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterDeviceBusID");
  WriteDebug(funcName);

  if (xorg.Devices().GetSize()==0) return;

  MyString newId=(const char*)deviceBusID->GetValue().mb_str();

  if (xorg.Devices().Current().GetBus()!=newId)
  {
    xorg.Devices().Current().SetBus(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterDeviceScreen(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterDeviceScreen");
  WriteDebug(funcName);

  if (xorg.Devices().GetSize()==0) return;

  MyString newId=(const char*)deviceScreen->GetValue().mb_str();

  if (xorg.Devices().Current().GetScreen()!=newId)
  {
    xorg.Devices().Current().SetScreen(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterDevicVideoRam(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterDevicVideoRam");
  WriteDebug(funcName);

  if (xorg.Devices().GetSize()==0) return;

  MyString newId=(const char*)deviceVideoRam->GetValue().mb_str();

  if (xorg.Devices().Current().GetVideoRam()!=newId)
  {
    xorg.Devices().Current().SetVideoRam(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterDeviceOptValue(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterDeviceOptValue");
  WriteDebug(funcName);

  if (xorg.Devices().GetSize()==0 || xorg.Devices().Current().Options().GetSize()==0) return;

  MyString newId=(const char*)deviceOptValue->GetValue().mb_str();

  if (xorg.Devices().Current().Options().Current().GetValue()!=newId)
  {
    xorg.Devices().Current().Options().Current().SetValue(newId);
    SetSaved(false);

  }
}

void GuiFrame::EnterMonitorHorizSync(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterMonitorHorizSync");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0) return;

  MyString newId=(const char*)monitorHorizSync->GetValue().mb_str();

  if (xorg.Monitors().Current().GetHorSync()!=newId)
  {
    xorg.Monitors().Current().SetHorSync(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorVertRefresh(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterMonitorVertRefresh");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0) return;

  MyString newId=(const char*)monitorVertRefresh->GetValue().mb_str();

  if (xorg.Monitors().Current().GetVertRef()!=newId)
  {
    xorg.Monitors().Current().SetVertRef(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorDisplay(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterMonitorDisplay");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0) return;

  MyString newId=(const char*)monitorDisplay->GetValue().mb_str();

  if (xorg.Monitors().Current().GetDisplaySize()!=newId)
  {
    xorg.Monitors().Current().SetDisplaySize(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorGamma(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterMonitorGamma");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0) return;

  MyString newId=(const char*)monitorGamma->GetValue().mb_str();

  if (xorg.Monitors().Current().GetGamma()!=newId)
  {
    xorg.Monitors().Current().SetGamma(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorOptValue(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterMonitorOptValue");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0 || xorg.Monitors().Current().Options().GetSize()==0) return;

  MyString newId=(const char*)monitorOptValue->GetValue().mb_str();

  if (xorg.Monitors().Current().Options().Current().GetValue()!=newId)
  {
    xorg.Monitors().Current().Options().Current().SetValue(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterMonitorModelineOption(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterMonitorModelineOption");
  WriteDebug(funcName);

  if (xorg.Monitors().GetSize()==0 || xorg.Monitors().Current().Modelines().GetSize()==0) return;

  MyString newId=(const char*)monitorModelineOption->GetValue().mb_str();

  if (xorg.Monitors().Current().Modelines().Current().GetValue()!=newId)
  {
    xorg.Monitors().Current().Modelines().Current().SetValue(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterInputDeviceDriver(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterInputDeviceDriver");
  WriteDebug(funcName);

  if (xorg.InputDevices().GetSize()==0) return;

  MyString newId=(const char*)inputDeviceDriver->GetValue().mb_str();

  if (xorg.InputDevices().Current().GetDriver()!=newId)
  {
    xorg.InputDevices().Current().SetDriver(newId);
    SetSaved(false);

    if (UpdateInputDeviceOption())
    {
      MyMessage::Append(funcName);
      if (ShowError()) return;
    }
  }
}

void GuiFrame::EnterInputDeviceOptValue(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterInputDeviceOptValue");
  WriteDebug(funcName);

  if (xorg.InputDevices().GetSize()==0 || xorg.InputDevices().Current().Options().GetSize()==0) return;

  MyString newId=(const char*)inputDeviceOptValue->GetValue().mb_str();

  if (xorg.InputDevices().Current().Options().Current().GetValue()!=newId)
  {
    xorg.InputDevices().Current().Options().Current().SetValue(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenDepth(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterScreenDepth");
  WriteDebug(funcName);

  if (xorg.Screens().GetSize()==0) return;

  MyString newId=(const char*)screenDepth->GetValue().mb_str();

  if (xorg.Screens().Current().GetDefaultDepth()!=newId)
  {
    xorg.Screens().Current().SetDefaultDepth(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenOptValue(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterScreenOptValue");
  WriteDebug(funcName);

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Options().GetSize()==0) return;

  MyString newId=(const char*)screenOptValue->GetValue().mb_str();

  if (xorg.Screens().Current().Options().Current().GetValue()!=newId)
  {
    xorg.Screens().Current().Options().Current().SetValue(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenDisplayViewport(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterScreenDisplayViewport");
  WriteDebug(funcName);

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Displays().GetSize()==0) return;

  MyString newId=(const char*)screenDisplayViewport->GetValue().mb_str();

  if (xorg.Screens().Current().Displays().Current().GetViewPort()!=newId)
  {
    xorg.Screens().Current().Displays().Current().SetViewPort(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenDisplayVirtual(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterScreenDisplayVirtual");
  WriteDebug(funcName);

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Displays().GetSize()==0) return;

  MyString newId=(const char*)screenDisplayVirtual->GetValue().mb_str();

  if (xorg.Screens().Current().Displays().Current().GetVirtual()!=newId)
  {
    xorg.Screens().Current().Displays().Current().SetVirtual(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterScreenDisplayOptValue(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterScreenDisplayOptValue");
  WriteDebug(funcName);

  if (xorg.Screens().GetSize()==0 || xorg.Screens().Current().Displays().GetSize()==0 || xorg.Screens().Current().Displays().Current().Options().GetSize()==0) return;

  MyString newId=(const char*)screenDisplayOptValue->GetValue().mb_str();

  if (xorg.Screens().Current().Displays().Current().Options().Current().GetValue()!=newId)
  {
    xorg.Screens().Current().Displays().Current().Options().Current().SetValue(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterServerLayoutScreenNumber(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterServerLayoutScreenNumber");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().Screens().GetSize()==0) return;

  MyString newId=(const char*)serverLayoutScreenNumber->GetValue().mb_str();

  if (xorg.ServerLayouts().Current().Screens().Current().GetNumber()!=newId)
  {
    xorg.ServerLayouts().Current().Screens().Current().SetNumber(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterServerLayoutScreenPosition(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterServerLayoutScreenPosition");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().Screens().GetSize()==0) return;

  MyString newId=(const char*)serverLayoutScreenPosition->GetValue().mb_str();

  if (xorg.ServerLayouts().Current().Screens().Current().GetPosition()!=newId)
  {
    xorg.ServerLayouts().Current().Screens().Current().SetPosition(newId);
    SetSaved(false);

    newId.MakeLower();
    newId.StripSpaces();
    if (newId=="rightof" || newId=="leftof" || newId=="above" || newId=="below")
    {
      serverLayoutScreen2->Enable(true);
    }
    else
    {
      serverLayoutScreen2->Enable(false);
      // xorg.ServerLayouts().Current().Screens().Current().SetSecond("");
    }
  }
}

void GuiFrame::EnterServerLayoutOptValue(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterServerLayoutOptValue");
  WriteDebug(funcName);

  if (xorg.ServerLayouts().GetSize()==0 || xorg.ServerLayouts().Current().Options().GetSize()==0) return;

  MyString newId=(const char*)serverLayoutOptValue->GetValue().mb_str();

  if (xorg.ServerLayouts().Current().Options().Current().GetValue()!=newId)
  {
    xorg.ServerLayouts().Current().Options().Current().SetValue(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterServerFlagsOptValue(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterServerFlagsOptValue");
  WriteDebug(funcName);

  if (xorg.ServerFlags().GetSize()==0) return;

  MyString newId=(const char*)serverFlagsOptValue->GetValue().mb_str();

  if (xorg.ServerFlags().Current().GetValue()!=newId)
  {
    xorg.ServerFlags().Current().SetValue(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterExtensionOptValue(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterExtensionOptValue");
  WriteDebug(funcName);

  if (xorg.Extensions().GetSize()==0) return;

  MyString newId=(const char*)extensionOptValue->GetValue().mb_str();

  if (xorg.Extensions().Current().GetValue()!=newId)
  {
    xorg.Extensions().Current().SetValue(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterModuleOptValue(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterModuleOptValue");
  WriteDebug(funcName);

  if (xorg.Modules().GetSize()==0 || xorg.Modules().Current().Options().GetSize()==0) return;

  MyString newId=(const char*)moduleOptValue->GetValue().mb_str();

  if (xorg.Modules().Current().Options().Current().GetValue()!=newId)
  {
    xorg.Modules().Current().Options().Current().SetValue(newId);
    SetSaved(false);
  }
}

void GuiFrame::EnterDriMode(wxCommandEvent& event)
{
  MyString funcName("GuiFrame::EnterDriMode");
  WriteDebug(funcName);

  MyString newId=(const char*)driMode->GetValue().mb_str();

  if (xorg.Dri().GetMode()!=newId)
  {
    xorg.Dri().SetMode(newId);
    SetSaved(false);
  }
}
